<?php
  // Create menu
  $title = __('Admin configuration', 'admin_permission');
  apm_menu($title);

  $id = Params::getParam('adminId');

  if(Params::getParam('plugin_action') == 'done' && $id > 0) {
    $perms = Params::getParam('perms');
    ModelAPM::newInstance()->deleteData($id);

    if(count($perms) > 0) {
      foreach($perms as $key => $val) {
        ModelAPM::newInstance()->replaceData(array(
          'fk_i_admin_id' => osc_esc_html($id),
          'fk_i_permission_id' => osc_esc_html($key),
          'i_grantor_id' => osc_logged_admin_id(),
          's_type' => osc_esc_html(Params::getParam('s_type')),
          'dt_date' => date('Y-m-d H:i:s')
        ));
      }        
    }
    
    osc_add_flash_ok_message(__('Admin permission successfully updated.', 'admin_permission'), 'admin');
    header('Location:' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=admin_permission/admin/admin_edit.php&adminId=' . $id);
    exit;
  }

  $admin = Admin::newInstance()->findByPrimaryKey($id);
  $perms = array();
  
  if($id <> '' && $id > 0) {
    $perms = ModelAPM::newInstance()->getAdminPerm($id);
    
    if($id == apm_param('primary_admin')) {
      osc_add_flash_error_message(__('Permissions for primary admin cannot be set.', 'admin_permission'), 'admin');
      header('Location:' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=admin_permission/admin/admins.php');
      exit;
    }
  }


  // REMOVE PERMISSIONS
  if(Params::getParam('what') == 'remove') { 
    ModelAPM::newInstance()->deleteData($id);
    osc_add_flash_ok_message(__('Admin permissions successfully removed', 'admin_permission'), 'admin');
    
    header('Location:' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=admin_permission/admin/admins.php');
    exit;
  }
  
  $perm_ids = array();
  if(is_array($perms) && !empty($perms)) {
    $perm_ids = array_column($perms, 'fk_i_permission_id');
  }
?>


<div class="mb-body">
  <!-- CONFIGURE SECTION -->
  <div class="mb-box mb-account-detail">
    <div class="mb-head">
      <i class="fa fa-user"></i> <?php _e('Admin permission configuration', 'admin_permission'); ?>
    </div>

    <div class="mb-inside">
      <form name="promo_form" id="promo_form" action="<?php echo osc_admin_base_url(true); ?>" method="POST" enctype="multipart/form-data" >
        <input type="hidden" name="page" value="plugins" />
        <input type="hidden" name="action" value="renderplugin" />
        <input type="hidden" name="file" value="<?php echo osc_plugin_folder(__FILE__); ?>admin_edit.php" />
        <input type="hidden" name="plugin_action" value="done" />
        <input type="hidden" name="adminId" value="<?php echo $id; ?>" />
        
        <div class="mb-notes">
          <div class="mb-line"><?php _e('You can add new permissions there, usually to restrict access to plugin pages or different custom sections.', 'admin_permission'); ?></div>
        </div>

        <div class="mb-row">
          <label for="pk_i_id"><span><?php _e('Admin ID', 'admin_permission'); ?></span></label> 
          <input size="10" disabled id="pk_i_id" type="text" value="<?php echo $id; ?>" />
        </div>

        <div class="mb-row">
          <label for="s_section"><span><?php _e('Name', 'admin_permission'); ?></span></label> 
          <input disabled size="50" id="s_name" type="text" value="<?php echo osc_esc_html($admin['s_name']); ?>" />
        </div>
        
        <div class="mb-row">
          <label for="s_role"><span><?php _e('Role', 'admin_permission'); ?></span></label> 
          <input disabled size="30" id="s_role" type="text" value="<?php echo osc_esc_html($admin['b_moderator'] == 1 ? __('Moderator', 'admin_permission') : ($admin['pk_i_id'] == apm_param('primary_admin') ? __('Super admin', 'admin_permission') : __('Admin', 'admin_permission'))); ?>" />
        </div>

        <div class="mb-row">
          <label for="s_name"><span><?php _e('Access type', 'admin_permission'); ?></span></label> 
          <select name="s_type" required>
            <option value="GRANT" <?php if(@$perms[0]['s_type'] == 'GRANT') { ?>selected="selected"<?php } ?>><?php _e('Grant access', 'admin_permission'); ?></option>
            <option value="RESTRICT" <?php if(@$perms[0]['s_type'] == 'RESTRICT') { ?>selected="selected"<?php } ?>><?php _e('Restrict access', 'admin_permission'); ?></option>
          </select>

          <div class="mb-explain"><?php _e('Select if you want to grant or restrict access to admin.', 'admin_permission'); ?></div>
        </div>
        
        <div class="mb-row">
          <label for=""><span><?php _e('Permissions', 'admin_permission'); ?></span></label> 
          
          <div class="mb-perms-sel">
            <?php $sections = ModelAPM::newInstance()->getSections(); ?>
            
            <?php foreach($sections as $s) { ?>
              <div class="mb-sec">
                <label class="seclab xn" title="<?php echo osc_esc_html(__('Select all', 'admin_permission')); ?>">
                  <input type="checkbox" class="sec-check" value="1" />
                  <?php echo $s['s_section']; ?></label>
                </label>
                
                <div class="mb-ps">
                  <?php $ps = ModelAPM::newInstance()->getPerms($s['s_section']); ?>
                  
                  <?php foreach($ps as $p) { ?>
                    <label class="mb-has-tooltip-topleft xn" title="<?php echo osc_esc_html($p['s_params']); ?>">
                      <input type="checkbox" name="perms[<?php echo $p['pk_i_id']; ?>]" value="1" <?php if(in_array($p['pk_i_id'], $perm_ids)) { ?>checked<?php } ?> />
                      <?php echo $p['s_name']; ?>
                    </label>
                  <?php } ?>
                </div>
              </div>
            <?php } ?>
          </div>
          
          <div class="mb-explain"><?php _e('Select permissions which you would like to grant or restrict.', 'admin_permission'); ?></div>
        </div>

        
        <div class="mb-row">&nbsp;</div>

        <div class="mb-foot">
          <button type="submit" class="mb-button"><?php _e('Update', 'admin_permission');?></button>
        </div>
      </form>
    </div>
  </div>
  
</div>

<?php echo apm_footer(); ?>