<?php
  // Create menu
  $title = __('Permission configuration', 'admin_permission');
  apm_menu($title);

  $id = Params::getParam('permId');

  if(Params::getParam('plugin_action') == 'done') {
    $params = Params::getParam('s_params');
    $params_sanitized = array();
    
    foreach($params as $row) {
      $key = trim(osc_esc_html($row['param']));

      if($key != '') {
        $val = trim(osc_esc_html($row['value']));
        $val = implode(',', array_filter(array_map('trim', explode(',', $val))));
        $params_sanitized[$key] = $val;
      }
    }
    
    $data = array(
      's_section' => osc_esc_html(Params::getParam('s_section')),
      's_name' => osc_esc_html(Params::getParam('s_name')),
      's_params' => json_encode($params_sanitized)
    );

    if(trim(Params::getParam('s_section')) == '' || trim(Params::getParam('s_name')) == '') {
      osc_add_flash_error_message(__('Name and section cannot be empty.', 'admin_permission'), 'admin');
    } else if($id == '' || $id == 0) {
      $data['i_core'] = 0;
      $id = ModelAPM::newInstance()->insertPermData($data);
      osc_add_flash_ok_message(__('Permission successfully created.', 'admin_permission'), 'admin');
    } else {
      ModelAPM::newInstance()->updatePermData($id, $data);
      osc_add_flash_ok_message(__('Permission successfully updated.', 'admin_permission'), 'admin');
    }

    header('Location:' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=admin_permission/admin/permission_edit.php&permId=' . $id);
    exit;
  }


  $perm = false;
  
  if($id <> '' && $id > 0) {
    $perm = ModelAPM::newInstance()->getPerm($id);
    
    if(@$perm['i_core'] == 1 && !apm_is_debug()) {
      osc_add_flash_error_message(__('Core permissions cannot be edited.', 'admin_permission'), 'admin');
      header('Location:' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=admin_permission/admin/permissions.php');
      exit;
    }
  }


  // REMOVE PERMISSION
  if(Params::getParam('what') == 'delete') { 
    $rem = ModelAPM::newInstance()->getPerm($id);

    if($rem !== false && $rem['i_core'] != 1 && !apm_is_demo()) {
      ModelAPM::newInstance()->deletePermData($id);
      osc_add_flash_ok_message(__('Permission successfully removed', 'admin_permission'), 'admin');
    } else {
      osc_add_flash_error_message(__('Permission could not be removed', 'admin_permission'), 'admin');
    }
    
    header('Location:' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=admin_permission/admin/permissions.php');
    exit;
  }
  
  
  $params_prepared = array();

  if(isset($perm['s_params'])) {
    $params = json_decode($perm['s_params'], true);
    
    if(count($params) > 0) {
      foreach($params as $key => $val) {
        $params_prepared[] = array('param' => $key, 'value' => $val);
      }
    }
  }
?>


<div class="mb-body">
  <!-- CONFIGURE SECTION -->
  <div class="mb-box mb-account-detail">
    <div class="mb-head">
      <i class="fa fa-key"></i> <?php _e('Permission configuration', 'admin_permission'); ?>
    </div>

    <div class="mb-inside">
      <form name="promo_form" id="promo_form" action="<?php echo osc_admin_base_url(true); ?>" method="POST" enctype="multipart/form-data" >
        <input type="hidden" name="page" value="plugins" />
        <input type="hidden" name="action" value="renderplugin" />
        <input type="hidden" name="file" value="<?php echo osc_plugin_folder(__FILE__); ?>permission_edit.php" />
        <input type="hidden" name="plugin_action" value="done" />
        <input type="hidden" name="permId" value="<?php echo $id; ?>" />
        
        <div class="mb-notes">
          <div class="mb-line"><?php _e('You can add new permissions there, usually to restrict access to plugin pages or different custom sections.', 'admin_permission'); ?></div>
        </div>

        <?php if($id > 0) { ?>
          <div class="mb-row">
            <label for="pk_i_id"><span><?php _e('Permission ID', 'admin_permission'); ?></span></label> 
            <input size="10" disabled id="pk_i_id" type="text" value="<?php echo $id; ?>" />
          </div>
        <?php } ?>

        <div class="mb-row">
          <label for="s_section"><span><?php _e('Section', 'admin_permission'); ?></span></label> 
          <input required size="40" name="s_section" id="s_section" type="text" value="<?php echo isset($perm['s_section']) ? $perm['s_section'] : Params::getParam('section'); ?>" />

          <div class="mb-explain"><?php _e('Define in which section (grouping) will permission be shown.', 'admin_permission'); ?></div>
        </div>
        
        <div class="mb-row">
          <label for="s_name"><span><?php _e('Name', 'admin_permission'); ?></span></label> 
          <input required size="70" name="s_name" id="s_name" type="text" value="<?php echo isset($perm['s_name']) ? $perm['s_name'] : ''; ?>" />

          <div class="mb-explain"><?php _e('Name that identifies and describes this permission.', 'admin_permission'); ?></div>
        </div>

        <div class="mb-row">
          <label for="parameters"><span><?php _e('Parameters', 'admin_permission'); ?></span></label> 

          <div class="mb-params-list">
            <?php for($i=0;$i<10;$i++) { ?>
              <div class="mb-line">
                <input size="20" <?php if($i == 0) { ?>required<?php } ?> name="s_params[<?php echo $i; ?>][param]" type="text" class="p1" value="<?php echo @$params_prepared[$i]['param']; ?>" placeholder="<?php echo osc_esc_html(__('Parameter name', 'admin_permission')); ?>"/>
                <input size="40" name="s_params[<?php echo $i; ?>][value]" type="text" class="p2" value="<?php echo @$params_prepared[$i]['value']; ?>" placeholder="<?php echo osc_esc_html(__('Parameter value', 'admin_permission')); ?>"/>
              </div>
            <?php } ?>
          </div>
          
          <div class="mb-explain">
            <div class="mb-line"><?php _e('You can get parameter names and values from URL of section. Example: To restrict access into /oc-admin/index.php?page=items&action=item_edit&... you would define this set of params & vals - page: items; action: item_edit', 'admin_permission'); ?></div>
            <div class="mb-line"><?php _e('Enter "*" (without quotes) into value to apply on all variations of parameter values (wildcard).', 'admin_permission'); ?></div>
            <div class="mb-line"><?php _e('Enter "-" (without quotes) into value to apply in case parameter is defined and is empty/blank.', 'admin_permission'); ?></div>
            <div class="mb-line"><?php _e('Enter "" (without quotes) into value to apply in case parameter is not defined or is empty/blank.', 'admin_permission'); ?></div>
            <div class="mb-line"><?php _e('You can enter multiple values for same parameter, delimit them with comma (,), no white spaces needed, do not use for special chars (*, -, empty space).', 'admin_permission'); ?></div>
          </div>
        </div>

        
        <div class="mb-row">&nbsp;</div>

        <div class="mb-foot">
          <?php if($id == '' || $id == 0) { ?>
            <button type="submit" class="mb-button"><?php _e('Create', 'admin_permission');?></button>
          <?php } else { ?>
            <button type="submit" class="mb-button"><?php _e('Update', 'admin_permission');?></button>
          <?php } ?>
        </div>
      </form>
    </div>
  </div>
  
</div>

<?php echo apm_footer(); ?>