<?php

// MANAGE ACCESS
function apm_manage_access() {
  $restrict = false;

  $params = Params::getParamsAsArray();
  $data = ModelAPM::newInstance()->getAdminPerm(osc_logged_admin_id());
  
  if(!is_array($data) || empty($data) || count($data) <= 0 || apm_param('enable') != 1) {
    return false;
  }
  
  if(osc_logged_admin_id() == apm_param('primary_admin')) {
    return false; 
  }

  // Do not block ajax calls
  if(Params::getParam('page') == 'ajax' || Params::getParam('page') == 'login' || Params::getParam('action') == 'logout') {
    return false;
  }

  // Home page accesse cannot be blocked
  // if(Params::getParam('page') == '' && Params::getParam('action') == '') {
    // osc_add_flash_warning_message(sprintf(__('You don\'t have enough permissions to access this section or do this action (%s).', 'admin_permission'), __('HOME', 'admin_permission')), 'admin');
    // return false;
  // }
  
  $redirect1 = (Params::getParam('apmRedirectFirst') == 1 ? true : false);
  $redirect2 = (Params::getParam('apmRedirectSecond') == 1 ? true : false);
  
  foreach($data as $line) {
    $type = $line['s_type'];
    $rule_params = json_decode($line['s_params'], true);

    $restrict_line = ($type == 'RESTRICT' ? false : true);
    $matched = '';

    if(is_array($rule_params) && count($rule_params) > 0) {
      if($type == 'RESTRICT') {
        $check = apm_params_match_all($rule_params, true);

        if($check === true) {
          $restrict_line = true;
          $matched = $line['s_section'] . ' / ' . $line['s_name'] . ': ' . $line['s_params'];
          break;
        }
      } else {
        $check = apm_params_match_all($rule_params, true);

        if($check === true) {
          $restrict_line = false;
          $matched = $line['s_section'] . ' / ' . $line['s_name'] . ': ' . $line['s_params'];
          break;
        }
      }
    }
  }
  
  // All parameters matched restriction rule
  if($restrict_line === true) {
    osc_add_flash_warning_message(__('You don\'t have enough permissions to access this section or do this action.', 'admin_permission'), 'admin');
    $restrict = true;
  }
    
  if($restrict === true && $type == 'RESTRICT' || $restrict === true && $type == 'GRANT') {
    if($redirect2) {
      echo __('You don\'t have enough permissions to access section you wanted, but also do not have permissions to access home page. Please contact administrator to provide you link where you have access.', 'admin_permission');
      exit;
    } else if(Params::getParam('page') == 'main' && Params::getParam('action') == 'widget') {
      echo __('You don\'t have enough permissions to access this section', 'admin_permission');
      exit;
    } else if($redirect1) {
      $redirect = apm_admin_redirect_url() . '&apmRedirectSecond=1';
    } else {
      $redirect = osc_get_http_referer();

      if($redirect == '') {
        $redirect = apm_admin_redirect_url();
      }
      
      if(strpos($redirect, 'page=login') !== false) {
        $redirect = apm_admin_redirect_url();
        Session::newInstance()->_dropMessage('admin');
      }

      if(count(explode('apmRedirectFirst', $redirect)) == 1 && count(explode('apmRedirectSecond', $redirect)) == 1) {
        if(count(explode('?', $redirect)) == 1) {
          $redirect .= '?apmRedirectFirst=1';
        } else {
          $redirect .= '&apmRedirectFirst=1';
        }
      }
    }

    // echo ($matched <> '' ? $matched : 'No match');
    // exit;
  
    header('Location: ' . $redirect);
    exit;
  }
}

osc_add_hook('init_admin', 'apm_manage_access', 1);


// CHECK IF ALL PARAMS ARE MATCHED
function apm_params_match_all($params, $type = true, $partial = false) {
  $results = array();
  
  if(is_array($params) && count($params) > 0) {
    foreach($params as $key => $vals) {
      $vals_ = explode(',', $vals);

      if(
        Params::getParam($key) != '' && in_array(Params::getParam($key), $vals_)
        || $key == 'page' && $vals == 'main' && count($params) == 1 && Params::getParam('page') == '' && Params::getParam('action') == ''  // access to home page
        || Params::existParam($key) && (in_array('*', $vals_) || $vals == '*')   // works for all variations of parameter value
        || Params::existParam($key) && Params::getParam($key) == '' && (in_array('-', $vals_) || $vals == '-')   // works only when parameter exists and is blank
        || Params::getParam($key) == '' && $vals == ''      // i.e. if action = ""
      ) {
        $results[] = true;
      } else {
        $results[] = false;
      }
    }
  } else {
    return true;
  }
  
  if(!$partial) {
    if($type === true && count(array_unique($results)) == 1 && $results[0] == true) {
      return true;  // all rules matched "true"
    }
    
    if($type === false && count(array_unique($results)) == 1 && $results[0] == false) {
      return true;  // all rules matched "false"
    }
  } else {
    if($type === true && in_array(true, $results)) {
      return true;  // at least one rule matched "true"
    }
    
    if($type === false && in_array(false, $results)) {
      return true;  // at least one rule matched "false"
    }    
  }
  
  return false; 
}


// GET ADMIN REDIRECT PAGE
function apm_admin_redirect_url($admin_id = NULL) {
  if($admin_id === NULL) {
    $admin_id = osc_logged_admin_id();
  }
  
  $all = ModelAPM::newInstance()->getPerms2();
  $current = ModelAPM::newInstance()->getAdminPerm($admin_id);
  $current_ids = array_column($current, 'fk_i_permission_id');
  $type = (@$current[0]['s_type'] == 'GRANT' ? 'GRANT' : 'RESTRICT');
  
  if($type == 'GRANT') {
    return apm_url_from_params($current[0]['s_params']);
    
  } else if($type == 'RESTRICT') {
    foreach($all as $a) {
      if(!in_array($a['pk_i_id'], $current_ids)) {
        return apm_url_from_params($a['s_params']);
      }
    }
  }
}


// CREATE URL FROM PARAMS
function apm_url_from_params($params_json) {
  $array = json_decode($params_json, true);

  if(!is_array($array) || count($array) <= 0) {
    return osc_admin_base_url();
  }
  
  $url = osc_admin_base_url(true);
  
  $search_params = array();
  foreach($array as $key => $val) {
    $search_params[] = $key . '=' . apm_translate_val($val);
  }
  
  if(count($search_params) > 0) {
    $url .= '?' . implode('&', $search_params);
  }
  
  return $url;
}


// TRANSLATE VALUES TO URL
function apm_translate_val($val) {
  $vals = explode(',', $val);
  $v = $vals[0];
  
  if($v == '-' || $v == '*') {
    return '';
  }
  
  return $v;
}


// CORE FUNCTIONS
function apm_param($name) {
  return osc_get_preference($name, 'plugin-admin_permission');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}

// CHECK IF DEBUG MODE IS ON
function apm_is_debug() {
  if(defined('APM_DEBUG') && APM_DEBUG == true) {
    return true;
  }
  
  return false;
}


// CHECK IF RUNNING ON DEMO
function apm_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}



// CATEGORIES WORK
function apm_cat_tree($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
      $array[$c['pk_i_id']]['sub'] = apm_cat_sub($list, $c['pk_i_id']);
    }
  }

  return $array;
}

function apm_cat_sub($list, $parent_id) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {  echo $c['s_name'];
        $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
        $array[$c['pk_i_id']]['sub'] = apm_cat_sub($list, $c['pk_i_id']);
      }
    }
  }
      
  return $array;
}

function apm_cat_list($selected = array(), $categories = '', $level = 0) {
  if($categories == '' || $level == 0) {
    $categories = apm_cat_tree($categories);
  }


  foreach($categories as $c) {
    echo '<option value="' . $c['pk_i_id'] . '" ' . (in_array($c['pk_i_id'], $selected) ? 'selected="selected"' : '') . '>' . str_repeat('-', $level) . ($level > 0 ? ' ' : '') . $c['s_name'] . '</option>';

    if(@count($c['sub']) > 0) {
      apm_cat_list($selected, $c['sub'], $level + 1);
    }
  }
}


function apm_list_values_ol($values) {
 if(count($values) > 0 && is_array($values)) {
    foreach($values as $v) {
      ?>

      <li class="mb-val" id="val_<?php echo $v['pk_i_id']; ?>">
        <?php apm_div_value($v); ?>
      
        <ol>
          <?php 
            if(isset($v['values']) && count($v['values']) > 0) { 
              apm_list_values_ol($v['values']); 
            }
          ?>
        </ol>
      </li>
    <?php
    }
  }
}

?>