<?php
class ModelAPM extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_admin_user() {
  return DB_TABLE_PREFIX.'t_admin';
}

public function getTable_perm() {
  return DB_TABLE_PREFIX.'t_apm_permission';
}


public function getTable_admin() {
  return DB_TABLE_PREFIX.'t_apm_admin';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelAPM<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install($version = '') {
  if($version == '') {
    $this->import('admin_permission/model/struct.sql');
    osc_set_preference('version', 100, 'plugin-admin_permission', 'INTEGER');
  }

}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_perm()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_admin()));


  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-admin_permission'";
  $this->dao->query($query);
}


// GET ADMINS
public function getAdmins($only_admin = true) {
  $this->dao->select();
  $this->dao->from($this->getTable_admin_user());
  
  if($only_admin) {
    $this->dao->where('b_moderator', 0);
  }
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}



// GET ADMIN RESTRICTION DATA
public function getAdminPerm($id) {
  if($id <= 0) {
    return array();
  }
  
  $this->dao->select('a.*, p.*');
  $this->dao->from($this->getTable_admin() . ' as a');
  $this->dao->join($this->getTable_perm() . ' as p', '(a.fk_i_permission_id = p.pk_i_id)', 'INNER');

  $this->dao->where('a.fk_i_admin_id', $id);
  $this->dao->orderby('p.pk_i_id', 'ASC');

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// GET ADMIN RESTRICTION DATA
public function getAdminDataRaw($id) {
  if($id <= 0) {
    return array();
  }
  
  $this->dao->select();
  $this->dao->from($this->getTable_admin());
  $this->dao->where('fk_i_admin_id', $id);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// GET SECTIONS
public function getSections() {
  $this->dao->select('DISTINCT s_section');
  $this->dao->from($this->getTable_perm());
  $this->dao->orderby('i_core DESC, pk_i_id', 'ASC');
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// GET ADMIN PERMISSIONS
public function getPerms($section = NULL) {
  $this->dao->select();
  $this->dao->from($this->getTable_perm());
  
  if($section !== NULL) {
    $this->dao->where('s_section', $section);
  }
  
  $this->dao->orderby('i_core DESC, s_section ASC, pk_i_id', 'ASC');
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// GET ADMIN PERMISSIONS
public function getPerms2() {
  $this->dao->select();
  $this->dao->from($this->getTable_perm());

  $this->dao->orderby('pk_i_id', 'ASC');
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}



// GET PERMISSION
public function getPerm($id) {
  $this->dao->select();
  $this->dao->from($this->getTable_perm());
  $this->dao->where('pk_i_id', $id);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }
  
  return false;
}




// INSERT ADMIN DATA
public function insertData($data) {
  $this->dao->insert($this->getTable_admin(), $data);
}


// UPDATE ADMIN DATA
public function updateData($id, $data) {
  $this->dao->update($this->getTable_admin(), $data, array('fk_i_admin_id' => $id));
}


// REPLACE ADMIN DATA
public function replaceData($data) {
  $this->dao->replace($this->getTable_admin(), $data);
}


// DELETE ADMIN DATA
public function deleteData($id) {
  return $this->dao->delete($this->getTable_admin(), array('fk_i_admin_id' => $id));
}


// INSERT PERMISSION DATA
public function insertPermData($data) {
  $this->dao->insert($this->getTable_perm(), $data);
  return $this->dao->insertedId();
}


// UPDATE PERMISSION DATA
public function updatePermData($id, $data) {
  $this->dao->update($this->getTable_perm(), $data, array('pk_i_id' => $id));
}


// REPLACE PERMISSION DATA
public function replacePermData($data) {
  $this->dao->replace($this->getTable_perm(), $data);
}


// DELETE PERMISSION
public function deletePermData($id) {
  return $this->dao->delete($this->getTable_perm(), array('pk_i_id' => $id));
}


}
?>