<?php
/*
  Plugin Name: Auto Registration Plugin
  Plugin URI: https://osclasspoint.com/osclass-plugins/social-and-authentication/auto-registration-plugin-i55
  Description: Register non-logged users when adding new listing
  Version: 1.1.3
  Author: MB Themes
  Author URI: http://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: auto_reg
  Plugin update URI: auto-registration
  Product Key: SdWSefTyq9twKToM6qPk
*/


require_once 'model/ModelAR.php';


function ar_call_after_install() {

  //upload email templates
  foreach(osc_listLocales() as $loc) {
    //new user registered
    $des_register[$loc['code']]['s_title'] = '{WEB_TITLE} - Registration successful';
    $des_register[$loc['code']]['s_text'] = '<p>Hi {CONTACT_NAME}!</p> <p>We have created account for you on: <strong>{WEB_LINK}</strong></p> <p><br /></p> <p>User name: <strong>{CONTACT_NAME}</strong> <br />Email: <strong>{CONTACT_EMAIL}</strong> <br />Password: <strong>{PASSWORD}</strong></p> <p><br /></p> <p>Regards, <br />{WEB_TITLE}</p>';
  }

  Page::newInstance()->insert( array('s_internal_name' => 'ar_user_registered', 'b_indelible' => '1'), $des_register);
}


function ar_call_after_uninstall() {
  Page::newInstance()->deleteByInternalName('ar_new_user');
}


// REGISTER NEW USER
function ar_register_user( $item ) {
  if( $item['pk_i_id'] > 0 ) {
    $item = Item::newInstance()->findByPrimaryKey( $item['pk_i_id'] );

    // only when there is no user on item
    if( $item['fk_i_user_id'] == NULL ) {
      $user_data = array();


      // Save user data to array
      $user_data['s_name'] = $item['s_contact_name'];
      $user_data['s_email'] = $item['s_contact_email'];
      $user_data['s_password'] = osc_genRandomPassword();
      $user_data['s_secret'] = osc_genRandomPassword();
      $user_data['countryId'] = $item['fk_c_country_code'];
      $user_data['country'] = $item['s_country'];
      $user_data['regionId'] = $item['fk_i_region_id'];
      $user_data['region'] = $item['s_region'];
      $user_data['cityId'] = $item['fk_i_city_id'];
      $user_data['city'] = $item['s_city'];

      // Get phone
      $user_data['s_phone_mobile'] = (Params::getParam('sPhone') <> '' ? Params::getParam('sPhone') : Params::getParam('contactPhone'));


      $pass = true;

      if(!osc_validate_email( $user_data['s_email'] )) {
        osc_add_flash_error_message( _m('Entered contact email is not valid, user account has not been created', 'auto_reg') );
        $pass = false;
      }

      if($pass && trim($user_data['s_name']) == '') {
        osc_add_flash_error_message( _m('Contact name is empty, user account has not been created', 'auto_reg') );
        $pass = false;
      }


      if( $pass ) {
        ModelAR::newInstance()->insertUser( $user_data );

        $user = User::newInstance()->findByEmail( $user_data['s_email'] );
        ModelAR::newInstance()->updateUsername( $user['pk_i_id'], $user['pk_i_id'] );
        ModelAR::newInstance()->updateUserId( $item['pk_i_id'], $user['pk_i_id'] );

        Alerts::newInstance()->update(array('fk_i_user_id' => $user['pk_i_id']), array('s_email' => $user['s_email']));

        ar_send_mail( $user_data['s_name'], $user_data['s_email'], $user_data['s_password'] );

        // When user needs to validate it's listing
        if( osc_user_validation_enabled() ) {
          osc_run_hook('hook_email_user_validation', $user, $user_data);
        } else {
          ModelAR::newInstance()->updateUserActive( $user['pk_i_id'], 1 );
        }

        // User registration complete
        osc_run_hook('user_register_completed', $user['pk_i_id']);

        // Login user if allowed - problem if user account is inactive on start
        // ModelAR::newInstance()->loginUserById( $user['pk_i_id'] );
      }
    }
  }
}

osc_add_hook('posted_item', 'ar_register_user', 10);


// SEND EMAIL
function ar_send_mail( $name, $email, $pass ) {
  $page = new Page() ;
  $page = $page->findByInternalName('ar_user_registered');
  //if(empty($page)) { exit(); }

  $locale = osc_current_user_locale() ;
  $content = array();
  if(isset($page['locale'][$locale]['s_title'])) {
    $content = $page['locale'][$locale];
  } else {
    $content = current($page['locale']);
  }


  $words   = array();
  $words[] = array('{CONTACT_NAME}', '{CONTACT_EMAIL}', '{PASSWORD}', '{WEB_TITLE}', '{WEB_LINK}');
  $words[] = array( $name, $email, $pass, osc_page_title(), '<a href="' . osc_base_url() . '">' . osc_page_title() . '</a>' );

  $title = osc_mailBeauty($content['s_title'], $words) ;
  $body  = osc_mailBeauty($content['s_text'], $words) ;

  $email_build = array(
    'subject'  => $title, 
    'to' => $email, 
    'to_name'  => $name,
    'body' => $body,
    'alt_body' => $body
  );

  osc_sendMail($email_build);
}



// Do not send email about new listing for non-registered user
osc_remove_hook('hook_email_new_item_non_register_user', 'fn_email_new_item_non_register_user');

// This is needed in order to be able to activate the plugin
osc_register_plugin(osc_plugin_path(__FILE__), 'ar_call_after_install') ;

// This is a hack to show a Uninstall link at plugins table (you could also use some other hook to show a custom option panel)
osc_add_hook(osc_plugin_path(__FILE__) . '_uninstall', 'ar_call_after_uninstall');
?>