<?php
class ModelBO extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() { 
  parent::__construct(); 
}

public function getTable_Item() {
  return DB_TABLE_PREFIX.'t_item';
}

public function getTable_ItemDescription() {
  return DB_TABLE_PREFIX.'t_item_description';
}

public function getTable_ItemStats() {
  return DB_TABLE_PREFIX.'t_item_stats';
}

public function getTable_ItemBO() {
  return DB_TABLE_PREFIX.'t_item_bo_mgr';
}

public function getTable_User() {
  return DB_TABLE_PREFIX.'t_user';
}

public function getTable_Comment() {
  return DB_TABLE_PREFIX.'t_item_comment';
}

public function getTable_Page() {
  return DB_TABLE_PREFIX.'t_pages';
}

public function getTable_PaymentPublish() {
  return DB_TABLE_PREFIX.'t_payments_publish';
}

public function getTable_Searches() {
  return DB_TABLE_PREFIX.'t_latest_searches';
}


public function import($file) {
  $path = osc_plugin_resource($file) ;
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql)){ throw new Exception("Error importSQL::ModelBO<br>".$file.'<br>'.$path.'<br><br>Please check your database for table t_item_bo_mgr. <br>If any of those tables exist in your database, delete them!');}
}

public function uninstall() {
  $this->dao->query('DROP TABLE '.$this->getTable_ItemBO());
}

public function getItem($item_id) {
  if($item_id <= 0) { return array(); }
  
  $this->dao->select('i.*, b.*');
  //$this->dao->from($this->getTable_Item() . ' i, ' . $this->getTable_ItemBO() . ' b');
  $this->dao->from($this->getTable_Item() . ' i');
  $this->dao->join($this->getTable_ItemBO() . ' as b', 'i.pk_i_id = b.item_id', 'LEFT OUTER');
  $this->dao->where('i.pk_i_id', $item_id );

  $result = $this->dao->get();
  if( !$result ) { return array(); }
  return $result->row();
}

public function getPages() {
  $this->dao->select('pk_i_id');
  $this->dao->from( $this->getTable_Page() );
  $this->dao->where('s_internal_name like "bo_mgr%"');

  $result = $this->dao->get();

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function getStats( $item_id ) {
  if($item_id <= 0) { return array(); }

  $this->dao->select('fk_i_item_id as item_id, sum(i_num_views) as regular_views_total, sum(i_num_premium_views) as premium_views_total');
  $this->dao->from( $this->getTable_ItemStats() );
  $this->dao->where('fk_i_item_id', $item_id );

  $result = $this->dao->get();
  if( !$result ) { return array(); }
  return $result->row();
}

public function addItem( $item_id, $republish_times = 0, $form_used = 0, $pass = NULL, $mobile = NULL, $status = 0, $republish_auto = 0 ) {
  $aSet = array(
    'item_id' => $item_id,
    'republish_times' => $republish_times,
    'republish_auto' => $republish_auto,
    'form_used' => $form_used,
    'passwrd' => $pass,
    'mobile' => $mobile,
    'status_id' => $status
  );

  return $this->dao->insert( $this->getTable_ItemBO(), $aSet);
}

public function updateItem( $item_id, $republish_times = 0, $form_used = 0, $pass = NULL, $mobile = NULL, $status = 0, $republish_auto = 0 ) {
  $aSet = array(
    'item_id' => $item_id,
    'republish_times' => $republish_times,
    'republish_auto' => $republish_auto,
    'form_used' => $form_used,
    'passwrd' => $pass,
    'mobile' => $mobile,
    'status_id' => $status
  );

  $aWhere = array( 'item_id' => $item_id);
  return $this->_update($this->getTable_ItemBO(), $aSet, $aWhere);
}

public function updateItemStatus( $item_id, $status_id ) {
  $aSet = array('status_id' => $status_id);
  $aWhere = array('item_id' => $item_id);

  return $this->dao->update($this->getTable_ItemBO(), $aSet, $aWhere);
}

public function replaceItemStatus( $item_id, $status_id ) {
  $data = array(
    'item_id' => $item_id,
    'status_id' => $status_id
  );

  return $this->dao->replace($this->getTable_ItemBO(), $data);
}

public function updateUserDetails( $user_id, $field, $value = null ) {
  $aSet = array(
    $field  => $value
  );

  $aWhere = array( 'pk_i_id' => $user_id);
  return $this->_update($this->getTable_User(), $aSet, $aWhere);
}

public function republish_times( $item_id ) {
  return $this->dao->query('UPDATE '.$this->getTable_ItemBO().' SET republish_times=coalesce(republish_times, 0)+1 WHERE item_id='.$item_id);
}

public function republish_auto( $item_id ) {
  return $this->dao->query('UPDATE '.$this->getTable_ItemBO().' SET republish_auto=coalesce(republish_auto, 0)-1 WHERE item_id='.$item_id);
}

public function contact_form_times( $item_id ) {
  return $this->dao->query('UPDATE '.$this->getTable_ItemBO().' SET form_used=coalesce(form_used, 0)+1 WHERE item_id='.$item_id);
}

public function republish_item( $item_id, $pub_date, $exp_date, $premium_set, $republish_inactive, $auto_republish = false ) {
  $aSet = array(
    'dt_pub_date'  => $pub_date,
    'dt_mod_date' => null,
    'dt_expiration' => $exp_date,
    'b_premium' => $premium_set,
    'b_active' => $republish_inactive
  );

  $aWhere = array( 'pk_i_id' => $item_id);

  //Lower auto republish
  if($auto_republish) {
    $this->republish_auto( $item_id );
  }

  //Add +1 republish times
  $this->republish_times( $item_id );

  return $this->_update($this->getTable_Item(), $aSet, $aWhere);
}

public function getItems($today, $user_notify_before_date, $delete_date) {
  $result = $this->dao->query('SELECT pk_i_id FROM '.$this->getTable_Item().' WHERE (extract(year from dt_expiration)*10000 + extract(month from dt_expiration)*100 + extract(day from dt_expiration)) in ("' . $today . '", "' . $user_notify_before_date . '", "' . $delete_date . '")');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function updatePaymentTable( $item_id, $paid ) {
  $aSet = array(
    'dt_date'  => date('Y-m-d H:i:s'),
    'b_paid' => $paid
  );

  $aWhere = array( 'fk_i_item_id' => $item_id);
  return $this->_update($this->getTable_PaymentPublish(), $aSet, $aWhere);
}

public function insertPaymentTable( $item_id, $paid ) {
  $aSet = array(
    'fk_i_item_id' => $item_id,
    'dt_date'  => date('Y-m-d H:i:s'),
    'b_paid' => $paid
  );

  return $this->dao->insert( $this->getTable_PaymentPublish(), $aSet);
}

public function getExpiredListingsByUserId($user_id) {
  if($user_id <= 0) { return array(); }

  $result = $this->dao->query('SELECT DISTINCT i.*, d.* FROM '.$this->getTable_Item().' as i, '.$this->getTable_ItemDescription().' as d WHERE (extract(year from i.dt_expiration)*10000 + extract(month from i.dt_expiration)*100 + extract(day from i.dt_expiration)) <= (extract(year from current_date)*10000 + extract(month from current_date)*100 + extract(day from current_date)) AND i.fk_i_user_id = ' . $user_id . ' AND i.pk_i_id = d.fk_i_item_id AND d.fk_c_locale_code="' . osc_current_user_locale() . '"');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function getMyItemsWithComments($user_id) {
  if($user_id <= 0) { return array(); }
  
  $result = $this->dao->query('SELECT i.*, d.*, count(c.pk_i_id) as comment_count FROM '.$this->getTable_Item().' as i, '.$this->getTable_ItemDescription().' as d, '.$this->getTable_Comment().' as c WHERE i.fk_i_user_id = ' . $user_id . ' AND i.b_enabled = 1 AND i.b_active = 1 AND i.b_spam = 0 AND c.b_enabled = 1 AND c.b_active = 1 AND c.b_spam = 0 AND i.pk_i_id = c.fk_i_item_id AND i.pk_i_id = d.fk_i_item_id GROUP BY i.pk_i_id HAVING count(c.pk_i_id) > 0');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function getOtherItemsWithMyComments($user_id) {
  if($user_id <= 0) { return array(); }
  
  $result = $this->dao->query('SELECT i.*, d.*, count(c.pk_i_id) as comment_count FROM '.$this->getTable_Item().' as i, '.$this->getTable_ItemDescription().' as d, '.$this->getTable_Comment().' as c WHERE coalesce(i.fk_i_user_id, 0) <> ' . $user_id . ' AND c.fk_i_user_id = ' . $user_id . ' AND i.b_enabled = 1 AND i.b_active = 1 AND i.b_spam = 0 AND c.b_enabled = 1 AND c.b_active = 1 AND c.b_spam = 0 AND i.pk_i_id = c.fk_i_item_id AND i.pk_i_id = d.fk_i_item_id GROUP BY i.pk_i_id HAVING count(c.pk_i_id) > 0');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function getCommentsByItemId($item_id) {
  if($item_id <= 0) { return array(); }
  
  $result = $this->dao->query('SELECT * FROM '.$this->getTable_Comment().' WHERE fk_i_item_id = ' . $item_id . ' AND b_enabled = 1 AND b_active = 1 AND b_spam = 0');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}


/* ----------------------- Clear inactive, banned and spam Users, Comments & Listings ------------------------- */
public function getInactiveUsersToDelete($user_delete_date) {
  $result = $this->dao->query('SELECT pk_i_id FROM '.$this->getTable_User().' WHERE (extract(year from dt_reg_date)*10000 + extract(month from dt_reg_date)*100 + extract(day from dt_reg_date)) <= ' . $user_delete_date . ' AND b_active = 0');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function getNotLoggedUsersToDelete($user_delete_date) {
  $result = $this->dao->query('SELECT pk_i_id FROM '.$this->getTable_User().' WHERE ((extract(year from dt_access_date)*10000 + extract(month from dt_access_date)*100 + extract(day from dt_access_date)) <= ' . $user_delete_date . ' AND extract(year from dt_access_date) <> 0) or ((extract(year from dt_reg_date)*10000 + extract(month from dt_reg_date)*100 + extract(day from dt_reg_date)) <= ' . $user_delete_date . ' AND extract(year from dt_access_date) = 0)');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}  

public function getDisabledUsersToDelete() {
  $result = $this->dao->query('SELECT pk_i_id FROM '.$this->getTable_User().' WHERE b_enabled = 0');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function getInactiveCommentsToDelete($comment_delete_date) {
  $result = $this->dao->query('SELECT pk_i_id FROM '.$this->getTable_Comment().' WHERE (extract(year from dt_pub_date)*10000 + extract(month from dt_pub_date)*100 + extract(day from dt_pub_date)) <= ' . $comment_delete_date . ' AND b_active = 0');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function getDisabledSpamCommentsToDelete() {
  $result = $this->dao->query('SELECT pk_i_id FROM '.$this->getTable_Comment().' WHERE b_enabled = 0 or b_spam = 1');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function getInactiveListingsToDelete($listing_delete_date) {
  $result = $this->dao->query('SELECT pk_i_id FROM '.$this->getTable_Item().' WHERE (extract(year from dt_pub_date)*10000 + extract(month from dt_pub_date)*100 + extract(day from dt_pub_date)) <= ' . $listing_delete_date . ' AND b_active = 0');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

public function getDisabledSpamListingsToDelete() {
  $result = $this->dao->query('SELECT pk_i_id FROM '.$this->getTable_Item().' WHERE b_enabled = 0 or b_spam = 1');

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}
/* ----------------------- End of Clear inactive, banned and spam Users, Comments & Listings ------------------ */


public function ajax($query) {
  $this->dao->select('distinct left(s_search, 30) as id, left(s_search, 30) as label, left(s_search, 30) as value');
  $this->dao->from($this->getTable_Searches());
  $this->dao->like('lower(s_search)', strtolower(trim($query)));
  $this->dao->limit(10);
  $result = $this->dao->get();

  if( !$result ) { return array(); }
  $prepare = $result->result();
  return $prepare;
}
        
// update
function _update($table, $values, $where) {
  $this->dao->from($table);
  $this->dao->set($values);
  $this->dao->where($where);
  return $this->dao->update();
}
}
?>