<?php
class ModelFB extends DAO {
  private static $instance;

  public static function newInstance() {
    if( !self::$instance instanceof self ) {
      self::$instance = new self;
    }
    return self::$instance;
  }

  function __construct() { 
    parent::__construct(); 
  }

  public function getTable_User() {
    return DB_TABLE_PREFIX.'t_user';
  }

  public function getTable_UserBO() {
    return DB_TABLE_PREFIX.'t_user_bo_mgr_feedback';
  }

  public function getTable_BanList() {
    return DB_TABLE_PREFIX.'t_ban_rule' ;
  }

  public function addFeedback( $user_id, $commentator_id, $commentator_name, $commentator_title, $commentator_message, $score_quality, $score_delivery, $score_communication, $active, $pub_date ) {
    $aSet = array(
      'user_id'  => $user_id,
      'commentator_id'  => $commentator_id,
      'commentator_name'  => $commentator_name,
      'commentator_title'  => $commentator_title,
      'commentator_message'  => $commentator_message,      
      'active'  => $active,
      'pub_date'  => $pub_date
    );

    if($score_quality <> '') {
      $aSet['score_quality'] = $score_quality;
    }

    if($score_delivery <> '') {
      $aSet['score_delivery'] = $score_delivery;
    }

    if($score_communication <> '') {
      $aSet['score_communication'] = $score_communication;
    }

    return $this->dao->insert( $this->getTable_UserBO(), $aSet);
  }

  public function activateFeedback( $feedback_id, $active ) {
    $aSet = array(
      'active'  => $active
    );

    $aWhere = array( 'feedback_id' => $feedback_id );

    return $this->_update($this->getTable_UserBO(), $aSet, $aWhere);
  }

  public function deleteFeedback( $feedback_id ) {
    return $this->dao->delete($this->getTable_UserBO(), array('feedback_id' => $feedback_id) ) ;
  }

  public function getFeedbackByUserId( $user_id, $active ) {
    $this->dao->select('s.*, u.s_name as user_name');
    $this->dao->from( $this->getTable_UserBO() . ' s, ' . $this->getTable_User() . ' u' );
    $this->dao->where( 's.user_id = u.pk_i_id' );
    $this->dao->where( 's.active', $active );

    if($user_id <> '' and $user_id <> 0) {
      $this->dao->where( 's.user_id', $user_id );
    }
    $this->dao->orderBy('s.pub_date DESC');

    $result = $this->dao->get();
    if( !$result ) { return array(); }

    $prepare = $result->result();
    return $prepare;
  }

  public function getFeedbackScoreByUserId( $user_id, $active ) {
    $this->dao->select( 'user_id, avg(score_quality) as score_quality, avg(score_delivery) as score_delivery, avg(score_communication) as score_communication, (sum(score_quality) + sum(score_delivery) + sum(score_communication))/(count(score_quality) + count(score_delivery) + count(score_communication)) as score_overall' );
    $this->dao->from( $this->getTable_UserBO() );
    $this->dao->where( 'active', $active );
    $this->dao->where( 'user_id', $user_id );
    $this->dao->groupBy( 'user_id' );

    $result = $this->dao->get();
    if( !$result ) { return array(); }

    return $result->row();
  }

  public function getLatestFeedback( $limit = 5, $active = 1 ) {
    $this->dao->select();
    $this->dao->from( $this->getTable_UserBO() );

    $this->dao->where( 'active', $active );

    $this->dao->orderBy('pub_date DESC');

    $this->dao->limit($limit);

    $result = $this->dao->get();
    if( !$result ) { return array(); }

    $prepare = $result->result();
    return $prepare;
  }


  public function getFeedbackScoreForBest($wilson = 1, $limit = 5) {
    if($wilson == 1) {
      $this->dao->select('user_id, s_name as user_name, (sum(score_quality) + sum(score_delivery) + sum(score_communication))/(count(score_quality) + count(score_delivery) + count(score_communication)) as score_overall, sum(case round((score_quality + score_delivery + score_communication)/3) when 5 then 2 when 4 then 1 else 0 end) as positive, sum(case round((score_quality + score_delivery + score_communication)/3) when 2 then 1 when 1 then 2 else 0 end) as negative, ((sum(case round((score_quality + score_delivery + score_communication)/3) when 5 then 2 when 4 then 1 else 0 end) + 1.9208) / (sum(case round((score_quality + score_delivery + score_communication)/3) when 5 then 2 when 4 then 1 else 0 end) + sum(case round((score_quality + score_delivery + score_communication)/3) when 2 then 1 when 1 then 2 else 0 end)) - 1.96 * SQRT((sum(case round((score_quality + score_delivery + score_communication)/3) when 5 then 2 when 4 then 1 else 0 end) * sum(case round((score_quality + score_delivery + score_communication)/3) when 2 then 1 when 1 then 2 else 0 end)) / (sum(case round((score_quality + score_delivery + score_communication)/3) when 5 then 2 when 4 then 1 else 0 end) + sum(case round((score_quality + score_delivery + score_communication)/3) when 2 then 1 when 1 then 2 else 0 end)) + 0.9604) / (sum(case round((score_quality + score_delivery + score_communication)/3) when 5 then 2 when 4 then 1 else 0 end) + sum(case round((score_quality + score_delivery + score_communication)/3) when 2 then 1 when 1 then 2 else 0 end))) / (1 + 3.8416 / (sum(case round((score_quality + score_delivery + score_communication)/3) when 5 then 2 when 4 then 1 else 0 end) + sum(case round((score_quality + score_delivery + score_communication)/3) when 2 then 1 when 1 then 2 else 0 end))) as final_order');
      $this->dao->having( 'sum(case round((score_quality + score_delivery + score_communication)/3) when 5 then 2 when 4 then 1 else 0 end) + sum(case round((score_quality + score_delivery + score_communication)/3) when 2 then 1 when 1 then 2 else 0 end) > 0' );
      $this->dao->orderBy('final_order DESC');
    } else {
      $this->dao->select('user_id, s_name as user_name, (sum(score_quality) + sum(score_delivery) + sum(score_communication))/(count(score_quality) + count(score_delivery) + count(score_communication)) as score_overall');
      $this->dao->orderBy('score_overall DESC');
    }

    $this->dao->from( $this->getTable_UserBO() . ' s, ' .  $this->getTable_User() . ' u');
    $this->dao->where( 's.user_id = u.pk_i_id' );
    $this->dao->where( 's.active', 1 );
    $this->dao->limit( $limit );
    $this->dao->groupBy( 'user_id, s_name' );

    $result = $this->dao->get();
    if( !$result ) { return array(); }

    $prepare = $result->result();
    return $prepare;
  }


  public function getBestUsers() {
    $this->dao->select('u.pk_i_id, u.s_name, s.positive');

    $this->dao->from( $this->getTable_User() . ' u, ' . $this->getFeedbackScoreForBest() . ' s' );
    $this->dao->where( 's.user_id = u.pk_i_id' );


    $this->dao->orderBy('pub_date DESC');

    //$this->dao->limit($limit);

    $result = $this->dao->get();
    if( !$result ) { return array(); }

    $prepare = $result->result();
    return $prepare;
  }

  public function countFeedbackByCommentatorId( $user_id, $active = NULL ) {
    $this->dao->select('count(*) as count');
    $this->dao->from( $this->getTable_UserBO() );

    if($user_id <> 0 and $user_id <> '') {
      $this->dao->where( 'commentator_id', $user_id );
    }

    if($active <> '') {
      $this->dao->where( 'active', $active );
    }

    $result = $this->dao->get();
    if( !$result ) { return array(); }

    return $result->row();
  }


  public function countNotValidatedFeedback() {
    $this->dao->select('count(*) as count');
    $this->dao->from( $this->getTable_UserBO() );

    $this->dao->where( 'active', 0 );

    $result = $this->dao->get();
    if( !$result ) { return array(); }

    return $result->row();
  }


  public function getFeedbackByCommentatorId( $user_id, $limit = NULL, $from = NULL, $active = NULL ) {
    $this->dao->select();
    $this->dao->from( $this->getTable_UserBO() );

    if($user_id <> '' and $user_id <> 0) {
      $this->dao->where( 'commentator_id', $user_id );
    }

    if($active <> '') {
      $this->dao->where( 'active', $active );
    }

    if($from <> '') {
      $this->dao->where( 'pub_date >= STR_TO_DATE("' . $from . '", "%Y-%m-%d %H:%i:%s")' );
    }

    if($limit <> 0 and $limit <> '') {
      $this->dao->limit($limit);
    }

    $this->dao->orderBy('pub_date DESC');

    $result = $this->dao->get();
    if( !$result ) { return array(); }

    if($limit == 1) {
      return $result->row();
    }

    $prepare = $result->result();
    return $prepare;
  }

  public function insertBan( $reason = null, $ip = '' ) {
    $aSet = array(
      's_name'  => 'Spam > IP (' . $reason . ')',
      's_ip'  => $ip,
      's_email' => ''
    );

    return $this->dao->insert( $this->getTable_BanList(), $aSet);
  }

  // update
  function _update($table, $values, $where) {
    $this->dao->from($table);
    $this->dao->set($values);
    $this->dao->where($where);
    return $this->dao->update();
  }
}
?>