<?php

// ITEM DETAIL BOX
function cap_item_detail($item) {
  $item_id = (int)(isset($item['pk_i_id']) ? $item['pk_i_id'] : 0);

  if($item_id <= 0) {
    return false;
  }

  $item_attr = ModelCAP::newInstance()->getItemAttr($item_id);

  if(!is_array($item_attr) || !isset($item_attr['fk_i_type_id']) || $item_attr['fk_i_type_id'] <= 0) {
    return false;
  }

  $car_type = cap_get_type($item_attr['fk_i_type_id']);
  ?>
  
  <div id="cap-item">
    <?php foreach(cap_get_sections('item', $car_type['pk_i_id']) as $skey => $section) { ?>
      <?php 
        $sid = $section['s_id'];
        
        $section_props_all = cap_get_section_properties('item', $skey, $car_type); 
        $section_props = cap_filter_nonempty_item_attr($section_props_all, $item_attr);
        $section_props_gtype = cap_filter_nonempty_item_attr_group($section_props_all, $item_attr);
        
        $options = array();
        $options['style'] = ($section['s_item_style'] <> '' ? $section['s_item_style'] : $sid);   // define properties style
      ?>
      
      <?php if(is_array($section_props) && (count($section_props) > 0 || cap_param('enable_empty_sections') == 1)) { ?>
        <?php if($sid == 'main') { ?>
          <strong class="cap-head"><?php _e('Car attributes', 'car_attr_pro'); ?></strong>
          
        <?php } else if(!in_array($sid, array('main'))) { ?>
          <strong class="cap-subhead"><?php echo cap_get_name($section); ?></strong>
        <?php } ?>
        
        <div class="cap-section cap-section-<?php echo $sid; ?>">
          <?php if(count($section_props) > 0) { ?>
            <?php foreach($section_props as $key => $prop) { ?>
              <?php cap_create_elem($key, $car_type, $item_attr, $options); ?>
            <?php } ?>
          <?php } ?>
        </div>
      <?php } ?>
      

      <?php if(is_array($section_props_gtype) && count($section_props_gtype) > 0) { ?>
        <?php foreach($section_props_gtype as $key => $prop) { ?>
          <strong class="cap-subhead"><?php echo cap_get_name($prop); ?></strong>

          <div class="cap-section cap-section-<?php echo $sid; ?> cap-section-group-<?php echo $prop['s_id']; ?>">
            <?php cap_create_elem($key, $car_type, $item_attr, $options); ?>
          </div>
        <?php } ?>
      <?php } ?>
    <?php } ?>
    
    <?php osc_run_hook('cap_item_detail', $item); ?>
  </div>
 
  <?php  
}

// ITEM DETAIL MANAGEMENT
function cap_item_detail_hooks() {
  $hooks = array_filter(array_map('trim', explode(',', cap_param('item_detail_hooks'))));

  if(is_array($hooks) && count($hooks) > 0) {
    foreach($hooks as $hook) {
      osc_add_hook($hook, function() {
        cap_item_detail(osc_item()); 
      });
    }
  }
}

osc_add_hook('init', 'cap_item_detail_hooks', 2);


// ITEM MAIN ATTRIBUTES DISPLAY
// Define section or prop_ids (keys)
// Display: main / sub / label_value / label_value_alt / icon_value / icon_value_alt 
//          icon_value_label / icon_label_value / value_only / checks / bool / standard
// Compact: true / false
// Limit: 1-99
// Align: left / center

function cap_item_attributes($item_id, $options = array(), $section_key = 0, $prop_ids = array()) {
  $display = isset($options['display']) ? $options['display'] : 'main';
  $with_header = isset($options['with_header']) ? $options['with_header'] : false;
  $compact = isset($options['compact']) ? $options['compact'] : false;
  $align = isset($options['align']) ? $options['align'] : '';
  $limit = isset($options['limit']) ? $options['limit'] : false;
  
  if($item_id <= 0) {
    return false;
  }

  $item_attr = ModelCAP::newInstance()->getItemAttr($item_id);

  if(!is_array($item_attr) || !isset($item_attr['fk_i_type_id']) || $item_attr['fk_i_type_id'] <= 0) {
    return false;
  }

  $car_type = cap_get_type($item_attr['fk_i_type_id']);
  $prop_ids = array_filter(!is_array($prop_ids) ? array($prop_ids) : $prop_ids);
  $props_final = array();
  
  // User defined IDs or Keys
  if(!empty($prop_ids) && count($prop_ids) > 0) {
    $props = cap_get_properties();
    foreach($props as $key => $prop) {
      if(in_array($key, $prop_ids) || in_array($prop['s_id'], $prop_ids)) {
        $props_final[$key] = $prop;
      }
    }
    
  // User defined section
  } else if($section_key > 0 && in_array($section_key, cap_get_sections('item', $car_type['pk_i_id']))) {
    $props = cap_get_section_properties('item', $section_key, $car_type); 
    $props_final = cap_filter_nonempty_item_attr($props, $item_attr, true);
    
  // User not defined IDs nor section
  } else {
    $props = cap_get_properties();
    $props_final = cap_filter_nonempty_item_attr($props, $item_attr, true);
  }
  
  if(!is_array($props_final) || count($props_final) <= 0) {
    return false;
  }
  
  if($limit > 0) {
    $props_final = array_slice($props_final, 0, $limit, true);
  }
  ?>

  <div id="cap-item" class="cap-item-attr cap-display-<?php echo osc_esc_html($display); ?> cap-compact-<?php echo $compact ? 'yes' : 'no'; ?> <?php if($align != '') { ?>cap-align-<?php echo $align; ?><?php } ?>">
    <?php if($with_header) { ?>
      <strong class="cap-subhead"><?php _e('Attributes', 'car_attr_pro'); ?></strong>
    <?php } ?>
    
    <div class="cap-section cap-section-<?php echo $display; ?>">
      <?php 
        $options = array(
          'style' => $display
        );
        
        foreach($props_final as $key => $prop) {
          cap_create_elem($key, $car_type, $item_attr, $options);
        }
      ?>
    </div>
    
    <?php osc_run_hook('cap_item_attributes', $item_id, $item_attr); ?>
  </div>

  <?php  
}

// ITEM ATTRIBUTES MANAGEMENT
function cap_item_attributes_hooks() {
  $hooks = array_filter(array_map('trim', explode(',', cap_param('item_attr_hooks'))));

  if(is_array($hooks) && count($hooks) > 0) {
    foreach($hooks as $hook) {
      osc_add_hook($hook, function() {
        cap_item_attributes(osc_item_id(), array('display' => 'main', 'compact' => 'true', 'limit' => 6)); 
      });
    }
  }
}

osc_add_hook('init', 'cap_item_attributes_hooks', 2);


// ADD PROPERTIES INTO ITEM TITLE - MANUALLY ONLY!
function cap_props_to_item_title($item_id = -1) {
  if($item_id == -1) {
    $item_id = osc_item_id();
  }
  
  $output = array();
  
  if($item_id > 0 && cap_param('item_title_props') != '') {
    $item_attr = ModelCAP::newInstance()->getItemAttr($item_id);
    $car_type = cap_get_type($item_attr['fk_i_type_id']);
    $prop_ids = explode(',', cap_param('item_title_props'));
    $props_final = array();
    
    if(!empty($prop_ids) && count($prop_ids) > 0) {
      $props = cap_get_properties();
      foreach($props as $key => $prop) {
        if(in_array($key, $prop_ids) || in_array($prop['s_id'], $prop_ids)) {
          $props_final[$key] = $prop;
        }
      }

      $props_final = cap_filter_nonempty_item_attr($props_final, $item_attr, true);

      foreach($props_final as $key => $prop) {
        $data = cap_create_elem($key, $car_type, $item_attr, array('return_only' => true));
        
        if(isset($data['value']) && $data['value'] <> '') {
          $output[] = $data['value'];
        }
      }
    }
  }
  
  $output = array_filter($output);
  return implode(' - ', $output);
}


// SHOWCASE ITEM ATTRIBUTE
function cap_item_attributes_showcase($item_id) {
  ?>
  <div class="cap-showcase">
    <strong class="cap-shw-hdr"><?php _e('Car Attributes PRO Plugin Showcase', 'car_attr_pro'); ?></strong>

    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_item_attributes(osc_item_id(), array('display' => 'main', 'with_header' => false, 'limit' => 5, 'compact' => true, 'align' => 'center')); ?&gt;</div>
      <?php cap_item_attributes($item_id, array('display' => 'main', 'with_header' => false, 'limit' => 5, 'compact' => true, 'align' => 'center')); ?>
    </div>

    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_item_attributes(osc_item_id(), array('display' => 'icon_value', 'with_header' => true, 'limit' => 5, 'compact' => false, 'align' => 'center')); ?&gt;</div>
      <?php cap_item_attributes($item_id, array('display' => 'icon_value', 'with_header' => true, 'limit' => 5, 'compact' => false, 'align' => 'center')); ?>
    </div>
    
    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_item_attributes(osc_item_id(), array('display' => 'icon_label_value', 'with_header' => false, 'limit' => 6)); ?&gt;</div>
      <?php cap_item_attributes($item_id, array('display' => 'icon_label_value', 'with_header' => false, 'limit' => 6)); ?>
    </div>

    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_item_attributes(osc_item_id(), array('display' => 'value_only'), '', array('type','make','model','body_type')); ?&gt;</div>
      <?php cap_item_attributes($item_id, array('display' => 'value_only'), '', array('type','make','model','body_type')); ?>
    </div>
    
    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_item_attributes(osc_item_id(), array('display' => 'label_value_alt', 'limit' => 5), 'main'); ?&gt;</div>
      <?php cap_item_attributes($item_id, array('display' => 'label_value_alt', 'limit' => 5), 'main'); ?>
    </div>
    
    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_item_attributes(osc_item_id(), array('display' => 'checks', 'limit' => 5), 'sub'); ?&gt;</div>
      <?php cap_item_attributes($item_id, array('display' => 'checks', 'limit' => 5), 'sub'); ?>
    </div>
    
    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_item_attributes(osc_item_id(), array('display' => 'sub', 'limit' => 3)); ?&gt;</div>
      <?php cap_item_attributes($item_id, array('display' => 'sub', 'limit' => 3)); ?>
    </div>
    
    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_item_attributes(osc_item_id(), array('display' => 'sub', 'compact' => true, 'limit' => 10)); ?&gt;</div>
      <?php cap_item_attributes($item_id, array('display' => 'sub', 'compact' => true, 'limit' => 10)); ?>
    </div>
    
    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_item_attributes(osc_item_id(), array('display' => 'bool', 'with_header' => true)); ?&gt;</div>
      <?php cap_item_attributes($item_id, array('display' => 'bool', 'with_header' => true)); ?>
    </div>
    


    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_widget_types(true, false); ?&gt;</div>
      <?php cap_widget_types(true, false); ?>
    </div>
    

    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_widget_makes('cars', true); ?&gt;</div>
      <?php cap_widget_makes('cars', true); ?>
    </div>
    
    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_widget_models(1); ?&gt;</div>
      <?php cap_widget_models(1); ?>
    </div>
    
    
    <div class="cap-shw-box">
      <div class="cap-code">&lt?php cap_search_box(array('with_form' => true, 'design' => 'complex', 'menu' => 'advanced', 'menu_items' => 'standard', 'style' => 'boxed', 'width' => 'medium')); ?&gt;</div>
      <?php cap_search_box(array('with_form' => true, 'design' => 'complex', 'menu' => 'advanced', 'menu_items' => 'standard', 'style' => 'boxed', 'width' => 'medium')); ?>
    </div>

  </div>
  <?php
}

// Hook showcase to item page
osc_add_hook('item_detail', function($item) {
  if(cap_param('enable_showcase') == 1) {
    cap_item_attributes_showcase($item['pk_i_id']);
  }
});


// SEARCH BOX WIDGET
function cap_search_box($options = array()) {
  $with_form = (isset($options['with_form']) ? $options['with_form'] : false);
  $ignore_form_tag = (isset($options['ignore_form_tag']) ? $options['ignore_form_tag'] : false);

  $design = (isset($options['design']) ? $options['design'] : cap_param('search_box_design'));
  $menu = (isset($options['menu']) ? $options['menu'] : cap_param('search_box_menu'));
  $menu_items = (isset($options['menu_items']) ? $options['menu_items'] : cap_param('search_box_menu_items'));
  $style = (isset($options['style']) ? $options['style'] : cap_param('search_box_style'));
  $width = (isset($options['width']) ? $options['width'] : cap_param('search_box_width'));
  $close_btn = (isset($options['close_btn']) ? $options['close_btn'] : false);
  $search_params = (isset($options['search_params']) ? $options['search_params'] : array());

  $design = strtolower($design == '' ? 'complex' : $design);
  $menu = strtolower($menu == '' ? 'advanced' : $menu);
  $menu_items = strtolower($menu_items == '' ? 'condensed' : $menu_items);
  $style = strtolower($style == '' ? 'borderless' : $style);
  $width = strtolower($width == '' ? 'large' : $width); //medium

  $cat_id = 0;
  
  if(osc_is_search_page()) {
    $cat_id = osc_search_category_id();
    $cat_id = isset($cat_id[0]) ? $cat_id[0] : '';
  }

  $types = cap_get_types($cat_id);
  
  if(!is_array($types) || count($types) <= 0) {
    return false;
  }
  ?>
  <?php if($with_form) { ?>
    <?php if(!$ignore_form_tag) { ?>
    <form action="<?php echo osc_base_url(true); ?>" method="GET" class="nocsrf">
    <?php } ?>
      <input type="hidden" name="page" value="search">
      <input type="hidden" name="sOrder" value="<?php echo osc_esc_html(osc_search_order()); ?>"/>
      <input type="hidden" name="iOrderType" value="<?php $allowedTypesForSorting = Search::getAllowedTypesForSorting(); echo isset($allowedTypesForSorting[osc_search_order_type()]) ? $allowedTypesForSorting[osc_search_order_type()] : ''; ?>" />
      <input type="hidden" name="sCountry" value="<?php echo osc_esc_html(cap_get_param('sCountry', $search_params)); ?>"/>
      <input type="hidden" name="sRegion" value="<?php echo osc_esc_html(cap_get_param('sRegion', $search_params)); ?>"/>
      <input type="hidden" name="sCity" value="<?php echo osc_esc_html(cap_get_param('sCity', $search_params)); ?>"/>
      <input type="hidden" name="iPage" value=""/>
      <input type="hidden" name="sShowAs" value="<?php echo osc_esc_html(cap_get_param('sShowAs', $search_params)); ?>"/>
      <input type="hidden" name="userId" value="<?php echo osc_esc_html(cap_get_param('userId', $search_params)); ?>"/>
      <input type="hidden" name="notFromUserId" value="<?php echo osc_esc_html(cap_get_param('notFromUserId', $search_params)); ?>"/>
      <input type="hidden" name="sPattern" value="<?php echo osc_esc_html(cap_get_param('sPattern', $search_params)); ?>"/>
      <input type="hidden" name="sCategory[]" value="<?php echo osc_esc_html(cap_get_param('sCategory', $search_params)); ?>"/>
      <input type="hidden" name="sCompany" value="<?php echo osc_esc_html(cap_get_param('sCompany', $search_params)); ?>"/>
      <input type="hidden" name="sCondition" value="<?php echo osc_esc_html(cap_get_param('sCondition', $search_params)); ?>"/>
      <input type="hidden" name="sTransaction" value="<?php echo osc_esc_html(cap_get_param('sTransaction', $search_params)); ?>"/>
      <input type="hidden" name="sCompany" value="<?php echo osc_esc_html(cap_get_param('sCompany', $search_params)); ?>"/>
      <input type="hidden" name="bPic" value="<?php echo osc_esc_html(cap_get_param('bPic', $search_params)); ?>"/>
      <input type="hidden" name="bPremium" value="<?php echo osc_esc_html(cap_get_param('bPremium', $search_params)); ?>"/>
      <input type="hidden" name="bPhone" value="<?php echo osc_esc_html(cap_get_param('bPhone', $search_params)); ?>"/>
      <input type="hidden" name="sPriceMin" value="<?php echo osc_esc_html(cap_get_param('sPriceMin', $search_params)); ?>"/>
      <input type="hidden" name="sPriceMax" value="<?php echo osc_esc_html(cap_get_param('sPriceMax', $search_params)); ?>"/>
      
      <?php if($ignore_form_tag) { ?><input type="hidden" name="capDummyPlaceholder" value=""/><?php } ?>
  <?php } ?>
  
  <div class="cap-search cap-is-search cap-<?php echo osc_esc_html($design); ?> cap-style-<?php echo osc_esc_html($style); ?> cap-width-<?php echo osc_esc_html($width); ?>" data-theme="<?php echo osc_current_web_theme(); ?>">
    <div class="cap-section-top cap-menu-<?php echo osc_esc_html($menu); ?> cap-menu-items-<?php echo osc_esc_html($menu_items); ?>"<?php if(count($types) <= 1) { ?> style="display:none;"<?php } ?>>
      <?php 
        $selected_id = cap_get_param(cap_get_input_name('type'), $search_params);

        if($selected_id == '') {
          $selected_id = $types[array_keys($types)[0]]['pk_i_id'];
        }
        
        $selected_type = cap_get_type($selected_id);
        $selected_id = (isset($selected_type['pk_i_id']) ? $selected_type['pk_i_id'] : 0);
      ?>
      <?php if(is_array($types) && count($types) > 0) { ?>
        <?php foreach($types as $car_type) { ?>
          <a href="#" data-id="<?php echo $car_type['pk_i_id']; ?>" data-sid="<?php echo osc_esc_html($car_type['s_slug']); ?>" class="<?php if($selected_id == $car_type['pk_i_id']) { ?>cap-active<?php } ?>" <?php if($car_type['s_color'] != '') { ?>style="border-top-color:<?php echo $car_type['s_color']; ?>;"<?php } ?>>
            <?php echo cap_get_icon($car_type, 'type'); ?>
            <div><span><?php echo cap_get_name($car_type); ?></span></div>
          </a>
        <?php } ?>
      <?php } ?>
      
      <?php if(count($types) > 3 && $menu == 'advanced') { ?>
        <a href="#" data-id="-1" class="cap-special" data-alt-text="<?php echo osc_esc_html(__('Less', 'car_attr_pro')); ?>">
          <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 -50 512 512"><path d="M328 256c0 39.8-32.2 72-72 72s-72-32.2-72-72 32.2-72 72-72 72 32.2 72 72zm104-72c-39.8 0-72 32.2-72 72s32.2 72 72 72 72-32.2 72-72-32.2-72-72-72zm-352 0c-39.8 0-72 32.2-72 72s32.2 72 72 72 72-32.2 72-72-32.2-72-72-72z"/></svg>
          <div><span><?php _e('More', 'car_attr_pro'); ?></span></div>
        </a>
        
        <div class="cap-more-types">
          <?php if(is_array($types) && count($types) > 0) { ?>
            <?php foreach($types as $car_type) { ?>
              <a href="#" data-id="<?php echo $car_type['pk_i_id']; ?>" class="<?php if($selected_id == $car_type['pk_i_id']) { ?>cap-active<?php } ?>" <?php if($car_type['s_color'] != '') { ?>style="border-top-color:<?php echo $car_type['s_color']; ?>;"<?php } ?>>
                <?php echo cap_get_icon($car_type, 'type'); ?>
                <div><span><?php echo cap_get_name($car_type); ?></span></div>
              </a>
            <?php } ?>
          <?php } ?>
        </div>
      <?php } ?>
    </div>

    <?php $i = 0; ?>
    <?php if(is_array($types) && count($types) > 0) { ?>
      <?php foreach($types as $car_type) { ?>
        <?php 
          $j = 0;
          $id = $car_type['pk_i_id']; 
          $is_selected_type = ($selected_id == $car_type['pk_i_id'] ? true : false);
          $sections = cap_get_sections('widget', $car_type['pk_i_id']);
        ?>
        
        <div class="cap-section-body<?php if($is_selected_type) { ?> cap-active<?php } ?>" data-id="<?php echo $id; ?>">
          <input type="hidden" name="<?php echo cap_get_input_name('type'); ?>" value="<?php echo cap_get_slug($car_type); ?>" <?php if(!$is_selected_type) { ?>disabled<?php } ?>/>
          
          <strong class="cap-head"><?php _e('What are you looking for?', 'car_attr_pro'); ?></strong>
          
          <?php if(trim((string)$car_type['s_description']) != '') { ?>
            <div class="cap-text"><?php echo $car_type['s_description']; ?></div>
          <?php } ?>
          
          <?php if(is_array($sections) && count($sections) > 0) { ?>
            <?php foreach($sections as $skey => $section) { ?>
              <?php if($section['s_widget_style'] == 'BOX') { ?>
                <?php
                  $sid = $section['s_id'];
                  $header = cap_get_help_text_search($section);
                  $section_props = cap_get_section_properties('widget', $skey, $car_type);
                ?>
                
                <?php if(is_array($section_props) && (count($section_props) > 0 || cap_param('enable_empty_sections') == 1)) { ?>
                  <?php if($header != '-') { ?>
                    <strong class="cap-subhead cap-bot"><?php echo ($header != '' ? $header : cap_get_name($section)); ?></strong>
                  <?php } ?>
                  
                  <div class="cap-section-mid<?php if($j > 0) { ?> cap-bot<?php } ?>" data-section-id="<?php echo $sid; ?>">
                    <?php if(count($section_props) > 0) { ?>
                      <?php foreach($section_props as $pkey => $prop) { ?>
                        <?php cap_create_input($pkey, $car_type, array(), array('is_active' => $is_selected_type, 'section' => 'widget')); ?>
                      <?php } ?>
                    <?php } ?>
                  </div>
                <?php } ?>
                
                <?php $j++; ?>
              <?php } ?>
            <?php } ?>
          <?php } ?>
          

          <strong class="cap-subhead"><?php _e('Additional properties', 'car_attr_pro'); ?></strong>

          <div class="cap-section-mid cap-buttons">
            <?php if(is_array($sections) && count($sections) > 0) { ?>
              <?php foreach($sections as $skey => $section) { ?>
                <?php if($section['s_widget_style'] == 'BUTTON' || $section['s_widget_style'] == '') { ?>
                  <?php
                    $sid = $section['s_id'];
                    $header = cap_get_help_text_search($section);
                    $section_props = cap_get_section_properties('widget', $skey, $car_type);
                    $icon = cap_get_icon($section);
                    $desc = trim(cap_get_description($section));
                  ?>
                  
                  <?php if(is_array($section_props) && (count($section_props) > 0 || cap_param('enable_empty_sections') == 1)) { ?>
                    <?php 
                      $counter = 0;
                      
                      // Count only for selected type, avoid mixing properties across different types
                      if($is_selected_type) {
                        $props_nonepty = cap_filter_nonempty_search_props($section_props, 'widget');
                        
                        if(is_array($props_nonepty) && count($props_nonepty) > 0) {
                          foreach($props_nonepty as $prop_ne) {
                            $xval = Params::getParam(cap_get_input_name($prop_ne['s_id']));
                            $xval_ = explode(' ', $xval);
                            $counter += (count($xval_) > 1 ? count($xval_) : 1);
                            
                            if(isset($prop_ne['is_range']) && $prop_ne['is_range'] === true) {
                              if($prop_ne['is_range_from'] && $prop_ne['is_range_to']) {
                                $counter++;
                              }                      
                            }
                          }
                        }
                      }
                    ?>
                    
                    <div class="cap-input-box">
                      <a href="#" class="cap-open-detail" data-section="<?php echo $sid; ?>">
                        <?php echo ($icon <> '' ? $icon : CAP_SECTION_DEFAULT); ?>
                        
                        <div class="cap-btn-text<?php if($desc <> '') { ?> cap-has-desc<?php } ?>">
                          <strong><?php echo cap_get_name($section); ?></strong>
                          
                          <?php if($desc <> '') { ?>
                            <span><?php echo $desc; ?></span>
                          <?php } ?>
                        </div>
                        
                        <em class="cap-counter2" data-count="<?php echo $counter; ?>"><?php echo $counter; ?></em>
                        <?php echo CAP_ANGLE_RIGHT; ?>
                      </a>
                      
                      <div class="cap-details" data-section="<?php echo $sid; ?>">
                        <div class="cap-details-box" data-section="<?php echo $sid; ?>">
                          <?php if($header != '-') { ?>
                            <strong class="cap-head"><?php echo ($header <> '' ? $header : cap_get_name($section)); ?></strong>
                          <?php } ?>
                          
                          <div class="cap-details-values">
                            <?php if(count($section_props) > 0) { ?>
                              <?php foreach($section_props as $pkey => $prop) { ?>
                                <?php cap_create_input($pkey, $car_type, array(), array('is_active' => $is_selected_type, 'section' => 'widget')); ?>
                              <?php } ?>
                            <?php } ?>
                          </div>
                          
                          <div class="cap-detail-footer">
                            <div class="cap-box-filter">
                              <?php echo CAP_SEARCH_SVG; ?>
                              <input type="text" class="cap-filter-results" placeholder="<?php echo osc_esc_html(__('Filter properties...', 'car_attr_pro')); ?>"/>
                              <?php echo CAP_CLEAR_SVG; ?>
                            </div>
                            
                            <div class="cap-box-clear">
                              <a href="#" class="cap-details-clear"><?php _e('Clear', 'car_attr_pro'); ?></a>
                            </div>
                            
                            <div class="cap-box-apply">
                              <a href="#" class="cap-details-apply"><?php _e('Apply', 'car_attr_pro'); ?></a>
                            </div>
                          </div>
                        </div>
                      </div>
                    </div>
                  <?php } ?>
                <?php } ?>
              <?php } ?>
            <?php } ?>
          </div>

          <div class="cap-section-action">
            <div class="cap-btn-adv">
              <a href="<?php echo osc_search_url(); ?>" class="cap-more" data-alt-text="<?php echo osc_esc_html(__('Simplified search', 'car_attr_pro')); ?>">
                <?php echo CAP_ADV_SEARCH_SVG; ?>
                <span><?php _e('Advanced search', 'car_attr_pro'); ?></span>
              </a>
            </div>
            
            <div class="cap-btn-sub"><button type="submit" class="cap-submit"><?php _e('Search', 'car_attr_pro'); ?></button></div>
            
            <?php if($close_btn) { ?>
              <div class="cap-btn-close">
                <a href="#" class="cap-close">
                  <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 352 512" height="14px"><path d="M242.72 256l100.07-100.07c12.28-12.28 12.28-32.19 0-44.48l-22.24-22.24c-12.28-12.28-32.19-12.28-44.48 0L176 189.28 75.93 89.21c-12.28-12.28-32.19-12.28-44.48 0L9.21 111.45c-12.28 12.28-12.28 32.19 0 44.48L109.28 256 9.21 356.07c-12.28 12.28-12.28 32.19 0 44.48l22.24 22.24c12.28 12.28 32.2 12.28 44.48 0L176 322.72l100.07 100.07c12.28 12.28 32.2 12.28 44.48 0l22.24-22.24c12.28-12.28 12.28-32.19 0-44.48L242.72 256z"/></svg>
                  <?php _e('Close modal', 'car_attr_pro'); ?>
                </a>
              </div>
            <?php } ?>
          </div>
        </div>
        
        <?php $i++; ?>
      <?php } ?>
    <?php } ?>
    
    <?php osc_run_hook('cap_search_box', $options); ?>
  </div>
  
  <?php if($with_form && !$ignore_form_tag) { ?>
    </form>
  <?php } ?>
  
  <?php
}


// SEARCH BOX HOOKS MANAGEMENT
function cap_search_box_hooks() {
  $hooks = array_filter(array_map('trim', explode(',', cap_param('search_box_hooks'))));

  if(is_array($hooks) && count($hooks) > 0) {
    foreach($hooks as $hook) {
      osc_add_hook($hook, 'cap_search_box', 4);
    }
  }
}

osc_add_hook('init', 'cap_search_box_hooks', 2);


// Search box hook - search page
osc_add_hook('search_items_top', function() { 
  if(cap_param('hook_search_top') == 1) { 
    cap_search_box(array('with_form' => true, 'design' => 'simple', 'menu' => 'advanced', 'menu_items' => 'standard', 'style' => 'borderless', 'width' => 'medium'));
  }
});

// Search gox hook - home page
osc_add_hook('home_search_top', function() { 
  if(cap_param('hook_home_search') == 1) { 
    cap_search_box(array('with_form' => false, 'design' => 'simple', 'menu' => 'advanced', 'menu_items' => 'condensed', 'style' => 'boxed', 'width' => 'large'));
  }
});


// SEARCH HOOK - Add properties to search via hook
function cap_search_hook($catId = 0) { 
  if(cap_param('disable_search_hook') == 1) {
    return false;
  }

  $cat_id = (int)(isset($catId[0]) ? $catId[0] : 0);
  
  if(cap_param('hide_search_hook_empty_cat') == 1 && $cat_id == 0) {
    return false;
  }
  
  $type_id = Params::getParam(cap_get_input_name('type'));
  $props = cap_get_properties_enabled('search');

  $car_types = array();
  
  if(is_array($props) && count($props) > 0) {
    $input_created = false;
    $car_type = array();
    
    if($type_id > 0) {
      $car_type = cap_get_type($type_id);

    } else {
      $car_types = cap_get_types($cat_id);
      
      if(is_array($car_types) && count($car_types) > 0) {
        $car_type = $car_types[array_keys($car_types)[0]];
      }
    }

    if(is_array($car_type) && isset($car_type['pk_i_id'])) {
    ?>
      <div class="cap-search-hook" data-category-id="<?php echo osc_esc_html($cat_id); ?>">
        <strong class="cap-head"><?php _e('Car attributes', 'car_attr_pro'); ?></strong>
      
        <?php
          foreach($props as $pkey => $prop) {
            $pid = $prop['s_id'];
            
            if(!in_array($pid, array('location','category','price','keywords'))) {
              if($pid == 'type' || in_array(0, $prop['types']) || in_array($car_type['pk_i_id'], $prop['types'])) {
                if(in_array(0, $prop['categories']) || in_array($cat_id, $prop['categories'])) {      // This one may not be needed
                  cap_create_input($pkey, $car_type, array(cap_get_prop_db_column(cap_get_property_keys('type')) => $car_type['pk_i_id']), array('section' => 'search', 'category_id' => $cat_id));
                  $input_created = true;
                }
              }
            }
          }
        ?>
      
        <?php if(cap_param('more_filters_button') == 1 && $input_created) { ?>
          <a href="#" class="cap-search-more-prop">
            <svg width="16" height="16" viewBox="0 0 24 24" focusable="false" aria-hidden="true" xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" clip-rule="evenodd" d="M21 6h-8.337a3.5 3.5 0 00-6.326 0H3v3h3.337a3.5 3.5 0 006.326 0H21V6zM9.5 9a1.5 1.5 0 100-3 1.5 1.5 0 000 3zM21 15h-3.337a3.5 3.5 0 00-6.326 0H3v3h8.337a3.5 3.5 0 006.326 0H21v-3zm-6.5 3a1.5 1.5 0 100-3 1.5 1.5 0 000 3z"></path></svg>
            <?php _e('More filters', 'car_attr_pro'); ?>
          </a>
          
          <!-- SEARCH WIDGET BOX IS PLACED IN FOOTER -->
        <?php } ?>
        
        <?php osc_run_hook('cap_search_hook', $cat_id); ?>
      </div>
    <?php 
    }
 
    View::newInstance()->_exportVariableToView('cap_search_params', Params::getParamsAsArray());
  }
}

osc_add_hook('search_form', 'cap_search_hook');



// SEARCH BOX FOR SEARCH HOOK IN FOOTER
function cap_search_hook_footer() {
  if(osc_is_search_page() && cap_param('disable_search_hook') != 1 && cap_param('more_filters_button') == 1) {
  ?>
  <div class="cap-search-more-footer-holder" style="display:none;">
    <?php
      $params = array();
      
      if(View::newInstance()->_exists('cap_search_params')) {
        $params = View::newInstance()->_get('cap_search_params');
      }
    
      cap_search_box(array(
        'with_form' => true,
        'ignore_form_tag' => true,
        'design' => 'complex',
        'menu' => 'advanced',
        'menu_items' => 'condensed',
        'style' => 'borderless',
        'width' => 'large',
        'close_btn' => true,
        'search_params' => $params
      ));
    ?>
  </div>
  <?php
  }
}

osc_add_hook('footer', 'cap_search_hook_footer', 3);



// USE SEARCH PARAMS TO FILTER ITEMS
function cap_search_extend() {
  $props = cap_get_properties();
  $params = Params::getParamsAsArray();
  $filters = array();
  
  foreach($params as $pname => $val) {
    if(substr($pname, 0, strlen(CAP_PARAM_PREFIX)+1) == CAP_PARAM_PREFIX . '-') {
      $id_ = explode('-', $pname);
      $last = end($id_);

      $range = '';
      if($last == 'from' || $last == 'to') {
        $range = $last;
        $pname = str_replace('-' . $range, '', $pname);
      }
      
      $multi = false;
      $selected_ids = array_filter(explode(' ', str_replace(array('+','%20'), ' ', $val)), 'strlen');
      
      if(count($selected_ids) > 1) {
        $multi = true;
      } else if(count($selected_ids) == 1) {
        $selected_ids = $selected_ids[0];
      } else {
        continue;
      }
      
      $id = substr($pname, strlen(CAP_PARAM_PREFIX)+1);   // identifier
      $key = cap_get_property_keys($id);
      $prop = $props[$key];
      
      if($key > 0 && cap_property_enabled($key) !== false) {
        if(cap_get_prop_db_column($key) != '' && $val != '') {
          $filters[$id] = array(
            'db_col' => cap_get_prop_db_column($key),
            'value' => $selected_ids,
            'data_type' => $prop['s_data_type'],
            'filter_type' => $prop['s_search_engine'],
            'range' => $range,
            'multi' => $multi,
            'value_keys_by_id' => $prop['values_key_by_id']
          );
        }
      }
    }
  }
  
  if(count($filters) > 0) {
    Search::newInstance()->addJoinTable('cap.fk_i_item_id', DB_TABLE_PREFIX.'t_cap_item_attr as cap', DB_TABLE_PREFIX.'t_item.pk_i_id = cap.fk_i_item_id', 'INNER');

    foreach($filters as $id => $data) {
      if($data['data_type'] == 'FK_INTEGER') {
        $value = $data['value'];
        $sql_values = cap_separate_string_int_arr($data['value'], true);

        if($sql_values['string'] == '"DUMMY_CONDITION"') {
          $ids_selector = $sql_values['integer'];
        } else {
          if($id == 'type') {
            $ids_selector = sprintf(
              'SELECT fk_i_type_id FROM %st_cap_type_locale tl WHERE tl.fk_c_locale_code = "%s" AND (tl.s_slug in (%s) OR tl.fk_i_type_id in (%s))',
              DB_TABLE_PREFIX, cap_get_locale(), $sql_values['string'], $sql_values['integer']
            );
            
          } else if ($id == 'make') {
            $ids_selector = sprintf('SELECT pk_i_id FROM %st_cap_make WHERE (s_slug in (%s) OR pk_i_id in (%s))', DB_TABLE_PREFIX, $sql_values['string'], $sql_values['integer']);
            
          } else if ($id == 'model') {
            $ids_selector = sprintf('SELECT pk_i_id FROM %st_cap_model WHERE (s_slug in (%s) OR pk_i_id in (%s))', DB_TABLE_PREFIX, $sql_values['string'], $sql_values['integer']);
          } else {
            $keys_list = array();
            
            if(is_array($value) && count($value) > 0) {
              foreach($value as $vid) {
                if(isset($data['value_keys_by_id'][$vid])) {
                  $keys_list[] = (int)$data['value_keys_by_id'][$vid];
                }
              }
            } else {
              if(isset($data['value_keys_by_id'][$value])) {
                $keys_list[] = (int)$data['value_keys_by_id'][$value];
              }
            }
            
            $ids_selector = implode(',', $keys_list);
          }
        }
      
        Search::newInstance()->addConditions(sprintf('cap.%s in (%s)', $data['db_col'], $ids_selector));
      }
      
      if($data['data_type'] == 'INTEGER' || $data['data_type'] == 'DECIMAL') {
        if($data['range'] == 'from') {
          Search::newInstance()->addConditions(sprintf('cap.%s >= %d', $data['db_col'], (int)$data['value']));
        } else if($data['range'] == 'to') {
          Search::newInstance()->addConditions(sprintf('cap.%s <= %d', $data['db_col'], (int)$data['value']));
        } else if($data['multi'] === true) {
          Search::newInstance()->addConditions(sprintf('cap.%s in (%s)', $data['db_col'], (is_array($data['value']) ? implode(',', $data['value']) : (string)$data['value'])));
        } else {
          Search::newInstance()->addConditions(sprintf('cap.%s = %d', $data['db_col'], (int)$data['value']));
        }
      }


      // if(($data['data_type'] == 'TEXTAREA' || substr($data['data_type'], 0, 7) == 'VARCHAR') && $data['filter_type'] == 'like') {
      if($data['data_type'] == 'TEXTAREA' || (substr($data['data_type'], 0, 7) == 'VARCHAR' && $multi == false)) {
        if($data['filter_type'] == '' || $data['filter_type'] == 'EQUAL') {
          Search::newInstance()->addConditions(sprintf('cap.%s = "%s"', $data['db_col'], $data['value']));

        } else if ($data['filter_type'] == 'LIKE1') {
          Search::newInstance()->addConditions(sprintf('cap.%s like "%s%%"', $data['db_col'], $data['value']));

        } else if ($data['filter_type'] == 'LIKE2') {
          Search::newInstance()->addConditions(sprintf('cap.%s like "%%%s%%"', $data['db_col'], $data['value']));
        }
        
      } else if(substr($data['data_type'], 0, 7) == 'VARCHAR' && $multi === true) {
        Search::newInstance()->addConditions(sprintf('cap.%s in (%s)', $data['db_col'], '"' . implode('","', $data['value']) . '"'));
      }


      if($data['data_type'] == 'BOOLEAN') {
        Search::newInstance()->addConditions(sprintf('cap.%s = %d', $data['db_col'], (int)($data['value'] == 1 ? 1 : 0)));
      }
    }
  }
}

osc_add_hook('search_conditions', 'cap_search_extend');


// PUBLISH BOX
function cap_item_publish($cat_id, $item_id = '') {
  if($cat_id <= 0) {
    return false;
  }
  
  $item_attr = array();
  
  if($item_id <= 0) {
    $item_id = osc_item_id();
  }
  
  if($item_id > 0) {
    $item_attr = ModelCAP::newInstance()->getItemAttr($item_id);
  }
  
  $selected_type_id = 0;
  
  if(isset($item_attr['fk_i_type_id']) && $item_attr['fk_i_type_id'] > 0) {
    $selected_type_id = $item_attr['fk_i_type_id'];
  } else if(isset($types[0]) && isset($types[0]['fk_i_type_id'])) {
    $selected_type_id = $types[0]['fk_i_type_id'];
  }

  
  $types = cap_get_types($cat_id);
  ?>
  
  <?php if(is_array($types) && count($types) > 0) { ?>
    <?php
      $arr_type_key_id = array_search($selected_type_id, array_column($types, 'pk_i_id'));
      $type_selected = cap_get_type($selected_type_id);

      if(!isset($type_selected['pk_i_id'])) {
        $selected_type_id = array_keys($types)[0];
        $type_selected = $types[$selected_type_id];
      }
    ?>
    
    <div class="cap-publish cap-search" data-item-id="<?php echo $item_id; ?>" data-theme="<?php echo osc_current_web_theme(); ?>">
      <input type="hidden" name="<?php echo cap_get_input_name('type'); ?>" value="<?php echo $type_selected['pk_i_id']; ?>"/>

      <strong class="cap-head"><?php _e('Car attributes', 'car_attr_pro'); ?></strong>

      <div class="cap-section-top"<?php if(count($types) <= 1) { ?> style="display:none;"<?php } ?>>
        <?php foreach($types as $car_type) { ?>
          <?php $id = $car_type['pk_i_id']; ?>
          
          <a href="#" data-id="<?php echo $id; ?>" data-item-id="<?php echo $item_id; ?>" class="<?php if($selected_type_id == $id) { ?>cap-active<?php } ?>" <?php if($car_type['s_color'] != '') { ?>style="border-top-color:<?php echo $car_type['s_color']; ?>;"<?php } ?>>
            <?php echo cap_get_icon($car_type, 'type'); ?>
            <div><span><?php echo cap_get_name($car_type); ?></span></div>
          </a>
        <?php } ?>
      </div>
    
      <?php 
        cap_item_publish_type_section($type_selected, $item_attr);
      ?>
      
      <?php osc_run_hook('cap_item_publish', $cat_id, $item_id, $type_selected, $item_attr); ?>
    </div>
  <?php } ?>

  <?php  
}

// HOOKS MANAGEMENT
osc_add_hook('item_form', function($cat_id) {
  if(cap_param('hook_publish') == 1) {
    cap_item_publish($cat_id);
  }
});

osc_add_hook('item_edit', function($cat_id, $item_id) {
  if(cap_param('hook_publish') == 1) {
    cap_item_publish($cat_id, $item_id);
  }
});



// CREATE ONE TYPE SECTION FOR PUBLISH PAGE
function cap_item_publish_type_section($car_type, $item_attr = array()) {
  if(isset($car_type['pk_i_id'])) {
  ?>
    <?php $tid = $car_type['pk_i_id']; ?>
    
    <div class="cap-type-wrap" data-id="<?php echo $tid; ?>">
      <?php $j = 1; ?>
      <?php foreach(cap_get_sections('publish', $car_type['pk_i_id']) as $skey => $section) { ?>
        <?php 
          $sid = $section['s_id']; 
          $header = cap_get_help_text_publish($section);
          $section_props = cap_get_section_properties('publish', $skey, $car_type);
        ?>
        
        <?php if(is_array($section_props) && (count($section_props) > 0 || cap_param('enable_empty_sections') == 1)) { ?>
          <?php if(1==2 && $sid == 'main') { ?>
            <strong class="cap-head"><?php _e('Car attributes', 'car_attr_pro'); ?></strong>
            
          <?php } else if(!in_array($sid, array('main')) && $header != '-') { ?>
            <strong class="cap-subhead"><?php echo ($header <> '' ? $header : cap_get_name($section)); ?></strong>
          <?php } ?>
          
          <div class="cap-section cap-section-<?php echo $sid; ?>">
            <?php if(count($section_props) > 0) { ?>
              <?php foreach($section_props as $pkey => $prop) { ?>
                <?php 
                  $pid = $prop['s_id'];
                  $pstyle = $prop['s_style'];

                  $input_type = '';
                  if($prop['s_data_type'] == 'INTEGER' || $prop['s_data_type'] == 'DECIMAL') {
                    $input_type = 'number';
                    
                    if($prop['s_style'] != 'input') {
                      $prop['s_style'] = 'input';
                    }
                  }
                  
                  //} else if($prop['s_style'] == 'checkbox' || $prop['s_style'] == 'select' || $prop['s_style'] == 'slider') {
                  if($prop['b_multi_select'] == 1 && ($prop['s_style'] == 'checkbox' || $prop['s_style'] == 'slider')) {
                    $pstyle = 'select';
                  }

                  if(!in_array($pid, array('location','category','price','type'))) {
                    cap_create_input($pkey, $car_type, ($item_attr === false ? array() : $item_attr), array('style' => $pstyle, 'multi' => 'one', 'range' => 'norange', 'section' => 'publish'));
                  }
                ?>
              <?php } ?>
            <?php } ?>
            
            <?php osc_run_hook('cap_item_publish_type_section', $car_type, $item_attr, $section); ?>
          </div>
          
          <?php $j++; ?>
        <?php } ?>
      <?php } ?>
    </div>
  <?php
  }
}


// SAVE PUBLISHED DATA TO ITEM ATTRIBUTE
function cap_item_publish_data($item) {
  if(!isset($item['pk_i_id']) || $item['pk_i_id'] <= 0) {
    return false;
  }
  
  $item_id = $item['pk_i_id'];
  $params = Params::getParamsAsArray();
  $data = array();
  
  if(is_array($params) && count($params) > 0) {
    foreach($params as $pname => $val) {
      if(substr($pname, 0, strlen(CAP_PARAM_PREFIX)+1) == CAP_PARAM_PREFIX . '-') {
        $id = substr($pname, strlen(CAP_PARAM_PREFIX)+1);
        $key = cap_get_property_keys($id);
        
        if(cap_get_prop_db_column($key) != '' && cap_convert_prop_value($key, $val) !== null) {
          $data[cap_get_prop_db_column($key)] = cap_convert_prop_value($key, $val);
        }
      }
    }
  }
  
  // Update database
  if(is_array($data) && count($data) > 0) {
    ModelCAP::newInstance()->updateItemAttr((int)$item_id, $data);
  }
}

osc_add_hook('posted_item', 'cap_item_publish_data');
osc_add_hook('edited_item', 'cap_item_publish_data');


// GENERATE CAR TYPES WIDGET
function cap_widget_types($with_header = true, $wrap_elements = false) {
  $types = cap_get_types();
  ?>

  <?php if(is_array($types) && count($types) > 0) { ?>
    <?php if($with_header) { ?>
      <strong class="cap-wiget-header"><?php _e('Types', 'car_attr_pro'); ?></strong>
    <?php } ?>
    
    <div class="cap-widget cap-nice-scroll-box cap-types<?php if($wrap_elements) { ?> cap-wrap-elem<?php } ?><?php if(cap_param('widget_hide_scroll') == 1) { ?> cap-nice-scroll-no-overflow-force<?php } ?>">
      <a href="#" class="cap-nice-scroll-btn cap-nice-scroll-prev"><?php echo CAP_SCROLL_PREV; ?></a>
      
      <div class="cap-wrap cap-nice-scroll">
        <?php foreach($types as $car_type) { ?>
          <?php $id = $car_type['pk_i_id']; ?>
          
          <a href="<?php echo osc_search_url(array(cap_get_input_name('type') => cap_get_slug($car_type))); ?>" data-id="<?php echo $id; ?>">
            <?php echo cap_get_icon($car_type, 'type', false); ?>
            <div><span><?php echo cap_get_name($car_type); ?></span></div>
          </a>
        <?php } ?>
        
        <?php osc_run_hook('cap_widget_types', $with_header, $wrap_elements, $types); ?>
      </div>
      
      <a href="#" class="cap-nice-scroll-btn cap-nice-scroll-next"><?php echo CAP_SCROLL_NEXT; ?></a>
    </div>
  <?php } ?>
  <?php
}



// GENERATE CAR MAKES WIDGET
function cap_widget_makes($car_type_id = 0, $with_icon = true, $with_header = true, $wrap_elements = false) {
  $car_type = cap_get_type($car_type_id);
  
  if(!isset($car_type['pk_i_id'])) {
    return false;
  }
  
  $makes = ModelCAP::newInstance()->getMakes((int)$car_type['pk_i_id'], true, $with_icon);
  ?>

  <?php if(is_array($makes) && count($makes) > 0) { ?>
    <?php if($with_header) { ?>
      <strong class="cap-widget-header"><?php _e('Makes', 'car_attr_pro'); ?></strong>
    <?php } ?>
    
    <div class="cap-widget cap-nice-scroll-box cap-makes<?php if($wrap_elements) { ?> cap-wrap-elem<?php } ?><?php if(cap_param('widget_hide_scroll') == 1) { ?> cap-nice-scroll-no-overflow-force<?php } ?>">
      <a href="#" class="cap-nice-scroll-btn cap-nice-scroll-prev"><?php echo CAP_SCROLL_PREV; ?></a>
    
      <div class="cap-wrap cap-nice-scroll">
        <?php foreach($makes as $make) { ?>
          <?php 
            $id = $make['pk_i_id']; 
            $search_params = array();
            
            if(isset($car_type['pk_i_id'])) {
              $search_params[cap_get_input_name('type')] = cap_get_slug($car_type);
            }
            
            $search_params[cap_get_input_name('make')] = cap_get_slug($make);
          ?>
          
          <a href="<?php echo osc_search_url($search_params); ?>" data-id="<?php echo $id; ?>">
            <?php echo cap_get_icon($make, 'make'); ?>
            <div><span><?php echo ($make['s_name'] <> '' ? $make['s_name'] : __('N/A', 'car_attr_pro')); ?></span></div>
          </a>
        <?php } ?>
        
        <?php osc_run_hook('cap_widget_makes', $car_type_id, $with_icon, $with_header, $wrap_elements, $makes); ?>
      </div>
      
      <a href="#" class="cap-nice-scroll-btn cap-nice-scroll-next"><?php echo CAP_SCROLL_NEXT; ?></a>
    </div>
  <?php } ?>
  <?php
}



// GENERATE CAR MODELS WIDGET
function cap_widget_models($make_id, $car_type_id = 0, $with_header = true, $wrap_elements = false) {
  $make = ModelCAP::newInstance()->getMake($make_id);
  $models = ModelCAP::newInstance()->getModels($make_id);
  ?>

  <?php if(is_array($models) && count($models) > 0) { ?>
    <?php if($with_header) { ?>
      <strong class="cap-widget-header">
        <?php echo sprintf(__('%s models', 'car_attr_pro'), cap_get_name($make)); ?>
      </strong>
    <?php } ?>
    
    <div class="cap-widget2 cap-models<?php if($wrap_elements) { ?> cap-wrap-elem<?php } ?>">
      <div class="cap-wrap">
        <?php foreach($models as $model) { ?>
          <?php 
            $id = $model['pk_i_id']; 
            $search_params = array();
            
            if($car_type_id > 0) {
              $car_type = cap_get_type($car_type_id);
              $search_params[cap_get_input_name('type')] = cap_get_slug($car_type);
            }
            
            $search_params[cap_get_input_name('make')] = cap_get_slug($make);
            $search_params[cap_get_input_name('model')] = cap_get_slug($model);
          ?>
          
          <a href="<?php echo osc_search_url($search_params); ?>" data-id="<?php echo $id; ?>">
            <?php echo cap_get_icon($model, 'model'); ?>
            <div><span><?php echo ($model['s_name'] <> '' ? $model['s_name'] : __('N/A', 'car_attr_pro')); ?></span></div>
          </a>
        <?php } ?>
        
        <?php osc_run_hook('cap_widget_models', $make_id, $car_type_id, $with_header, $wrap_elements, $make, $models); ?>
      </div>
    </div>
  <?php } ?>
  <?php
}



// CREATE INPUT/SELECT/AUTOCOMPLETE
// From-to, add these at start of $style, ie from_select_multi
//function cap_create_input($id, $car_type, $style = '', $range = '', $category_id = 0, $selected_values = false, $is_active = true) {
function cap_create_input($key, $car_type, $selected_values = false, $options = array()) {
  $properties = cap_get_properties();
  
  // Apply filters for customization
  $properties = osc_apply_filter('cap_create_input_properties', $properties, $key, $car_type, $selected_values, $options);
  $selected_values = osc_apply_filter('cap_create_input_selected_values', $selected_values, $key, $car_type, $options);
  $options = osc_apply_filter('cap_create_input_options', $options, $key, $car_type, $selected_values);
  
  if($key == '' || !isset($properties[$key])) {
    return false;
  }
  
  $prop = $properties[$key];
  $id = $prop['s_id'];

  
  // Get options
  $range = isset($options['range']) ? $options['range'] : ($prop['b_range'] == 1 ? 'range' : '');
  $category_id = isset($options['category_id']) ? $options['category_id'] : 0;
  $is_active = isset($options['is_active']) ? $options['is_active'] : true;
  $section = isset($options['section']) ? $options['section'] : '';
  $multi = isset($options['multi']) ? $options['multi'] : ($prop['b_multi_select'] == 1 ? 'multi' : 'one');
  $style = isset($options['style']) ? $options['style'] : $prop['s_style'];
  $groups = isset($options['groups']) ? $options['groups'] : ($prop['s_group_list'] <> '' ? array_filter(explode(',', $prop['s_group_list'])) : '');
  $subgroup = isset($options['subgroup']) ? $options['subgroup'] : false;
  $group_parent = isset($options['group_parent']) ? $options['group_parent'] : '';
  $input_type = isset($options['input_type']) ? $options['input_type'] : 'text';
  $box_maximize = isset($options['box_maximize']) ? $options['box_maximize'] : '';

  // No check for subgroup
  if(cap_property_enabled($key, $section) === false) {
    return false;
  }

  //$is_publish = (Params::getParam('page') == 'ajax' && Params::getParam('action') == 'runhook' && substr(Params::getParam('hook'), 0, 4) == 'item');
  $is_publish = ($section == 'publish' ? true : false);
  $is_search = ($section == 'search' ? true : false);
  $is_widget = ($section == 'widget' ? true : false);
  
  
  // Subgroups without initiation from parent groups disabled
  $prop_in_groups = cap_get_properties_groups();
  
  if((in_array($key, $prop_in_groups) || in_array($id, $prop_in_groups)) && $group_parent == '') {
    return false;
  }

  
  if($id == 'type' && empty($car_type)) {
    $car_type = array('pk_i_id' => 0);
    $type_id = -1;
  } else {  
    $type_id = $car_type['pk_i_id'];
  }


  $db_type = $prop['s_data_type'];
  

  // if($id == 'keywords' && $is_publish) {
    // $style = 'textarea';
    // $range = '';
  // }
  
  // Textarea allowed just on publish page
  if($style == 'textarea' && !$is_publish) {
    $style = 'input';
  }
  
  
  // Change integer selects to inputs on publish
  $required = false;
  $required_html5 = false;
  $required_jqval = false;
  
  if($is_publish) {
    if($db_type == 'INTEGER' || $db_type == 'DECIMAL') {
      $style = 'input';
      $input_type = 'number';
    } else if(substr($db_type, 0, 7) == 'VARCHAR' && $style != 'textarea') {
      $style = 'input';
      $input_type = 'text';
    }
    
    if($prop['b_required'] == 1) {
      $required = true; 
      
      if(cap_param('required_validation') == '') {
        $required_jqval = true;
      } else {
        $required_html5 = true;
      }
    }
  }
  
  // Allow location & category just as simple selects
  if($id == 'location' || $id == 'category') {
    $style = 'select';
    $range = '';
  }
  
  if($box_maximize == '') {
    if(
      $prop['s_size'] == 'MAX_WIDGET' && $is_widget 
      || $prop['s_size'] == 'MAX_SEARCH' && $is_search 
      || $prop['s_size'] == 'MAX_PUBLISH' && $is_publish 
      || $prop['s_size'] == 'MAX_ALL'
    ) {
      $box_maximize = 1;
    } else if (
      $prop['s_size'] == 'MAX_WIDGET_MOBILE' && $is_widget 
      || $prop['s_size'] == 'MAX_SEARCH_MOBILE' && $is_search 
      || $prop['s_size'] == 'MAX_PUBLISH_MOBILE' && $is_publish 
      || $prop['s_size'] == 'MAX_ALL_MOBILE'    
    ) {
      $box_maximize = 2;
    }
  }
  
  $box_maximize = ($box_maximize == '' ? 0 : $box_maximize);
  $box_max_class = '';

  if($box_maximize == 1) {
    $box_max_class = ' cap-maxsize';
  } else if ($box_maximize == 2) {
    $box_max_class = ' cap-maxsize-mobile';
  }

  
  if(!in_array($input_type, array('text','number','url'))) {
    $input_type = 'text';
  }
  
  if(in_array($style, array('input','textarea'))) {
    $multi = 'one';
  }
  
  $slug = $id;
  $range_string = '';


  // Disable range for multi-select options
  if($multi != 'one' || $range == 'norange') {
    $range = '';
  } else {
    if($range == 'from' || $range == 'to') {
      $slug .= '-' . $range;
      $range_string = ($range == 'from' ? __('from', 'car_attr_pro') : __('to', 'car_attr_pro'));
    }
  }

  // Disable multi if style is HTML select
  if($style == 'select' && cap_param('select_box_type') == 'HTML') {
    $multi = 'one';
  }
  
  // Check if input is disabled
  $disabled = ($is_active === false ? true : false);

  // Get selected values
  $select_all = true;
  $deselect_all = true;
  
  $selected_ids = '';
  $selected_slugs = '';
  $selected_ids_count = 0;
  $selected_ids_name = '';
  $selected_ids_first = '';
  

  // Try to get selected list
  if($selected_values === false) {
    $item_id = (osc_item_id() > 0 ? osc_item_id() : Params::getParam('itemId'));
    if($item_id > 0) {
      $selected_values = ModelCAP::newInstance()->getItemAttr($item_id);
    }
  }
  
  // Get slug of type
  $selected_type = array();
  if(is_array($selected_values) && !empty($selected_values) && isset($selected_values[cap_get_prop_db_column($key)]) && $id == 'type') {
    $selected_type = cap_get_type($selected_values[cap_get_prop_db_column($key)]);
    
    if(isset($selected_type['s_slug'])) {
      $selected_slugs = $selected_type['s_slug'];
    }
  }
  
  // Get list of selected IDs
  if(is_array($selected_values) && !empty($selected_values) && isset($selected_values[cap_get_prop_db_column($key)])) {
    $selected_ids = $selected_values[cap_get_prop_db_column($key)];
    
  } else {   
    $selected_ids = Params::getParam(cap_get_input_name($slug));
  }
  
  if($multi != 'one' && $style != 'input') {
    if(!is_array($selected_ids)) {
      $selected_ids = array_filter(explode(' ', str_replace(array('+','%20'), ' ', $selected_ids)));
    }
    
    $selected_ids_count = count($selected_ids);
    
  } else {
    if(is_array($selected_ids)) {
      $selected_ids = isset($selected_ids[0]) ? $selected_ids[0] : '';
    }
  }

  // Avoid mixing selected values across different types on search
  if($disabled) {
    $selected_ids = array();
  }

  // Get property/attribute values
  if($id == 'type') {
    $values = cap_get_types((int)$category_id);

  } else if($id == 'make') {
    $values = ModelCAP::newInstance()->getMakes((int)$type_id);

  } else if ($id == 'model') {
    if(is_array($selected_values) && !empty($selected_values) && isset($selected_values[cap_get_prop_db_column(cap_get_property_keys('make'))])) {
      $make_ids = $selected_values[cap_get_prop_db_column(cap_get_property_keys('make'))];
    } else {   
      $make_ids = Params::getParam(cap_get_input_name('make'));
      $make_ids = explode(' ', str_replace(array('+','%20'), ' ', (string)$make_ids));
    }
  
    //$make_ids = explode(' ', str_replace(array('+','%20'), ' ', (string)Params::getParam(cap_get_input_name('make'))));
    $values = ModelCAP::newInstance()->getModels($make_ids);
    
    if(count($values) == (cap_param('models_limit') > 0 ? cap_param('models_limit') : 200)) {
      $values[-1] = array('pk_i_id' => -1,'fk_i_make_id' => -1,'s_name' => __('Limit reached, refine makes to get more models', 'car_attr_pro'),'s_slug' => '');
    }

    if(!is_array($values) || count($values) == 0) {
      $disabled = true;
    }
    
  } else if ($id == 'category') {
    $car_type_categories = ModelCAP::newInstance()->getTypeCategories($type_id);
    $category_ids = is_array($car_type_categories) ? array_column($car_type_categories, 'fk_i_category_id') : array();
    
    $values = array();
    if(!in_array(0, $category_ids)) {    // means "all categories"
      if(count($category_ids) > 0) {
        foreach($category_ids as $cid) {
          if(!in_array($cid, array_keys($values))) {
            $crow = Category::newInstance()->findByPrimaryKey($cid);
            
            if(isset($crow['pk_i_id'])) {
              $values[$crow['pk_i_id']] = array(
                'pk_i_id' => $crow['pk_i_id'],
                's_name' => $crow['s_name'],
                'level' => 0
              );
              
              $sub_tree = Category::newInstance()->toSubTree($crow['pk_i_id']);
              $crow['categories'] = $sub_tree;
              
              $sub_values = cap_val_category_sub($crow, 1, array_keys($values));
              
              if(is_array($sub_values) && count($sub_values) > 0) {
                $values = $values + $sub_values;  // array merge with keys preserve
              }
            }
          }
        }
      }
    } else {
      $cat_tree = cap_get_categories_to_tree();

      $dummy = array();
      $dummy['categories'] = $cat_tree;
      $values = cap_val_category_sub($dummy, 0);
    }

  } else if ($id == 'location') {
    $values = array();
    
    $countries = cap_get_countries();

    if(is_array($countries) && count($countries) > 1) {
      foreach($countries as $ctr) {
        $values[] = array(
          'pk_i_id' => $ctr['pk_c_code'],
          's_name' => osc_location_native_name_selector($ctr, 's_name'),
          'type' => 'country',
          'level' => 0
        );

        $regions = cap_get_regions($ctr['pk_c_code']);
        
        if(is_array($regions) && count($regions) > 0) {
          foreach($regions as $reg) {
            $values[] = array(
              'pk_i_id' => $reg['pk_i_id'],
              's_name' => osc_location_native_name_selector($reg, 's_name'),
              'type' => 'region',
              'level' => 1
            );
          }
        }
      }
    } else {
      $regions = cap_get_regions();
      
      if(is_array($regions) && count($regions) > 0) {
        foreach($regions as $reg) {
          $values[] = array(
            'pk_i_id' => $reg['pk_i_id'],
            's_name' => osc_location_native_name_selector($reg, 's_name'),
            'type' => 'region',
            'level' => 0
          );
        }
      }
    }
    
  } else {
    $values = cap_get_property_values($key, $car_type['pk_i_id']);
  }

  
  // Get selected value name
  if(is_array($values) && count($values) > 0) {
    if(is_array($selected_ids)) {
      $selected_ids_first = (isset($selected_ids[0]) ? $selected_ids[0] : '');
    } else if(trim((string)$selected_ids) != '') {
      $selected_ids_first = $selected_ids;
    }
    
    if($selected_ids_first != '') {
      if(isset($values[$selected_ids_first]) && isset($values[$selected_ids_first]['s_name'])) {
        $selected_ids_name = cap_get_name($values[$selected_ids_first]);
      } else {

        // Selected ID may be SLUG
        foreach($values as $svl) {
          //if(isset($svl['s_slug']) && ($svl['s_slug'] == $selected_ids_first || $svl['pk_i_id'] == $selected_ids_first)) {
          if(isset($svl['s_id']) && ($svl['s_id'] == $selected_ids_first || $svl['pk_i_id'] == $selected_ids_first)) {
            $selected_ids_name = cap_get_name($svl);
            break;
          }
          
          // Makes, models, types use slug
          if(isset($svl['s_slug']) && ($svl['s_slug'] == $selected_ids_first || $svl['pk_i_id'] == $selected_ids_first)) {
            $selected_ids_name = cap_get_name($svl);
            break;
          }
        }
      }
    }
  }
  
  // Select only one for input/textarea
  if(is_array($selected_ids) && (in_array($style, array('input','textarea')) || $multi == 'one')) {
    $selected_ids = array_filter($selected_ids);
    $selected_ids = end($selected_ids);
  }
  
  // If values are essential for type of attribute and none were found
  if($id != 'model' && $id != 'make' && cap_param('hide_if_no_values') == 1) {
    if($db_type == 'INTEGER' || $db_type = 'FK_INTEGER' || substr($db_type, 0, 7) == 'VARCHAR') {
      if(!is_array($values) || count($values) <= 0) {
        if($style == 'select' || ($style == 'slider' && $prop['s_data_type'] != 'BOOLEAN') || ($style == 'checkbox' && $multi != 'one')) {
          echo '<!-- ' . sprintf(__('[CAP] Attribute "%s" not generated due to missing values', 'car_attr_pro'), strtoupper($id)) . '-->';
          return false;
        }
      }
    }
  }
  

  // Range integer inputs - use entered value even if not in list
  if($selected_ids_name == '' && ($is_search || $is_widget) && ($db_type == 'INTEGER' || $db_type == 'DECIMAL')) {
    $selected_ids_name = $selected_ids;
  }


  if(isset($prop['delimiter']) && $prop['delimiter'] != '') {
    ?>
    <div class="cap-input-box cap-delimiter cap-del-<?php echo $prop['delimiter']; ?>"></div>
    <?php
  }

  // Define measure (km, kg, ...)
  $measure_text = '';
  if($prop['s_measure_unit'] <> '') { 
    $measure_text = ' ' . $prop['s_measure_unit']; 
    
    if($selected_ids_name <> '') {
      $selected_ids_name .= $measure_text;
    }
  }
  
  // Label text and placeholder text
  $name_text = cap_get_name($prop);
  $name_alt_text = cap_get_name_alt($prop);
  $label_text = $name_text;
  $description_text = trim(cap_get_description($prop));
  $placeholder_text = $label_text;

  
  if($is_publish && cap_get_help_text_publish($prop) != '') {
    $placeholder_text = cap_get_help_text_publish($prop);
  } else if(($is_search || $is_widget) && cap_get_help_text_search($prop) != '') {
    $placeholder_text = cap_get_help_text_search($prop);
  }

  if($is_publish && $required) {
    $label_text .= '<em class="cap-req">*</em>';
  }
  
  // Create help icon for description text
  $description_html = ' ';
  
  if($description_text != '') {
    $description_html .= '<div class="cap-help-elem">';
    $description_html .= CAP_HELP_SVG;
    $description_html .= '<div class="cap-help-elem-text">' . $description_text . '</div>';
    $description_html .= '</div>';
  }
  
  $label_text .= $description_html;


  // Start building inputs
  if(is_array($groups) && count($groups) > 0) {
  ?>
    <div class="cap-input-box cap-groups" data-id="<?php echo $slug; ?>">
      <strong class="cap-line"><?php echo $label_text; ?></strong>
      <div class="cap-groups-wrap">
        <?php
          $new_opt = $options;
          unset($new_opt['style']);
          $new_opt['groups'] = array();
          $new_opt['group_parent'] = $id;
          $new_opt['input_type'] = 'number';
          
          foreach($groups as $group_member_key) {
            $new_opt['subgroup'] = true;
            cap_create_input($group_member_key, $car_type, ($selected_values === false ? array() : $selected_values), $new_opt);
          }
        ?>
      </div>
    </div>
  <?php

  } else if($range == 'both' || $range == 'range') {
  ?>
    <div class="cap-input-box cap-range" data-id="<?php echo $slug; ?>">
      <strong class="cap-line"><?php echo $label_text; ?></strong>
      <div class="cap-range-wrap">
        <?php
          $new_opt = $options;
          $new_opt['style'] = $style;
          $new_opt['range'] = 'from';
          cap_create_input($key, $car_type, ($selected_values === false ? array() : $selected_values), $new_opt);

          $new_opt['range'] = 'to';
          cap_create_input($key, $car_type, ($selected_values === false ? array() : $selected_values), $new_opt);
        ?>
      </div>
    </div>
  <?php
  
  } else if($style == 'textarea') {
  ?>
    <div class="cap-input-box cap-<?php echo $style; ?> cap-<?php echo $multi; ?><?php echo $box_max_class; ?>" data-id="<?php echo $slug; ?>" data-type="<?php echo $car_type['pk_i_id']; ?>" data-section="<?php echo $section; ?>">
      <strong class="cap-line"><?php echo $label_text; ?></strong>
      <textarea name="<?php echo cap_get_input_name($slug); ?>" class="cap-textarea<?php if($required_jqval) { ?> cap-required<?php } ?>" placeholder="<?php echo osc_esc_html($placeholder_text); ?>" data-label="<?php echo osc_esc_html($name_text); ?>"<?php if($required_html5) { ?> required<?php } ?>><?php echo osc_esc_html((string)$selected_ids); ?></textarea>
    </div>
  <?php
  
  } else if($style == 'input') {
  ?>
    <div class="cap-input-box cap-<?php echo $style; ?> cap-<?php echo $multi; ?><?php echo $box_max_class; ?>" data-id="<?php echo $slug; ?>" data-type="<?php echo $car_type['pk_i_id']; ?>"<?php if($range != '') { ?> data-range="<?php echo $range; ?>"<?php } ?> data-section="<?php echo $section; ?>">
      <?php if($range == '') { ?>
        <strong class="cap-line"><?php echo $label_text; ?></strong>
      <?php } ?>
      
      <input name="<?php echo cap_get_input_name($slug); ?>" class="cap-input<?php if($is_publish && trim($measure_text) != '') { ?> cap-has-meas<?php } ?><?php if($required_jqval) { ?> cap-required<?php } ?>" type="<?php echo $input_type; ?>" <?php if($input_type == 'number') { ?>min="0" step="0.01"<?php } ?> value="<?php echo osc_esc_html(($input_type == 'number' ? ($selected_ids != '' ? (float)$selected_ids : '') : (string)$selected_ids)); ?>" placeholder="<?php echo osc_esc_html($range_string <> '' ? $placeholder_text . ' ' . $range_string : $placeholder_text); ?>" data-label="<?php echo osc_esc_html($name_text); ?>"<?php if($required_html5) { ?> required<?php } ?>/>

      <?php if(trim($measure_text) != '') { ?>
        <em class="cap-meas"><?php echo trim($measure_text); ?></em>
      <?php } ?>
    </div>
  <?php
  
  } else if($style == 'checkbox') { 
    if($multi == 'one') {
    ?>
      <div class="cap-input-box cap-<?php echo $style; ?> cap-<?php echo $multi; ?><?php echo $box_max_class; ?>" data-id="<?php echo $slug; ?>" data-type="<?php echo $car_type['pk_i_id']; ?>" data-section="<?php echo $section; ?>">
        <label class="cap-label">
          <input name="<?php echo cap_get_input_name($slug); ?>" type="checkbox" value="1" <?php if((int)$selected_ids == 1) { ?>checked<?php } ?>/>
          <span><?php echo $label_text; ?></span>
        </label>
      </div>
    <?php 
    
    } else if(is_array($values) && count($values) > 0) { 
    ?>
      <div class="cap-input-box cap-<?php echo $style; ?> cap-<?php echo $multi; ?><?php echo $box_max_class; ?>" data-id="<?php echo $slug; ?>" data-type="<?php echo $car_type['pk_i_id']; ?>" data-section="<?php echo $section; ?>">
        <input name="<?php echo cap_get_input_name($slug); ?>" type="hidden" <?php if($disabled) { ?>disabled<?php } ?> value="<?php echo osc_esc_html(is_array($selected_ids) ? implode(' ', $selected_ids) : $selected_ids); ?>"/>

        <?php if($range == '') { ?>
          <strong class="cap-line"><?php echo $label_text; ?></strong>
        <?php } ?>
        
        <?php foreach($values as $vid => $val) { ?>
          <label class="cap-label">
            <input data-id="cap-<?php echo $slug; ?>-<?php echo cap_get_slug($val, $vid, $section); ?>" type="checkbox" value="<?php echo cap_get_slug($val, $vid, $section); ?>" <?php if(in_array(cap_get_slug($val, $vid, $section), $selected_ids) || in_array($vid, $selected_ids)) { ?>checked<?php } ?>/>
            <span><?php echo cap_get_name($val); ?></span>
          </label>
        <?php } ?>
      </div>
    <?php
    }

  } else if($style == 'slider') {
    if($prop['s_data_type'] == 'BOOLEAN') {
      $values = array(
        '1' => array('s_name' => $name_text),
        '0' => array('s_name' => ($name_alt_text <> '' ? $name_alt_text : __('No', 'car_attr_pro')))
      );
      
    } 
  ?>
    <div class="cap-input-box cap-<?php echo $style; ?> cap-<?php echo $multi; ?><?php echo $box_max_class; ?>" data-id="<?php echo $slug; ?>" data-type="<?php echo $car_type['pk_i_id']; ?>" data-section="<?php echo $section; ?>">
      <?php if($range == '') { ?>
        <strong class="cap-line"><?php echo $label_text; ?></strong>
      <?php } ?>
      
      <div class="cap-slider-wrap">
        <?php $j = 0; ?>
        <?php foreach($values as $vid => $val) { ?>
          <label class="cap-label">
            <input class="<?php if($required_jqval) { ?>cap-required<?php } ?>" name="<?php echo cap_get_input_name($slug); ?>" <?php if($disabled) { ?>disabled<?php } ?> data-id="cap-<?php echo $slug; ?>-<?php echo cap_get_slug($val, $vid, $section); ?>" type="radio" value="<?php echo cap_get_slug($val, $vid, $section); ?>" <?php if(in_array(cap_get_slug($val, $vid, $section), (is_array($selected_ids) ? $selected_ids : array($selected_ids))) || (empty(is_array($selected_ids) ? $selected_ids : array($selected_ids)) && $j == 0)) { ?>checked<?php } ?> data-label="<?php echo osc_esc_html($name_text); ?>"<?php if($required_html5) { ?> required<?php } ?>/>
            <span><?php echo cap_get_name($val); ?></span>
          </label>
          
          <?php $j++; ?>
        <?php } ?>
      </div>
    </div>
  <?php
  
  } else if($style == 'select' && cap_param('select_box_type') == 'HTML') { 
    // Multiselect not available here
    $sel_multiple = ''; //($multi == 'one' ? '' : ' multiple');
    ?>
      <div class="cap-input-box cap-<?php echo $style; ?> cap-<?php echo $multi; ?><?php if($disabled) { ?> cap-disabled<?php } ?><?php echo $box_max_class; ?>" data-id="<?php echo $slug; ?>" data-type="<?php echo $car_type['pk_i_id']; ?>"<?php if($range != '') { ?> data-range="<?php echo $range; ?>"<?php } ?> data-db-type="<?php echo strtolower($db_type); ?>" data-section="<?php echo $section; ?>">
        <?php if($range == '') { ?>
          <label class="cap-label"><?php echo $label_text; ?></label>
        <?php } ?>
        
        <div class="cap-input-wrap">
          <select class="cap-input<?php if($required_jqval) { ?> cap-required<?php } ?>" name="<?php echo cap_get_input_name($slug); ?>" <?php if($disabled) { ?>disabled<?php } ?><?php echo $sel_multiple; ?> data-label="<?php echo osc_esc_html($name_text); ?>"<?php if($required_html5) { ?> required<?php } ?>/>
            <option value=""<?php if($selected_ids == '') { ?>selected="selected"<?php } ?>><?php echo ($range_string <> '' ? $placeholder_text . ' ' . $range_string : $placeholder_text); ?></option>

            <?php 
              $current_section = ''; 
              $prev_section = '';
            ?>
            
            <?php foreach($values as $vid => $val) { ?>
              <?php 
                // Identify if top section has ended
                if(isset($val['s_section']) && $val['s_section'] != '') { 
                  $current_section = $val['s_section'];
                } else {
                  $current_section = '';
                }
              ?>
                
              <?php if($current_section != $prev_section && $prev_section != '') { ?>
                <option disabled data-end="<?php echo $prev_section; ?>">--------</option>
              <?php } ?>
              
              <option value="<?php echo cap_get_slug($val, $vid, $section); ?>"<?php if($selected_ids == cap_get_slug($val, $vid, $section) || $vid == $selected_ids) { ?> selected="selected"<?php } ?><?php if(isset($val['level'])) { ?> data-level="<?php echo $val['level']; ?>"<?php } ?><?php if($id == 'location') { ?> data-country="<?php echo $val['type'] == 'country' ? $val['pk_i_id'] : ''; ?>" data-region="<?php echo $val['type'] == 'region' ? $val['pk_i_id'] : ''; ?>" data-city=""<?php } ?>>
                <?php 
                  if(isset($val['level']) && intval($val['level']) > 0) { 
                    echo str_repeat('&middot;', intval($val['level'])) . (intval($val['level']) > 0 ? ' ' : '');;
                  }

                  echo cap_get_name($val) . $measure_text;
                ?>
              </option>
            <?php } ?>
          </select>
        </div>
      </div>
    <?php
  
  } else if($style == 'select') { 
  
    if($multi == 'one') {
    ?>
      <div class="cap-input-box cap-<?php echo $style; ?> cap-<?php echo $multi; ?><?php if($disabled) { ?> cap-disabled<?php } ?><?php echo $box_max_class; ?>" data-id="<?php echo $slug; ?>" data-type="<?php echo $car_type['pk_i_id']; ?>"<?php if($range != '') { ?> data-range="<?php echo $range; ?>"<?php } ?> data-db-type="<?php echo strtolower($db_type); ?>" data-section="<?php echo $section; ?>">
        <?php if($range == '') { ?>
          <label class="cap-label"><?php echo $label_text; ?></label>
        <?php } ?>
        
        <?php if($id == 'location') { ?>
          <input type="hidden" name="sCountry" value="<?php echo osc_esc_html(Params::getParam('sCountry')); ?>"/>
          <input type="hidden" name="sRegion" value="<?php echo osc_esc_html(Params::getParam('sRegion')); ?>"/>
          <input type="hidden" name="sCity" value="<?php echo osc_esc_html(Params::getParam('sCity')); ?>"/>
        <?php } ?>
        
        <div class="cap-input-wrap">
          <input name="<?php echo cap_get_input_name($slug); ?>" type="hidden" <?php if($disabled) { ?>disabled<?php } ?> value="<?php echo osc_esc_html((string)($selected_slugs <> '' ? $selected_slugs : $selected_ids)); ?>"/>
          <input data-id="<?php echo cap_get_input_name($slug); ?>-display" readonly <?php if($disabled) { ?>disabled<?php } ?> type="text" value="<?php echo osc_esc_html($selected_ids_name); ?>" class="cap-input cap-display<?php if($required_jqval) { ?> cap-required<?php } ?>" placeholder="<?php echo osc_esc_html($range_string <> '' ? $placeholder_text . ' ' . $range_string : $placeholder_text); ?>" data-label="<?php echo osc_esc_html($name_text); ?>"<?php if($required_html5) { ?> required<?php } ?>/>
          <input data-id="<?php echo cap_get_input_name($slug); ?>-filter" <?php if($disabled) { ?>disabled<?php } ?> type="text" value="" class="cap-input cap-filter" placeholder="<?php echo osc_esc_html(sprintf(__('Refine %s', 'car_attr_pro'), strtolower($name_text))); ?>" />
          <?php echo CAP_CARET_SVG; ?>
          <?php echo CAP_CLEAR_SVG; ?>
        </div>

        <?php if(is_array($values) && count($values) > 0) { ?>
          <div class="cap-values-list">
            <div class="cap-values-wrap">
              <?php if($deselect_all) { ?>
                <div class="cap-value cap-deselect" data-id=""><?php _e('Clear selection', 'car_attr_pro'); ?></div>              
              <?php } ?>

              <?php 
                $current_section = ''; 
                $prev_section = '';
              ?>
              
              <?php foreach($values as $vid => $val) { ?>
                <?php 
                  // Identify if top section has ended
                  if(isset($val['s_section']) && $val['s_section'] != '') { 
                    $current_section = $val['s_section'];
                  } else {
                    $current_section = '';
                  }
                ?>
                
                <?php if($vid == -1) { ?>
                  <label class="cap-value cap-disabled">
                    <span><em><?php echo cap_get_name($val); ?></em></span>
                  </label>
                  
                <?php } else { ?>
                  <?php if($current_section != $prev_section && $prev_section != '') { ?>
                    <em class="cap-top-section-end" data-end="<?php echo $prev_section; ?>"></em>
                  <?php } ?>
                
                  <div class="cap-value<?php if($selected_ids == cap_get_slug($val, $vid, $section) || $vid == $selected_ids) { ?> cap-active<?php } ?>" data-id="<?php echo cap_get_slug($val, $vid, $section); ?>"<?php if(isset($val['level'])) { ?> data-level="<?php echo $val['level']; ?>"<?php } ?><?php if($id == 'location') { ?> data-country="<?php echo $val['type'] == 'country' ? $val['pk_i_id'] : ''; ?>" data-region="<?php echo $val['type'] == 'region' ? $val['pk_i_id'] : ''; ?>" data-city=""<?php } ?>>
                    <?php 
                      if(isset($val['level']) && intval($val['level']) > 0) { 
                        echo str_repeat('&middot;', intval($val['level'])) . (intval($val['level']) > 0 ? ' ' : '');
                      }

                      echo cap_get_name($val) . $measure_text;
                    ?>
                  </div>
                <?php } ?>
              <?php } ?>
            </div>
            
            <?php if(cap_param('hide_values_ok_btn') != 1) { ?>
              <div class="cap-values-ok">
                <a href="#" class="cap-values-ok-btn"><?php _e('Ok', 'cap_attr_pro'); ?></a>
              </div>
            <?php } ?>
          </div>
        <?php } ?>
      </div>
    <?php 
    
    } else { 
    ?>
      <div class="cap-input-box cap-<?php echo $style; ?> cap-<?php echo $multi; ?><?php if($disabled) { ?> cap-disabled<?php } ?><?php echo $box_max_class; ?>" data-id="<?php echo $slug; ?>" data-type="<?php echo $car_type['pk_i_id']; ?>"<?php if($range != '') { ?> data-range="<?php echo $range; ?>"<?php } ?> data-db-type="<?php echo strtolower($db_type); ?>" data-section="<?php echo $section; ?>">
        <?php if($range == '') { ?>
          <label class="cap-label"><?php echo $label_text; ?></label>
        <?php } ?>
        
        <?php $counter = (is_array($selected_ids) ? count($selected_ids) : 0); ?>
        
        <div class="cap-input-wrap">
          <input name="<?php echo cap_get_input_name($slug); ?>" type="hidden" <?php if($disabled) { ?>disabled<?php } ?> value="<?php echo osc_esc_html(is_array($selected_ids) ? implode(' ', $selected_ids) : $selected_ids); ?>"/>
          <input data-id="<?php echo cap_get_input_name($slug); ?>-display" readonly <?php if($disabled) { ?>disabled<?php } ?> type="text" value="<?php echo osc_esc_html($selected_ids_name); ?>" class="cap-input cap-display<?php if($required_jqval) { ?> cap-required<?php } ?>" placeholder="<?php echo osc_esc_html($placeholder_text); ?>" data-label="<?php echo osc_esc_html($name_text); ?>"<?php if($required_html5) { ?> required<?php } ?>/>
          <input data-id="<?php echo cap_get_input_name($slug); ?>-filter" <?php if($disabled) { ?>disabled<?php } ?> type="text" value="" class="cap-input cap-filter" placeholder="<?php echo osc_esc_html(sprintf(__('Refine %s', 'car_attr_pro'), strtolower($name_text))); ?>"/>
          <div class="cap-counter" data-count="<?php echo $counter; ?>"><?php echo $counter-1; ?></div>
          <?php echo CAP_CARET_SVG; ?>
          <?php echo CAP_CLEAR_SVG; ?>
        </div>
        
        <?php if(is_array($values) && count($values) > 0) { ?>
          <div class="cap-values-list">
            <div class="cap-values-wrap">
              <?php if($select_all) { ?>
                <div class="cap-value cap-select-all<?php if($deselect_all) { ?> cap-w50<?php } ?>" data-id=""><?php _e('Select all', 'car_attr_pro'); ?></div>              
              <?php } ?>
              
              <?php if($deselect_all) { ?>
                <div class="cap-value cap-deselect<?php if($select_all) { ?> cap-w50<?php } ?>" data-id=""><?php _e('Clear all', 'car_attr_pro'); ?></div>              
              <?php } ?>
              
              <?php 
                $current_section = ''; 
                $prev_section = '';
              ?>
              
              <?php foreach($values as $vid => $val) { ?>
                <?php 
                  // Identify if top section has ended
                  if(isset($val['s_section']) && $val['s_section'] != '') { 
                    $current_section = $val['s_section'];
                  } else {
                    $current_section = '';
                  }
                ?>

                <?php if($vid == -1) { ?>
                  <label class="cap-value cap-disabled">
                    <span><em><?php echo cap_get_name($val); ?></em></span>
                  </label>
                  
                <?php } else { ?>
                  <?php if($current_section != $prev_section && $prev_section != '') { ?>
                    <em class="cap-top-section-end" data-end="<?php echo $prev_section; ?>"></em>
                  <?php } ?>
                    
                  <label class="cap-label cap-value"<?php if(isset($val['level'])) { ?> data-level="<?php echo $val['level']; ?>"<?php } ?>>
                    <input xname="<?php echo cap_get_input_name($slug); ?>" data-id="cap-<?php echo $slug; ?>-<?php echo cap_get_slug($val, $vid, $section); ?>" type="checkbox" value="<?php echo cap_get_slug($val, $vid, $section); ?>" <?php if(in_array(cap_get_slug($val, $vid, $section), $selected_ids) || in_array($vid, $selected_ids)) { ?>checked<?php } ?>/>
                    <span><?php echo cap_get_name($val) . $measure_text; ?></span>
                  </label>
                  
                  <?php $prev_section = $current_section; ?>
                <?php } ?>
              <?php } ?>
            </div>
            
            <?php if(cap_param('hide_values_ok_btn') != 1) { ?>
              <div class="cap-values-ok">
                <a href="#" class="cap-values-ok-btn"><?php _e('Ok', 'cap_attr_pro'); ?></a>
              </div>
            <?php } ?>
          </div>
        <?php } ?>
      </div>

    <?php
    }
  }
}


// CREATE PROPERTY DISPLAY
function cap_create_elem($key, $car_type, $values = array(), $options = array()) {
  $properties = cap_get_properties();
  
  // Apply filters for customization
  $properties = osc_apply_filter('cap_create_elem_properties', $properties, $key, $car_type, $values, $options);
  $values = osc_apply_filter('cap_create_elem_values', $values, $key, $car_type, $options);
  $options = osc_apply_filter('cap_create_elem_options', $options, $key, $car_type, $values);
  
  $prop = $properties[$key];
  $id = $prop['s_id'];
  
  if(cap_property_enabled($key) === false) {
    return false;
  }
  
  if(in_array($id, array('location','category','price'))) {
    return false;
  }
  
  // options
  $def_style = $prop['s_style'];
  $db_type = $prop['s_data_type'];
  $name = cap_get_name($prop);
  $label = $name;
  $input_name = cap_get_input_name($id);
  $db_col_name = cap_get_prop_db_column($key);
  $style = (isset($options['style']) ? $options['style'] : 'standard');
  $subgroup = (isset($options['subgroup']) ? $options['subgroup'] : false);
  $class = (isset($options['class']) ? $options['class'] : '');
  $group_parent = (isset($options['group_parent']) ? $options['group_parent'] : '');
  $data_type = strtolower(isset($options['data_type']) ? $options['data_type'] : 'text');
  $return_only = (isset($options['return_only']) ? $options['return_only'] : false);

  $prop_in_groups = cap_get_properties_groups();
  
  if((in_array($key, $prop_in_groups) || in_array($id, $prop_in_groups)) && $group_parent == '') {
    return false;
  }

  // Check groups
  $group_ids = $prop['s_group_list'];
  
  if($group_ids != '') {
    $group_ids = array_filter(explode(',', $group_ids));

    if(is_array($group_ids) && count($group_ids) > 0) {
      //echo '<strong class="cap-subhead">' . $name . '</strong>';
      //echo '<div class="cap-section cap-section-subgroup">';
      
      $options['class'] = 'cap-is-subgroup';
      $options['group_parent'] = $key;
      $options['subgroup'] = true;

      foreach($group_ids as $group_member_key) {
        cap_create_elem($group_member_key, $car_type, $values, $options);
      }
      
      //echo '</div>';
    }
    
    return false;
  }

  // Create help icon for description text
  $description_text = trim(cap_get_description($prop));
  $description_html = ' ';
  
  if($description_text != '') {
    $description_html .= '<div class="cap-help-elem">';
    $description_html .= CAP_HELP_SVG;
    $description_html .= '<div class="cap-help-elem-text">' . $description_text . '</div>';
    $description_html .= '</div>';
  }
  
  $label .= $description_html;


  if($data_type == 'text') {
    $data_type = ($db_type == 'DECIMAL' ? 'number' : $data_type);
  }
  
  $selected_value_arr = array();
  $selected_value_name = '';
  $selected_value_id = (isset($values[$db_col_name]) ? $values[$db_col_name] : '');


  //$prop_values_all = cap_properties_values_list($car_type['s_id']);
  //$prop_values_ = isset($prop_values_all[$id]) ? $prop_values_all[$id] : array();
  $prop_values_ = cap_get_property_values($key, $car_type['pk_i_id']);

  if(is_array($prop_values_) && count($prop_values_) > 0) {
    if(isset($prop_values_[$selected_value_id])) {
      $selected_value_name = cap_get_name($prop_values_[$selected_value_id]);
    }
  }
  
  
  if($id == 'type') {
    $selected_value_arr = cap_get_type($selected_value_id);
  } else if($id == 'make') {
    $selected_value_arr = ModelCAP::newInstance()->getMake($selected_value_id);
  } else if($id == 'model') {
    $selected_value_arr = ModelCAP::newInstance()->getModel($selected_value_id);
  }
  
  if($selected_value_arr !== false && is_array($selected_value_arr) && isset($selected_value_arr['pk_i_id'])) {
    $selected_value_name = cap_get_name($selected_value_arr);
  }

  // Get different default icon for MAIN
  if($style == 'main') {
    $icon = cap_get_prop_icon($key, true);
    $icon = ($icon == '' ? CAP_CHECK_ALT_SVG : $icon);
  } else {
    $icon = cap_get_prop_icon($key);
  }



  if($db_type == 'BOOLEAN' && $selected_value_id == 1) {
    $selected_value_name = $name;
    $label = '';
  } else if($selected_value_name == '') {
    $selected_value_name = $selected_value_id;
  }

  if($data_type == 'number' && $selected_value_name != '') {
    $selected_value_name = number_format((float)$selected_value_name, 1, '.', ' ');
  }
  
  // if(substr($id, 0, 9) == 'fuel_cons' && $style != 'fuel' && $selected_value_name != '') {
    // $selected_value_name .= ' ' . cap_param('fuel_consumption_unit_long');
  // }

  $measure_text = '';
  if($prop['s_measure_unit'] <> '') { 
    $measure_text = ' <em>' . $prop['s_measure_unit'] . '</em>'; 
    $selected_value_name .= $measure_text;
  }
  
  if($selected_value_name == '') {
    return false;
  }
  
  $label = trim((string)$label);
  
  if($return_only) {
    return array(
      'label' => $label,
      'value' => $selected_value_name,
      'icon' => $icon
    );
  }
  ?>
  
  <div class="cap-item-elem cap-style-<?php echo osc_esc_html($style); ?> <?php echo $class; ?>" data-id="<?php echo $id; ?>" data-key="<?php echo $key; ?>" data-type="<?php echo strtolower($db_type); ?>">
    <?php if($style == 'main') { ?>
      <?php echo $icon; ?>
      
      <div class="cap-show-data">
        <div class="cap-show-label"><?php echo $label; ?></div>
        <div class="cap-show-value"><?php echo $selected_value_name; ?></div>
      </div>

    <?php } else if($style == 'sub') { ?>
      <?php echo CAP_ASTERISK_SVG; ?>
      <div class="cap-show5-value"><?php echo $selected_value_name; ?></div>

    <?php } else if($style == 'extra') { ?>
      <?php echo $icon; ?>
      
      <div class="cap-show-data">
        <div class="cap-show-value"><?php echo ($selected_value_name != '' ? $selected_value_name : '-'); ?></div>
        <div class="cap-show-label"><?php echo $label; ?></div>
      </div>
      
    <?php } else if($style == 'icon_value') { ?>
      <?php echo $icon; ?>
      <div class="cap-show9-value"><span><?php echo $selected_value_name; ?></span></div>

    <?php } else if($style == 'icon_value_alt') { ?>
      <?php echo $icon; ?>
      <div class="cap-show12-value"><?php echo $selected_value_name; ?></div>
      
    <?php } else if($style == 'icon_value_label') { ?>
      <?php echo $icon; ?>
      <div class="cap-show10-value"><?php echo $selected_value_name; ?></div>
      <div class="cap-show10-label"><?php echo $label; ?></div>

    <?php } else if($style == 'icon_label_value') { ?>
      <?php echo $icon; ?>
      <div class="cap-show10-label"><?php echo $label; ?></div>
      <div class="cap-show10-value"><?php echo $selected_value_name; ?></div>
      
    <?php } else if($style == 'value_only') { ?>
      <div class="cap-show6-value"><?php echo $selected_value_name; ?></div>
      
    <?php } else if($style == 'label_value') { ?>
      <div class="cap-show7-label"><?php echo $label; ?></div>
      <div class="cap-show7-value"><?php echo $selected_value_name; ?></div>

    <?php } else if($style == 'label_value_alt') { ?>
      <div class="cap-show8-label"><?php echo $label; ?>: </div>
      <div class="cap-show8-value"><?php echo $selected_value_name; ?></div>
      
    <?php } else if($style == 'checks' || $style == 'bool' || $db_type == 'BOOLEAN') { ?>
      <?php echo CAP_CHECK_SVG; ?>
      <div class="cap-show2-value"><?php echo $selected_value_name; ?></div>

    <?php //} else if($id == 'keywords') { ?>
    <?php } else if($prop['s_data_type'] == 'TEXTAREA' || $prop['s_data_type'] == 'VARCHAR_1000') { ?>
      <div class="cap-show4-value"><?php echo $selected_value_name; ?></div>
      
    <?php } else { ?>
      <div class="cap-show3-label"><?php echo $label; ?>:</div>
      <div class="cap-show3-value"><?php echo $selected_value_name; ?></div>
    
    <?php } ?>
  </div>
  
  <?php
}


// GET SEARCH HOOK DATA AJAX
function cap_ajax_get_search_hook() {
  if(Params::getParam('capGetSearchHook') == 1) {
    cap_search_hook(array(Params::getParam('sCategory')));
    exit;
  } 
}

osc_add_hook('init', 'cap_ajax_get_search_hook', 2);


// GET MODELS VIA AJAX
function cap_ajax_get_models() {
  if(Params::getParam('capGetModels') == 1) {
    $car_type = cap_get_type(Params::getParam(cap_get_input_name('type')));
    
    if(Params::getParam('capIsPublish') == 1) {
      cap_create_input(cap_get_property_keys('model'), $car_type, array(), array('style' => 'select', 'section' => 'publish', 'multi' => 'one'));
    } else {
      cap_create_input(cap_get_property_keys('model'), $car_type, array(), array('section' => osc_esc_html(Params::getParam('capSection'))));
    }
    
    exit;
  } 
}

osc_add_hook('init', 'cap_ajax_get_models', 2);


// DELETE MAKE
function cap_admin_delete_make() {
  $id = Params::getParam('id');
  ModelCAP::newInstance()->deleteModelsByMakeId($id);
  ModelCAP::newInstance()->deleteMake($id);
  exit;
}

osc_add_hook('ajax_admin_cap_admin_delete_make', 'cap_admin_delete_make');


// DELETE MODEL
function cap_admin_delete_model() {
  $id = Params::getParam('id');
  ModelCAP::newInstance()->deleteModel($id);
  exit;
}

osc_add_hook('ajax_admin_cap_admin_delete_model', 'cap_admin_delete_model');


// DELETE PROPERTY VALUE
function cap_admin_delete_property_value() {
  $id = Params::getParam('id');
  ModelCAP::newInstance()->deletePropertyValue($id);
  exit;
}

osc_add_hook('ajax_admin_cap_admin_delete_property_value', 'cap_admin_delete_property_value');


// UPDATE TYPES POSITION
function cap_admin_update_types_position() {
  $orders = Params::getParam('cap');

  if(is_array($orders) && count($orders) > 0) {
    foreach($orders as $order => $id) {
      ModelCAP::newInstance()->updateTypeRow($id, array('i_order' => $order));
    }
  }

  exit;
}

osc_add_hook('ajax_admin_cap_admin_update_types_position', 'cap_admin_update_types_position');


// UPDATE SECTION POSITION
function cap_admin_update_sections_position() {
  $orders = Params::getParam('cap');

  if(is_array($orders) && count($orders) > 0) {
    foreach($orders as $order => $id) {
      if($id == 1) {  // main is always first
        $order = 0;
      } else if($id == 2) {  // bottom is always first
        $order = 1;
      } else {
        $order = $order + 2;
      }
      
      ModelCAP::newInstance()->updateSectionRow($id, array('i_order' => $order));
    }
  }

  exit;
}

osc_add_hook('ajax_admin_cap_admin_update_sections_position', 'cap_admin_update_sections_position');


// UPDATE PROPERTIES POSITION
function cap_admin_update_properties_position() {
  $orders = Params::getParam('cap');

  if(is_array($orders) && count($orders) > 0) {
    foreach($orders as $order => $id) {
      ModelCAP::newInstance()->updatePropertyRow($id, array('i_order' => $order));
    }
  }

  exit;
}

osc_add_hook('ajax_admin_cap_admin_update_properties_position', 'cap_admin_update_properties_position');


// UPDATE PROPERTIES POSITION
function cap_admin_update_property_value_position() {
  $orders = Params::getParam('cap');

  if(is_array($orders) && count($orders) > 0) {
    foreach($orders as $order => $id) {
      ModelCAP::newInstance()->updatePropertyValueRow($id, array('i_order' => $order));
    }
  }

  exit;
}

osc_add_hook('ajax_admin_cap_admin_update_property_value_position', 'cap_admin_update_property_value_position');


// UPDATE MAKES POSITION
function cap_admin_update_makes_position() {
  $orders = Params::getParam('cap');

  if(is_array($orders) && count($orders) > 0) {
    foreach($orders as $order => $id) {
      ModelCAP::newInstance()->updateMake($id, array('i_order' => $order));
    }
  }

  exit;
}

osc_add_hook('ajax_admin_cap_admin_update_makes_position', 'cap_admin_update_makes_position');


// UPDATE MODELS POSITION
function cap_admin_update_models_position() {
  $orders = Params::getParam('cap');

  if(is_array($orders) && count($orders) > 0) {
    foreach($orders as $order => $id) {
      ModelCAP::newInstance()->updateModel($id, array('i_order' => $order));
    }
  }

  exit;
}

osc_add_hook('ajax_admin_cap_admin_update_models_position', 'cap_admin_update_models_position');


// GET MAKES IN OC-ADMIN
function cap_admin_get_makes() {
  $type_id = Params::getParam('id');
  $new = (Params::getParam('new') == 1 ? true : false);
  
  if($new) {
    ModelCAP::newInstance()->insertEmptyMake($type_id);
  }
  
  $car_type = cap_get_type($type_id);
  
  if(!isset($car_type['pk_i_id'])) {
    exit;
  }
  
  $makes = ModelCAP::newInstance()->getMakes((int)$car_type['pk_i_id'], false);
  
  if(is_array($makes) && count($makes) > 0) {
    foreach($makes as $make) {
      cap_admin_generate_elem('make', $make['pk_i_id'], $make['s_name'], 'capmakes[' . $make['pk_i_id'] . ']', $make['b_enabled'], $make['s_icon'], $make['s_color'], $make['s_section'], $make['s_slug']);
    }
  } else {
    cap_admin_generate_empty(__('No makes found for selected type', 'car_attr_pro'));
  }

  exit;
}

osc_add_hook('ajax_admin_cap_admin_get_makes', 'cap_admin_get_makes');


// GET MODELS IN OC-ADMIN
function cap_admin_get_models() {
  $make_id = Params::getParam('id');
  $new = (Params::getParam('new') == 1 ? true : false);
  
  if($new) {
    ModelCAP::newInstance()->insertEmptyModel($make_id);
  }
  
  $models = ModelCAP::newInstance()->getModels($make_id, false);
  
  if(is_array($models) && count($models) > 0) {
    foreach($models as $model) {
      cap_admin_generate_elem('model', $model['pk_i_id'], $model['s_name'], 'capmodels[' . $model['pk_i_id'] . ']', $model['b_enabled'], '', '', '', $model['s_slug']);
    }
  } else {
    cap_admin_generate_empty(__('No models found for selected make', 'car_attr_pro'));
  }

  exit;
}

osc_add_hook('ajax_admin_cap_admin_get_models', 'cap_admin_get_models');


// ADD NEW PROPERTY VALUE ADMIN
function cap_admin_add_property_value() {
  $props = cap_get_properties();

  $prop_id = Params::getParam('propId');
  $prop = $props[$prop_id];

  $type_id = Params::getParam('typeId');
  $name = Params::getParam('name');
  $list = array_filter(array_unique(explode(';', str_replace(',', ';', Params::getParam('list')))));
  $locale = Params::getParam('locale');
  
  $max_value_id = ModelCAP::newInstance()->getMaxPropertyValueId();
  $max_value_id = $max_value_id + 1;
  
  $id = (osc_sanitizeString($name) <> '' ? osc_sanitizeString($name) . '-' : '') . $max_value_id;
  
  if(empty($list)) {
    $val_id = ModelCAP::newInstance()->insertPropertyValue($prop_id, $type_id, $id, $locale, $name, $max_value_id);
    
    if($val_id > 0) {
      cap_admin_generate_value_elem($val_id, array('s_name' => $name, 's_id' => $id, 'fk_i_type_id' => $type_id), $prop);
    }
  } else {
    foreach($list as $name) {
      $id = (osc_sanitizeString($name) <> '' ? osc_sanitizeString($name) . '-' : '') . $max_value_id;
      $val_id = ModelCAP::newInstance()->insertPropertyValue($prop_id, $type_id, $id, $locale, $name, $max_value_id);
    
      if($val_id > 0) {
        cap_admin_generate_value_elem($val_id, array('s_name' => $name, 's_id' => $id, 'fk_i_type_id' => $type_id), $prop);
      }
    
      $max_value_id++;
    }
  }
  
  exit;
}

osc_add_hook('ajax_admin_cap_admin_add_property_value', 'cap_admin_add_property_value');


// GET SECTION VIA AJAX
function cap_ajax_get_item_publish_type_section() {
  if(Params::getParam('capGetItemPublishTypeSection') == 1) {
    $item_attr = array();
    $item_id = (Params::getParam('itemId') > 0 ? Params::getParam('itemId') : osc_item_id());
    
    if($item_id > 0) {
      $item_attr = ModelCAP::newInstance()->getItemAttr($item_id);
    }
  
    $car_type = cap_get_type(Params::getParam(cap_get_input_name('type')));
    

    cap_item_publish_type_section($car_type, $item_attr);
    exit;
  } 
}

osc_add_hook('init', 'cap_ajax_get_item_publish_type_section', 2);



// SEPARATE STRING & INTEGER VALUES FROM GIVE ARRAY
function cap_separate_string_int_arr($data, $as_in_statement = false) {
  $integer = array();
  $string = array();
  
  if(!is_array($data)) {
    $result = cap_separate_string_int_one($data);
    
    if($result !== false) {
      $integer[] = $result['integer'];
      $string[] = $result['string'];
    }    
  } else {
    if(count($data) > 0) {
      foreach($data as $d) {
        $result = cap_separate_string_int_one($d);
        
        if($result !== false) {
          $integer[] = $result['integer'];
          $string[] = $result['string'];
        }
      }
    }
  }
  
  $string = array_filter(array_unique($string));
  $integer = array_filter(array_unique($integer), 'strlen');
  
  
  if($as_in_statement === true) {
    if(!empty($string)) {
      $string = '"' . implode('","', $string) . '"';
    } else {
      $string = '"DUMMY_CONDITION"'; 
    }
    
    if(!empty($integer)) {
      $integer = implode(',', $integer);
    } else {
      $integer = -1;  // or 0?
    }
  }
  
  return array(
    'string' => $string,
    'integer' => $integer
  );
}


// CHECK WHAT KIND OF DATA IT IS - INTEGER OR STRING
function cap_separate_string_int_one($data) {
  $integer = '';
  $string = '';
  
  if(filter_var($data, FILTER_VALIDATE_INT) !== false && (int)$data > 0) {
    $integer = (int)$data;
  } else if (trim((string)$data) != '') {
    $string = trim((string)$data);
  } else {
    return false;
  }

  return array(
    'string' => $string,
    'integer' => $integer
  );
}
  

// GET SEARCH HOOK PROPERTIES
function cap_get_properties_enabled($location = '') {
  $properties = cap_get_properties();
  $output = array();

  if(is_array($properties) && count($properties) > 0) {
    foreach($properties as $key => $prop) {
      if(cap_property_enabled($key, $location)) {
        $output[$key] = $prop;
      }
    }
  }
  
  return $output;
}


// CHECK IF PROPERTY ENABLED
function cap_property_enabled($key, $location = '') {
  $props = cap_get_properties();
  $prop = $props[$key];
  
  if(!isset($props[$key])) {
    return true;
  }
  
  // Disable general
  if($prop['b_enabled'] != 1) {
    return false;
  }

  // Add to search hook
  if($location == 'search' && $prop['b_enabled_search'] != 1) {
    return false;
  }

  // Disable widget
  if($location == 'widget' && $prop['b_enabled_widget'] != 1) {
    return false;
  }
  
  return true;
}


// RETURN THOSE PROPERTIES THAT HAS VALUE ON ITEM ATTRIBUTE
function cap_filter_nonempty_item_attr($properties, $item_attr = array(), $group_ingore = false) {
  if(!is_array($properties) || empty($properties) || empty($item_attr)) {
    return array();
  }
  
  $output = array();
  $prop_in_groups = array();
  
  if($group_ingore == false) {
    $prop_in_groups = cap_get_properties_groups();
  }

  foreach($properties as $key => $prop) {
    if(cap_property_enabled($key) && !in_array($key, $prop_in_groups)) {    // ignore subgroup properties as they are managed differently
      if(isset($item_attr[cap_get_prop_db_column($key)]) && $item_attr[cap_get_prop_db_column($key)] != '') {
        $output[$key] = $prop;
      }
    }
  }
  
  return $output;
}


// RETURN NON-EMPTY GROUP ATTRIBUTES
function cap_filter_nonempty_item_attr_group($properties, $item_attr = array()) {
  if(!is_array($properties) || empty($properties) || empty($item_attr)) {
    return array();
  }
  
  $output = array();
  
  foreach($properties as $key => $prop) {
    if(cap_property_enabled($key)) {
      if($prop['s_style'] == 'group' && $prop['s_group_list'] != '') {
        $group_ids = array_filter(explode(',', $prop['s_group_list']));
        
        if(is_array($group_ids) && count($group_ids) > 0) {
          foreach($group_ids as $gid) {
            if(cap_property_enabled($gid)) {
              if(isset($item_attr[cap_get_prop_db_column($gid)]) && $item_attr[cap_get_prop_db_column($gid)] != '') {
                $output[$key] = $prop;    // add group prop, not subgroup!
                break;
              }
            }
          }
        }
      }
    }
  }
  
  return $output;
}


// RETURN THOSE PROPERTIES THAT ARE NOT EMPTY FOR SEARCH
function cap_filter_nonempty_search_props($properties, $location = 'widget') {
  if(!is_array($properties) || empty($properties)) {
    return array();
  }
  
  $params = Params::getParamsAsArray();
  $output = array();
  
  foreach($properties as $key => $prop) {
    if(cap_property_enabled($key, $location)) {
      if($prop['s_id'] != '') {
        // Exact match
        if(isset($params[cap_get_input_name($prop['s_id'])]) && $params[cap_get_input_name($prop['s_id'])] != '') {
          $output[$key] = $prop;
          $output[$key]['is_range'] = false;
          $output[$key]['is_range_from'] = false;
          $output[$key]['is_range_to'] = false;
        }
        
        // Is range (from-to)
        if(
          isset($params[cap_get_input_name($prop['s_id']) . '-from']) && $params[cap_get_input_name($prop['s_id']) . '-from'] != ''
          || isset($params[cap_get_input_name($prop['s_id']) . '-to']) && $params[cap_get_input_name($prop['s_id']) . '-to'] != ''
        ) {
          $output[$key] = $prop;
          $output[$key]['is_range'] = true;
          $output[$key]['is_range_from'] = (isset($params[cap_get_input_name($prop['s_id']) . '-from']) && $params[cap_get_input_name($prop['s_id']) . '-from'] != '');
          $output[$key]['is_range_to'] = (isset($params[cap_get_input_name($prop['s_id']) . '-to']) && $params[cap_get_input_name($prop['s_id']) . '-to'] != '');
        }
      }
    }
  }
  
  return $output;
}


// GET PROPERTY ICON
function cap_get_prop_icon($key, $exact = false) {
  //$list = cap_property_icon_list();
  //$properties = cap_properties_list();
  $properties = cap_get_properties();
  $prop = $properties[$key];
  $icon = $prop['s_icon'];
  
  if(strtolower(substr($icon, 1, 3)) == 'svg') {
    return $icon;
  } else if(strtolower(substr($icon, 0, 2)) == 'fa')  {
    return '<i class="' . $icon . '"></i>';
  } else if(strtolower(substr($icon, 0, 4) == 'http'))  {
    return '<img src="' . $icon . '" alt="' . osc_esc_html(cap_get_name($prop)) . '"/>';
  }
  
  // asterisk
  if($exact) {
    return '';
  }
  
  return '<svg xmlns="http://www.w3.org/2000/svg" height="28px" viewBox="0 0 512 512"><path d="M478.21 334.093L336 256l142.21-78.093c11.795-6.477 15.961-21.384 9.232-33.037l-19.48-33.741c-6.728-11.653-21.72-15.499-33.227-8.523L296 186.718l3.475-162.204C299.763 11.061 288.937 0 275.48 0h-38.96c-13.456 0-24.283 11.061-23.994 24.514L216 186.718 77.265 102.607c-11.506-6.976-26.499-3.13-33.227 8.523l-19.48 33.741c-6.728 11.653-2.562 26.56 9.233 33.037L176 256 33.79 334.093c-11.795 6.477-15.961 21.384-9.232 33.037l19.48 33.741c6.728 11.653 21.721 15.499 33.227 8.523L216 325.282l-3.475 162.204C212.237 500.939 223.064 512 236.52 512h38.961c13.456 0 24.283-11.061 23.995-24.514L296 325.282l138.735 84.111c11.506 6.976 26.499 3.13 33.227-8.523l19.48-33.741c6.728-11.653 2.563-26.559-9.232-33.036z"/></svg>';  
}


// GET ICON OF STANDARD OBJECT
function cap_get_icon($data, $type = 'type', $with_color = true) {
  $icon = '';
  $alt_text = '';
  $color = '';
  
  if($type == 'type') {
    $alt_text = osc_esc_html(cap_get_name($data));
  } else if ($type == 'make' || $type == 'model') {
    $alt_text = osc_esc_html(isset($data['s_name']) ? $data['s_name'] : '');
  }
  
  if($with_color && isset($data['s_color']) && $data['s_color'] != '') {
    $color = $data['s_color'];
  }
  
  if(isset($data['s_icon']) && (string)$data['s_icon'] != '') {
    if(strtolower(substr($data['s_icon'], 1, 3)) == 'svg') {
      $icon = $data['s_icon'];
      
      if($color != '' && strpos($icon, 'style') === false) {
        $icon = str_replace('<svg', '<svg style="fill:' . $color . ';"', $icon);
      }
    } else if(strtolower(substr($data['s_icon'], 0, 2)) == 'fa')  {
      $icon = '<i class="' . $data['s_icon'] . '"' . ($color != '' ? ' style="color:' . $color . ';"' : '') . '></i>';
    } else if(strtolower(substr($data['s_icon'], 0, 4) == 'http'))  {
      $icon = '<img src="' . $data['s_icon'] . '" alt="' . $alt_text . '"/>';
    }
  }
  
  return $icon;
}


// GET SECTION PROPERTIES
function cap_get_section_properties($section_type, $section_key, $car_type = array(), $with_type_details = false) {
  $car_type_id = '';
  
  if(isset($car_type['pk_i_id']) && $car_type['pk_i_id'] > 0) {
    $car_type_id = $car_type['pk_i_id'];
  }
  
  if($with_type_details) {
    $props = ModelCAP::newInstance()->getProperties(false, $car_type_id, $with_type_details);
  } else {
    $props = cap_get_properties($car_type_id);
  }
  
  $output = array();
  
  if(is_array($props) && count($props) > 0) {
    foreach($props as $key => $prop) {
      $publish_section_id = ($prop['fk_i_section_publish_id'] > 0 ? $prop['fk_i_section_publish_id'] : $prop['fk_i_section_widget_id']);
      $publish_section_id = ($publish_section_id > 0 ? $publish_section_id : $prop['fk_i_section_item_id']);
      $publish_section_id = ($publish_section_id > 0 ? $publish_section_id : 1);    // map to main if none found
      
      if(
        (($section_type == 'widget' || $section_type == 'search') && $prop['fk_i_section_widget_id'] == $section_key) 
        || ($section_type == 'item' && $prop['fk_i_section_item_id'] == $section_key) 
        || ($section_type == 'publish' && $publish_section_id == $section_key) 
      ) {
        if($prop['s_id'] != 'type') {
          $output[$key] = $prop;
        }
      }
    }
  }

  return $output;
}


// GET PARAMETERS
function cap_get_param($name, $params = array()) {
  if(!is_array($params) || empty($params) || !isset($params[$name])) {
    return Params::getParam($name);
  }
  
  return $params[$name];
}
  

// GET PROPERTY DATABASE COLUMN
function cap_get_prop_db_column($key, $db_type = '', $enabled = true, $props = array()) {
  //$props = cap_properties_list();
  
  if($enabled) {
    $props = cap_get_properties();
  } else if(empty($props)) {
    $props = ModelCAP::newInstance()->getProperties(false);     // for database fields updates
  }
  
  $prop = $props[$key];
  
  $id = $prop['s_id'];
  $slug = trim(str_replace('-', '_', $id));
  
  if($db_type == '') {
    $db_type = @$prop['s_data_type'];
  }
  
  $db_type = trim($db_type);

  if($db_type != '' && $slug != '') {
    if($db_type == 'FK_INTEGER') {
      return 'fk_i_' . $slug . '_id';
    } else if($db_type == 'BOOLEAN') {
      return 'b_' . $slug;
    } else if($db_type == 'INTEGER') {
      return 'i_' . $slug;
    } else if(substr($db_type, 0, 7) == 'VARCHAR') {
      return 's_' . $slug;
    } else if($db_type == 'DECIMAL') {
      return 'd_' . $slug;
    }
  }
  
  return '';
}


// CONVERT PROPERTY VALUE
function cap_convert_prop_value($key, $value) {
  //$props = cap_properties_list();
  $props = cap_get_properties();
  $prop = $props[$key];
  
  $db_type = $prop['s_data_type'];
  
  if($db_type == 'FK_INTEGER') {
    $mval = (int)$value;
    return ($mval > 0 ? $mval : null);
    
  } else if($db_type == 'BOOLEAN') {
    $mval = (boolean)($value == 'on' ? 1 : $value);
    return $mval;

  } else if($db_type == 'INTEGER') {
    $mval = (int)$value;
    return ($value === '' ? null : $mval);

  } else if($db_type == 'DECIMAL') {
    $mval = (float)$value;
    return ($value === '' ? null : $mval);
    
  } else if(substr($db_type, 0, 7) == 'VARCHAR') {
    $mval = trim((string)$value);
    return ($mval != '' ? $mval : null);
    
  }
  
  return '';
}


// GET STYLE NAME
function cap_get_style_name($id) {
  $styles2 = cap_styles_list2();
  
  if(isset($styles2[$id])) {
    return $styles2[$id];
  }
  
  return __('Unknown', 'car_attr_pro');  
}


// GET DBTYPE NAME
function cap_get_dbtype_name($id) {
  $dbtypes = cap_dbtypes_list();
  
  if(isset($dbtypes[$id])) {
    return $dbtypes[$id];
  }
  
  return __('Unknown', 'car_attr_pro');  
}



// List of items sections
function cap_item_section_styles_list() {
  $list = array(
    'main' => __('Main', 'car_attr_pro'),
    'sub' => __('Sub', 'car_attr_pro'),
    'extra' => __('Extra', 'car_attr_pro'),
    'data' => __('Data', 'car_attr_pro'),
    'bool' => __('Bool', 'car_attr_pro'),
    'detail' => __('Detail', 'car_attr_pro'),
    'label_value' => __('Label & value', 'car_attr_pro'),
    'label_value_alt' => __('Label & value (alt)', 'car_attr_pro'),
    'icon_value' => __('Icon & value', 'car_attr_pro'),
    'icon_value_alt' => __('Icon & value (alt)', 'car_attr_pro'),
    'icon_value_label' => __('Icon, value & label', 'car_attr_pro'),
    'icon_label_value' => __('Icon, label & value', 'car_attr_pro'),
    'value_only' => __('Value only', 'car_attr_pro'),
    'checks' => __('Checkboxes', 'car_attr_pro'),
    'detail' => __('Detail', 'car_attr_pro'),
    'standard' => __('Standard', 'car_attr_pro')
  );
  
  return osc_apply_filter('cap_item_section_styles_list', $list); 
}


// List of styles
function cap_type_styles_list() {
  $list = array(
    'predefined' => __('Select from pre-defined values', 'car_attr_pro'),
    'userdefined' => __('User define values', 'car_attr_pro'),
    'group' => __('Group other properties', 'car_attr_pro')
  );
  
  return osc_apply_filter('cap_type_styles_list', $list); 
}


// List of styles
function cap_styles_list2() {
  $list = array(
    'group' => __('Group', 'car_attr_pro'),
    'select' => __('Select box', 'car_attr_pro'),
    'slider' => __('Slider', 'car_attr_pro'),
    'checkbox' => __('Checkbox', 'car_attr_pro'),
    'input' => __('Input (short text, number)', 'car_attr_pro'),
    'textarea' => __('Textarea (medium text)', 'car_attr_pro')
  );
  
  return osc_apply_filter('cap_styles_list2', $list); 
}


// List of styles
function cap_dbtypes_list() {
  $list = array(
    'FK_INTEGER' => __('Values list (FK)', 'car_attr_pro'),
    'BOOLEAN' => __('Boolean', 'car_attr_pro'),
    'INTEGER' => __('Integer', 'car_attr_pro'),
    'DECIMAL' => __('Decimal', 'car_attr_pro'),
    'VARCHAR_50' => __('Varchar (50)', 'car_attr_pro'),
    'VARCHAR_200' => __('Varchar (200)', 'car_attr_pro'),
    'VARCHAR_1000' => __('Varchar (1000)', 'car_attr_pro')
  );
  
  return osc_apply_filter('cap_dbtypes_list', $list); 
}


// ADMIN GENERATE ELEMENT
function cap_admin_generate_elem($type, $key, $name, $input_name = '', $enabled = true, $icon = '', $color = '', $section = '', $slug = '') {
?>
  <div class="mb-elem<?php if($input_name == '') { ?> mb-elem-light<?php } ?>" data-id="<?php echo $key; ?>" id="cap_<?php echo $key; ?>">
    <i class="mb-move fa fa-arrows-alt"></i>
    
    <?php if($type == 'type') { ?>
      <?php $count_makes = ModelCAP::newInstance()->countTypeMakes($key); ?>
      <span class="mb-text">
        <?php echo $name; ?>
        <em title="<?php echo osc_esc_html(__('Number of makes assigned to this type', 'car_attr_pro')); ?>">(<?php echo $count_makes; ?>)</em>
      </span>
    <?php } else { ?>
      <div class="mb-inputs mb-inputs-<?php echo $type; ?>">
        <?php if($type == 'model') { ?>
          <input type="text" class="mb-i-name" name="<?php echo $input_name; ?>[s_name]" value="<?php echo osc_esc_html($name); ?>" placeholder="<?php echo osc_esc_html(__('Model name', 'car_attr_pro')); ?>" required/>
          <input type="text" class="mb-i-slug" name="<?php echo $input_name; ?>[s_slug]" value="<?php echo osc_esc_html($slug); ?>" placeholder="<?php echo osc_esc_html(__('Slug', 'car_attr_pro')); ?>" pattern="^[a-z]{1}[a-z0-9\-]{2,49}$" required/>
        <?php } else if ($type == 'make') { ?>
          <input type="text" class="mb-i-name" name="<?php echo $input_name; ?>[s_name]" value="<?php echo osc_esc_html($name); ?>" placeholder="<?php echo osc_esc_html(__('Make name', 'car_attr_pro')); ?>" required/>
          <input type="text" class="mb-i-slug" name="<?php echo $input_name; ?>[s_slug]" value="<?php echo osc_esc_html($slug); ?>" placeholder="<?php echo osc_esc_html(__('Slug', 'car_attr_pro')); ?>" pattern="^[a-z]{1}[a-z0-9\-]{2,49}$" required/>
          <input type="text" class="mb-i-icon" name="<?php echo $input_name; ?>[s_icon]" value="<?php echo osc_esc_html($icon); ?>" placeholder="<?php echo osc_esc_html(__('FA class, url or svg', 'car_attr_pro')); ?>"/>
          <input type="color" class="mb-i-color" name="<?php echo $input_name; ?>[s_color]" value="<?php echo osc_esc_html($color); ?>"/>
        <?php } ?>
        
        <?php if($type == 'make') { ?>
          <label class="mb-check mb-sect">
            <input type="checkbox" name="<?php echo $input_name; ?>[s_section]" value="top" <?php if($section == 'top') { ?>checked<?php } ?>/>
            <span><?php _e('Top', 'car_attr_pro'); ?></span>
          </label>
        <?php } ?>
        
        <label class="mb-check">
          <input type="checkbox" name="<?php echo $input_name; ?>[b_enabled]" value="1" <?php if($enabled == 1) { ?>checked<?php } ?>/>
          <span title="<?php echo osc_esc_html(__('Enabled', 'car_attr_pro')); ?>"><?php _e('Enb', 'car_attr_pro'); ?></span>
        </label>
      </div>
    <?php } ?>
    
    <a href="#" class="mb-delete" data-id="<?php echo $key; ?>"><i class="fa fa-trash mb-delete-icon"></i></a>
    <a href="#" class="mb-expand" data-id="<?php echo $key; ?>"><i class="fa fa-chevron-right mb-expand-icon"></i></a>
  </div>
<?php
}

function cap_admin_generate_empty($text) {
?>
  <div class="mb-empty-box"><?php echo $text; ?></div>
<?php
}


// GENERATE PROPERTY VALUE
function cap_admin_generate_value_elem($id, $data = array(), $prop = array()) {
  $data_type = '';
  $pattern = '[a-z]{1}[a-z0-9\-]{2,49}';

  if(isset($prop['s_data_type'])) {
    $data_type = $prop['s_data_type'];
  }
  
  if($data_type == 'INTEGER') {
    $pattern = '[0-9]{1,50}';
  }
  ?>
  <div class="mb-row" id="cap_<?php echo $id; ?>" data-id="<?php echo $id; ?>" data-sid="<?php echo osc_esc_html($data['s_id']); ?>">
    <input type="hidden" name="capval[<?php echo $id; ?>][pk_i_id]" value="<?php echo osc_esc_html(isset($data['pk_i_id']) ? $data['pk_i_id'] : 0); ?>"/>
    <input type="hidden" name="capval[<?php echo $id; ?>][fk_i_type_id]" value="<?php echo osc_esc_html(isset($data['fk_i_type_id']) ? $data['fk_i_type_id'] : 0); ?>"/>
  
    <div class="mb-col-move"><i class="fa fa-arrows mb-has-tooltip move ui-sortable-handle" title=""></i></div>
    <div class="mb-col-id"><input type="text" name="capval[<?php echo $id; ?>][s_id]" minlength="3" maxlength="50" required pattern="<?php echo $pattern; ?>" value="<?php echo osc_esc_html(isset($data['s_id']) ? $data['s_id'] : ''); ?>" placeholder="<?php echo osc_esc_html(__('Unique ID', 'car_attr_pro')); ?>"/></div>
    <div class="mb-col-name"><input type="text" name="capval[<?php echo $id; ?>][s_name]" required value="<?php echo osc_esc_html(isset($data['s_name']) ? $data['s_name'] : ''); ?>" placeholder="<?php echo osc_esc_html(__('Value name', 'car_attr_pro')); ?>"/></div>
    <div class="mb-col-icon"><input type="text" name="capval[<?php echo $id; ?>][s_icon]" value="<?php echo osc_esc_html(isset($data['s_icon']) ? $data['s_icon'] : ''); ?>" placeholder="<?php echo osc_esc_html(__('FA class, url or svg', 'car_attr_pro')); ?>"/></div>
    <div class="mb-col-del"><a href="#" class="mb-del-value" data-id="<?php echo $id; ?>"><i class="fa fa-times"></i></a></div>
  </div>
  <?php
}

function cap_admin_generate_value_empty($text) {
?>
  <div class="mb-empty-box"><?php echo $text; ?></div>
<?php
}


// ADD FOOTER CSS/JS
function cap_footer_css_js() {
?>
<script type="text/javascript">var capBaseUrl = '<?php echo osc_base_url(true); ?>'; var capParamPrefix = '<?php echo CAP_PARAM_PREFIX; ?>'; capCloseAfterSelect = <?php echo (cap_param('close_after_select') == 1 ? 1 : 0); ?>; var capBaseAjaxUrl = '<?php echo osc_base_url(true); ?>?ajaxRequest=1&nolog=1&capAjax=1';var capIsRtl = <?php echo (cap_is_rtl() ? 'true' : 'false'); ?>;</script>
<style>
  .cap-values-wrap:before {content:"<?php echo osc_esc_html(__('No results', 'car_attr_pro')); ?>";}
  <?php if(cap_param('color_primary') != '') { ?>
  :root body {--cap-primary-color: <?php echo cap_param('color_primary'); ?>;--cap-primary-color-hover: <?php echo cap_param('color_primary_hover') != '' ? cap_param('color_primary_hover') : cap_param('color_primary'); ?>;--set-where: functions.php;}
  <?php } ?>
</style>

<?php if((osc_is_publish_page() || osc_is_edit_page()) && count(cap_get_properties_required()) > 0) { ?>
<script>
  // Required fields validation - publish page
  $(document).ready(function() {
    jQuery.validator.addMethod("capRequired", jQuery.validator.methods.required, 'Required field');
    jQuery.validator.messages.capRequired = function(_,elem) { 
      return jQuery.validator.format("<?php echo osc_esc_html(__('{0}: this field is required', 'image_required')); ?>", $(elem).attr('data-label')); 
    };
    
    jQuery.validator.addClassRules("cap-required", {capRequired: true});
  });
</script>
<?php } ?>
<?php
}

osc_add_hook('footer', 'cap_footer_css_js');
osc_add_hook('admin_footer', 'cap_footer_css_js');


// GET INPUT/SELECT UNIQUE NAME
function cap_get_input_name($slug) {
  if($slug == 'category') {
    $name = 'sCategory';
  } else if($slug == 'price-from') {
    $name = 'sPriceMin';
  } else if($slug == 'price-to') {
    $name = 'sPriceMax';
  } else {
    $name = implode('-', array_filter(array(CAP_PARAM_PREFIX, $slug)));
  }
  
  $name = osc_esc_html($name);
  
  return osc_apply_filter('cap_get_input_name', $name, $slug); 
}


// GET SLUG FROM DATA (OR ID)
function cap_get_slug($data, $id = '', $section = '') {
  if($section == 'publish' || cap_param('use_key_instead_id') == 1) {
    if(isset($data['pk_i_id']) ) {
      return $data['pk_i_id'];
    } else {
      return $id;
    }
  }
  
  if(isset($data['s_slug']) && osc_sanitizeString($data['s_slug']) <> '') {
    return osc_apply_filter('cap_get_slug', osc_sanitizeString($data['s_slug']), $data);
  }
  
  if(isset($data['s_id']) && osc_sanitizeString($data['s_id']) <> '') {
    return osc_apply_filter('cap_get_slug', osc_sanitizeString($data['s_id']), $data);
  }
  
  return osc_apply_filter('cap_get_slug', (isset($data['pk_i_id']) ? $data['pk_i_id'] : osc_sanitizeString($id)), $data);
}


// GET NAME FROM OBJECT
function cap_get_name($data, $is_admin = false) {
  if(isset($data['locales'])) {
    if(cap_field('s_name', $data['locales']) <> '') {
      return osc_apply_filter('cap_get_name', cap_field('s_name', $data['locales']), $data);
    }
  }

  if(isset($data['s_name']) && trim((string)$data['s_name']) <> '') {
    return osc_apply_filter('cap_get_name', $data['s_name'], $data);
  }
  
  if($is_admin && isset($data['s_id']) && trim((string)$data['s_id']) <> '') {
    return osc_apply_filter('cap_get_name', $data['s_id'], $data);
  }
  
  if($is_admin && isset($data['pk_i_id'])) {
    return __('Row', 'car_attr_pro') . ' #' . $data['pk_i_id'];
  }
  
  return __('N/A', 'car_attr_pro');
}


// GET ALT NAME FROM OBJECT
function cap_get_name_alt($data) {
  if(isset($data['locales'])) {
    if(cap_field('s_name_alt', $data['locales']) <> '') {
      return osc_apply_filter('cap_get_name_alt', cap_field('s_name_alt', $data['locales']), $data);
    }
  }

  if(isset($data['s_name_alt']) && trim((string)$data['s_name_alt']) <> '') {
    return osc_apply_filter('cap_get_name_alt', $data['s_name_alt'], $data);
  }
  
  return osc_apply_filter('cap_get_name_alt', '', $data);
}


// GET DESCRIPTION FROM OBJECT
function cap_get_description($data) {
  if(isset($data['s_description']) && trim((string)$data['s_description']) <> '') {
    return osc_apply_filter('cap_get_description', $data['s_description'], $data);
  }
  
  return osc_apply_filter('cap_get_description', '', $data);
}


// GET HELP TEXT FROM OBJECT
function cap_get_help_text_publish($data) {
  if(isset($data['s_text_publish']) && trim((string)$data['s_text_publish']) <> '') {
    return osc_apply_filter('cap_get_help_text_publish', $data['s_text_publish'], $data);
  }
  
  return osc_apply_filter('cap_get_help_text_publish', '', $data);
}


// GET HELP TEXT FROM OBJECT
function cap_get_help_text_search($data) {
  if(isset($data['s_text_search']) && trim((string)$data['s_text_search']) <> '') {
    return osc_apply_filter('cap_get_help_text_search', $data['s_text_search'], $data);
  }
  
  return osc_apply_filter('cap_get_help_text_search', '', $data);
}


// GET HELP TEXT FROM OBJECT
function cap_get_help_text_item($data) {
  if(isset($data['s_text_item']) && trim((string)$data['s_text_item']) <> '') {
    return osc_apply_filter('cap_get_help_text_item', $data['s_text_item'], $data);
  }
  
  return osc_apply_filter('cap_get_help_text_item', '', $data);
}


// GET CORRECT LOCALE VALUE
function cap_field($column, $data, $locale = '') {
  if ($locale == '') {
    $locale = osc_current_user_locale();
  }
 
  $value = @$data[$locale][$column];

  if($value == '') {
    $value = @$data[osc_language()][$column];

    if($value == '') {
      $aLocales = osc_get_locales();
      foreach($aLocales as $locale) {
        $value = @$data[@$locale['pk_c_code']][$column];
        if($value != '') {
          break;
        }
      }
    }
  }

  return (string) $value;
}


// GET LOCATION NAMES
function cap_get_section_location_names($data) {
  $output = array();
  
  if(isset($data['s_section_type'])) {
    $locs = explode(',', $data['s_section_type']);
    
    if(count($locs) > 0) {
      foreach($locs as $loc) {
        $output[] = cap_get_section_location_name($loc);
      }
    } else {
      $output[] = __('None (hidden)', 'car_attr_pro');
    }
  }
  
  return implode(', ', $output);
}


// GET LOCATION NAME
function cap_get_section_location_name($id) {
  if($id == 'item') {
    return __('Item page', 'car_attr_pro');
  } else if($id == 'widget') {
    return __('Widget', 'car_attr_pro');
  } else if($id == 'publish') {
    return __('Publish page', 'car_attr_pro');
  }
}


// ITERRATE CATEGORY TREE
function cap_val_category_sub($tree, $level = 1, $existing_ids = array()) {
  $output = array();
  
  if(isset($tree['categories']) && is_array($tree['categories']) && count($tree['categories']) > 0) {
    foreach($tree['categories'] as $sub_tree) {
      if(!in_array($sub_tree['pk_i_id'], $existing_ids)) {
        $output[$sub_tree['pk_i_id']] = array(
          'pk_i_id' => $sub_tree['pk_i_id'],
          's_name' => $sub_tree['s_name'],
          'level' => $level
        );
        
        $existing_ids[] = $sub_tree['pk_i_id'];
        $sub_output = cap_val_category_sub($sub_tree, $level+1, $existing_ids);
        
        if(is_array($sub_output) && count($sub_output) > 0) {
          //$output = array_merge($output, $sub_output);
          $output = $output+ $sub_output;
        }
      }
    }
  }

  return $output;
}


// BANNED PROPERTY TYPES (RESERVED)
function cap_banned_properties($what = 'id') {
  if($what == 'id') {
    return array('item', 'type', 'make', 'model', 'category', 'location', 'price');
  } else {
    return array('fk_i_item_id', 'fk_i_type_id', 'fk_i_make_id', 'fk_i_model_id', 's_category', 'fk_i_category_id', 's_location', 'fk_i_location_id', 'i_price');
  }
}


// BANNED SECTIONS (RESERVED)
function cap_banned_sections() {
  return array('main', 'bottom');
}


// GENERATE INSERT FOR PROPERTIES SQL CONTENT
function cap_properties_basic_data() {
  $sql = '';

  $i = 1;
  $vi = 1;
  
  $types_list = array(
    'all' => 0,
    'cars' => 1,
    'motorcycles' => 2,
    'trucks' => 3,
    'vans' => 4,
    'trailers' => 5,
    'parts' => 6,
    'construction' => 7,
    'agricultural' => 8
  );

  
  foreach(cap_properties_list() as $sid => $prop) {
    $style = $prop['style'];
    $style_ = explode('_', $style);
    $multi = @$style_[1] == 'multi' ? 1 : 0;
    $range = @$style_[1] == 'range' ? 1 : 0;
    $db_type = ($prop['s_data_type'] == 'STRING' ? 'VARCHAR(30)' : $prop['s_data_type']);
    //$db_type = ($db_type == 'DECIMAL' ? 'DECIMAL(10,3)' : $db_type);
    //$db_type = ($db_type == 'FK_INTEGER' ? 'INTEGER' : $db_type);
    
    $measure = '';
    
    if($sid == 'engine_power') {
      $measure = 'kW';
    } else if($sid == 'engine_size') {
      $measure = 'cm3';
    } else if($sid == 'mileage') {
      $measure = 'km';
    } else if($sid == 'max_payload' || $sid == 'max_weight') {
      $measure = 'kg';
    } else if(substr($sid, 0, 9) == 'fuel_cons') {
      $measure = 'l/100km';
    }
    
    $prio_props = array('type','make','model','body_type','application','fuel','year','drive','gearbox','engine_power','engine_type','mileage','doors','vin','color','color_type');
    $subprio_props = array('cruise_control','connectivity_system','wheels_type','rent','financing','shipping','warranty','new','no_accident','first_owner','serviced','veteran','tuning','vat_deduction','vat_invoice','authorized_dealer','leasing','imported','for_import');
    $fuel_props = array('fuel_cons','fuel_cons_combined','fuel_cons_in_city','fuel_cons_out_city');

    $item_section = '';
    $item_section = (in_array($sid, $prio_props) ? 'main' : '');
    $item_section = (in_array($sid, $subprio_props) ? 'sub' : $item_section);
    $item_section = (in_array($sid, $fuel_props) ? 'fuel' : $item_section);
    $item_section = ($sid == 'keywords' ? 'detail' : $item_section);
    
    if($item_section == '' && $db_type == 'BOOLEAN') {
      $item_section = 'bool';
    }
    
    $stl = $style_[0];
    if($item_section == '') {
      $item_section = 'data';
    }
    
    
    $group_list = isset($prop['groups']) ? implode(',', $prop['groups']) : '';
    
    if($group_list != '') {
      $stl = 'group';
      $multi = 0;
      $range = 0;
    }
    
    $chkvals = cap_properties_values_list();
    if(is_array($chkvals) && count($chkvals) > 0) {
      if(($stl == 'select' || $stl == 'checkbox' || $stl == 'slider') && substr($db_type, 0, 7) == 'VARCHAR') {
        $db_type = 'FK_INTEGER';
        
        if($stl == 'select' || $stl == 'checkbox') {
          $multi = 0;
          $range = 0;
        }
      }
    }
    
    if($sid == 'keywords') {
      $stl = 'textarea';
    }


    $sql .= PHP_EOL;
    $sql .= '/* Property: ' . $sid . ' */' . PHP_EOL;
    $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_property ';
    $sql .= '(pk_i_id,s_id,s_icon,s_style,b_multi_select,b_range,b_required,s_section_widget,s_section_item,s_data_type,s_measure_unit,s_group_list,i_order) ';
    $sql .= 'VALUES (';
    
    $sql .= $i . ',';
    $sql .= '"' . osc_esc_html(osc_sanitizeString(str_replace('_', '-', $sid))) . '",';
    $sql .= '"' . osc_esc_html(str_replace('"', "'", cap_get_prop_icon($sid, true))) . '",';
    $sql .= '"' . osc_esc_html($stl) . '",';
    $sql .= $multi . ',';
    $sql .= $range . ',';
    $sql .= 0 . ',';
    $sql .= '"' . osc_esc_html($prop['section']) . '",';       // s_section_widget
    $sql .= '"' . osc_esc_html($item_section) . '",';         // s_section_item
    $sql .= '"' . osc_esc_html($db_type) . '",'; 
    $sql .= '"' . osc_esc_html($measure) . '",'; 
    $sql .= '"' . osc_esc_html($group_list) . '",'; 
    $sql .= $i;
    
    $sql .= ');' . PHP_EOL;
    
    
    // LOAD LOCALES HERE
    $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_property_locale (fk_i_property_id,fk_c_locale_code,s_name,s_name_alt) ';
    $sql .= 'VALUES (' . $i . ',"/*LOCALE_CODE*/", "' . osc_esc_html(cap_get_name($prop)) . '", "' . osc_esc_html($sid == 'rent' ? __('Sale', 'car_attr_pro') : '') . '");' . PHP_EOL;
  
  
    // LOAD TYPES HERE
    $ptypes = $prop['types'];
    $enabled_types = array();
    
    if(is_array($ptypes) && count($ptypes) > 0) {
      $last = end($ptypes);
      
      if(substr($last, 0, 1) == '~') {
        foreach($types_list as $pt => $pid) {
          if(!in_array('~' . $pt, $ptypes)) {
            $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_property_type (fk_i_property_id,fk_i_type_id) ';
            $sql .= 'VALUES (' . $i . ',' . $pid . ');' . PHP_EOL;
            $enabled_types[$pid] = $pt;
          }
        }
      } else {
        foreach($ptypes as $pt) {
          $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_property_type (fk_i_property_id,fk_i_type_id) ';
          $sql .= 'VALUES (' . $i . ',' . $types_list[$pt] . ');' . PHP_EOL;
          $enabled_types[$types_list[$pt]] = $pt;
        }
      }

    } else {
      $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_property_type (fk_i_property_id,fk_i_type_id) ';
      $sql .= 'VALUES (' . $i . ',0);' . PHP_EOL;
    }
    


    // LOAD VALUES HERE
    $enabled_types[0] = 'all';
    ksort($enabled_types);
    
    $property_val_used_ids = array();
    $ui = 1;

    if(is_array($enabled_types) && count($enabled_types) > 0) {
      foreach($enabled_types as $tp => $tid) {
        $vals = cap_properties_values_list($tid, true);
        
        if(is_array($vals) && count($vals) > 0) {
          foreach($vals as $val_prop_sid => $prop_vals) {
            // only add values related to this property
            if($val_prop_sid == $sid) {
              if(is_array($prop_vals) && count($prop_vals) > 0) {
                foreach($prop_vals as $pval_sid => $pval_name) {
                  $pval_sid_clean = osc_esc_html(osc_sanitizeString($pval_sid));
                  
                  if(in_array($pval_sid_clean, $property_val_used_ids)) {
                    $pval_sid_clean .= '-' . $vi;
                    $ui++;
                  } else {
                    $property_val_used_ids[] = $pval_sid_clean;
                  }
  
                  $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_property_value (pk_i_id,fk_i_property_id,fk_i_type_id,s_id,i_order) ';
                  $sql .= 'VALUES (' . $vi . ',' . $i . ',' . $tp . ',"' . $pval_sid_clean . '",' . $vi . ');' . PHP_EOL;
          
                  $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_property_value_locale (fk_i_property_value_id,fk_c_locale_code,s_name) ';
                  $sql .= 'VALUES (' . $vi . ',"/*LOCALE_CODE*/", "' . osc_esc_html($pval_name) . '");' . PHP_EOL;
                  
                  $vi++;
                }
              }
            }
          }
        }
      }
    }
    
    $i++;
  }

  echo '<pre>';
  echo $sql;
  echo '</pre>';
  exit;
}

// cap_properties_basic_data();


// GENERATE MAKES & MODELS BASIC DATA SQL CONTENT
function cap_makes_models_basic_data() {
  $sql = '';

  echo '<pre>';
  
  // CARS
  $car_type_id = 1;
  $data = file_get_contents(osc_base_path() . 'oc-content/plugins/car_attr_pro/model/json/car_makes_models.json');
  $data = json_decode($data, true);


  $make_ids = array();
  $makes = array();
  $models = array();

  $i = 1;
  foreach($data as $row) {
    $j = 1;
    
    if(!in_array(strtolower($row['brand']), $makes)) {
      $section = (in_array(strtolower($row['brand']), array('mercedes','volkswagen','skoda','bmw','audi')) ? 'top' : '');
      $sql .= '/* Make: ' . osc_esc_html($row['brand']) . ' */'.PHP_EOL;
      $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_make (pk_i_id,fk_i_type_id,s_name,s_slug,s_icon,s_color,s_section,i_order,b_enabled) VALUES ('.$i.','.$car_type_id.',"'.osc_esc_html($row['brand']).'","'.osc_sanitizeString($row['brand']).'","","","'.$section.'",'.$i.',1);'.PHP_EOL.PHP_EOL;
      $makes[] = strtolower($row['brand']);
      $make_ids[strtolower($row['brand'])] = $i;
      $i++;
    } else {
      $sql .= '/* Make: ' . osc_esc_html($row['brand']) . ' - already exists */'.PHP_EOL;
    }
    
    $sql .= '/* Models for make: ' . osc_esc_html($row['brand']) . ' */'.PHP_EOL;
    foreach($row['models'] as $mdl) {
      if(!in_array(strtolower($row['brand']).'_'.strtolower($mdl), $models)) {
        $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_model (fk_i_make_id,s_name,s_slug,i_order,b_enabled) VALUES ('.$make_ids[strtolower($row['brand'])].',"'.osc_esc_html($mdl).'","'.osc_sanitizeString(str_replace('.', '', $mdl)).'",'.$j.',1);'.PHP_EOL;
        $models[] = strtolower($row['brand']).'_'.strtolower($mdl);
        $j++;
      }
    }
    
    $sql .= PHP_EOL;
  }
  
  
  
  // MOTORCYCLES
  $car_type_id = 2;
  $data = file_get_contents(osc_base_path() . 'oc-content/plugins/car_attr_pro/model/json/moto_makes_models.json');
  $data = json_decode($data, true);

  $make_ids = array();
  $makes = array();
  $prev_make = '';

  foreach($data as $row) {
    $j = 1;
    $models = array();
    
    $mk = $row[0];
    $mdl = $row[1];

    if($prev_make != $mk) {
      $sql .= PHP_EOL;
    }
    
    if(!in_array(strtolower($mk), $makes)) {
      $section = (in_array(strtolower($mk), array('mercedes','volkswagen','skoda','bmw','audi')) ? 'top' : '');
      $sql .= '/* Make: ' . osc_esc_html($mk) . ' */'.PHP_EOL;
      $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_make (pk_i_id,fk_i_type_id,s_name,s_slug,s_icon,s_color,s_section,i_order,b_enabled) VALUES ('.$i.','.$car_type_id.',"'.osc_esc_html($mk).'","'.osc_sanitizeString($mk).'","","","'.$section.'",'.$i.',1);'.PHP_EOL.PHP_EOL;
      $makes[] = strtolower($mk);
      $make_ids[strtolower($mk)] = $i;
      $i++;
    }
    
    if($prev_make != $mk) {
      $sql .= '/* Models for make: ' . osc_esc_html($mk) . ' */'.PHP_EOL;
    }
    
    if(!in_array(strtolower($mk).'_'.strtolower($mdl), $models)) {
      $sql .= 'INSERT INTO /*TABLE_PREFIX*/t_cap_model (fk_i_make_id,s_name,s_slug,i_order,b_enabled) VALUES ('.$make_ids[strtolower($mk)].',"'.osc_esc_html($mdl).'","'.osc_sanitizeString(str_replace('.', '', $mdl)).'",'.$j.',1);'.PHP_EOL;
      $models[] = strtolower($mk).'_'.strtolower($mdl);
      $j++;
    }
    
    $prev_make = $mk;
  }

  echo $sql;
  echo '</pre>';
}




// INCLUDE MAILER SCRIPT
function cap_include_mailer() {
  if(file_exists(osc_lib_path() . 'phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'phpmailer/class.phpmailer.php';
  } else if(file_exists(osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php';
  }
}


// CREATE LOCALE SELECT BOX
function cap_locale_box( $file, $param = '', $id = -1 ) {
  $html = '';
  $locales = OSCLocale::newInstance()->listAllEnabled();
  $current = cap_get_locale();
  $id_string = '';

  if($id > 0 && $param <> '') {
    $id_string = '&' . $param . '=' . $id;
  }

  $html .= '<select rel="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=car_attr_pro/admin/' . $file . $id_string . '" class="mb-select mb-select-locale" id="capLocale" name="capLocale">';

  foreach( $locales as $l ) {
    $html .= '<option value="' . $l['pk_c_code'] . '" ' . ($current == $l['pk_c_code'] ? 'selected="selected"' : '') . '>' . $l['s_name'] . '</option>';
  }
 
  $html .= '</select>';
  return $html;
}


// GET CURRENT OR DEFAULT ADMIN LOCALE
function cap_get_locale() {
  $locales = OSCLocale::newInstance()->listAllEnabled();

  if(Params::getParam('capLocale') <> '') {
    $current = Params::getParam('capLocale');
  } else {
    $current = (osc_current_user_locale() <> '' ? osc_current_user_locale() : osc_current_admin_locale());
    $current_exists = false;

    // check if current locale exist in front-office
    foreach( $locales as $l ) {
      if($current == $l['pk_c_code']) {
        $current_exists = true;
      }
    }

    if( !$current_exists ) {
      $i = 0;
      foreach( $locales as $l ) {
        if( $i==0 ) {
          $current = $l['pk_c_code'];
        }

        $i++;
      }
    }
  }

  return $current;
}


// GET TYPE
// ID can be key or identifier
function cap_get_type($id) {
  if($id == '') { 
    return false; 
  }
  
  $types = cap_get_types(); 

  if(!is_array($types) || count($types) <= 0) {
    return false;
  }
  
  if(isset($types[$id])) {
    return $types[$id];
  }
  
  foreach($types as $type) {
    if($type['s_slug'] == $id) {
      return $type;
    }
  }
  
  return false;
}


// GET TYPES
function cap_get_types($category_id = 0) {
  $category_id = (int)$category_id;
  
  if(View::newInstance()->_exists('cap_' . $category_id . '_types')) {
    return View::newInstance()->_get('cap_' . $category_id . '_types');
  } else {
    $types = ModelCAP::newInstance()->getTypes(true, $category_id);
    
    View::newInstance()->_exportVariableToView('cap_' . $category_id . '_types', $types);
    return $types;
  }
}


// GET TYPES ALL
function cap_get_types_all() {
  if(View::newInstance()->_exists('cap_types_all')) {
    return View::newInstance()->_get('cap_types_all');
  } else {
    $types = ModelCAP::newInstance()->getTypes(true);
    View::newInstance()->_exportVariableToView('cap_types_all', $types);
    
    return $types;
  }
}


// GET PROPERTIES
function cap_get_properties($type_id = -1) {
  $type_id_ = strtolower($type_id == -1 ? 'all' : $type_id);

  if(View::newInstance()->_exists('cap_' . $type_id_ . '_properties')) {
    return View::newInstance()->_get('cap_' . $type_id_ . '_properties');
  } else {
    //$properties = ModelCAP::newInstance()->getProperties(true, $type_id);
    $properties = cap_get_properties_all();
    $output = array();

    if($type_id > 0) {
      if(is_array($properties) && count($properties) > 0) {
        foreach($properties as $key => $prop) {
          if(in_array($type_id, $prop['types']) || in_array(0, $prop['types']) || $type_id <= 0) {
            $output[$key] = $prop;
          }
        }
      }
    } else {
      $output = $properties;
    }
    
    View::newInstance()->_exportVariableToView('cap_' . $type_id_ . '_properties', $output);
    return $output;
  }
}


// GET PROPERTIES ALL
function cap_get_properties_all() {
  if(View::newInstance()->_exists('cap_properties_all')) {
    return View::newInstance()->_get('cap_properties_all');
  } else {
    $properties = ModelCAP::newInstance()->getProperties(true);
    View::newInstance()->_exportVariableToView('cap_properties_all', $properties);
    
    return $properties;
  }
}


// GET ALL REQUIRED PROPERTIES
function cap_get_properties_required() {
  if(View::newInstance()->_exists('cap_properties_required')) {
    return View::newInstance()->_get('cap_properties_required');
  } else {
    $properties = cap_get_properties_all();
    $output = array();
    
    if(is_array($properties) && count($properties) > 0) {
      foreach($properties as $key => $prop) {
        if($prop['b_required'] == 1) {
          $output[$key] = $prop;
        }
      }
    }
    
    View::newInstance()->_exportVariableToView('cap_properties_required', $output);
    return $output;
  }
}


// GET PROPERTIES GROUPS
function cap_get_properties_groups() {
  if(View::newInstance()->_exists('cap_properties_groups')) {
    return View::newInstance()->_get('cap_properties_groups');
  } else {
    $groups = ModelCAP::newInstance()->getPropertiesGroups();
    View::newInstance()->_exportVariableToView('cap_properties_groups', $groups);
    
    return $groups;
  }
}


// GET PROPERTY KEY FROM ID
function cap_get_property_keys($id = '') {
  $props = cap_get_properties();
  $data = array_combine(array_column($props, 's_id'), array_column($props, 'pk_i_id'));

  if($id != '') {
    if(isset($data[$id])) {
      return $data[$id];
    }
    
    return false;
  }
  
  return $data;
}


// GET PROPERTY VALUES
function cap_get_property_values($prop_key, $type_id = 0) {
  $properties = cap_get_properties();
  $prop = $properties[$prop_key];
  $id = $prop['s_id'];
  
  if(isset($prop['values'][$type_id])) {
    return $prop['values'][$type_id];
  } else if($type_id > 0 && isset($prop['values'][0])) {
    return $prop['values'][0];
  }

  return array();
}


// GET SECTIONS
function cap_get_sections($location = 'item', $type_id = -1) {
  $type_id_ = strtolower($type_id == -1 ? 'all' : $type_id);

  if(View::newInstance()->_exists('cap_' . $location . '_' . $type_id_ . '_sections')) {
    return View::newInstance()->_get('cap_' . $location . '_' . $type_id_ . '_sections');
  } else {
    //$properties = ModelCAP::newInstance()->getProperties(true, $type_id);
    $sections = cap_get_sections_all_location($location);
    $output = array();

    if($type_id > 0) {
      if(is_array($sections) && count($sections) > 0) {
        foreach($sections as $key => $section) {
          if(in_array($type_id, $section['types']) || in_array(0, $section['types']) || $type_id <= 0) {
            $output[$key] = $section;
          }
        }
      }
    } else {
      $output = $sections;
    }
    
    View::newInstance()->_exportVariableToView('cap_' . $location . '_' . $type_id_ . '_sections', $output);
    return $output;
  }
}

// GET SECTIONS BY LOCATION (item, search, publish)
function cap_get_sections_all_location($location = 'item') {
  $location_ = ($location <> '' ? $location : 'all');
  
  if(View::newInstance()->_exists('cap_' . $location_ . '_sections')) {
    return View::newInstance()->_get('cap_' . $location_ . '_sections');
  } else {
    $sections = cap_get_sections_all();
    $output = array();

    if($location <> '') {
      if(is_array($sections) && count($sections) > 0) {
        foreach($sections as $key => $section) {
          $locations = explode(',', $section['s_section_type']);
          
          if(!empty($locations) && in_array($location, $locations)) {
            $output[$key] = $section;
          }
        }
      }
    } else {
      $output = $sections; 
    }
    
    View::newInstance()->_exportVariableToView('cap_' . $location_ . '_sections', $output);
    return $output;
  }
}

// GET SECTIONS ALL
function cap_get_sections_all() {
  if(View::newInstance()->_exists('cap_sections_all')) {
    return View::newInstance()->_get('cap_sections_all');
  } else {
    $sections = ModelCAP::newInstance()->getSections(true);
    View::newInstance()->_exportVariableToView('cap_sections_all', $sections);
    
    return $sections;
  }
}

 

// GET CATEGORIES TREE AND STORE THEM IN SESSION
function cap_get_categories_to_tree() {
  if(View::newInstance()->_exists('cap_categories_to_tree')) {
    return View::newInstance()->_get('cap_categories_to_tree');
  } else {
    $categories = Category::newInstance()->toTree();
    View::newInstance()->_exportVariableToView('cap_categories_to_tree', $categories);
    
    return $categories;
  }
}


// GET COUNTRIES AND STORE THEM IN SESSION
function cap_get_countries() {
  if(View::newInstance()->_exists('cap_countries')) {
    return View::newInstance()->_get('cap_countries');
  } else {
    $countries = Country::newInstance()->listAll();
    View::newInstance()->_exportVariableToView('cap_countries', $countries);
    
    return $countries;
  }
}


// GET REGIONS AND STORE THEM IN SESSION
function cap_get_regions($country_code = '') {
  $country_code = trim((string)$country_code);
  $country_code_ = strtolower($country_code == '' ? 'all' : $country_code);
  
  if(View::newInstance()->_exists('cap_' . $country_code_ . '_regions')) {
    return View::newInstance()->_get('cap_' . $country_code_ . '_regions');
  } else {
    if($country_code == '') {
      $regions = Region::newInstance()->listAll();
      View::newInstance()->_exportVariableToView('cap_' . $country_code_ . '_regions', $regions);

    } else {
      $regions = Region::newInstance()->findByCountry($country_code);
      View::newInstance()->_exportVariableToView('cap_' . $country_code_ . '_regions', $regions);
    }
    
    return $regions;
  }
}


// RTL LANGUAGE SUPPORT
function cap_is_rtl() {
  $current_lang = strtolower(osc_current_user_locale());
  $locale = osc_get_current_user_locale();
  
  if(isset($locale['b_rtl']) && $locale['b_rtl'] == 1) {
    return true;
  } else if(in_array(osc_current_user_locale(), cap_rtl_languages())) {
    return true;
  } else {
    return false;
  }
}

// GET DIRECTION STRING
function cap_language_dir() {
  return cap_is_rtl() ? 'rtl' : 'ltr';
}

// LIST ALL RTL LANGUAGES/LOCALES FOR OLDER OSCLASS VERSIONS
function cap_rtl_languages() {
  $langs = array('ar_LB','ar_DZ','ar_BH','ar_EG','ar_IQ','ar_JO','ar_KW','ar_LY','ar_MA','ar_OM','ar_SA','ar_SY','fa_IR','ar_TN','ar_AE','ar_YE','ar_TD','ar_CO','ar_DJ','ar_ER','ar_MR','ar_SD');
  return $langs;
}


// CORE FUNCTIONS
function cap_param($name) {
  return osc_get_preference($name, 'plugin-car_attr_pro');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF CACHE ENABLED
function cap_cache_enabled() {
  // Disable in backoffice
  if(defined('OC_ADMIN') && OC_ADMIN) {
    return false;
  }
  
  if(cap_param('enable_cache') == 1) {
    return true;
  }

  return false;
}


// GET LIFETIME OF CACHE
function cap_cache_ttl() {
  return OSC_CACHE_TTL;
}

// CHECK IF RUNNING ON DEMO
function cap_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if( !function_exists('osc_is_contact_page') ) {
  function osc_is_contact_page() {
    $location = Rewrite::newInstance()->get_location();
    $section = Rewrite::newInstance()->get_section();
    if( $location == 'contact' ) {
      return true ;
    }

    return false ;
  }
}


// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}



// CATEGORIES WORK
function cap_cat_tree($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
      $array[$c['pk_i_id']]['sub'] = cap_cat_sub($list, $c['pk_i_id']);
    }
  }

  return $array;
}

function cap_cat_sub($list, $parent_id) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {
        $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
        $array[$c['pk_i_id']]['sub'] = cap_cat_sub($list, $c['pk_i_id']);
      }
    }
  }
      
  return $array;
}

function cap_cat_list($selected = array(), $categories = '', $level = 0) {
  if($categories == '' || $level == 0) {
    $categories = cap_cat_tree($categories);
  }

  if($level == 0) {
    echo '<option value="0" ' . (in_array(0, $selected) ? 'selected="selected"' : '') . '>' . __('All categories', 'car_attr_pro') . '</option>';
  }
  
  foreach($categories as $c) {
    echo '<option value="' . $c['pk_i_id'] . '" ' . (in_array($c['pk_i_id'], $selected) ? 'selected="selected"' : '') . '>' . str_repeat('-', $level) . ($level > 0 ? ' ' : '') . $c['s_name'] . '</option>';

    if(@count($c['sub']) > 0) {
      cap_cat_list($selected, $c['sub'], $level + 1);
    }
  }
}


function cap_list_values_ol($values) {
 if(count($values) > 0 && is_array($values)) {
    foreach($values as $v) {
      ?>

      <li class="mb-val" id="val_<?php echo $v['pk_i_id']; ?>">
        <?php cap_div_value($v); ?>
      
        <ol>
          <?php 
            if(isset($v['values']) && count($v['values']) > 0) { 
              cap_list_values_ol($v['values']); 
            }
          ?>
        </ol>
      </li>
    <?php
    }
  }
}


// CATEGORIES WORK FLAT
function cap_cat_tree_flat($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  $level = 0;
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name'], 'i_level' => $level);
      $array = array_merge($array, cap_cat_sub_flat($list, $c['pk_i_id'], $level + 1));
    }
  }

  return $array;
}

function cap_cat_sub_flat($list, $parent_id, $level = 0) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {
        $array[] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name'], 'i_level' => $level);
        $array = array_merge($array, cap_cat_sub_flat($list, $c['pk_i_id'], $level + 1));
      }
    }
  }

  return $array;
}



// GENERATE PAGINATION
function cap_admin_paginate($file, $page_id, $per_page, $count_all, $class = '', $params = '') {
  $html = '';
  $page_id = (int)$page_id;
  $page_id = ($page_id <= 0 ? 1 : $page_id);
  $base_link = osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=' . $file . $params;

  if($per_page < $count_all) {
    $html .= '<div id="mb-pagination" class="' . $class . '">';
    $html .= '<div class="mb-pagination-wrap">';
    $html .= '<div>' . __('Page:', 'car_attr_pro') . '</div>';

    $pages = ceil($count_all/$per_page); 
    $page_actual = ($page_id == '' ? 1 : $page_id);

    if($pages > 6) {

      // Too many pages to list them all
      if($page_id == 1) { 
        $ids = array(1,2,3, $pages);

      } else if ($page_id > 1 && $page_id < $pages) {
        $ids = array(1,$page_id-1, $page_id, $page_id+1, $pages);

      } else {
        $ids = array(1, $page_id-2, $page_id-1, $page_id);
      }

      $old = -1;
      $ids = array_unique(array_filter($ids));

      foreach($ids as $i) {
        $url = $base_link . '&pageId=' . $i;
        
        if($old <> -1 && $old <> $i - 1) {
          $html .= '<span>&middot;&middot;&middot;</span>';
        }

        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="mb-active"' : '') . '>' . $i . '</a>';
        $old = $i;
      }

    } else {

      // List all pages
      for ($i = 1; $i <= $pages; $i++) {
        $url = $base_link . '&pageId=' . $i;
        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="mb-active"' : '') . '>' . $i . '</a>';
      }
    }

    $html .= '</div>';
    $html .= '</div>';
  }

  return $html;
}


if(!function_exists('mb_generate_rand_int')) {
  function mb_generate_rand_int($length = 18) {
    $characters = '0123456789';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}


if(!function_exists('osc_get_current_user_locations_native')) {
  function osc_get_current_user_locations_native() {
    return false;
  }
}

if(!function_exists('osc_location_native_name_selector')) {
  function osc_location_native_name_selector($array, $column = 's_name') {
    return @$array[$column];
  }
}

?>