<?php
class ModelCAP extends DAO {
private static $instance;

public static function newInstance() {
  if(!self::$instance instanceof self) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_item() {
  return DB_TABLE_PREFIX.'t_item';
}

public function getTable_type() {
  return DB_TABLE_PREFIX.'t_cap_type';
}

public function getTable_type_locale() {
  return DB_TABLE_PREFIX.'t_cap_type_locale';
}

public function getTable_type_category() {
  return DB_TABLE_PREFIX.'t_cap_type_category';
}

public function getTable_make() {
  return DB_TABLE_PREFIX.'t_cap_make';
}

public function getTable_model() {
  return DB_TABLE_PREFIX.'t_cap_model';
}

public function getTable_section() {
  return DB_TABLE_PREFIX.'t_cap_section';
}

public function getTable_section_locale() {
  return DB_TABLE_PREFIX.'t_cap_section_locale';
}

public function getTable_section_type() {
  return DB_TABLE_PREFIX.'t_cap_section_type';
}

public function getTable_property() {
  return DB_TABLE_PREFIX.'t_cap_property';
}

public function getTable_property_locale() {
  return DB_TABLE_PREFIX.'t_cap_property_locale';
}

public function getTable_property_type() {
  return DB_TABLE_PREFIX.'t_cap_property_type';
}

public function getTable_property_value() {
  return DB_TABLE_PREFIX.'t_cap_property_value';
}

public function getTable_property_value_locale() {
  return DB_TABLE_PREFIX.'t_cap_property_value_locale';
}

public function getTable_item_attr() {
  return DB_TABLE_PREFIX.'t_cap_item_attr';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);
  
  $sql = str_replace('/*LOCALE_CODE*/', osc_language(), $sql);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelCAP<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install() {
  $this->import('car_attr_pro/model/struct.sql');
  
  // Default data
  if(!defined('CAP_INSTALL_BASIC_DATA') || CAP_INSTALL_BASIC_DATA == 1) {
    $this->import('car_attr_pro/model/basic_data.sql');
  }
  
  $this->updateItemAttrStructure(true);
}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_make()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_model()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_item_attr()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_type()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_type_locale()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_type_category()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_section()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_section_locale()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_section_type()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_property()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_property_locale()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_property_type()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_property_value()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_property_value_locale()));
  

  
  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-car_attr_pro'";
  $this->dao->query($query);
}


// EXECUTE QUERIES ON VERSION UPDATE
public function versionUpdate($ignore_error = false) {
  $version = (int)cap_param('version');     // v100 is initial
  $version = ($version >= 100 ? $version : 0);
  $plugin = 'car_attr_pro';
  
  // Version not yet available - it's installation process now
  if($version == 0) {
    return true;
  }
  
  // $queries = array(
    // array('version' => 101, 'query' => sprintf("CREATE TABLE %st_cap_values (pk_i_id INT NOT NULL AUTO_INCREMENT, fk_c_locale_code CHAR(5) NOT NULL, s_type VARCHAR(20), s_name VARCHAR(100) NOT NULL, PRIMARY KEY(pk_i_id, fk_c_locale_code)) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';", DB_TABLE_PREFIX)),
    // array('version' => 102, 'query' => sprintf("ALTER TABLE %st_user_car_attr_pro ADD COLUMN s_gallery VARCHAR(2000);", DB_TABLE_PREFIX)),
  // );
  
  $queries = array();
  
  if(is_array($queries) && count($queries) > 0) {
    foreach($queries as $query) {
      if($version < $query['version'] && $query['version'] <= cap_VERSION_ID) {
        $result = $this->dao->query($query['query']);
        
        if($result === false && $ignore_error !== true) {
          $message  = sprintf(__('Update of plugin "%s" failed on DB version "%s". Please enable %s to see error details. Failed query is listed below.', 'car_attr_pro'), __('Business Profile Plugin', 'car_attr_pro'), $query['version'], '<a href="https://docs.osclasspoint.com/debug-mode" target="_blank">' . __('DB debug mode', 'car_attr_pro') . '</a>');
          $message .= '<pre style="font-size:11px;">' . $query['query'] . '</pre>';
          $message .= '<a href="' . osc_admin_base_url(true) . '?page=plugins&forceupdateplugin=' . $plugin . '">' . __('Ignore error and force plugin update', 'car_attr_pro') . '</a>. ';
          $message .= __('Never force update plugin until you are sure that your database structure match to model/struct.sql file! It may lead to unexpected plugin functionality. Try to reinstall plugin.', 'car_attr_pro');

          osc_add_flash_error_message($message, 'admin');
          return false;
        }
      }
    }
  }
  
  return true;
}



// GET ITEM DATA
public function getItemAttr($id) {
  if($id <= 0) { return false; }

  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getPropertiesGroups::' . (string)$id);
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
  
  } else {
    $this->dao->select();
    $this->dao->from($this->getTable_item_attr());
    $this->dao->where('fk_i_item_id', $id);

    $result = $this->dao->get();
    
    if($result) {
      $data = $result->row();
      osc_cache_set($cache_key, $data, cap_cache_ttl());
      return $data;
    }
    
    osc_cache_set($cache_key, false, cap_cache_ttl());
    return false;
  } 
  
  return false;
}


// GET MAKES DATA
public function getMakes($type_id = '', $enabled = true, $with_icon = false) {
  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getMakes::' . (string)$type_id . (string)$enabled . (string)$with_icon);
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
  
  } else {
    $this->dao->select('mk.*');
    $this->dao->from($this->getTable_make() . ' as mk');
    $this->dao->join($this->getTable_type() . ' as tp', '(mk.fk_i_type_id = tp.pk_i_id)', 'INNER');

    if($enabled) {
      $this->dao->where('mk.b_enabled', 1);
      $this->dao->where('tp.b_enabled', 1);
    }
    
    if($with_icon) {
      $this->dao->where('trim(mk.s_icon) <> ""');
    }

    if(filter_var($type_id, FILTER_VALIDATE_INT) !== false && (int)$type_id > 0) {
      $this->dao->where('tp.pk_i_id', (int)$type_id);
    } else if ((string)$type_id != '') {
      //$this->dao->where('tp.s_type', (string)$type_id);
      //$this->dao->where('tp.s_id', (string)$type_id);
      $this->dao->join($this->getTable_type_locale() . ' as tpl', '(tpl.fk_i_type_id = tp.pk_i_id AND tpl.fk_c_locale_code = "' . cap_get_locale() . '")', 'LEFT OUTER');
      $this->dao->where('tpl.s_slug', (string)$type_id);
    }
    
    $this->dao->orderby('case when mk.s_section = "top" then 1 else 0 end DESC, mk.i_order ASC, mk.s_name ASC', '');
    
    $result = $this->dao->get();
    
    if($result) {
      $data = $result->result();
      $output = array();
      
      if(is_array($data) && count($data) > 0) {
        foreach($data as $dat) {
          $output[$dat['pk_i_id']] = $dat;
        }
      }

      osc_cache_set($cache_key, $output, cap_cache_ttl());
      return $output;
    }
    
    osc_cache_set($cache_key, array(), cap_cache_ttl());
    return array();
  } 
  
  return array();
}


// GET MAKE
public function getMake($id) {
  if($id <= 0) { return false; }

  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getTypeCategories::' . (string)$id);
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
    
  } else {
    $this->dao->select();
    $this->dao->from($this->getTable_make());
    
    if(filter_var($id, FILTER_VALIDATE_INT) !== false && (int)$id > 0) {
      $this->dao->where('pk_i_id', (int)$id);
    } else if ((string)$id != '') {
      $this->dao->where('s_slug', (string)$id);
    }

    $result = $this->dao->get();
    
    if($result) {
      $data = $result->row();
      osc_cache_set($cache_key, $data, cap_cache_ttl());
      return $data;
    }

    osc_cache_set($cache_key, false, cap_cache_ttl());
    return false;
  } 
  
  return false;
}


// GET MODEL
public function getModel($id) {
  if($id <= 0) { return false; }
  
  $this->dao->select();
  $this->dao->from($this->getTable_model());

  if(filter_var($id, FILTER_VALIDATE_INT) !== false && (int)$id > 0) {
    $this->dao->where('pk_i_id', (int)$id);
  } else if ((string)$id != '') {
    $this->dao->where('s_slug', (string)$id);
  }
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }

  return false;
}


// GET MODELS DATA
// Make ID can be integer or array of integers. If not provided, return empty array.
public function getModels($make_id = array(), $enabled = true) {
  if((is_array($make_id) && empty($make_id)) || $make_id <= 0) {
    return array();
  }

  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getModels::' . (string)json_encode($make_id) . (string)$enabled);
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
  
  } else {
    $sql_values = cap_separate_string_int_arr($make_id, true);
    
    // Check if both values are not dummy
    if($sql_values['string'] == '"DUMMY_CONDITION"' && $sql_values['integer'] == -1) {
      return array();
    }
    
    $this->dao->select('mdl.*, mk.s_name as s_make_name, mk.pk_i_id as s_section');
    $this->dao->from($this->getTable_model() . ' as mdl');
    $this->dao->join($this->getTable_make() . ' as mk', '(mk.pk_i_id = mdl.fk_i_make_id)', 'INNER');
    $this->dao->join($this->getTable_type() . ' as tp', '(mk.fk_i_type_id = tp.pk_i_id)', 'INNER');

    if($enabled) {
      $this->dao->where('tp.b_enabled', 1);
      $this->dao->where('mk.b_enabled', 1);
      $this->dao->where('mdl.b_enabled', 1);
    }
    
    $this->dao->where('(mk.s_slug in (' . $sql_values['string'] . ') OR mk.pk_i_id in (' . $sql_values['integer'] . '))');
    $this->dao->limit(cap_param('models_limit') > 0 ? cap_param('models_limit') : 200);
    $this->dao->orderby('mk.i_order ASC, mk.s_name ASC, mdl.i_order ASC, mdl.s_name ASC', '');

    $result = $this->dao->get();
    
    if($result) {
      $data = $result->result();
      $output = array();
      
      if(is_array($data) && count($data) > 0) {
        foreach($data as $dat) {
          $output[$dat['pk_i_id']] = $dat;
        }
      }
      
      osc_cache_set($cache_key, $output, cap_cache_ttl());
      return $output;
    }
    
    osc_cache_set($cache_key, array(), cap_cache_ttl());
    return array();
  } 
  
  return array();
}



// GET CAR TYPES
public function getTypes($enabled = true, $category_id = 0, $limit = 0) {
  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getTypes::' . (string)$enabled . (string)$category_id . (string)$limit);
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
    
  } else {
    $this->dao->select('t.*, tl.*');
    $this->dao->from($this->getTable_type() . ' as t');
    $this->dao->join($this->getTable_type_locale() . ' as tl', '(tl.fk_i_type_id = t.pk_i_id AND tl.fk_c_locale_code = "' . cap_get_locale() . '")', 'LEFT OUTER');

    if($enabled) {
      $this->dao->where('t.b_enabled', 1);
    }

    if((int)$category_id > 0) {
      //$this->dao->join($this->getTable_type_category() . ' as tc', '((tc.fk_i_type_id = t.pk_i_id AND tc.fk_i_category_id = ' . (int)$category_id . ') OR tc.fk_i_category_id = 0)', 'INNER');
      //$this->dao->where('tc.fk_i_category_id', (int)$category_id);
      $in_query = sprintf('SELECT fk_i_type_id FROM %s WHERE fk_i_category_id in (0,%d)', $this->getTable_type_category(),(int)$category_id);
      $this->dao->where(sprintf('t.pk_i_id in (%s)', $in_query));
    }

    if($limit > 0) {
      $this->dao->limit($limit);
    }
    
    $this->dao->orderby('t.i_order', 'ASC');

    $result = $this->dao->get();
    
    if($result) {
      $data = $result->result();
      $output = array();

      $locales = $this->getTypesLocales();
      
      if(is_array($data) && count($data) > 0) {
        foreach($data as $d) {
          $output[$d['pk_i_id']] = $d;
          $output[$d['pk_i_id']]['locales'] = isset($locales[$d['pk_i_id']]) ? $locales[$d['pk_i_id']] : array();
        }
      }

      osc_cache_set($cache_key, $output, cap_cache_ttl());
      return $output;
    }
    
    osc_cache_set($cache_key, array(), cap_cache_ttl());
    return array();
  } 
  
  return array();
}


// GET TYPE LOCALES
public function getTypeLocales($id) {
  if($id <= 0) { return array(); }
  
  $this->dao->select();
  $this->dao->from($this->getTable_type_locale());
  $this->dao->where('fk_i_type_id', (int)$id);

  $result = $this->dao->get();
  $array = array();

  if($result) {
    $locales = $result->result();

    if(is_array($locales) && count($locales) > 0) {
      foreach($locales as $l) {
        $array[$l['fk_c_locale_code']] = array(
          's_name' => $l['s_name'], 
          's_description' => $l['s_description'], 
          's_slug' => $l['s_slug']
        );
      }
    }
  }

  return $array;
}


// GET TYPE LOCALES FOR ALL TYPES
public function getTypesLocales() {
  $this->dao->select();
  $this->dao->from($this->getTable_type_locale());

  $result = $this->dao->get();
  $array = array();

  if($result) {
    $locales = $result->result();

    if(is_array($locales) && count($locales) > 0) {
      foreach($locales as $l) {
        $array[$l['fk_i_type_id']][$l['fk_c_locale_code']] = array(
          's_name' => $l['s_name'], 
          's_description' => $l['s_description'], 
          's_slug' => $l['s_slug']
        );
      }
    }
  }

  return $array;
}


// GET CAR TYPES ROWS
public function getTypesRows($enabled = 1) {
  $this->dao->select();
  $this->dao->from($this->getTable_type());

  if($enabled == 1) {
    $this->dao->where('b_enabled', 1);
  }

  $this->dao->orderby('i_order', 'ASC');

  $result = $this->dao->get();
  
  if($result) {
    return $result->result();
  }

  return array();
}


// GET CAR TYPE
public function getType($id) {
  if($id <= 0) { return false; }

  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getType::' . (string)$id);
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
    
  } else {
    $this->dao->select('t.*, tl.*');
    $this->dao->from($this->getTable_type() . ' as t');
    $this->dao->join($this->getTable_type_locale() . ' as tl', '(tl.fk_i_type_id = t.pk_i_id AND tl.fk_c_locale_code = "' . cap_get_locale() . '")', 'LEFT OUTER');

    if(filter_var($id, FILTER_VALIDATE_INT) !== false && (int)$id > 0) {
      $this->dao->where('t.pk_i_id', (int)$id);
    } else if ((string)$id != '') {
      $this->dao->where('tl.s_slug', (string)$id);
    } else {
      return false;
    }
    
    $result = $this->dao->get();
    
    if($result) {
      $data = $result->row();
      
      if(isset($data['pk_i_id'])) {
        $data['locales'] = $this->getTypeLocales($data['pk_i_id']);
        osc_cache_set($cache_key, $data, cap_cache_ttl());
        return $data;
      }
    }
    
    osc_cache_set($cache_key, false, cap_cache_ttl());
    return false;
  } 
  
  return false;
}

// GET CAR TYPE RECORD
public function getTypeRow($id) {
  if($id <= 0) { return false; }
  
  $this->dao->select();
  $this->dao->from($this->getTable_type());

  $this->dao->where('pk_i_id', $id);
  
  $result = $this->dao->get();
  
  if($result) {
    return $result->row();
  }

  return false;
}


// GET CATEGORY TYPE
public function getCategoryTypes($id) {
  if($id <= 0) { return array(); }
  
  $this->dao->select();
  $this->dao->from($this->getTable_type_category());
  
  $this->dao->where(sprintf('(fk_i_category_id in (0,%d)', (int)$id));
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
  }

  return array();
}


// GET CATEGORIES OF TYPE
public function getTypeCategories($id) {
  if($id <= 0) { return array(); }

  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getTypeCategories::' . (string)$id);
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
    
  } else {
    $this->dao->select();
    $this->dao->from($this->getTable_type_category());
    
    $this->dao->where('fk_i_type_id', $id);
    
    $result = $this->dao->get();
    
    if($result) {
      $output = $result->result();
      osc_cache_set($cache_key, $output, cap_cache_ttl());
      return $output;
    }

    osc_cache_set($cache_key, array(), cap_cache_ttl());
    return array();
  } 
  
  return array();
}



// COUNT CATEGORIES OF TYPE
public function countTypeCategories($id) {
  if($id <= 0) { return 0; }
  
  $this->dao->select('count(*) as i_count');
  $this->dao->from($this->getTable_type_category());
  
  $this->dao->where('fk_i_type_id', $id);
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['i_count']) && $data['i_count'] > 0) {
      return (int)$data['i_count'];
    }
  }

  return 0;
}


// COUNT MAKES OF TYPE
public function countTypeMakes($id) {
  if($id <= 0) { return 0; }
  
  $this->dao->select('count(*) as i_count');
  $this->dao->from($this->getTable_make());
  
  $this->dao->where('fk_i_type_id', $id);
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['i_count']) && $data['i_count'] > 0) {
      return (int)$data['i_count'];
    }
  }

  return 0;
}


// GET PROPERTIY ROW
public function getPropertyRow($id) {
  if($id <= 0) { return false; }
  
  $this->dao->select();
  $this->dao->from($this->getTable_property());
  $this->dao->where('pk_i_id', $id);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }
  
  return false;
}


// GET PROPERTIES ROWS
public function getPropertiesRows() {
  $this->dao->select();
  $this->dao->from($this->getTable_property());

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// GET PROPERTIES KEYS THOSE ARE PART OF GROUP
public function getPropertiesGroups() {
  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getPropertiesGroups');
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
    
  } else {
    $this->dao->select('s_group_list');
    $this->dao->from($this->getTable_property());
    $this->dao->where('coalesce(s_group_list, "") <> ""');

    $result = $this->dao->get();
    
    if($result) {
      $data = $result->result();
      $output = array();
      
      if(is_array($data) && count($data) > 0) {
        foreach($data as $row) {
          $grps = explode(',', str_replace('_', '-', $row['s_group_list']));
          $output = $output + $grps;
        }
      }

      $output = array_filter(array_unique($output));
      osc_cache_set($cache_key, $output, cap_cache_ttl());
      return $output;
    }
    
    osc_cache_set($cache_key, array(), cap_cache_ttl());
    return array();
  } 
  
  return array();
}


// GET PROPERTIES
public function getProperties($enabled = true, $type_id = -1, $with_type_details = false) {
  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getProperties::' . (string)$enabled . (string)$type_id . (string)$with_type_details);
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
    
  } else {
    $this->dao->select('p.*, pl.*');
    $this->dao->from($this->getTable_property() . ' as p');
    $this->dao->join($this->getTable_property_locale() . ' as pl', '(pl.fk_i_property_id = p.pk_i_id AND pl.fk_c_locale_code = "' . cap_get_locale() . '")', 'LEFT OUTER');

    if($enabled) {
      $this->dao->where('p.b_enabled', 1);
    }

    if((int)$type_id >= 0) {
      $in_query = sprintf('SELECT fk_i_property_id FROM %s WHERE fk_i_type_id in (0,%d)', $this->getTable_property_type(),(int)$type_id);
      $this->dao->where(sprintf('p.pk_i_id in (%s)', $in_query));
    }

    $this->dao->orderby('p.i_order ASC, pl.s_name ASC', '');

    $result = $this->dao->get();
    
    if($result) {
      $data = $result->result();
      $locales = $this->getPropertiesLocales();
      $values = $this->getPropertiesValues(true, $type_id);
      $types_all = array();
      $types_prop_all = $this->getPropertiesTypes();
      $prop_types_categories = $this->getPropertiesTypesCategories();
      
      if($with_type_details) {
        $types_all = $this->getTypes(false);
      }
      
      $output = array();
      
      if(is_array($data) && count($data) > 0) {
        foreach($data as $dat) {
          $prop_types = array();
          $key = $dat['pk_i_id'];
          
          if(isset($types_prop_all[$key]) && is_array($types_prop_all[$key]) && count($types_prop_all[$key]) > 0) {
            if($with_type_details) {
              foreach($types_prop_all[$key] as $prop_id) {
                if(isset($types_all[$prop_id])) {
                  $prop_types[$prop_id] = $types_all[$prop_id];
                } else if($prop_id == 0) {
                  $prop_types[$prop_id] = array('pk_i_id' => 0, 's_name' => __('All types', 'car_attr_pro'));
                }
              }
            } else {
              $prop_types = $types_prop_all[$key];
            }
          }

          
          $output[$key] = $dat;
          $output[$key]['locales'] = isset($locales[$key]) ? $locales[$key] : array();
          $output[$key]['values'] = isset($values[$key]) ? $values[$key] : array();
          
          $val_count = 0;
          $vals_by_id = array();
          
          if(count($output[$key]['values']) > 0) {
            foreach($output[$key]['values'] as $type_vals) {
              $val_count += count(is_array($type_vals) ? $type_vals : array());
              
              if(is_array($type_vals) && count($type_vals) > 0) {
                foreach($type_vals as $tval) {
                  $tkey = ($tval['s_id'] <> '' ? $tval['s_id'] : $tval['pk_i_id']);
                  //$vals_by_id[$tkey] = $tval;   // no need for whole value array
                  $vals_by_id[$tkey] = $tval['pk_i_id'];
                }
              }
            }
          }
          
          $output[$key]['values_key_by_id'] = $vals_by_id;
          $output[$key]['values_types_count'] = count($output[$key]['values']);
          $output[$key]['values_count'] = $val_count;

          $output[$key]['types'] = $prop_types;
          //$output[$key]['categories'] = array_filter(array_unique($prop_types_categories), 'strlen');
          
          if(isset($prop_types_categories[$key]) && is_array($prop_types_categories[$key])) {
            $output[$key]['categories'] = $prop_types_categories[$key];
          } else {
            $output[$key]['categories'] = array();
          }
        }
      }
      
      osc_cache_set($cache_key, $output, cap_cache_ttl());
      return $output;
    }
    
    osc_cache_set($cache_key, array(), cap_cache_ttl());
    return array();
  } 
  
  return array();
}


// GET LOCALES OF ALL PROPERTIES
public function getPropertiesLocales() {
  $this->dao->select();
  $this->dao->from($this->getTable_property_locale());

  $result = $this->dao->get();
  $array = array();

  if($result) {
    $locales = $result->result();

    if(is_array($locales) && count($locales) > 0) {
      foreach($locales as $l) {
        $array[$l['fk_i_property_id']][$l['fk_c_locale_code']] = array(
          's_name' => $l['s_name'], 
          's_name_alt' => $l['s_name_alt'], 
          's_description' => $l['s_description']
        );
      }
    }
  }

  return $array;
}


// GET TYPES OF ALL PROPERTIES
public function getPropertiesTypes() {
  $this->dao->select();
  $this->dao->from($this->getTable_property_type());

  $result = $this->dao->get();
  $output = array();

  if($result) {
    $data = $result->result();

    if(is_array($data) && count($data) > 0) {
      foreach($data as $d) {
        $output[$d['fk_i_property_id']][] = $d['fk_i_type_id'];
      }
    }
  }

  return $output;
}

// GET TYPES OF ALL PROPERTIES
public function getPropertiesTypesCategories() {
  $this->dao->select('DISTINCT t.fk_i_property_id, CASE WHEN t.fk_i_type_id = 0 THEN 0 ELSE c.fk_i_category_id END as fk_i_category_id');
  $this->dao->from($this->getTable_property_type() . ' as t');
  $this->dao->join($this->getTable_type_category() . ' as c', '(t.fk_i_type_id = c.fk_i_type_id)', 'LEFT OUTER');

  $this->dao->orderby('t.fk_i_property_id ASC, c.fk_i_category_id ASC', '');
  
  $result = $this->dao->get();
  $output = array();

  if($result) {
    $data = $result->result();

    if(is_array($data) && count($data) > 0) {
      foreach($data as $d) {
        if(!isset($output[$d['fk_i_property_id']])) {
          $output[$d['fk_i_property_id']] = array();
        }
        
        if($d['fk_i_category_id'] !== null) {
          $output[$d['fk_i_property_id']][] = $d['fk_i_category_id'];
        }
      }
    }
  }

  return $output;
}



// GET PROPERTY TYPES
public function getPropertyTypes($id) {
  if($id <= 0) { return array(); }
  
  $this->dao->select();
  $this->dao->from($this->getTable_property_type());
  
  $this->dao->where('fk_i_property_id', $id);
  
  $result = $this->dao->get();
  
  if($result) {
    return $result->result();
  }

  return array();
}



// GET PROPERTIES
public function getPropertiesValues($enabled = true, $type_id = -1) {
  $this->dao->select('pv.*, pvl.*');
  $this->dao->from($this->getTable_property_value() . ' as pv');
  $this->dao->join($this->getTable_property_value_locale() . ' as pvl', '(pvl.fk_i_property_value_id = pv.pk_i_id AND pvl.fk_c_locale_code = "' . cap_get_locale() . '")', 'LEFT OUTER');

  if($enabled) {
    $this->dao->where('pv.b_enabled', 1);
  }

  // if((int)$type_id >= 0) {
    // $in_query = sprintf('SELECT fk_i_property_value_id FROM %s WHERE fk_i_type_id in (0,%d)', $this->getTable_property_value_type(),(int)$type_id);
    // $this->dao->where(sprintf('pv.pk_i_id in (%s)', $in_query));
  // }
  
  if((int)$type_id >= 0) {
    $this->dao->where('pv.fk_i_type_id', (int)$type_id);
  }

  $this->dao->orderby('pv.i_order ASC, pvl.s_name ASC', '');

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    $locales = $this->getPropertyValuesLocales();
    $output = array();
    
    if(is_array($data) && count($data) > 0) {
      foreach($data as $dat) {
        $output[$dat['fk_i_property_id']][$dat['fk_i_type_id']][$dat['pk_i_id']] = $dat;
        $output[$dat['fk_i_property_id']][$dat['fk_i_type_id']][$dat['pk_i_id']]['locales'] = isset($locales[$dat['pk_i_id']]) ? $locales[$dat['pk_i_id']] : array();
      }
    }
    
    return $output;
  }

  return array();
}


// GET GET MAX PROPERTY VALUE ID
public function getMaxPropertyValueId() {
  $this->dao->select('max(pk_i_id) as i_max_id');
  $this->dao->from($this->getTable_property_value());
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['i_max_id']) && $data['i_max_id'] > 0) {
      return (int)$data['i_max_id'];
    }
  }

  return 0;
}


// GET ALL PROPERTY VALUES LOCALES
public function getPropertyValuesLocales() {
  $this->dao->select();
  $this->dao->from($this->getTable_property_value_locale());

  $result = $this->dao->get();
  $array = array();

  if($result) {
    $locales = $result->result();

    if(is_array($locales) && count($locales) > 0) {
      foreach($locales as $l) {
        $array[$l['fk_i_property_value_id']][$l['fk_c_locale_code']] = array(
          's_name' => $l['s_name'], 
          's_description' => $l['s_description']
        );
      }
    }
  }

  return $array;
}




// GET SECTION ROW
public function getSectionRow($id) {
  if($id <= 0) { return false; }
  
  $this->dao->select();
  $this->dao->from($this->getTable_section());
  $this->dao->where('pk_i_id', $id);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }
  
  return false;
}


// GET SECTIONS ROWS
public function getSectionsRows() {
  $this->dao->select();
  $this->dao->from($this->getTable_section());

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// GET SECTIONS
public function getSections($enabled = true, $type_id = -1, $with_type_details = false, $location = '') {
  $cache_key = md5(osc_base_url() . 'Plugin::CarAttributesPro::getSections::' . (string)$enabled . (string)$type_id . (string)$with_type_details . (string)$location);
  $found = null;
  $cache = osc_cache_get($cache_key, $found);

  if(cap_cache_enabled() && $cache !== false) {
    return $cache;
    
  } else {
    $this->dao->select('s.*, sl.*');
    $this->dao->from($this->getTable_section() . ' as s');
    $this->dao->join($this->getTable_section_locale() . ' as sl', '(sl.fk_i_section_id = s.pk_i_id AND sl.fk_c_locale_code = "' . cap_get_locale() . '")', 'LEFT OUTER');

    if($enabled) {
      $this->dao->where('s.b_enabled', 1);
    }

    if((int)$type_id >= 0) {
      $in_query = sprintf('SELECT fk_i_section_id FROM %s WHERE fk_i_type_id in (0,%d)', $this->getTable_section_type(),(int)$type_id);
      $this->dao->where(sprintf('s.pk_i_id in (%s)', $in_query));
    }
    
    // item, publish, widget
    if($location != '') {
      $this->dao->where('concat(",", s.s_section_type, ",") like "%,' . $location . ',%"');
    }

    $this->dao->orderby('s.i_order ASC, sl.s_name ASC', '');

    $result = $this->dao->get();
    
    if($result) {
      $data = $result->result();
      $locales = $this->getSectionsLocales();
      $types_all = array();
      $types_prop_all = $this->getSectionsTypes();

      if($with_type_details) {
        $types_all = $this->getTypes(false);
      }
      
      $output = array();
      
      if(is_array($data) && count($data) > 0) {
        foreach($data as $dat) {
          $section_types = array();
          $key = $dat['pk_i_id'];
          
          if(isset($types_prop_all[$key]) && is_array($types_prop_all[$key]) && count($types_prop_all[$key]) > 0) {
            if($with_type_details) {
              foreach($types_prop_all[$key] as $type_id) {
                if(isset($types_all[$type_id])) {
                  $section_types[$type_id] = $types_all[$type_id];
                } else if($type_id == 0) {
                  $section_types[$type_id] = array('pk_i_id' => 0, 's_name' => __('All types', 'car_attr_pro'));
                }
              }
            } else {
              $section_types = $types_prop_all[$key];
            }
          }

          
          $output[$key] = $dat;
          $output[$key]['locales'] = isset($locales[$key]) ? $locales[$key] : array();
          $output[$key]['types'] = $section_types;
        }
      }
      
      osc_cache_set($cache_key, $output, cap_cache_ttl());
      return $output;
    }
    
    osc_cache_set($cache_key, array(), cap_cache_ttl());
    return array();
  } 
  
  return array();
}

// GET LOCALES OF ALL SECTIONS
public function getSectionsLocales() {
  $this->dao->select();
  $this->dao->from($this->getTable_section_locale());

  $result = $this->dao->get();
  $array = array();

  if($result) {
    $locales = $result->result();

    if(is_array($locales) && count($locales) > 0) {
      foreach($locales as $l) {
        $array[$l['fk_i_section_id']][$l['fk_c_locale_code']] = array(
          's_name' => $l['s_name'], 
          's_description' => $l['s_description'], 
          's_text_search' => $l['s_text_search'],
          's_text_publish' => $l['s_text_publish'],
          's_text_item' => $l['s_text_item']
        );
      }
    }
  }

  return $array;
}


// GET TYPES OF ALL SECTIONS
public function getSectionsTypes() {
  $this->dao->select();
  $this->dao->from($this->getTable_section_type());

  $result = $this->dao->get();
  $array = array();

  if($result) {
    $data = $result->result();

    if(is_array($data) && count($data) > 0) {
      foreach($data as $d) {
        $array[$d['fk_i_section_id']][] = $d['fk_i_type_id'];
      }
    }
  }

  return $array;
}


// GET SECTION TYPES
public function getSectionTypes($id) {
  if($id <= 0) { return array(); }
  
  $this->dao->select();
  $this->dao->from($this->getTable_section_type());
  
  $this->dao->where('fk_i_section_id', $id);
  
  $result = $this->dao->get();
  
  if($result) {
    return $result->result();
  }

  return array();
}





// UPDATE MAKE
public function updateMake($id, $data) {
  return $this->dao->update($this->getTable_make(), $data, array('pk_i_id' => $id));
}

// UPDATE MODEL
public function updateModel($id, $data) {
  return $this->dao->update($this->getTable_model(), $data, array('pk_i_id' => $id));
}

// UPDATE ITEM DATA
public function updateItemAttr($id, $data) {
  $data['fk_i_item_id'] = $id;
  return $this->dao->replace($this->getTable_item_attr(), $data);
}


// UPDATE TYPE ROW
public function updateTypeRow($id, $data) {
  return $this->dao->update($this->getTable_type(), $data, array('pk_i_id' => $id));
}

// UPDATE TYPE LOCALE ROW
public function updateTypeLocale($id, $locale, $data) {
  $this->dao->replace($this->getTable_type_locale(), $data); //, array('fk_i_type_id' => $id, 'fk_c_locale_code' => $locale));
}

// UPDATE TYPE CATEGORY ROW
public function updateTypeCategory($id, $data) {
  $this->deleteTypeCategories($id);
  
  if(is_array($data) && count($data) > 0) {
    foreach($data as $row) {
      $this->dao->insert($this->getTable_type_category(), $row);
    }
  }
}


// UPDATE TYPE DATA
public function updateType($id, $locale, $data_type, $data_locale, $data_category) {
  $this->updateTypeRow($id, $data_type);
  $this->updateTypeLocale($id, $locale, $data_locale);
  $this->updateTypeCategory($id, $data_category);
}





// UPDATE PROPERTY ROW
public function updatePropertyRow($id, $data) {
  return $this->dao->update($this->getTable_property(), $data, array('pk_i_id' => $id));
}

// UPDATE PROPERTY LOCALE ROW
public function updatePropertyLocale($id, $locale, $data) {
  $this->dao->replace($this->getTable_property_locale(), $data); //, array('fk_i_property_id' => $id, 'fk_c_locale_code' => $locale));
}

// UPDATE PROPERTY CATEGORY ROW
public function updatePropertyType($id, $data) {
  $this->deletePropertyTypes($id);
  
  if(is_array($data) && count($data) > 0) {
    foreach($data as $row) {
      $this->dao->insert($this->getTable_property_type(), $row);
    }
  }
}


// UPDATE PROPERTY DATA
public function updateProperty($id, $locale, $data_property, $data_locale, $data_type) {
  $this->updatePropertyRow($id, $data_property);
  $this->updatePropertyLocale($id, $locale, $data_locale);
  $this->updatePropertyType($id, $data_type);
}


// UPDATE PROPERTY VALUE ROW
public function updatePropertyValueRow($id, $data) {
  return $this->dao->update($this->getTable_property_value(), $data, array('pk_i_id' => $id));
}


// UPDATE PROPERTY VALUE LOCALE ROW
public function updatePropertyValueLocale($id, $locale, $data) {
  $this->dao->replace($this->getTable_property_value_locale(), $data); //, array('fk_i_property_value_id' => $id, 'fk_c_locale_code' => $locale));
}

// UPDATE PROPERTY VALUE DATA
public function updatePropertyValue($id, $locale, $data_property_value, $data_locale) {
  $this->updatePropertyValueRow($id, $data_property_value);
  $this->updatePropertyValueLocale($id, $locale, $data_locale);
}


// DELETE MAKE
public function deleteMake($id) {
  return $this->dao->delete($this->getTable_make(), array('pk_i_id' => $id));
}

// DELETE MODEL
public function deleteModel($id) {
  return $this->dao->delete($this->getTable_model(), array('pk_i_id' => $id));
}

// DELETE MODELS BY MAKE
public function deleteModelsByMakeId($id) {
  return $this->dao->delete($this->getTable_model(), array('fk_i_make_id' => $id));
}

// DELETE ITEM ATTR
public function deleteItemAttr($id) {
  return $this->dao->delete($this->getTable_item_attr(), array('fk_i_item_id' => $id));
}

// DELETE TYPE
public function deleteType($id) {
  $this->dao->delete($this->getTable_type_category(), array('fk_i_type_id' => $id));
  $this->dao->delete($this->getTable_type_locale(), array('fk_i_type_id' => $id));
  return $this->dao->delete($this->getTable_type(), array('pk_i_id' => $id));
}

public function deleteTypeCategories($id) {
  return $this->dao->delete($this->getTable_type_category(), array('fk_i_type_id' => $id));
}

public function deletePropertyTypes($id) {
  return $this->dao->delete($this->getTable_property_type(), array('fk_i_property_id' => $id));
}


// DELETE PROPERTY
public function deleteProperty($id) {
  $this->deletePropertyValues($id);
  $this->dao->delete($this->getTable_property_type(), array('fk_i_property_id' => $id));
  $this->dao->delete($this->getTable_property_locale(), array('fk_i_property_id' => $id));
  return $this->dao->delete($this->getTable_property(), array('pk_i_id' => $id));
}


// DELETE PROPERTY VALUE
public function deletePropertyValue($id) {
  $this->dao->delete($this->getTable_property_value_locale(), array('fk_i_property_value_id' => $id));
  return $this->dao->delete($this->getTable_property_value(), array('pk_i_id' => $id));
}


// DELETE PROPERTY VALUES (ALL)
public function deletePropertyValues($prop_id) {
  return $this->dao->delete($this->getTable_property_value(), array('fk_i_property_id' => $prop_id));
}



public function updateMakeEnabled($id, $enabled = 1) {
  return $this->dao->update($this->getTable_make(), array('b_enabled' => $enabled), array('pk_i_id' => $id));
}

public function updateModelEnabled($id, $enabled = 1) {
  return $this->dao->update($this->getTable_model(), array('b_enabled' => $enabled), array('pk_i_id' => $id));
}

public function updateCategoryType($id, $type_id = NULL) {
  return $this->dao->replace($this->getTable_type_category(), array('fk_i_category_id' => $id, 'fk_i_type_id' => $type_id), array('fk_i_category_id' => $id));
}



// UPDATE SECTION ROW
public function updateSectionRow($id, $data) {
  return $this->dao->update($this->getTable_section(), $data, array('pk_i_id' => $id));
}

// UPDATE SECTION LOCALE ROW
public function updateSectionLocale($id, $locale, $data) {
  $this->dao->replace($this->getTable_section_locale(), $data);
}

// UPDATE SECTION TYPE ROW
public function updateSectionType($id, $data) {
  $this->deleteSectionTypes($id);
  
  if(is_array($data) && count($data) > 0) {
    foreach($data as $row) {
      $this->dao->insert($this->getTable_section_type(), $row);
    }
  }
}


// UPDATE SECTION DATA
public function updateSection($id, $locale, $data_section, $data_locale, $data_type) {
  $this->updateSectionRow($id, $data_section);
  $this->updateSectionLocale($id, $locale, $data_locale);
  $this->updateSectionType($id, $data_type);
}


// DELETE 
public function deleteSectionTypes($id) {
  return $this->dao->delete($this->getTable_section_type(), array('fk_i_section_id' => $id));
}


// DELETE SECTION
public function deleteSection($id) {
  $this->dao->delete($this->getTable_section_type(), array('fk_i_section_id' => $id));
  $this->dao->delete($this->getTable_section_locale(), array('fk_i_section_id' => $id));
  return $this->dao->delete($this->getTable_section(), array('pk_i_id' => $id));
}




public function insertEmptyMake($type_id) {
  $this->dao->insert($this->getTable_make(), array('fk_i_type_id' => $type_id));
  return $this->dao->insertedId();
}

public function insertEmptyModel($make_id) {
  $this->dao->insert($this->getTable_model(), array('fk_i_make_id' => $make_id));
  return $this->dao->insertedId();
}

public function insertType($data) {
  $this->dao->insert($this->getTable_type(), $data);
  return $this->dao->insertedId();
}

public function insertProperty($data) {
  $this->dao->insert($this->getTable_property(), $data);
  return $this->dao->insertedId();
}

public function insertSection($data) {
  $this->dao->insert($this->getTable_section(), $data);
  return $this->dao->insertedId();
}


public function insertPropertyValueLocale($value_id, $locale, $name) {
  $this->dao->insert($this->getTable_property_value_locale(), array(
    'fk_i_property_value_id' => $value_id,
    'fk_c_locale_code' => $locale,
    's_name' => $name
  ));
  
  $val_id = $this->dao->insertedId();
}


public function insertPropertyValue($prop_id, $type_id, $id, $locale, $name, $order = NULL) {
  $data = array(
    'fk_i_property_id' => $prop_id,
    'fk_i_type_id' => $type_id,
    's_id' => $id
  );
  
  if($order !== NULL) {
    $data['i_order'] = $order;
  }
  
  $this->dao->insert($this->getTable_property_value(), $data);
  
  $val_id = $this->dao->insertedId();
  
  $this->insertPropertyValueLocale($val_id, $locale, $name);
  return $val_id;
}





// GET LIST OF TABLE COLUMNS
//SHOW COLUMNS FROM City;

public function getTableColumns($table) {
  $this->dao->select();
  $this->dao->from('INFORMATION_SCHEMA.COLUMNS');
  $this->dao->where('table_name', $table);
  
  $result = $this->dao->get();

  if($result) {
    $data = $result->result();
    return $data;
  }

  return array();
}


// UPDATE ITEM ATTR TABLE STRUCTURE - RECONCILIATION
public function updateItemAttrStructure($is_installation = false) {
  $banned_ids = cap_banned_properties();
  $banned_cols = cap_banned_properties('dbcol');
  $checked = array();

  $cdata = $this->getTableColumns($this->getTable_item_attr());
  $props = $this->getPropertiesRows();
  $props_fresh = $this->getProperties(false);
  
  $props_db_col = array();

  if(is_array($props) && count($props) > 0) {
    foreach($props as $prop) {
      $props_db_col[$prop['pk_i_id']] = cap_get_prop_db_column($prop['pk_i_id'], $prop['s_data_type'], false, $props_fresh);
    }
  }
  
  // First check every column of table if underlying property exists
  if(is_array($cdata) && count($cdata) > 0) {
    foreach($cdata as $col) {
      $colname = isset($col['COLUMN_NAME']) ? $col['COLUMN_NAME'] : '';
      
      if($colname != '' && strlen($colname) >= 5 && !in_array($colname, $banned_cols)) {
        $found = false;
        
        // Check if this column is generated by some property
        if(is_array($props) && count($props) > 0) {
          foreach($props as $prop) {
            $id = trim(osc_sanitizeString($prop['s_id']));

            if($id != '' && $prop['s_group_list'] == '') {
              // $colname_prop = cap_get_prop_db_column($prop['pk_i_id'], $prop['s_data_type'], false);
              $colname_prop = $props_db_col[$prop['pk_i_id']];
  
              if($colname == $colname_prop) {
                $found = true;
                break;
              }
            }
          }
        }
  
        // Column not found, drop it
        if($found === false) {
          $this->dao->query(sprintf('ALTER TABLE %s DROP COLUMN %s', $this->getTable_item_attr(), $colname));
        }
      }
    }
  }

  // Now check if every property has column
  if(is_array($props) && count($props) > 0) {
    foreach($props as $prop) {
      if($prop['s_group_list'] != '') {
        continue;
      }
      
      $id = trim(osc_sanitizeString($prop['s_id']));
      
      if($id != '' && strlen($id) >= 3 && !in_array($id, $banned_ids)) {
        // $colname = cap_get_prop_db_column($prop['pk_i_id'], $prop['s_data_type'], false);
        //echo $id . ' - ' . $prop['s_data_type'] . ', ';
        $colname = $props_db_col[$prop['pk_i_id']];

        if($colname != '' && strlen($colname) >= 5 && !in_array($colname, $banned_cols)) {
          $found = false;
          $alter = false;
          
          // Check each table column if we find match
          if(is_array($cdata) && count($cdata) > 0) {
            foreach($cdata as $col) {
              $colname_db = isset($col['COLUMN_NAME']) ? $col['COLUMN_NAME'] : '';
              $collength_db = isset($col['CHARACTER_MAXIMUM_LENGTH']) ? $col['CHARACTER_MAXIMUM_LENGTH'] : '';
              
              if($colname_db == $colname) {
                $found = true;
                
                // For varchar, check if length match
                if(substr($prop['s_data_type'], 0, 7) == 'VARCHAR') {
                  $prop_type_ = explode('_', $prop['s_data_type']);
                  $prop_length = isset($prop_type_[1]) ? (int)$prop_type_[1] : 64;

                  if($collength_db != $prop_length) {
                    $db_data_type = 'VARCHAR(' . $prop_length . ')';
                    $this->dao->query(sprintf('ALTER TABLE %s MODIFY COLUMN %s %s', $this->getTable_item_attr(), $colname, $db_data_type));
                  }
                }
                
                break;
              }
            }
          }
          
          // Column not found, add it
          if($found === false) {
            $db_data_type = strtoupper($prop['s_data_type']);
            
            if(substr($db_data_type, 0, 7) == 'VARCHAR') {
              $db_data_type_ = explode('_', $db_data_type);
              $prop_length = (isset($db_data_type_[1]) ? (int)$db_data_type_[1] : 64);
              $db_data_type = 'VARCHAR(' . $prop_length . ')';
            } else if($db_data_type == 'DECIMAL') {
              $db_data_type = 'DECIMAL(10,3)';
            } else if($db_data_type == 'FK_INTEGER') {
              $db_data_type = 'INTEGER';
            }
            
            $this->dao->query(sprintf('ALTER TABLE %s ADD %s %s DEFAULT NULL', $this->getTable_item_attr(), $colname, $db_data_type));
          }
        } else {
          if(!$is_installation && 1==2) {
            osc_add_flash_error_message(sprintf(__('Column name invalid: %s', 'car_attr_pro'), ($colname <> '' ? $colname : __('-blank-', 'car_attr_pro'))), 'admin');
          }
        }
      } else {
        if(!$is_installation && 1==2) {
          osc_add_flash_error_message(sprintf(__('Property ID invalid: %s', 'car_attr_pro'), $id), 'admin');
        }
      }
    }
  }
}


}
?>