SET FOREIGN_KEY_CHECKS=0;


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_type;
CREATE TABLE /*TABLE_PREFIX*/t_cap_type (
  pk_i_id INT NOT NULL AUTO_INCREMENT,
  s_id VARCHAR(20),
  s_icon VARCHAR(5000),
  s_color VARCHAR(20),
  i_order SMALLINT DEFAULT 9999,
  b_enabled BOOLEAN DEFAULT 1,

  PRIMARY KEY (pk_i_id)
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_type_locale;
CREATE TABLE /*TABLE_PREFIX*/t_cap_type_locale (
  fk_i_type_id INT NOT NULL,
  fk_c_locale_code CHAR(5) NOT NULL,
  s_name VARCHAR(255),
  s_description VARCHAR(2000),
  s_slug VARCHAR(255),

  PRIMARY KEY (fk_i_type_id,fk_c_locale_code),
  FOREIGN KEY (fk_i_type_id) REFERENCES /*TABLE_PREFIX*/t_cap_type (pk_i_id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_type_category;
CREATE TABLE /*TABLE_PREFIX*/t_cap_type_category (
  fk_i_type_id INT NOT NULL,
  fk_i_category_id INT(10) UNSIGNED NOT NULL,

  PRIMARY KEY (fk_i_type_id, fk_i_category_id)
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_section;
CREATE TABLE /*TABLE_PREFIX*/t_cap_section (
  pk_i_id INT NOT NULL AUTO_INCREMENT,
  s_id VARCHAR(20),
  s_icon VARCHAR(5000),
  s_section_type VARCHAR(20),
  s_item_style VARCHAR(20),
  s_widget_style VARCHAR(20),
  i_order SMALLINT DEFAULT 9999,
  b_enabled BOOLEAN DEFAULT 1,

  PRIMARY KEY (pk_i_id)
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_section_locale;
CREATE TABLE /*TABLE_PREFIX*/t_cap_section_locale (
  fk_i_section_id INT NOT NULL,
  fk_c_locale_code CHAR(5) NOT NULL,
  s_name VARCHAR(255),
  s_description VARCHAR(255),
  s_text_search VARCHAR(255),
  s_text_publish VARCHAR(255),
  s_text_item VARCHAR(255),
  
  PRIMARY KEY (fk_i_section_id,fk_c_locale_code),
  FOREIGN KEY (fk_i_section_id) REFERENCES /*TABLE_PREFIX*/t_cap_section (pk_i_id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_section_type;
CREATE TABLE /*TABLE_PREFIX*/t_cap_section_type (
  fk_i_section_id INT NOT NULL,
  fk_i_type_id INT NOT NULL DEFAULT 0,

  PRIMARY KEY (fk_i_section_id, fk_i_type_id)
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';



DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_make;
CREATE TABLE /*TABLE_PREFIX*/t_cap_make (
  pk_i_id INT NOT NULL AUTO_INCREMENT,
  fk_i_type_id INT NOT NULL,
  s_name VARCHAR(255),
  s_slug VARCHAR(255),
  s_icon VARCHAR(5000),
  s_color VARCHAR(20),
  s_section VARCHAR(10),
  i_order SMALLINT DEFAULT 9999,
  b_enabled BOOLEAN DEFAULT 1,

  PRIMARY KEY (pk_i_id),
  FOREIGN KEY (fk_i_type_id) REFERENCES /*TABLE_PREFIX*/t_cap_type (pk_i_id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_model;
CREATE TABLE /*TABLE_PREFIX*/t_cap_model (
  pk_i_id INT NOT NULL AUTO_INCREMENT,
  fk_i_make_id INT NOT NULL,
  s_name VARCHAR(255),
  s_slug VARCHAR(255),
  i_order SMALLINT DEFAULT 9999,
  b_enabled BOOLEAN DEFAULT 1,

  PRIMARY KEY (pk_i_id),
  FOREIGN KEY (fk_i_make_id) REFERENCES /*TABLE_PREFIX*/t_cap_make (pk_i_id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_property;
CREATE TABLE /*TABLE_PREFIX*/t_cap_property (
  pk_i_id INT NOT NULL AUTO_INCREMENT,
  s_id VARCHAR(50),
  s_icon VARCHAR(5000),
  s_style VARCHAR(30),
  b_multi_select BOOLEAN DEFAULT 0,
  b_range BOOLEAN DEFAULT 0,
  b_required BOOLEAN DEFAULT 0,
  s_section_widget VARCHAR(20),
  s_section_item VARCHAR(20),
  fk_i_section_widget_id INT,
  fk_i_section_item_id INT,
  fk_i_section_publish_id INT,
  s_data_type VARCHAR(20),
  s_measure_unit VARCHAR(20),
  s_group_list VARCHAR(500),
  s_search_engine VARCHAR(20),
  b_enabled BOOLEAN DEFAULT 1,
  b_enabled_widget BOOLEAN DEFAULT 1,
  s_size VARCHAR(20),
  b_enabled_search BOOLEAN DEFAULT 0,
  i_order SMALLINT DEFAULT 9999,

  PRIMARY KEY (pk_i_id),
  UNIQUE (s_id)
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_property_locale;
CREATE TABLE /*TABLE_PREFIX*/t_cap_property_locale (
  fk_i_property_id INT NOT NULL,
  fk_c_locale_code CHAR(5) NOT NULL,
  s_name VARCHAR(255),
  s_name_alt VARCHAR(255),
  s_description VARCHAR(1000),
  s_text_search VARCHAR(255),
  s_text_publish VARCHAR(255),

  PRIMARY KEY (fk_i_property_id,fk_c_locale_code),
  FOREIGN KEY (fk_i_property_id) REFERENCES /*TABLE_PREFIX*/t_cap_property (pk_i_id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_property_type;
CREATE TABLE /*TABLE_PREFIX*/t_cap_property_type (
  fk_i_property_id INT NOT NULL,
  fk_i_type_id INT NOT NULL DEFAULT 0,

  PRIMARY KEY (fk_i_property_id, fk_i_type_id)
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_property_value;
CREATE TABLE /*TABLE_PREFIX*/t_cap_property_value (
  pk_i_id INT NOT NULL AUTO_INCREMENT,
  fk_i_property_id INT NOT NULL,
  fk_i_type_id INT NOT NULL DEFAULT 0,
  s_id VARCHAR(50),
  s_icon VARCHAR(5000),
  i_order SMALLINT DEFAULT 9999,
  b_enabled BOOLEAN DEFAULT 1,

  PRIMARY KEY (pk_i_id),
  UNIQUE (fk_i_property_id,s_id),
  FOREIGN KEY (fk_i_property_id) REFERENCES /*TABLE_PREFIX*/t_cap_property (pk_i_id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_property_value_locale;
CREATE TABLE /*TABLE_PREFIX*/t_cap_property_value_locale (
  fk_i_property_value_id INT NOT NULL,
  fk_c_locale_code CHAR(5) NOT NULL,
  s_name VARCHAR(255),
  s_description VARCHAR(255),

  PRIMARY KEY (fk_i_property_value_id,fk_c_locale_code),
  FOREIGN KEY (fk_i_property_value_id) REFERENCES /*TABLE_PREFIX*/t_cap_property_value (pk_i_id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';





DROP TABLE IF EXISTS /*TABLE_PREFIX*/t_cap_item_attr;
CREATE TABLE /*TABLE_PREFIX*/t_cap_item_attr (
  fk_i_item_id INT UNSIGNED NOT NULL,
  fk_i_type_id INT DEFAULT NULL,
  fk_i_make_id INT DEFAULT NULL,
  fk_i_model_id INT DEFAULT NULL,
  
  fk_i_body_type_id INT DEFAULT NULL,
  fk_i_application_id INT DEFAULT NULL,
  fk_i_gearbox_id INT DEFAULT NULL,
  fk_i_drive_id INT DEFAULT NULL,
  fk_i_engine_type_id INT DEFAULT NULL,
  fk_i_fuel_id INT DEFAULT NULL,
  fk_i_seller_id INT DEFAULT NULL,
  fk_i_emission_id INT DEFAULT NULL,
  fk_i_color_id INT DEFAULT NULL,
  fk_i_color_type_id INT DEFAULT NULL,
  fk_i_aircondition_id INT DEFAULT NULL,
  fk_i_seats_material_id INT DEFAULT NULL,
  fk_i_headlights_type_id INT DEFAULT NULL,
  fk_i_cruise_control_id INT DEFAULT NULL,
  fk_i_connectivity_system_id INT DEFAULT NULL,
  fk_i_roof_id INT DEFAULT NULL,
  fk_i_alarm_type_id INT DEFAULT NULL,
  fk_i_interior_id INT DEFAULT NULL,
  fk_i_wheels_type_id INT DEFAULT NULL,
  fk_i_special_program_id INT DEFAULT NULL,

  i_year SMALLINT DEFAULT NULL,
  i_doors SMALLINT DEFAULT NULL,
  i_seats SMALLINT DEFAULT NULL,
  i_airbags SMALLINT DEFAULT NULL,
  i_axles SMALLINT DEFAULT NULL,
  i_max_payload INT DEFAULT NULL,
  i_max_weight INT DEFAULT NULL,
  i_max_capacity SMALLINT DEFAULT NULL,
  i_gears SMALLINT DEFAULT NULL,
  i_mileage INT DEFAULT NULL,
  i_engine_power INT DEFAULT NULL,
  i_engine_size INT DEFAULT NULL,
  
  d_fuel_cons_in_city DECIMAL(10,3) DEFAULT NULL,
  d_fuel_cons_out_city DECIMAL(10,3) DEFAULT NULL,
  d_fuel_cons_combined DECIMAL(10,3) DEFAULT NULL,
  
  s_keywords VARCHAR(64) DEFAULT NULL,
  s_vin VARCHAR(64) DEFAULT NULL,
  
  b_financing TINYINT(1) DEFAULT NULL,
  b_shipping TINYINT(1) DEFAULT NULL,
  b_warranty TINYINT(1) DEFAULT NULL,
  b_new TINYINT(1) DEFAULT NULL,
  b_no_accident TINYINT(1) DEFAULT NULL,
  b_registered TINYINT(1) DEFAULT NULL,
  b_first_owner TINYINT(1) DEFAULT NULL,
  b_serviced TINYINT(1) DEFAULT NULL,
  b_veteran TINYINT(1) DEFAULT NULL,
  b_tuning TINYINT(1) DEFAULT NULL,
  b_truck_homologation TINYINT(1) DEFAULT NULL,
  b_authorized_dealer TINYINT(1) DEFAULT NULL,
  b_vat_deduction TINYINT(1) DEFAULT NULL,
  b_vat_invoice TINYINT(1) DEFAULT NULL,
  b_leasing TINYINT(1) DEFAULT NULL,
  b_imported TINYINT(1) DEFAULT NULL,
  b_for_import TINYINT(1) DEFAULT NULL,
  b_bluetooth TINYINT(1) DEFAULT NULL,
  b_usb TINYINT(1) DEFAULT NULL,
  b_wireless_charge TINYINT(1) DEFAULT NULL,
  b_fast_charge TINYINT(1) DEFAULT NULL,
  b_navigation TINYINT(1) DEFAULT NULL,
  b_headup_display TINYINT(1) DEFAULT NULL,
  b_internet_access TINYINT(1) DEFAULT NULL,
  b_heated_seat_driver TINYINT(1) DEFAULT NULL,
  b_heated_seat_pass TINYINT(1) DEFAULT NULL,
  b_sport_seats TINYINT(1) DEFAULT NULL,
  b_ventilated_front_seats TINYINT(1) DEFAULT NULL,
  b_ventilated_back_seats TINYINT(1) DEFAULT NULL,
  b_memory_seat TINYINT(1) DEFAULT NULL,
  b_heated_windscreen TINYINT(1) DEFAULT NULL,
  b_independent_heat TINYINT(1) DEFAULT NULL,
  b_multifunct_stearing_wheel TINYINT(1) DEFAULT NULL,
  b_sport_stearing_wheel TINYINT(1) DEFAULT NULL,
  b_sport_suspension TINYINT(1) DEFAULT NULL,
  b_air_suspension TINYINT(1) DEFAULT NULL,
  b_keyless_entry TINYINT(1) DEFAULT NULL,
  b_keyless_go TINYINT(1) DEFAULT NULL,
  b_font_sensor TINYINT(1) DEFAULT NULL,
  b_rear_sensor TINYINT(1) DEFAULT NULL,
  b_park_assist TINYINT(1) DEFAULT NULL,
  b_rear_park_camera TINYINT(1) DEFAULT NULL,
  b_independent_park TINYINT(1) DEFAULT NULL,
  b_360_camera TINYINT(1) DEFAULT NULL,
  b_folding_mirrors TINYINT(1) DEFAULT NULL,
  b_blind_spot_assist TINYINT(1) DEFAULT NULL,
  b_lane_assist TINYINT(1) DEFAULT NULL,
  b_autonomous_stearing_sys TINYINT(1) DEFAULT NULL,
  b_isofix TINYINT(1) DEFAULT NULL,
  b_dynamic_corn_lights TINYINT(1) DEFAULT NULL,
  b_right_hand_drive TINYINT(1) DEFAULT NULL,
  b_abs TINYINT(1) DEFAULT NULL,
  b_esp TINYINT(1) DEFAULT NULL,
  b_asr TINYINT(1) DEFAULT NULL,
  b_central_lock TINYINT(1) DEFAULT NULL,
  b_tempomat TINYINT(1) DEFAULT NULL,
  b_distance_control_front TINYINT(1) DEFAULT NULL,
  b_stearing_booster TINYINT(1) DEFAULT NULL,
  b_towing_equip TINYINT(1) DEFAULT NULL,
  b_start_stop TINYINT(1) DEFAULT NULL,
  b_service_history TINYINT(1) DEFAULT NULL,
  b_fog_lights TINYINT(1) DEFAULT NULL,
  b_seasonal_tires TINYINT(1) DEFAULT NULL,
  b_new_tires TINYINT(1) DEFAULT NULL,
  b_immobilizer TINYINT(1) DEFAULT NULL,
  b_alarm TINYINT(1) DEFAULT NULL,
  b_trunk TINYINT(1) DEFAULT NULL,
  b_heated_grips TINYINT(1) DEFAULT NULL,
  b_radio TINYINT(1) DEFAULT NULL,
  b_rent TINYINT(1) DEFAULT NULL,
  b_cd_dvd TINYINT(1) DEFAULT NULL,
  b_parking_sensor TINYINT(1) DEFAULT NULL,
  b_electric_windows TINYINT(1) DEFAULT NULL,
  b_hook TINYINT(1) DEFAULT NULL,
  b_intarder TINYINT(1) DEFAULT NULL,
  b_sleeper_cabin TINYINT(1) DEFAULT NULL,
  b_onboard_computer TINYINT(1) DEFAULT NULL,
  b_kitchen TINYINT(1) DEFAULT NULL,
  b_refrigerator TINYINT(1) DEFAULT NULL,
  b_minibar TINYINT(1) DEFAULT NULL,
  b_airbag_driver TINYINT(1) DEFAULT NULL,
  b_airbag_passenger TINYINT(1) DEFAULT NULL,
  b_sliding_doors TINYINT(1) DEFAULT NULL,
  b_adjustable_suspension TINYINT(1) DEFAULT NULL,
  b_retarder TINYINT(1) DEFAULT NULL,
  b_tachograph TINYINT(1) DEFAULT NULL,
  b_cruise_control_system TINYINT(1) DEFAULT NULL,
  b_toilet TINYINT(1) DEFAULT NULL,
  b_tv_tuner TINYINT(1) DEFAULT NULL,
  b_elevator TINYINT(1) DEFAULT NULL,
  b_double_rear_wheels TINYINT(1) DEFAULT NULL,
  b_rain_sensor TINYINT(1) DEFAULT NULL,
  b_trunk_open TINYINT(1) DEFAULT NULL,
  b_trunk_close TINYINT(1) DEFAULT NULL,
  b_roof_rack TINYINT(1) DEFAULT NULL,
  b_tire_pressure TINYINT(1) DEFAULT NULL,
  b_bas TINYINT(1) DEFAULT NULL,
  
  PRIMARY KEY (fk_i_item_id),
  FOREIGN KEY (fk_i_item_id) REFERENCES /*TABLE_PREFIX*/t_item (pk_i_id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARACTER SET 'UTF8' COLLATE 'UTF8_GENERAL_CI';


SET FOREIGN_KEY_CHECKS=1;