<?php


// CONTROL WEB CONTACT FORM
function cfl_web_contact_check($params) {
  $email = strtolower(trim($params['reply_to']));
  $message = nl2br(strip_tags($params['body']));
  $error = '';
  $ip = cfl_get_ip();

  if($email == '') {
    return false;
  }
  
  if(!cfl_check_limit_hourly($email, $ip)) {
    $error = __('You have reached hourly limit for contact form usage!', 'contact_limit');
  } else if(!cfl_check_limit_daily($email, $ip)) {
    $error = __('You have reached daily limit for contact form usage!', 'contact_limit');
  } else if(!cfl_check_limit_weekly($email, $ip)) {
    $error = __('You have reached weekly limit for contact form usage!', 'contact_limit');
  } else if(!cfl_check_limit_monthly($email, $ip)) {
    $error = __('You have reached monthly limit for contact form usage!', 'contact_limit');
  } else if(cfl_check_block_words($message) !== false) {
    $error = sprintf(__('Your message contains blacklisted words (%s) and was not sent.', 'contact_limit'), cfl_check_block_words($message));
  } else if(cfl_check_ban_words($message) !== false) {
    $words = cfl_check_ban_words($message);
    $error = sprintf(__('Your message contains banned words (%s) and was not sent. For using these words in your message, you has been banned.', 'contact_limit'), $words);

    $ban_email = (cfl_param('ban_email') == 1 ? 1 : 0);
    $ban_ip = (cfl_param('ban_ip') == 1 ? 1 : 0);

    if(cfl_param('ban_ip_submask') == 1) {
      $ip_ = explode('.', $ip);
      
      if(count($ip_) == 4) {
        array_pop($ip_);
        $ip = implode('.', $ip_) . '.*';
      }
    }
    
    $ban_data = array(
      's_name' => sprintf(__('Contact Form Limit > Using banned words (%s)', 'contact_limit'), $words),
      's_email' => ($ban_email == 1 ? strtolower($email) : ''),
      's_ip' => ($ban_ip == 1 ? $ip : '')
    );
    
    if($ban_email == 1 || $ban_ip == 1) {
      BanRule::newInstance()->insert($ban_data);
    }
  }
  
  $data = array(
    's_email' => $email,
    's_ip' => cfl_get_ip(),
    's_type' => 'WEB',
    's_title' => $params['subject'],
    's_message' => osc_highlight($message, 1000),
    's_recipient' => $params['to'],
    's_error' => $error,
    'dt_date' => date('Y-m-d H:i:s')
  );
  
  if($error != '') {
    osc_add_flash_error_message($error);
    $mail_banned = cfl_manage_ban($email);
    
    if(!$mail_banned) {
      cfl_manage_ban(cfl_get_ip());
    }

    $data['s_status'] = 'BLOCKED';
    ModelCFL::newInstance()->insertLog($data);
    
    header('Location:' . osc_contact_url());
    exit;
  }
  
  $data['s_status'] = 'SENT';
  ModelCFL::newInstance()->insertLog($data);
}

osc_add_hook('pre_contact_post', 'cfl_web_contact_check', 2);



// CONTROL ITEM CONTACT FORM
function cfl_item_contact_check($params) {
  $item_id = $params['id'];
  $email = strtolower(trim($params['yourEmail']));
  $message = nl2br(strip_tags($params['message']));
  $error = '';
  $ip = cfl_get_ip();

  if($email == '') {
    return false;
  }
  
  if(!cfl_check_limit_hourly($email, $ip)) {
    $error = __('You have reached hourly limit for contact form usage!', 'contact_limit');
  } else if(!cfl_check_limit_daily($email, $ip)) {
    $error = __('You have reached daily limit for contact form usage!', 'contact_limit');
  } else if(!cfl_check_limit_weekly($email, $ip)) {
    $error = __('You have reached weekly limit for contact form usage!', 'contact_limit');
  } else if(!cfl_check_limit_monthly($email, $ip)) {
    $error = __('You have reached monthly limit for contact form usage!', 'contact_limit');
  } else if(cfl_check_block_words($message) !== false) {
    $error = sprintf(__('Your message contains blacklisted words (%s) and was not sent.', 'contact_limit'), cfl_check_block_words($message));
  } else if(cfl_check_ban_words($message) !== false) {
    $words = cfl_check_ban_words($message);
    $error = sprintf(__('Your message contains banned words (%s) and was not sent. For using these words in your message, you has been banned.', 'contact_limit'), $words);

    $ban_email = (cfl_param('ban_email') == 1 ? 1 : 0);
    $ban_ip = (cfl_param('ban_ip') == 1 ? 1 : 0);

    if(cfl_param('ban_ip_submask') == 1) {
      $ip_ = explode('.', $ip);
      
      if(count($ip_) == 4) {
        array_pop($ip_);
        $ip = implode('.', $ip_) . '.*';
      }
    }
    
    $ban_data = array(
      's_name' => sprintf(__('Contact Form Limit > Using banned words (%s)', 'contact_limit'), $words),
      's_email' => ($ban_email == 1 ? strtolower($email) : ''),
      's_ip' => ($ban_ip == 1 ? $ip : '')
    );
    
    if($ban_email == 1 || $ban_ip == 1) {
      BanRule::newInstance()->insert($ban_data);
    }
  }
  
  $item = Item::newInstance()->findByPrimaryKey($item_id);

  $data = array(
    's_email' => $email,
    's_ip' => cfl_get_ip(),
    's_type' => 'ITEM',
    'fk_i_item_id' => $item_id,
    's_title' => $item['s_title'],
    's_message' => osc_highlight($message, 1000),
    's_recipient' => $item['s_contact_email'],
    's_error' => $error,
    'dt_date' => date('Y-m-d H:i:s')
  );
  
  if($error != '') {
    osc_add_flash_error_message($error);
    $mail_banned = cfl_manage_ban($email);
    
    if(!$mail_banned) {
      cfl_manage_ban(cfl_get_ip());
    }
    
    $data['s_status'] = 'BLOCKED';
    ModelCFL::newInstance()->insertLog($data);
    
    header('Location:' . osc_item_url_ns($item_id));
    exit;
  }
  
  $data['s_status'] = 'SENT';
  ModelCFL::newInstance()->insertLog($data);
}

osc_add_hook('hook_email_item_inquiry', 'cfl_item_contact_check', 2);
osc_add_hook('hook_email_send_friend', 'cfl_item_contact_check', 2);


// MANAGE BAN FEATURES
function cfl_manage_ban($email) {
  $limit = (cfl_param('ban_limit') > 0 ? cfl_param('ban_limit') : 0);
  
  if($limit <= 0) {
    return false;
  }
  
  $limit_date = date('Y-m-d H:i:s', strtotime('-1 month'));
  $logs = ModelCFL::newInstance()->getLastLogs($email, '', '', $limit_date);
  
  $c = 1;
  $ban_status = array();
  
  // We are checking last XY logs. If there is sequence of blocked logs, user is banned
  if(is_array($logs) && count($logs) > 0) {
    foreach($logs as $log) {
      if($c <= $limit) {
        $ban_status[] = $log['s_status']; 
      } else {
        break;
      }
      
      $c++;
    }
  }
  
  // Check if all status were BLOCKED and we should ban user
  $ban = false;
  if(!in_array('SENT', $ban_status) && count($ban_status) >= $limit) {
    $ban = true;
  }
  
  
  // We should ban user
  if($ban === true) {
    $ban_email = (cfl_param('ban_email') == 1 ? 1 : 0);
    $ban_ip = (cfl_param('ban_ip') == 1 ? 1 : 0);
    
    // Email is actually IP, do not add ip into mail rules
    if(strpos($email, '@') === false) {
      $ban_email = 0;
    }

    $ip = cfl_get_ip();
    
    if(cfl_param('ban_ip_submask') == 1) {
      $ip_ = explode('.', $ip);
      
      if(count($ip_) == 4) {
        array_pop($ip_);
        $ip = implode('.', $ip_) . '.*';
      }
    }
    
    $data = array(
      's_name' => sprintf(__('Contact Form Limit > Reached blocked attempts limit (%d blocked mails)', 'contact_limit'), $limit),
      's_email' => ($ban_email == 1 ? strtolower($email) : ''),
      's_ip' => ($ban_ip == 1 ? $ip : '')
    );
    
    if($ban_email == 1 || $ban_ip == 1) {
      osc_add_flash_error_message(__('You have been banned due to enormous misuse of contact forms!', 'contact_limit'));
      BanRule::newInstance()->insert($data);
      return true;
    }
  }
  
  return false;
}


// CHECK BAN WORDS
function cfl_check_ban_words($message) {
  $words = cfl_param('ban_words');
  return cfl_check_words($message, $words);
}


// CHECK BAN WORDS
function cfl_check_block_words($message) {
  $words = cfl_param('block_words');
  return cfl_check_words($message, $words);
}

  
// RETURN WORDS THOSE ARE PART OF MESSAGE
function cfl_check_words($message, $words) {
  $words = array_map('trim', array_filter(explode(',', $words)));
  $message = ' ' . strtolower(trim($message)) . ' ';
  $found_words = array();

  if(trim($message) != '' && count($words) > 0) {
    foreach($words as $word) {
      if(strpos($message, $word) !== false) {
        $found_words[] = trim($word);
      }
    }
    
    if(count($found_words) > 0) {
      return implode(', ', $found_words);
    }
  }
  
  return false;
}


// GENERATE PAGINATION
function cfl_admin_paginate($file, $page_id, $per_page, $count_all, $class = '', $params = '') {
  $html = '';
  $page_id = (int)$page_id;
  $page_id = ($page_id <= 0 ? 1 : $page_id);
  $base_link = osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=' . $file . $params;

  if($per_page < $count_all) {
    $html .= '<div id="mb-pagination" class="' . $class . '">';
    $html .= '<div class="mb-pagination-wrap">';
    $html .= '<div>' . __('Page:', 'contact_limit') . '</div>';

    $pages = ceil($count_all/$per_page); 
    $page_actual = ($page_id == '' ? 1 : $page_id);

    if($pages > 6) {

      // Too many pages to list them all
      if($page_id == 1) { 
        $ids = array(1,2,3, $pages);

      } else if ($page_id > 1 && $page_id < $pages) {
        $ids = array(1,$page_id-1, $page_id, $page_id+1, $pages);

      } else {
        $ids = array(1, $page_id-2, $page_id-1, $page_id);
      }

      $old = -1;
      $ids = array_unique(array_filter($ids));

      foreach($ids as $i) {
        $url = $base_link . '&pageId=' . $i;
        
        if($old <> -1 && $old <> $i - 1) {
          $html .= '<span>&middot;&middot;&middot;</span>';
        }

        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="mb-active"' : '') . '>' . $i . '</a>';
        $old = $i;
      }

    } else {

      // List all pages
      for ($i = 1; $i <= $pages; $i++) {
        $url = $base_link . '&pageId=' . $i;
        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="mb-active"' : '') . '>' . $i . '</a>';
      }
    }

    $html .= '</div>';
    $html .= '</div>';
  }

  return $html;
}


// CHECK HOURLY LIMIT
function cfl_check_limit_hourly($email, $ip = '') {
  $limit = (cfl_param('per_hour') > 0 ? cfl_param('per_hour') : 0);
  $limit_date = date('Y-m-d H:i:s', strtotime('-1 hour'));
  
  return cfl_check_limit($email, $ip, $limit, $limit_date);
}

// CHECK DAILY LIMIT
function cfl_check_limit_daily($email, $ip = '') {
  $limit = (cfl_param('per_day') > 0 ? cfl_param('per_day') : 0);
  $limit_date = date('Y-m-d H:i:s', strtotime('-1 day'));
  
  return cfl_check_limit($email, $ip, $limit, $limit_date);
}

// CHECK WEEKLY LIMIT
function cfl_check_limit_weekly($email, $ip = '') {
  $limit = (cfl_param('per_week') > 0 ? cfl_param('per_week') : 0);
  $limit_date = date('Y-m-d H:i:s', strtotime('-7 day'));
  
  return cfl_check_limit($email, $ip, $limit, $limit_date);
}

// CHECK MONTHLY LIMIT
function cfl_check_limit_monthly($email, $ip = '') {
  $limit = (cfl_param('per_month') > 0 ? cfl_param('per_month') : 0);
  $limit_date = date('Y-m-d H:i:s', strtotime('-1 month'));

  return cfl_check_limit($email, $ip, $limit, $limit_date);
}


// CHECK LIMITS
function cfl_check_limit($email, $ip = '', $limit = 0, $limit_date = '') {
  if($limit > 0 && $limit_date != '') {
    $logs_email = ModelCFL::newInstance()->getLastLogs($email, 'SENT', '', $limit_date);
    
    if(is_array($logs_email) && count($logs_email) > 0 && count($logs_email) >= $limit) {
      return false;
    }
    
    if($ip != '') {
      $logs_ip = ModelCFL::newInstance()->getLastLogs($ip, 'SENT', '', $limit_date);

      if(is_array($logs_ip) && count($logs_ip) > 0 && count($logs_ip) >= $limit) {
        return false;
      }
    }
  }
  
  return true;
}


// GET CURRENT USER IP
function cfl_get_ip() {
  if(function_exists('osc_get_ip')) {
    return osc_get_ip();
  } else {
    if (isset($_SERVER["HTTP_CF_CONNECTING_IP"])) {
      $_SERVER['REMOTE_ADDR'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
      $_SERVER['HTTP_CLIENT_IP'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
    }
    
    $client = @$_SERVER['HTTP_CLIENT_IP'];
    $forward = @$_SERVER['HTTP_X_FORWARDED_FOR'];
    $remote = @$_SERVER['REMOTE_ADDR'];

    if(filter_var($client, FILTER_VALIDATE_IP)) {
      $ip = $client;
    } elseif(filter_var($forward, FILTER_VALIDATE_IP)) {
      $ip = $forward;
    } else {
      $ip = $remote;
    }

    return $ip;
  }
}


// CORE FUNCTIONS
function cfl_param($name) {
  return osc_get_preference($name, 'plugin-contact_limit');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}

// CATEGORIES LIST
function cfl_category_list() {
  $list = cfl_get_categories(Category::newInstance()->toTree());
  return $list;
}


function cfl_map_category_limit_options($type) {
  if($type == 'NONLOGGED') {
    return 'i_limit_nonlogged';
  } else if($type == 'PERSONAL') {
    return 'i_limit_personal';
  } else if($type == 'BUSINESS') {
    return 'i_limit_business';
  }
}


// GET MAIN CATEGORIES
function cfl_get_categories($categories) {
  $list = array();

  foreach($categories as $c) {
    $level = 1;
    $list[] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name'], 'level' => $level, 'fk_i_parent_id' => $c['fk_i_parent_id'] );
   
    if(isset($c['categories']) && is_array($c['categories'])) {
      $list = array_merge($list, cfl_get_subcategories($c['categories'], $level, $c['fk_i_parent_id']));
    }
  }

  return $list;
}


// GET SUBCATEGORIES
function cfl_get_subcategories($categories, $level = 0, $parent = 0) {
  $level++;
  $list = array();

  foreach($categories as $c) {
    if($level == 2) {
      $parent = $c['fk_i_parent_id'];
    }

    $list[] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name'], 'level' => $level, 'fk_i_parent_id' => $parent );
        
    if(isset($c['categories']) && is_array($c['categories'])) {
      $list = array_merge($list, cfl_get_subcategories($c['categories'], $level, $parent));
    }
  }

  return $list;
}


// CREATE CATEGORY TABS
function cfl_category_tabs( $level ) {
  $tab = '';

  if( $level == 2) {
    $tab = '<i class="fa fa-angle-right"></i>&nbsp;';
  } else if( $level == 3) {
    $tab = '&nbsp;&nbsp;<i class="fa fa-angle-right"></i><i class="fa fa-angle-right"></i>&nbsp;';
  } else if( $level == 4) {
    $tab = '&nbsp;&nbsp;&nbsp;&nbsp;<i class="fa fa-angle-right"></i><i class="fa fa-angle-right"></i><i class="fa fa-angle-right"></i>&nbsp;';
  }

  return $tab;
}



// CHECK IF RUNNING ON DEMO
function cfl_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}

?>