<?php
class ModelCFL extends DAO {
private static $instance;

public static function newInstance() {
  if(!self::$instance instanceof self) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_log() {
  return DB_TABLE_PREFIX.'t_cfl_log';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelCFL<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


// DO QUERIES ON VERSION UPDATE
public function versionUpdate() {
  $version = (cfl_param('version') <> '' ? cfl_param('cfl_param') : 100);    // v100 is initial

  // if($version == '' || $version < 101) { 
    // $this->import('contact_limit/model/struct_update_101.sql');
    // osc_set_preference('version', 101, 'plugin-contact_limit', 'INTEGER');
  // }
  
  osc_reset_preferences();
}


public function install($version = '') {
  if($version == '') {
    $this->import('contact_limit/model/struct.sql');
    osc_set_preference('version', 100, 'plugin-contact_limit', 'INTEGER');
  }
}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_log()));

  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-contact_limit'";
  $this->dao->query($query);
}


// GET LAST USER LOG
public function getLastLog($email, $status = '', $type = '') {
  $this->dao->select();
  $this->dao->from($this->getTable_log());
  $this->dao->where('s_email', $email);
  
  if($status != '') {
    $this->dao->where('s_status', $status);
  }
  
  if($type != '') {
    $this->dao->where('s_type', $type);
  }
  
  $this->dao->orderby('dt_date', 'DESC');
  $this->dao->limit(1);
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['s_email'])) {
      return $data;
    }
  }
  
  return false;
}


// GET LAST USER LOGS
public function getLastLogs($email, $status = '', $type = '', $date = '') {
  $this->dao->select();
  $this->dao->from($this->getTable_log());
  $this->dao->where('(s_email = "' . $email . '" OR s_ip = "' . $email . '")');
  
  if($status != '') {
    $this->dao->where('s_status', $status);
  }
  
  if($type != '') {
    $this->dao->where('s_type', $type);
  }
  
  if($date != '') {
    $this->dao->where('dt_date >= "' . $date . '"');
  }
  
  $this->dao->orderby('dt_date', 'DESC');

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// GET USER LOGS
public function getUserLogs($email) {
  $this->dao->select();
  $this->dao->from($this->getTable_log());
  $this->dao->where('s_email', $email);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// SEARCH LOGS
public function searchLogs($params, $only_count = false) {
  $selector = 'DISTINCT i.*';
  
  if($only_count === true) {
    $selector = 'count(DISTINCT i.pk_i_id) as i_count';
  }
  
  $this->dao->select($selector);
  $this->dao->from($this->getTable_log() . ' as i');

  if(isset($params['itemId']) && $params['itemId'] !== '') {
    $this->dao->like('i.fk_i_item_id', $params['itemId']);
  }
  
  if(isset($params['status']) && $params['status'] !== '') {
    $this->dao->where('i.s_status', $params['status']);
  }
  
  if(isset($params['type']) && $params['type'] !== '') {
    $this->dao->where('i.s_type', $params['type']);
  }
  
  if(isset($params['email']) && $params['email'] !== '') {
    $this->dao->like('i.s_email', $params['email']);
  }
  
  if(isset($params['error']) && $params['error'] !== '') {
    $this->dao->like('coalesce(i.s_error, "")', $params['error']);
  }
  
  if(isset($params['text']) && $params['text'] !== '') {
    $this->dao->like('concat(coalesce(i.s_title, ""), coalesce(i.s_message, ""), coalesce(i.s_recipient, ""))', $params['text']);
  }
  
  if(isset($params['ip']) && $params['ip'] !== '') {
    $this->dao->like('i.s_ip', $params['ip']);
  }

  if(isset($params['date']) && $params['date'] !== '') {
    $this->dao->like('i.dt_date', $params['date']);
  }
  

  if($only_count !== true) {
    // $limit[0] == page; $limit[1] == limit
    $page = (isset($params['pageId']) ? $params['pageId'] : 0);
    $per_page = (isset($params['per_page']) ? $params['per_page'] : 30);
    
    if($page > 0) {
      $this->dao->limit(($page-1)*$per_page, $per_page);
    } else if($per_page > 0) {
      $this->dao->limit($per_page);
    }


    if(isset($params['sort']) && $params['sort'] !== '') {
      if($params['sort'] == 'ID_ASC') {
        $this->dao->orderby('i.pk_i_id', 'ASC');
      } else if($params['sort'] == 'ID_DESC') {
        $this->dao->orderby('i.pk_i_id', 'DESC');
      } else if($params['sort'] == 'DATE_ASC') {
        $this->dao->orderby('i.dt_date', 'ASC');
      } else if($params['sort'] == 'DATE_DESC') {
        $this->dao->orderby('i.dt_date', 'DESC');
      }
    } else {
      $this->dao->orderby('i.dt_date', 'DESC');
    }
  }

  $result = $this->dao->get();
  
  if($result) {
    if($only_count === true) {
      $data = $result->row();
      return isset($data['i_count']) ? $data['i_count'] : 0;
    } else {
      return $result->result();
    }
  }

  return ($only_count ? 0 : array());
}


// INSERT NEW LOG
public function insertLog($data) {
  return $this->dao->insert($this->getTable_log(), $data);
}

// REMOVE OLD LOGS
public function cleanLogs($limit_date) {
  $this->dao->query(sprintf('DELETE FROM TABLE %s WHERE dt_date <= "%s"', $this->getTable_log(), $limit_date));
}


}
?>