<?php
class ModelEMT extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_page() {
  return DB_TABLE_PREFIX.'t_pages';
}

public function getTable_page_desc() {
  return DB_TABLE_PREFIX.'t_pages_description';
}


public function install() {
  $this->updateTemplateById('', file_get_contents(osc_base_path() . 'oc-content/plugins/email_template/model/templates/template1.php'));
  $this->updateTemplateById('', file_get_contents(osc_base_path() . 'oc-content/plugins/email_template/model/templates/template2.php'));
  $this->updateTemplateById('', file_get_contents(osc_base_path() . 'oc-content/plugins/email_template/model/templates/template3.php'));
}



public function uninstall() {
  $templates = $this->getTemplates();

  if($templates) {
    foreach($templates as $t) {
      Page::newInstance()->deleteByPrimaryKey($t['pk_i_id']);
    }
  }
}


// GET PLUGIN TEMPLATES
public function getTemplates() {
  $this->dao->select('*');
  $this->dao->from($this->getTable_page() . ' as p');
  $this->dao->where('p.s_internal_name like "emt_%"');

  $result = $this->dao->get();
  
  $j = 0;
  if($result) {
    $data = $result->result();

    if(count($data) > 0) {
      foreach($data as $d) {
        $data[$j]['locales'] = $this->getEmailText($d['pk_i_id']);
        $j++;
      }
    }

    return $data;
  }
  
  return false;
}


// GET PLUGIN TEMPLATE
public function getTemplateById($id) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_page() . ' as p');
  $this->dao->where('p.pk_i_id', $id);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    $data['locales'] = $this->getEmailText($id);
    return $data;
  }
  
  return false;
}


// UPDATE OR CREATE TEMPLATE
public function updateTemplateById($id = '', $desc = '') {
  $values_page = array(
    's_internal_name' => 'emt_template_' . $id,
    'b_indelible' => 1,
    'b_link' => 0,
    'dt_pub_date' => date("Y-m-d H:i:s") 
  );

  $values_desc = array(
    'fk_i_pages_id' => $id,
    'fk_c_locale_code' => emt_get_locale(),
    's_title' => 'emt_template_' . $id,
    's_text' => $desc
  );
  
  $where_desc = array(
    'fk_i_pages_id' => $id,
    'fk_c_locale_code' => emt_get_locale()
  );


  if($id > 0) {
    $page = $this->getTemplateById($id);

    if(isset($page['locales'][emt_get_locale()])) {
      $this->dao->update($this->getTable_page_desc(), $values_desc, $where_desc);
    } else {
      $this->dao->insert($this->getTable_page_desc(), $values_desc);
      $id = $this->dao->insertedId();
    }
  } else {
    $this->dao->insert($this->getTable_page(), $values_page);
    $id = $this->dao->insertedId();

    Params::setParam('templateId', $id);

    $where_desc['fk_i_pages_id'] = $id;
    $values_desc['fk_i_pages_id'] = $id;

    $this->dao->insert($this->getTable_page_desc(), $values_desc);
    return $id;
  }
}




// GET ATTRIBUTE LOCALES
public function getEmailText($page_id) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_page_desc());
  $this->dao->where('fk_i_pages_id', $page_id);

  $result = $this->dao->get();
  $array = array();

  if($result) {
    $locales = $result->result();

    if(count($locales) > 0) {
      foreach($locales as $l) {
        $array[$l['fk_c_locale_code']] = $l['s_text'];
      }
    }
  }

  return $array;
}


// REMOVE PAGE
public function deletePage($id) {
  $this->dao->delete($this->getTable_page_desc(), array('fk_i_pages_id' => $id));
  $this->dao->delete($this->getTable_page(), array('pk_i_id' => $id));
}




}
?>