<?php

// PREPARE CUSTOM CURRENCIES
function exr_get_custom_currencies() {
  $currencies = array_filter(explode(',', exr_param('custom_currencies'))); //;
  $output = array();
  
  if(is_array($currencies) && count($currencies) > 0) {
    foreach($currencies as $string) {
      $string_parts = explode('-', $string);
      
      $code = strtoupper(trim((string)$string_parts[0]));
      $symbol = (isset($string_parts[1]) ? trim((string)$string_parts[1]) : '');
      
      $output[$code] = $symbol;
    }
  }
  
  return $output;
}


// GET CURRENCY LIST
function exr_get_currency_codes($only_core = false) {
  $codes = array();
  $osc_currencies = ModelEXR::newInstance()->getCurrencies();
  $custom_currencies = exr_get_custom_currencies();
  
  if(is_array($osc_currencies) && count($osc_currencies) > 0) {
    foreach($osc_currencies as $currency) {
      $codes[] = $currency['pk_c_code'];
    }
  }
  
  if($only_core !== true) {
    if(is_array($custom_currencies) && count($custom_currencies) > 0) {
      foreach($custom_currencies as $code => $symbol) {
        $codes[] = $code;
      }
    }
  }
  
  $codes = array_filter(array_unique($codes));
  return $codes;
}


// ADD CONVERSIONS TO ITEM PRICE
function exr_add_prices($formatted_price) {
  if(exr_param('rate_to_item') == 1 && osc_item_price() != NULL && osc_item_price() != '' && osc_item_price() != 0) {
    $rates = exr_item_rates();
    $currencies = osc_get_currencies();
    $custom_currencies = exr_get_custom_currencies();
    $data = array();

    foreach($rates as $symbol => $rate) {
      $price = (osc_item_price()/1000000) * $rate;

      if(isset($custom_currencies[$symbol]) && $custom_currencies[$symbol] != '') {
        $symbol = $custom_currencies[$symbol];
      } else {
        $keys = array_keys(array_column($currencies, 'pk_c_code'), $symbol);
        $currency = array_map(function($k) use ($currencies){ return $currencies[$k];}, $keys);
        $currency = isset($currency[0]) ? $currency[0] : array();

        if(isset($currency['pk_c_code']) && $currency['s_description'] != '') {
          $symbol = $currency['s_description'];
        }
      }

      $currencyFormat = osc_locale_currency_format();
      $currencyFormat = str_replace('{NUMBER}', number_format($price, osc_locale_num_dec(), osc_locale_dec_point(), osc_locale_thousands_sep()), $currencyFormat);
      $currencyFormat = str_replace('{CURRENCY}', $symbol, $currencyFormat);
      $data[] = $currencyFormat;
    }

    return $formatted_price . ' <div class="MCtooltip"><i class="fa fa-caret-down"></i><span>'.implode("<br />", $data).'</span></div>';
  }

  return $formatted_price;
}


// ADD CONVERSIONS TO PREMIUM PRICE
function exr_add_prices_premium($formatted_price) {
  if(exr_param('rate_to_item') == 1 && osc_premium_price() != NULL && osc_premium_price() != '' && osc_premium_price() != 0) {
    $rates = exr_premium_rates();
    $data = array();

    foreach($rates as $symbol => $rate) {
      $price = (osc_premium_price()/1000000) * $rate;
      $symbol = $symbol;

      $currencyFormat = osc_locale_currency_format();
      $currencyFormat = str_replace('{NUMBER}', number_format($price, osc_locale_num_dec(), osc_locale_dec_point(), osc_locale_thousands_sep()), $currencyFormat);
      $currencyFormat = str_replace('{CURRENCY}', $symbol, $currencyFormat);
      $data[] = $currencyFormat;
    }

    return $formatted_price . ' <div class="MCtooltip"><i class="fa fa-caret-down"></i><span>'.implode("<br />", $data).'</span></div>';
  }
}


// PICK CORRECT FUNCTION
function exr_add_price_fix($formatted_price) {
  if(osc_item_price() > 0) {
    return exr_add_prices($formatted_price);
  } else {
    return exr_add_prices_premium($formatted_price);
  }
}

osc_add_filter('item_price', 'exr_add_price_fix');
osc_add_filter('premium_price', 'exr_add_price_fix');


// RETRO COMPATIBILITY WITH MCE
if(!function_exists('multicurrency_add_prices')) {
  function multicurrency_add_prices($formatted_price) {
    exr_add_prices($formatted_price);
  }
}


// GET RATE
function exr_get_rate($from, $to) {
  return ModelEXR::newInstance()->getRate($from, $to);
}


// GET RATES BASED ON FROM
function exr_get_rates($from) {
  return ModelEXR::newInstance()->getRate($from);
}


// GET ITEM RATES
function exr_item_rates() {
  $from = osc_item_currency();

  return ModelEXR::newInstance()->getRate($from);
}

// GET PREMIUM RATES
function exr_premium_rates() {
  $from = osc_premium_currency();

  return ModelEXR::newInstance()->getRate($from);
}

// CONVERT VALUE
function exr_convert($from, $to, $amount) {
  return exr_get_rate($from, $to) * $amount;
}



// GET RATES FROM API
function exr_feed_rate($from, $to, $from_row = array(), $to_row = array()) {
  $url = '';

  if($from == $to) {
    return 1.00;
  }


  // Amdoren.com
  if(exr_param('service') == 'amdoren') {
    $url = 'https://www.amdoren.com/api/currency.php?api_key=' . exr_param('amdoren_api_key') . '&from=' . strtoupper($from) . '&to=' . strtoupper($to);

    $content = json_decode(file_get_contents($url), true);

    // check for error
    if(isset($content['error']) && $content['error'] > 0) {
      osc_add_flash_error_message(sprintf(__('There was error using API (%s): %s', 'exchange_rate'), exr_param('service'), '[' . $content['error'] . '] ' . $content['error_message']), 'admin');

    } else if($content['amount'] <> '' && $content['amount'] > 0) {
      return $content['amount'];
    }
  }


  // Currencyconverterapi.com
  if(exr_param('service') == 'currencyconverterapi') {
    //$url = 'https://free.currencyconverterapi.com/api/v6/convert?apiKey=' . exr_param('cca_api_key') . '&q=' . strtoupper($from) . '_' . strtoupper($to);
    $url = 'https://free.currconv.com/api/v7/convert?apiKey=' . exr_param('cca_api_key') . '&q=' . strtoupper($from) . '_' . strtoupper($to);

    // If api key is invalid, url returns 400 and file_get_contents fails
    // $content = json_decode(file_get_contents($url), true);

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    $output = curl_exec($ch);

    // convert response
    $content = json_decode($output, true);


    if(isset($content['error'])) {
      osc_add_flash_error_message(sprintf(__('There was error using API (%s): %s', 'exchange_rate'), exr_param('service'), '[' . @$content['status'] . '] ' . @$content['error'] . ' (' . strtoupper($from) . '->' . strtoupper($to) . ')'), 'admin');

    } else if(isset($content['results']) && isset($content['results'][$from . '_' . $to])) {
      $rate = @$content['results'][$from . '_' . $to]['val'];

      if($rate <> '' && $rate > 0) {
        return $rate;
      } else {
        osc_add_flash_error_message(sprintf(__('There was error using API (%s)', 'exchange_rate'), exr_param('service')), 'admin');
      }

    } else {
      osc_add_flash_error_message(sprintf(__('There was error using API (%s)', 'exchange_rate'), exr_param('service')), 'admin');
    }
  }


  // Exchangeratesapi.io
  // SUNSET API
  if(exr_param('service') == 'exchangeratesapi') {
    $url = 'http://api.exchangeratesapi.io/latest?access_key=' . exr_param('exchangerates_api_key') . '&base=' . strtoupper($from) . '&symbols=' . strtoupper($to);

    $content = json_decode(file_get_contents($url), true);

    if(isset($content['error']) && isset($content['error']['code'])) {
      osc_add_flash_error_message(sprintf(__('There was error using API (%s): %s', 'exchange_rate'), exr_param('service'), '[' . @$content['error']['code'] . '] ' . @$content['error']['type'] . ' - ' . $content['error']['info'] . ' (' . strtoupper($from) . '->' . strtoupper($to) . ')'), 'admin');
    } else if(isset($content['rates'])) {
      $rate = @$content['rates'][$to];

      if($rate <> '' && $rate > 0) {
        return $rate;
      } else {
        osc_add_flash_error_message(sprintf(__('There was error using API (%s)', 'exchange_rate'), exr_param('service')), 'admin');
      }
    }
  }
  
  // Exchangeratesapi.io
  if(exr_param('service') == 'apilayer') {
    $curl = curl_init();

    curl_setopt_array($curl, array(
      CURLOPT_URL => 'https://api.apilayer.com/exchangerates_data/latest?base=' . strtoupper($from) . '&symbols=' . strtoupper($to),
      CURLOPT_HTTPHEADER => array(
        "Content-Type: text/plain",
        "apikey: " . exr_param('apilayer_api_key')
      ),
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => "",
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 0,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => "GET"
    ));

    $response = curl_exec($curl);

    curl_close($curl);

    $content = json_decode($response, true);

    if(isset($content['error']) && isset($content['error']['code'])) {
      osc_add_flash_error_message(sprintf(__('There was error using API (%s): %s', 'exchange_rate'), exr_param('service'), '[' . @$content['error']['code'] . '] ' . $content['error']['message'] . ' (' . strtoupper($from) . '->' . strtoupper($to) . ')'), 'admin');
    } else if(isset($content['rates'])) {
      $rate = @$content['rates'][$to];

      if($rate <> '' && $rate > 0) {
        return $rate;
      } else {
        osc_add_flash_error_message(sprintf(__('There was error using API (%s)', 'exchange_rate'), exr_param('service')), 'admin');
      }
    }
  }

  // Currencyconverterapi.com
  if(exr_param('service') == 'coinpaprika') {
    if(trim(exr_param('coinpaprika_api_key')) != '') {
      $url = 'https://api-pro.coinpaprika.com/v1';
    } else {
      $url = 'https://api.coinpaprika.com/v1';
    }
    
    $url .= '/coins';

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    
    if(trim(exr_param('coinpaprika_api_key')) != '') {
      $headers = ['Authorization: ' . exr_param('coinpaprika_api_key')];
      curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    }

    $output = curl_exec($ch);

    // convert response
    $list = json_decode($output, true);


    if(isset($list['error'])) {
      osc_add_flash_error_message(sprintf(__('There was error using API (%s): %s', 'exchange_rate'), exr_param('service'), @$content['error'] . ' (' . strtoupper($from) . '->' . strtoupper($to) . ')'), 'admin');

    } else {
      // Check by symbol, i.e. BTC
      $keys = array_keys(array_column($list, 'symbol'), $from);
      $coin = array_map(function($k) use ($list) { return $list[$k]; }, $keys);
      
      // Check by name, i.e: Tether
      if(!isset($coin[0]) && $from_row['s_name'] != '') {
        $keys = array_keys(array_column($list, 'name'), $from_row['s_name']);
        $coin = array_map(function($k) use ($list) { return $list[$k]; }, $keys);
      }
      
      // Check by symbol, i.e: USDT
      if(!isset($coin[0]) && $from_row['s_description'] != '') {
        $keys = array_keys(array_column($list, 'symbol'), $from_row['s_description']);
        $coin = array_map(function($k) use ($list) { return $list[$k]; }, $keys);
      }
      
      if(isset($coin[0]) && $coin[0]['id'] != '') {
        $coin_id = $coin[0]['id'];
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url . '/' . $coin_id . '/ohlcv/latest?quote=USD');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        
        if(trim(exr_param('coinpaprika_api_key')) != '') {
          $headers = ['Authorization: ' . exr_param('coinpaprika_api_key')];
          curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        }

        $output = curl_exec($ch);
        $coin_data = json_decode($output, true);
        
        if(isset($coin_data[0]) && @$coin_data[0]['close'] > 0) {
          $rate = $coin_data[0]['close'];
          return $rate;
        }
      } else {
        osc_add_flash_error_message(sprintf(__('There was error using API (%s): %s - %s', 'exchange_rate'), exr_param('service'), $from, __('Unable to find currency', 'exchange_rate')), 'admin');
      }
    }
    
    // Sleep for half second to avoid 429
    usleep(200000);
  }


  //return 1.00;
}



// UPDATE RATES IN DATABASE
function exr_update_rate() {
  //$cur_from = ModelEXR::newInstance()->getCurrencies();
  $cur_from = exr_get_currency_codes();
  $cur_to = $cur_from;

  // From
  if(is_array($cur_from) && count($cur_from) > 0) {
    foreach($cur_from as $f) {
      // To
      foreach($cur_to as $t) {
        if(exr_param('service') == 'coinpaprika' && $t == 'USD' || exr_param('service') != 'coinpaprika') {
          $rate = exr_feed_rate($f, $t, $f, $t);

          if($rate <> 0 && $rate <> '') {
            $data = array(
              's_from' => $f,
              's_to' => $t,
              'f_rate' => $rate
            );

            ModelEXR::newInstance()->updateCurrency($data);
            
            if(exr_param('service') == 'coinpaprika' && $t == 'USD') {
              $data = array(
                's_from' => $t,
                's_to' => $f,
                'f_rate' => 1/$rate
              );

              ModelEXR::newInstance()->updateCurrency($data);
            }
          }
        }
      }
    }
  }
}

osc_add_hook('cron_daily', 'exr_update_rate');


// DISABLE CURRENCIES ON PUBLISH/EDIT ITEM PAGE
function exr_disable_currencies() {
  $list = array_filter(array_unique(array_map('trim', explode(',', exr_param('disable_currencies')))));

  if(count($list) > 0) {
    foreach($list as $cur) {
    ?>
      <script>
        $(document).ready(function() {
          $('select[name="currency"]').children('option[value="<?php echo strtoupper($cur); ?>"]').remove();
        });
      </script>
      <?php
    }
  }
}

osc_add_hook('footer', 'exr_disable_currencies');



// CORE FUNCTIONS
function exr_param($name) {
  return osc_get_preference($name, 'plugin-exchange_rate');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function exr_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if( !function_exists('osc_is_contact_page') ) {
  function osc_is_contact_page() {
    $location = Rewrite::newInstance()->get_location();
    $section = Rewrite::newInstance()->get_section();
    if( $location == 'contact' ) {
      return true ;
    }

    return false ;
  }
}


// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}

?>