<?php
class ModelEXR extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_rate() {
  return DB_TABLE_PREFIX.'t_exchange_rate';
}

public function getTable_item() {
  return DB_TABLE_PREFIX.'t_item';
}

public function getTable_currency() {
  return DB_TABLE_PREFIX.'t_currency';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelEXR<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install($version = '') {
  if($version == '') {
    $this->import('exchange_rate/model/struct.sql');

    osc_set_preference('version', 100, 'plugin-exchange_rate', 'INTEGER');
  }

}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_rate()));


  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-exchange_rate'";
  $this->dao->query($query);
}



// GET CURRENCIES FROM OSCLASS
public function getCurrencies() {
  $this->dao->select();
  $this->dao->from($this->getTable_currency());

  $this->dao->where('b_enabled', 1);

  $result = $this->dao->get();
  
  if($result) {
    return $result->result();
  }

  return array();
}



// GET RATES
public function getRates() {
  $this->dao->select();
  $this->dao->from($this->getTable_rate());

  $this->dao->where('s_from != s_to');

  $result = $this->dao->get();
  
  if($result) {
    return $result->result();
  }

  return array();
}


// GET RATE
public function getRate($from, $to = '') {
  $this->dao->select();
  $this->dao->from($this->getTable_rate());

  $this->dao->where('s_from', $from);

  if($to <> '') {
    $this->dao->where('s_to', $to);
  } else {
    $this->dao->where('s_to != s_from');
  }

  $result = $this->dao->get();
  
  if($result) {
    if($to <> '') {
      $data = $result->row();
 
      if(isset($data['f_rate']) && $data['f_rate'] > 0) {
        return $data['f_rate'];
      } else {
        return 1.00;
      }

    } else {
      $data = $result->result();
      $array = array();

      if(count($data) > 0) {
        foreach($data as $d) {
          $array[$d['s_to']] = $d['f_rate'];
        }

        return $array;
      }
    }
  }

  return array();
}




// UPDATE CURRENCY RATE
public function updateCurrency($data) {
  return $this->dao->replace($this->getTable_rate(), $data);
}


// DELETE CURRENCY RATE
public function deleteCurrency($from, $to) {
  return $this->dao->delete($this->getTable_rate(), array('s_from' => $from, 's_to' => $to));
}



}
?>