<?php



// META TITLE
function faq_meta_title($tag) {
  $location = Rewrite::newInstance()->get_location();
  $section  = Rewrite::newInstance()->get_section();
  $route = Params::getParam('route');
  $page_title = ' - ' . osc_page_title();

  if($location == 'faq' && $section == 'home' || $route == 'faq-home') {
    $tag = __('Frequently Asked Questions (FAQ)', 'faq') . $page_title;

  } else if ($location == 'faq' && $section == 'search' || $route == 'faq-search') {
    $tag = sprintf(__('Search results for %s - FAQ', 'faq'), strip_tags(Params::getParam('pattern'))) . $page_title;

  } else if ($location == 'faq' && $section == 'category' || $route == 'faq-category') {
    $category = Modelfaq::newInstance()->getCategoryDetail(Params::getParam('categoryId'));
    $tag = sprintf(__('%s - FAQ', 'faq'), faq_category_title($category)) . $page_title;

  } 

  return $tag;
}

osc_add_filter('meta_title_filter', 'faq_meta_title', 10);


// META DESCRIPTION
function faq_meta_description($tag) {
  $location = Rewrite::newInstance()->get_location();
  $section  = Rewrite::newInstance()->get_section();
  $route = Params::getParam('route');

  if($location == 'faq' && $section == 'home' || $route == 'faq-home') {
    $tag = __('Welcome to our Frequently Asked Questions section, we believe you will find helpful information there.', 'faq');

  } else if($location == 'faq' && $section == 'search' || $route == 'faq-search') {
    $tag = __('Search results for your keyword in Frequently Asked Questions section.', 'faq');

  } else if ($location == 'faq' && $section == 'category' || $route == 'faq-category') {
    $category = Modelfaq::newInstance()->getCategoryDetail(Params::getParam('categoryId'));
    $tag = faq_category_description($category);

  }

  return $tag;
}

osc_add_filter('meta_description_filter', 'faq_meta_description', 10);


// CHECK AJAX ACTIONS
function faq_ajax_actions() {
  $action = trim(Params::getParam('faqAction'));
  $item_id = (int)Params::getParam('itemId');
  
  if($action != '' && $item_id > 0) {
    if($action == 'increaseView') {
      ModelFAQ::newInstance()->updateItemViews($item_id);
      exit;
      
    } else if($action == 'updateHelpful') {
      if(faq_param('helpful_enabled_only_logged') == 1 && !osc_is_web_user_logged_in()) {
        exit;
      } else {
        $positive = (int)Params::getParam('positive');
        $negative = (int)Params::getParam('negative');
      
        if(!osc_is_admin_user_logged_in()) {
          $positive = ($positive < -1 ? -1 : ($positive > 1 ? 1 : $positive)); 
          $negative = ($negative < -1 ? -1 : ($negative > 1 ? 1 : $negative)); 
        }
      
        ModelFAQ::newInstance()->updateItemRating($item_id, $positive, $negative);
      }

      exit;
    }
    
  } 
}

osc_add_hook('init', 'faq_ajax_actions');


// ADD JAVASCRIPT VARIABLES TO FOOTER
function faq_js_footer() {
?><script type="text/javascript">var faqBaseUrl='<?php echo osc_base_url(); ?>';</script><?php
}

osc_add_hook('footer', 'faq_js_footer');


// GENERATE SEARCH CLAUSE
function faq_search_clause($word, $table) {
  preg_match_all('/"(?:\\\\.|[^\\\\"])*"|\S+/', $word, $words);
  $words = $words[0];

  $clause_title = '(';
  $clause_desc = '(';

  $k = 0;
  if(count($words) > 0) {
    foreach($words as $w) {
      if($k > 0) {
        $clause_title .= ' OR ';
        $clause_desc .= ' OR ';
      }

      $clause_title .= sprintf("%s.%s like '%%" . str_replace('"', '', $w) . "%%'", $table, 's_title');
      $clause_desc .= sprintf("%s.%s like '%%" . str_replace('"', '', $w) . "%%'", $table, 's_description');
      $k++;
    }
  }

  $clause_title .= ')';
  $clause_desc .= ')';

  return '(' . $clause_title . ' OR ' . $clause_desc . ')';
}


// GET CATEGORY IMAGE
function faq_category_img($category, $use_default = true, $version = '') {
  $img = trim(isset($category['s_image']) ? $category['s_image'] : '');
  $version = (trim($version) <> '' ? '?v=' . trim($version) : '');
  
  if($img != '' && file_exists(osc_content_path() . 'plugins/faq/img/category/' . $img)) {
    return osc_base_url() . 'oc-content/plugins/faq/img/category/' . $img . $version;
  }
  
  if($use_default) {
    return osc_base_url() . 'oc-content/plugins/faq/img/category/default.png' . $version;
  }
  
  return false;
}


// GENERATE STATUS LABEL
function faq_status($status) {
  $html = '';

  if($status == 0) { 
    $html = '<span class="st0 mb-has-tooltip" title="' . osc_esc_html(__('Item is not visible', 'faq')) . '"><i class="fa fa-ban"></i> <span>' . __('Disabled', 'faq') . '</span></span>';
 } else {
    $html = '<span class="st3 mb-has-tooltip" title="' . osc_esc_html(__('Item is visible', 'faq')) . '"><i class="fa fa-check"></i> <span>' . __('Enabled', 'faq') . '</span></span>';
  }

  return $html;
}



// GET SINGLE POST
function faq_get_item($id) {
  return ModelFAQ::newInstance()->getItemDetail($id);
}


// GET ITEM TITLE IN USER LOCALE
function faq_item_title($item, $locale = '') {
  if(isset($item['locales'])) {
    if(faq_field('title', $item['locales'], $locale) <> '') {
      return faq_field('title', $item['locales'], $locale);
    }
  }

  return (@$item['s_title'] <> '' ? $item['s_title'] : __('New item', 'faq'));
}


// GET ITEM DESCRIPTION IN USER LOCALE
function faq_item_description($item, $locale = '') {
  if(isset($item['locales'])) {
    if(faq_field('description', $item['locales'], $locale) <> '') {
      return faq_field('description', $item['locales'], $locale);
    }
  }

  return (@$item['s_description'] <> '' ? $item['s_description'] : '');
}


// CATEGORY TITLE IN USER LOCALE
function faq_category_title($category, $locale = '') {
  if(isset($category['locales'])) {
    if(faq_field('title', $category['locales'], $locale) <> '') {
      return faq_field('title', $category['locales'], $locale);
    }
  }

  return (@$category['s_title'] <> '' ? $category['s_title'] : __('New category', 'faq'));
}


// CATEGORY DESCRIPTION IN USER LOCALE
function faq_category_description($category, $locale = '') {
  if(isset($category['locales'])) {
    if(faq_field('description', $category['locales'], $locale) <> '') {
      return faq_field('description', $category['locales'], $locale);
    }
  }

  return (@$category['s_description'] <> '' ? $category['s_description'] : '');
}


// GET SLUG FOR CATEGORY
function faq_category_slug($category, $locale = '') {
  $text = osc_sanitizeString(faq_category_title($category, $locale));

  if($text == '') {
    $text = 'category';
  }

  return $text;
}


// GET SINGLE ITEM
function faq_item($item, $class = '') {
  require 'form/loop-single.php';
}

// HOME LINK
function faq_home_link() {
  return osc_route_url('faq-home');
}

// HOME BUTTON
function faq_home_button($text = '') {
  if($text == '') {
    $text = __('FAQ', 'faq');
  }

  return '<a href="' . faq_home_link() . '" class="faq-button faq-button-home">' . $text . '</a>';
}


// UPDATE CATEGORY POSITION - AJAX
function faq_item_position() {
  $order = Params::getParam('item');

  if(is_array($order) && count($order) > 0) {
    $i = 0;
    foreach($order as $o) {
      ModelFAQ::newInstance()->updateItemPosition($o, $i+1);
      $i++;
    }
  }

  exit;
}

osc_add_hook('ajax_admin_faq_item_position', 'faq_item_position');

// UPDATE CATEGORY POSITION - AJAX
function faq_category_position() {
  $order = Params::getParam('cat');

  if(is_array($order) && count($order) > 0) {
    $i = 0;
    foreach($order as $o) {
      ModelFAQ::newInstance()->updateCategoryPosition($o, $i+1);
      $i++;
    }
  }

  exit;
}

osc_add_hook('ajax_admin_faq_category_position', 'faq_category_position');

// GET CORRECT LOCALE VALUE
function faq_field($column, $data, $locale = '') {
  if ($locale == '') {
    $locale = osc_current_user_locale();
  }
 
  $value = @$data[$locale][$column];

  if($value == '') {
    $value = @$data[osc_language()][$column];

    if($value == '') {
      $aLocales = osc_get_locales();
      foreach($aLocales as $locale) {
        $value = @$data[@$locale['pk_c_code']][$column];
        if($value != '') {
          break;
        }
      }
    }
  }

  return (string) $value;
}

// GET CURRENT OR DEFAULT ADMIN LOCALE
function faq_get_locale() {
  $locales = OSCLocale::newInstance()->listAllEnabled();

  if(Params::getParam('faqLocale') <> '') {
    $current = Params::getParam('faqLocale');
  } else {
    $current = (osc_current_user_locale() <> '' ? osc_current_user_locale() : osc_current_admin_locale());
    $current_exists = false;

    // check if current locale exist in front-office
    foreach( $locales as $l ) {
      if($current == $l['pk_c_code']) {
        $current_exists = true;
      }
    }

    if( !$current_exists ) {
      $i = 0;
      foreach( $locales as $l ) {
        if( $i==0 ) {
          $current = $l['pk_c_code'];
        }

        $i++;
      }
    }
  }

  return $current;
}

// CREATE LOCALE SELECT BOX
function faq_locale_box( $file, $param = '', $id = -1 ) {
  $html = '';
  $locales = OSCLocale::newInstance()->listAllEnabled();
  $current = faq_get_locale();

  $id_string = '';
  if($id > 0 && $param <> '') {
    $id_string = '&' . $param . '=' . $id;
  }

  $html .= '<select rel="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=faq/admin/' . $file . $id_string . '" class="mb-select mb-select-locale" id="faqLocale" name="faqLocale">';

  foreach( $locales as $l ) {
    $html .= '<option value="' . $l['pk_c_code'] . '" ' . ($current == $l['pk_c_code'] ? 'selected="selected"' : '') . '>' . $l['s_name'] . '</option>';
  }
 
  $html .= '</select>';
  return $html;
}



// CORE FUNCTIONS
function faq_param($name) {
  return osc_get_preference($name, 'plugin-faq');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function faq_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}

?>