<?php
class ModelFAQ extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_category() {
  return DB_TABLE_PREFIX.'t_faq_category';
}

public function getTable_category_locale() {
  return DB_TABLE_PREFIX.'t_faq_category_locale';
}

public function getTable_item() {
  return DB_TABLE_PREFIX.'t_faq_item';
}

public function getTable_item_locale() {
  return DB_TABLE_PREFIX.'t_faq_item_locale';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelFAQ<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install($version = '') {
  if($version == '') {
    $this->import('faq/model/struct.sql');
    osc_set_preference('version', 100, 'plugin-faq', 'INTEGER');
  }
}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_category()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_category_locale()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_item()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_item_locale()));

  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-faq'";
  $this->dao->query($query);
}


// GET CATEGORIES
public function getCategories() {
  $this->dao->select();
  $this->dao->from($this->getTable_category());
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return false;
}

// GET CATEGORY
public function getCategory($id) {
  $this->dao->select();
  $this->dao->from($this->getTable_category());
  $this->dao->where('pk_i_id', $id);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }
  
  return false;
}


// GET CATEGORY LOCALE
public function getCategoryLocale($id, $locale = '') {
  $this->dao->select('*');
  $this->dao->from($this->getTable_category_locale());
  $this->dao->where('fk_i_category_id', $id);

  if($locale <> '') {
    $this->dao->where('fk_c_locale_code', $locale);
  }

  $result = $this->dao->get();
  
  if($result) {
    return $result->row();
  }

  return false;
}


// GET CATEGORY LOCALES
public function getCategoryLocales($category_id) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_category_locale());
  $this->dao->where('fk_i_category_id', $category_id);

  $result = $this->dao->get();
  $array = array();

  if($result) {
    $locales = $result->result();

    if(is_array($locales) && count($locales) > 0) {
      foreach($locales as $l) {
        $array[$l['fk_c_locale_code']] = array('title' => $l['s_title'], 'description' => $l['s_description']);
      }
    }
  }

  return $array;
}


// GET ITEMS
public function getItems($category_id = '') {
  $this->dao->select();
  $this->dao->from($this->getTable_item());
  $result = $this->dao->get();
  
  if((int)$category_id > 0) {
    $this->dao->where('fk_i_category_id', $category_id);
  }
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return false;
}

// GET ITEM
public function getItem($id) {
  $this->dao->select();
  $this->dao->from($this->getTable_item());
  $this->dao->where('pk_i_id', $id);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }
  
  return false;
}


// GET ITEM LOCALE
public function getItemLocale($id, $locale = '') {
  $this->dao->select('*');
  $this->dao->from($this->getTable_item_locale());
  $this->dao->where('fk_i_item_id', $id);

  if($locale <> '') {
    $this->dao->where('fk_c_locale_code', $locale);
  }

  $result = $this->dao->get();
  
  if($result) {
    return $result->row();
  }

  return false;
}


// COUNT CATEGORY ITEMS
public function countCategoryItems($id) {
  $this->dao->select('count(pk_i_id) as i_count');
  $this->dao->from($this->getTable_item());
  $this->dao->where('fk_i_category_id', $id);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return (isset($data['i_count']) ? (int)$data['i_count'] : 0); 
  }

  return 0;
}


// GET ITEM LOCALES
public function getItemLocales($item_id) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_item_locale());
  $this->dao->where('fk_i_item_id', $item_id);

  $result = $this->dao->get();
  $array = array();

  if($result) {
    $locales = $result->result();

    if(is_array($locales) && count($locales) > 0) {
      foreach($locales as $l) {
        $array[$l['fk_c_locale_code']] = array('title' => $l['s_title'], 'description' => $l['s_description']);
      }
    }
  }

  return $array;
}


// GET CATEGORY WITH DETAILS
public function getCategoryDetail($category_id) {
  $this->dao->select('DISTINCT c.*, cl.s_title, cl.s_description, cl.fk_c_locale_code');
  $this->dao->from($this->getTable_category() . ' as c');
  $this->dao->join($this->getTable_category_locale() . ' as cl', '(cl.fk_i_category_id = c.pk_i_id AND cl.fk_c_locale_code = "' . faq_get_locale() . '")', 'LEFT OUTER');

  $this->dao->where('c.pk_i_id', $category_id);

  $result = $this->dao->get();
  
  if($result) {
    $category = $result->row();

    if(isset($category['pk_i_id'])) {
      $category['locales'] = $this->getCategoryLocales($category['pk_i_id']);
    }

    return $category;
  }

  return array();
}

// GET ITEMS WITH DETAILS
public function getItemDetail($item_id) {
  $this->dao->select('DISTINCT c.*, cl.s_title, cl.s_description, cl.fk_c_locale_code');
  $this->dao->from($this->getTable_item() . ' as c');
  $this->dao->join($this->getTable_item_locale() . ' as cl', '(cl.fk_i_item_id = c.pk_i_id AND cl.fk_c_locale_code = "' . faq_get_locale() . '")', 'LEFT OUTER');

  $this->dao->where('c.pk_i_id', $item_id);

  $result = $this->dao->get();
  
  if($result) {
    $item = $result->row();

    if(isset($item['pk_i_id'])) {
      $item['locales'] = $this->getItemLocales($item['pk_i_id']);
    }

    return $item;
  }

  return array();
}


// GET CATEGORIES
public function getCategoriesDetail() {
  $this->dao->select('DISTINCT c.*, cl.s_title, cl.s_description, cl.fk_c_locale_code');
  $this->dao->from($this->getTable_category() . ' as c');
  $this->dao->join($this->getTable_category_locale() . ' as cl', '(cl.fk_i_category_id = c.pk_i_id AND cl.fk_c_locale_code = "' . faq_get_locale() . '")', 'LEFT OUTER');

  $this->dao->orderby('c.i_order ASC, cl.s_title ASC, c.pk_i_id ASC');

  $result = $this->dao->get();
  
  if($result) {
    $categories = $result->result();

    if(is_array($categories) && count($categories) > 0) {
      $i = 0;
      
      foreach($categories as $c) { 
        $categories[$i]['locales'] = $this->getCategoryLocales($c['pk_i_id']);
        $categories[$i]['item_count'] = $this->countCategoryItems($c['pk_i_id']);
        $i++;
      }
    }

    return $categories;
  }

  return array();
}

// GET ITEMS
public function getItemsDetail($type = '', $category_id = '', $pattern = '', $limit = 0, $status = 1) {
  $this->dao->select('DISTINCT c.*, cl.s_title, cl.s_description, cl.fk_c_locale_code');
  $this->dao->from($this->getTable_item() . ' as c');
  $this->dao->join($this->getTable_item_locale() . ' as cl', '(cl.fk_i_item_id = c.pk_i_id AND cl.fk_c_locale_code = "' . faq_get_locale() . '")', 'LEFT OUTER');

  if($status != -1) {
    $this->dao->where('c.b_enabled', $status);
  }

  if((int)$category_id > 0) {
    $this->dao->where('c.fk_i_category_id', $category_id);
  }
  
  if(trim($pattern) <> '') {
    //$this->dao->where('cl.s_title like "%' . $keyword . '%" OR cl.s_description like "%' . $keyword . '%"');
    $clause = faq_search_clause($pattern, 'cl');
    $this->dao->where($clause);
  }
  
  if((int)$limit > 0) {
    $this->dao->limit($limit);
  }

  if($type == 'POPULAR') {
    $this->dao->orderby('c.i_helpful DESC, c.i_view DESC, c.dt_date DESC');

  } else if($type == 'HELPFUL') {
    $this->dao->orderby('c.i_helpful DESC, c.i_view DESC, c.dt_date DESC');
    
  } else if($type == 'VIEW') {
    $this->dao->orderby('c.i_view DESC, c.dt_date DESC');
    
  } else if($type == 'DATE' || $type == 'LATEST') {
    $this->dao->orderby('c.dt_date DESC, c.pk_i_id DESC');
    
  } else {
    $this->dao->orderby('c.i_order ASC, c.dt_date DESC');
  }

  $result = $this->dao->get();
  
  if($result) {
    $items = $result->result();

    if(is_array($items) && count($items) > 0) {
      $i = 0;
      
      foreach($items as $itm) { 
        $items[$i]['locales'] = $this->getItemLocales($itm['pk_i_id']);
        $i++;
      }
    }

    return $items;
  }

  return array();
}



// UPDATE ITEM
public function updateItem($data_item, $data_locale) {
  if(!isset($data_item['pk_i_id']) || $data_item['pk_i_id'] <= 0) {
    if(isset($data_item['pk_i_id'])) {
      unset($data_item['pk_i_id']);
    }

    $this->dao->insert($this->getTable_item(), $data_item);
    $id = $this->dao->insertedId();

    Params::setParam('itemId', $id);
    $data_locale['fk_i_item_id'] = $id;

    $this->updateItemLocale($data_locale);

  } else {
    Params::setParam('itemId', $data_item['pk_i_id']);
    $where = array('pk_i_id' => $data_item['pk_i_id']);

    $this->dao->update($this->getTable_item(), $data_item, $where);
    $this->updateItemLocale($data_locale);
  }
}

// UPDATE CATEGORY 
public function updateCategory($data_cat, $data_locale) {
  if(!isset($data_cat['pk_i_id']) || $data_cat['pk_i_id'] <= 0) {
    if(isset($data_cat['pk_i_id'])) {
      unset($data_cat['pk_i_id']);
    }

    $id = $this->insertCategory($data_cat);

    Params::setParam('editId', $id);
    $data_locale['fk_i_category_id'] = $id;

    $this->updateCategoryLocale($data_locale);

  } else {
    Params::setParam('editId', $data_cat['pk_i_id']);
    $where = array('pk_i_id' => $data_cat['pk_i_id']);

    $this->dao->update($this->getTable_category(), $data_cat, $where);
    $this->updateCategoryLocale($data_locale);
  }
}


// UPDATE CATEGORY LOCALE
public function updateCategoryLocale($data) {
  if(@$data['fk_i_category_id'] > 0 && trim(@$data['fk_c_locale_code']) <> '') {
    $this->dao->replace($this->getTable_category_locale(), $data);
  }
}

// UPDATE CATEGORY IMAGE
public function updateCategoryImage($id, $name) {
  $this->dao->update($this->getTable_category(), array('s_image' => $name), array('pk_i_id' => $id));
}

// UPDATE ITEM LOCALE
public function updateItemLocale($data) {
  if(@$data['fk_i_item_id'] > 0 && trim(@$data['fk_c_locale_code']) <> '') {
    $this->dao->replace($this->getTable_item_locale(), $data);
  }
}

// UPDATE CATEGORY POSITION
public function updateCategoryPosition($id, $order) {
  $this->dao->update($this->getTable_category(), array('i_order' => $order), array('pk_i_id' => $id));
}

// UPDATE ITEM POSITION
public function updateItemPosition($id, $order) {
  $this->dao->update($this->getTable_item(), array('i_order' => $order), array('pk_i_id' => $id));
}

// UPDATE ITEM VIEWS
public function updateItemViews($id) {
  if($id > 0) {
    return $this->dao->query('UPDATE '.$this->getTable_item() . ' SET i_view=coalesce(i_view, 0)+1 WHERE pk_i_id='.$id);
  }
}

// UPDATE ITEM HELPFUL/NOTHELPFUL
public function updateItemRating($id, $positive, $negative) {
  $positive = (int)$positive;
  $negative = (int)$negative;
  
  return $this->dao->query('UPDATE '.$this->getTable_item() . ' SET i_helpful=coalesce(i_helpful, 0)+' . $positive . ', i_nothelpful=coalesce(i_nothelpful, 0)+' . $negative . ' WHERE pk_i_id='.$id);
}


// INSERT CATEGORY
public function insertCategory($data) {
  $this->dao->insert($this->getTable_category(), $data);
  return $this->dao->insertedId();
}

// INSERT CATEGORY
public function insertItem($data) {
  $this->dao->insert($this->getTable_item(), $data);
  return $this->dao->insertedId();
}

// REMOVE CATEGORY
public function removeCategory($id) {
  $this->removeCategoryLocale($id);
  return $this->dao->delete($this->getTable_category(), array('pk_i_id' => $id));
}

// REMOVE ITEM
public function removeItem($id) {
  $this->removeItemLocale($id);
  return $this->dao->delete($this->getTable_item(), array('pk_i_id' => $id));
}

// REMOVE CATEGORY LOCALE
public function removeCategoryLocale($id, $locale = '') {
  if(trim($locale) <> '') { 
    $where = array('fk_i_category_id' => $id, 'fk_c_locale_code' => $locale);
  } else {
    $where = array('fk_i_category_id' => $id);
  }
  
  return $this->dao->delete($this->getTable_category_locale(), $where);
}

// REMOVE ITEM LOCALE
public function removeItemLocale($id, $locale = '') {
  if(trim($locale) <> '') { 
    $where = array('fk_i_item_id' => $id, 'fk_c_locale_code' => $locale);
  } else {
    $where = array('fk_i_item_id' => $id);
  }
  
  return $this->dao->delete($this->getTable_item_locale(), $where);
}

}
?>