<?php


// SUBSCRIBE-UNSUBSCRIBE FEATURE
function flw_create_alert_subscription($following_user_id, $follower_user_id, $action = 'ACTIVATE') {
  if(osc_is_web_user_logged_in() && $following_user_id > 0) {
    $aSearch = new Search();
    $aSearch->addConditions(sprintf('%st_item.fk_i_user_id = "%s"', DB_TABLE_PREFIX, $following_user_id));

    $alert_json = $aSearch->toJson();
    
    if($action == 'ACTIVATE') {
      $user = User::newInstance()->findByPrimaryKey($follower_user_id);
      $alert_id = Alerts::newInstance()->createAlert($follower_user_id, $user['s_email'], $alert_json, osc_genRandomPassword());

      if($alert_id > 0) {
        Alerts::newInstance()->activate($alert_id);
        return $alert_id;
      }
    } else {
      $alerts = Alerts::newInstance()->findBySearchAndUser($alert_json, $follower_user_id);
      
      if(is_array($alerts) && count($alerts) > 0) {
        foreach($alerts as $a) {
          Alerts::newInstance()->dao->delete(DB_TABLE_PREFIX . 't_alerts', array('pk_i_id' => $a['pk_i_id']));
        }
      }
    }
    
  }
  
  return false;
}



// GENERATE PAGINATION
function flw_paginate($type, $page_id, $per_page, $count_all, $class = '') {
  $html = '';
  $page_id = (int)$page_id;
  $page_id = ($page_id <= 0 ? 1 : $page_id);

  if($per_page < $count_all) {
    $html .= '<div class="flw-pagination ' . $class . '">';

    $pages = ceil($count_all/$per_page); 
    $page_actual = ($page_id == '' ? 1 : $page_id);

    if($pages > 6) {

      // Too many pages to list them all
      if($page_id == 1) { 
        $ids = array(1,2,3, $pages);

      } else if ($page_id > 1 && $page_id < $pages) {
        $ids = array(1,$page_id-1, $page_id, $page_id+1, $pages);

      } else {
        $ids = array(1, $page_id-2, $page_id-1, $page_id);
      }

      $old = -1;
      $ids = array_unique(array_filter($ids));

      foreach($ids as $i) {
        $url = osc_route_url('flw-paginate', array('pageType' => $type, 'pageId' => $i));

        if($old <> -1 && $old <> $i - 1) {
          $html .= '<span>&middot;&middot;&middot;</span>';
        }

        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="flw-active"' : '') . '>' . $i . '</a>';
        $old = $i;
      }

    } else {

      // List all pages
      for ($i = 1; $i <= $pages; $i++) {
        $url = osc_route_url('flw-paginate', array('pageType' => $type, 'pageId' => $i));

        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="flw-active"' : '') . '>' . $i . '</a>';
      }
    }

    $html .= '</div>';
  }

  return $html;
}


// CATCH AJAX USER LIST
function flw_ajax_list() {
  $user_id = Params::getParam('userId');
  $type = (Params::getParam('type') == '' ? 'follower' : Params::getParam('type'));  // follower - following
  $users = array();
  
  if($user_id <> '' && $user_id > 0) {
    $users = ModelFLW::newInstance()->getFollowerList($user_id, $type, $active = true, $limit = array(0, flw_param('limit_users')), $order = 'date_desc');
    $already_following = ModelFLW::newInstance()->getFollowerIds(osc_logged_user_id(), 'following');
  }
  
  ?>
  <div class="flw-head">
    <?php 
      if($type == 'follower') {
        _e('Followers', 'follow');
      } else {
        _e('Following', 'follow');
      }
    ?>
  </div>
  <?php

  if(is_array($users) && count($users) > 0) {   
    ?>
    <div class="flw-box-content flw-list">
      <?php 
        foreach($users as $u) { 
          flw_draw_user($u, $already_following);
        }
      ?>
    </div>
    
    <?php
  } else {
    ?>
    <div class="flw-box-content">
      <div class="flw-users-empty">
        <?php 
          if($type == 'follower') {
            _e('This user has no followers yet', 'follow');
          } else {
            _e('This user is not following anyone yet', 'follow');
          }
        ?>
      </div>
    <?php
  }

  exit;
}

osc_add_hook('ajax_flw_ajax_list', 'flw_ajax_list');


// DRAW USER CARD
function flw_draw_user($user, $already_following = array()) {
?>
  <div class="flw-one">
    <div class="flw-img">
      <img src="<?php echo flw_profile_img_url($user['pk_i_id'], $user); ?>" alt="<?php echo osc_esc_html($user['s_name']); ?>"/>
    </div>
    
    <div class="flw-about">
      <a href="<?php echo osc_user_public_profile_url($user['pk_i_id']); ?>" target="_blank"><?php echo $user['s_name']; ?></a>
      <span>
        <?php 
          if(flw_param('user_details') == '' || flw_param('user_details') == 'LOCATION') {
            $loc = implode(', ', array_filter(array($user['s_city'], $user['s_region'], $user['fk_c_country_code']))); 
            echo ($loc == '' ? '-' : trim($loc));
          } else if(flw_param('user_details') == 'TYPE') {
            echo ($user['b_company'] == 1 ? __('Professional seller', 'follow') : __('Personal seller', 'follow'));
          } else if(flw_param('user_details') == 'COUNT') {
            echo sprintf(__('%d listings', 'follow'), $user['i_items']);
          }
        ?>
      </span>
    </div>
    
    <div class="flw-buttons">
      <?php echo flw_create_link($user['pk_i_id'], (in_array($user['pk_i_id'], $already_following) ? 'following' : 'follow'), 'button'); ?>
    </div>
  </div>
<?php
}


// CATCH AJAX FOLLOW/UNFOLLOW ACTION
function flw_ajax_follow() {
  $following_user_id = Params::getParam('userId');
  $type = Params::getParam('type');
  
  if($following_user_id <> '' && $following_user_id > 0 && osc_is_web_user_logged_in()) {
    if($type == 'follow') {
      flw_follow_user($following_user_id, osc_logged_user_id());
    } else {
      flw_unfollow_user($following_user_id, osc_logged_user_id());
    }
    
    echo json_encode(array('status' => 'OK'));
  } else {
    echo json_encode(array('status' => 'ERR'));
  }

  exit;
}

osc_add_hook('ajax_flw_ajax_follow', 'flw_ajax_follow');



// CATCH AJAX UNFOLLOW ACTION
function flw_ajax_unfollow() {
  $following_user_id = Params::getParam('userId');
  
  if($following_user_id <> '' && $following_user_id > 0 && osc_is_web_user_logged_in()) {
    flw_unfollow_user($following_user_id, osc_logged_user_id());
    echo json_encode(array('status' => 'OK'));
  } else {
    echo json_encode(array('status' => 'ERR'));
  }

  exit;
}

osc_add_hook('ajax_flw_ajax_unfollow', 'flw_ajax_unfollow');


// ADD LINKS TO FOOTER
function flw_footer_js() {
?><script type="text/javascript">var flwFollow="<?php echo osc_esc_html(__('Follow', 'follow')); ?>";var flwFollowing="<?php echo osc_esc_html(__('Following', 'follow')); ?>";var flwError="<?php echo osc_esc_html(__('Error', 'follow')); ?>";</script><?php  
}

osc_add_hook('footer', 'flw_footer_js');



// GET USER PROFILE IMAGE URL
function flw_profile_img_url($user_id, $user = array()) {
  if($user_id <= 0) {
    return osc_base_url() . 'oc-content/plugins/follow/images/default-user-image.png';
  }
  
  if(flw_has_profile_img()) {
    if(!is_array($user) || empty($user) || !isset($user['pk_i_id'])) {
      $user = User::newInstance()->findByPrimaryKey($user_id);
    }
    
    if(isset($user['s_profile_img']) && $user['s_profile_img'] != '') {
      return osc_base_url() . 'oc-content/uploads/user-images/' . $user['s_profile_img'];
    }
  }
  
  if(function_exists('profile_picture_show')) {
    $picture = ModelUR::newInstance()->getPictureByUserId($user_id);
    $picture['pic_ext'] = isset($picture['pic_ext']) ? $picture['pic_ext'] : '.jpg';  

    if(file_exists(osc_content_path() . 'plugins/profile_picture/images/profile' . $user_id . $picture['pic_ext'])) { 
      return osc_base_url() . 'oc-content/plugins/profile_picture/images/profile' . $user_id . $picture['pic_ext'];
    }
  }
  
  if(function_exists('show_avatar')) {
    $picture = ModelAvatar::newInstance()->getAvatar($user_id); 

    if($picture <> '') {
      if(file_exists(osc_content_path() . 'plugins/avatar_plugin/avatar/' . $picture)) { 
        return osc_base_url() . 'oc-content/plugins/avatar_plugin/avatar/' . $picture;
      } 
    }
  }

  return osc_base_url() . 'oc-content/plugins/follow/images/default-user-image.png';
}


// CHECK IF OSCLASS HAS PROFILE PICTURE
function flw_has_profile_img() {
  if(defined('OSCLASS_AUTHOR') && defined('OSCLASS_AUTHOR') == 'OSCLASSPOINT' && osc_version() > 420) {
    return true;
  }
  
  return false;
}


// FOLLOW USER
function flw_follow_user($following_user_id, $follower_user_id) {
  ModelFLW::newInstance()->insertFollower($following_user_id, $follower_user_id);

  if(flw_param('subscribe') == 1) {
    flw_create_alert_subscription($following_user_id, $follower_user_id, 'ACTIVATE');
  }
}


// STOP FOLLOWING USER
function flw_unfollow_user($following_user_id, $follower_user_id) {
  ModelFLW::newInstance()->removeFollowerFollowing($following_user_id, $follower_user_id);

  if(flw_param('subscribe') == 1) {
    flw_create_alert_subscription($following_user_id, $follower_user_id, 'DECTIVATE');
  }
}


// FOLLOWERS COUNT
function flw_show_count($user_id, $action = 'follower', $type = 'button') {
  //if($user_id <= 0) {
  //  return false;
  //}
  
  $link = osc_base_url(true) . '?page=ajax&action=runhook&hook=flw_ajax_list';
  $count = ModelFLW::newInstance()->countFollowers($user_id, $action, $active = true);
  
  if($type == 'raw') {
    $text = ($action == 'follower' ? sprintf(__('%s followers', 'follow'), $count) : sprintf(__('%s following', 'follow'), $count));
    return $text;
  } else {
    $button  = '<a href="' . $link . '" class="flw-counts flw-' . ($action == 'follower' ? 'follower' : 'following') . '" data-id="' . $user_id . '" data-type="' . ($action == 'follower' ? 'follower' : 'following') . '">';
    $button .= '<strong>' . $count . '</strong> ';
    $button .= '<span>' . ($action == 'follower' ? __('Followers', 'follow') : __('Following', 'follow')) . '</span>';
    $button .= '</a>';
    
    return $button;
  }
}



// CREATE FOLLOW BUTTON/LINK
function flw_create_link($user_id, $action = 'auto', $type = 'button') {
  //if($user_id <= 0) {
  //  return false;
  //}
  
  $already_following = array();
  
  // auto-identify if follow or following button
  if($action == 'auto') {
    $already_following = ModelFLW::newInstance()->getFollowerIds(osc_logged_user_id(), 'following');
    
    if(in_array($user_id, $already_following)) {
      $action = 'unfollow';
    } else {
      $action = 'follow'; 
    }
  }
  
  $title = '';
  
  if(!osc_is_web_user_logged_in()) {
    $title = __('You must be logged-in to Follow/Unfollow', 'follow'); 
  }
  
  $link = osc_base_url(true) . '?page=ajax&action=runhook&hook=flw_ajax_follow&userId=' . $user_id . '&type=' . ($action == 'follower' ? 'follow' : 'unfollow');

  $button_design = (flw_param('button_design') <> '' ? flw_param('button_design') : 'DISABLED');
  
  $button_text = ($action == 'follow' ? __('Follow', 'follow') : __('Following', 'follow'));
  
  if($button_design == 'DISABLED_ALT' && !osc_is_web_user_logged_in()) {
    $button_text = __('Log in', 'follow'); 
  }
  
  if(osc_is_web_user_logged_in() && osc_logged_user_id() == $user_id) {
    $button_text = __('Your profile', 'follow'); 
  }
  
  if($type == 'raw') {
    return $link;
  } else {
    if($button_design == 'HIDE' && !osc_is_web_user_logged_in()) {
      return '';
    }
    
    $disable = false;
    
    if((osc_is_web_user_logged_in() && osc_logged_user_id() == $user_id) || !osc_is_web_user_logged_in()) {
      $disable = true;
    }
    
    
    $button = '<a href="' . $link . '" class="flw-subscribe flw-button flw-' . ($action == 'follow' ? 'do' : 'undo') . ' ' . ($disable ? 'flw-disabled' : '') . '" data-id="' . $user_id . '" data-type="' . ($action == 'follow' ? 'follow' : 'unfollow') . '" title="' . osc_esc_html($title) . '">' . $button_text . '</a>';
    return $button;
  }
}


// BUTTONS LINE
function flw_buttons_box() {
  if(osc_item_user_id() > 0) {  // this could be enabled to avoid blank buttons
    ?>
    <div id="flw-buttons" class="flw-item-buttons-wrap">
      <?php echo flw_show_count(osc_item_user_id(), $action = 'follower', $type = 'button'); ?>
      <?php echo flw_show_count(osc_item_user_id(), $action = 'following', $type = 'button'); ?>
      <?php echo flw_create_link(osc_item_user_id(), $action = 'auto', $type = 'button'); ?>
    </div>
    <?php
  }
}


// HOOK BUTTONS TO ITEM PAGE
function flw_hook_buttons() {
  if(flw_param('hook_item') == 1) {
    flw_buttons_box();
  }  
}

osc_add_hook('item_detail', 'flw_hook_buttons', 10);


// USER REMOVED
function flw_user_removed($id) {
  ModelFLW::newInstance()->removeFollower($id);
  ModelFLW::newInstance()->removeFollowing($id);
}
 
osc_add_hook('delete_user', 'flw_user_removed');


// CORE FUNCTIONS
function flw_param($name) {
  return osc_get_preference($name, 'plugin-follow');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function flw_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}

?>