<?php
class ModelFLW extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_user() {
  return DB_TABLE_PREFIX.'t_user';
}

public function getTable_follower() {
  return DB_TABLE_PREFIX.'t_user_follower';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelFLW<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install($version = '') {
  if($version == '') {
    $this->import('follow/model/struct.sql');
    osc_set_preference('version', 100, 'plugin-follow', 'INTEGER');
  }
}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_follower()));

  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-follow'";
  $this->dao->query($query);
}


// GET FOLLOWERS & FOLLOWING USERS
public function getFollowerIds($user_id, $type = 'follower') {
  $this->dao->select();
  $this->dao->from($this->getTable_follower());
  
  if($type == 'follower') {
    $this->dao->where('fk_i_following_user_id', $user_id);
  } else {
    $this->dao->where('fk_i_follower_user_id', $user_id);
  }
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    $output = array();
    
    if(is_array($data) && count($data) > 0) {
      foreach($data as $d) {
        if($type == 'follower') {
          $output[] = $d['fk_i_follower_user_id'];
        } else {
          $output[] = $d['fk_i_following_user_id'];
        }
      }

      $output = array_unique(array_filter($output));
    }
      
    return $output;
  }
  
  return array();
}


// GET FOLLOWERS & FOLLOWING USERS
public function getFollowerList($user_id, $type = 'follower', $active = true, $limit = array(0, 50), $order = 'date_desc') {
  if($type != 'follower') {
    $join = 'f.fk_i_following_user_id = u.pk_i_id';
  } else {
    $join = 'f.fk_i_follower_user_id = u.pk_i_id';
  }
  

  if(flw_has_profile_img()) {
    $this->dao->select('u.pk_i_id, u.s_name, u.s_profile_img, u.dt_reg_date, u.fk_c_country_code, u.s_country, u.s_region, u.s_city, u.b_company, u.i_items, u.i_comments');
  } else {
    $this->dao->select('u.pk_i_id, u.s_name, u.dt_reg_date, u.fk_c_country_code, u.s_country, u.s_region, u.s_city, u.b_company, u.i_items, u.i_comments');
  }

  $this->dao->from($this->getTable_follower() . ' as f');
  $this->dao->join($this->getTable_user() . ' as u', $join, 'INNER');

  if($active === true) {
    $this->dao->where('u.b_active', 1);
    $this->dao->where('u.b_enabled', 1);
  }
  
  if($type == 'follower') {
    $this->dao->where('fk_i_following_user_id', $user_id);
  } else {
    $this->dao->where('fk_i_follower_user_id', $user_id);
  }
  
  
  // $limit[0] == page; $limit[1] == limit
  $page = (isset($limit[0]) ? $limit[0] : 0);
  $per_page = (isset($limit[1]) ? $limit[1] : -1);
  
  if($page > 0) {
    $this->dao->limit(($page-1)*$per_page, $per_page);
  } else if($per_page > 0) {
    $this->dao->limit($per_page);
  }
  
  if($order == 'date_desc') {
    $this->dao->orderby('f.dt_date DESC');
  } else if($order == 'date_asc') {
    $this->dao->orderby('f.dt_date ASC');
  } else if($order == 'reg_desc') {
    $this->dao->orderby('u.dt_reg_date DESC');
  } else if($order == 'reg_asc') {
    $this->dao->orderby('u.dt_reg_date ASC');
  }

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// COUNT FOLLOWERS & FOLLOWING USERS
public function countFollowers($user_id, $type = 'follower', $active = true) {
  if($type != 'follower') {
    $join = 'f.fk_i_following_user_id = u.pk_i_id';
  } else {
    $join = 'f.fk_i_follower_user_id = u.pk_i_id';
  }
  
  $this->dao->select('count(*) as i_count');
  $this->dao->from($this->getTable_follower() . ' as f');

  if($active) {
    $this->dao->join($this->getTable_user() . ' as u', $join, 'INNER');
    $this->dao->where('u.b_active', 1);
    $this->dao->where('u.b_enabled', 1);
  }

  if($type == 'follower') {
    $this->dao->where('f.fk_i_following_user_id', $user_id);
  } else {
    $this->dao->where('f.fk_i_follower_user_id', $user_id);
  }
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['i_count']) && $data['i_count'] > 0) {
      return (int)$data['i_count'];
    }
  }
  
  return 0;
}



// INSERT FOLLOWER
public function insertFollower($following_user_id, $follower_user_id) {
  return $this->dao->replace($this->getTable_follower(), array('fk_i_following_user_id' => $following_user_id, 'fk_i_follower_user_id' => $follower_user_id, 'dt_date' => date('Y-m-d H:i:s')), array('fk_i_following_user_id' => $following_user_id, 'fk_i_follower_user_id' => $follower_user_id));
}

// REMOVE FOLLOWING COMPLETELY
public function removeFollowerFollowing($following_user_id, $follower_user_id) {
  return $this->dao->delete($this->getTable_follower(), array('fk_i_following_user_id' => $following_user_id, 'fk_i_follower_user_id' => $follower_user_id));
}

// REMOVE FOLLOWER COMPLETELY
public function removeFollower($id) {
  return $this->dao->delete($this->getTable_follower(), array('fk_i_follower_user_id' => $id));
}

// REMOVE FOLLOWING COMPLETELY
public function removeFollowing($id) {
  return $this->dao->delete($this->getTable_follower(), array('fk_i_following_user_id' => $id));
}

}
?>