<?php
/*
  Plugin Name: Google OneTap Login Osclass Plugin
  Plugin URI: https://osclasspoint.com/
  Description: Enable Google One Tap Login feature to your users to quickly login or register with their Google Account
  Version: 1.0.2
  Author: MB Themes
  Author URI: http://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: google_onetap_login
  Plugin update URI: google_onetap_login
  Support URI: https://forums.osclasspoint.com/general-plugins-discussion/
  Product Key: c7cq9Kdxlfn8tHIGjKvN
*/


require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'model/ModelGOT.php';
require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'functions.php';

// Library is big, require manually remove google api vendor folder and run composer cleanup

$php_major = explode('.', phpversion())[0];
$php_variant = ($php_major >= 8 ? '8.0' : '5.6');;

if($php_major >= 8) {
  define('GOT_API_PHP_VERSION', 'PHP8.0');
} else {
  define('GOT_API_PHP_VERSION', 'PHP5.6');
}

osc_add_route('got-redirect', 'got/redirect', 'got/redirect', osc_plugin_folder(__FILE__) . 'form/redirect.php', 'custom', 'got-login-callback');


// SHOW LOGIN BUTTON
function got_init_login_button() {
  if(!osc_is_web_user_logged_in() && got_param('enabled') == 1 && got_check_page()) { 
  ?>
  <script type="text/javascript" src="https://accounts.google.com/gsi/client" async defer></script>
  <div id="g_id_onload" class="google-onetap-login-box"
		data-client_id="<?php echo osc_esc_html(got_param('client_id')); ?>"
		data-context="signin"
		data-callback="gotLoginCallback"
    data-login_uri="<?php echo osc_esc_html(osc_user_login_url()); ?>"
		data-close_on_tap_outside="false">
	</div>

  <script type="text/javascript">
    // callback function that will be called when the user is successfully logged-in with Google
    function gotLoginCallback(googleUser) {
      //console.log(googleUser);
      var ajax = new XMLHttpRequest();

      ajax.open("POST", "<?php echo osc_esc_html(osc_base_url(true) . '?gotRedirect=1'); ?>", true);

      // callback when the status of AJAX is changed
      ajax.onreadystatechange = function() {
        if(this.readyState == 4) {       // request is completed
          if(this.status == 200) {
            //console.log(this.responseText);
          } 
          
          //if(this.status == 500) {
          if(this.status != 200) {
            console.log(this.responseText);
          }
          
          window.location.reload();
        }
      }

      // send google credentials in the AJAX request
      var formData = new FormData();
      formData.append("id_token", googleUser.credential);
      ajax.send(formData);
    }
  </script>
  <?php
  }
}

osc_add_hook('footer', 'got_init_login_button');



// CALLBACK AFTER LOGIN BUTTON IS HIT
// After successfull ajax call, user browser is refreshed
function got_login_callback() {
  if(Params::getParam('gotRedirect') == 1) {
    require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'src/google-api-php-client--' . GOT_API_PHP_VERSION . '/vendor/autoload.php';
    
    $user_id = '';
    $client_id = got_param('client_id');

    if(got_param('enabled') == 1 && $client_id <> '') {
      $client = new Google_Client([
        'client_id' => $client_id
      ]);

      // verify the token sent from AJAX
      $id_token = Params::getParam('id_token');
      
      try{
        // Payload keys: iss, nbf, aud, sub, email, email_verified, azp, name, given_name, picture, family_name, iat, exp, jti
        $payload = $client->verifyIdToken($id_token);
        
        if($payload && isset($payload['aud']) && $payload['aud'] == $client_id) {
          $gpUserData = array();
          $gpUserData['s_oauth_provider'] = 'google';
          $gpUserData['s_oauth_uid'] = isset($payload['sub']) ? $payload['sub'] : '';
          $gpUserData['s_name'] = isset($payload['name']) ? $payload['name'] : '';
          
          if(trim($gpUserData['s_name']) == '') {
            $gpUserData['s_name'] = (isset($payload['given_name']) ? $payload['given_name'] : '') . ' ' . (isset($payload['family_name']) ? $payload['family_name'] : '');
          }
          
          $gpUserData['s_email'] = isset($payload['email']) ? $payload['email'] : '';
          $gpUserData['i_email_verified'] = isset($payload['email_verified']) ? $payload['email_verified'] : 0;
          $gpUserData['s_picture'] = isset($payload['picture']) ? $payload['picture'] : '';
        
          $user_id = ModelGOT::newInstance()->updateUserOneTapData($gpUserData);
        }
      } catch (\Exception $e) {
        $error = $e->getMessage();

        osc_add_flash_error_message(sprintf(__('Login has failed: %s', 'google_onetap_login'), $error));
        
        //header('Location: ' . osc_base_url());
        exit;
      }

      // LOGIN NOW!
      if($user_id > 0) {
        $code = got_login_user($user_id);

        if($code == 3) {
          require_once osc_lib_path() . 'osclass/helpers/hSecurity.php';
          $secret = osc_genRandomPassword();

          ModelGOT::newInstance()->updateUserSecret($user_id, $secret);

          // mb_set_cookie('oc_userId', $user_id);
          // mb_set_cookie('oc_userSecret', $secret);
          Cookie::newInstance()->set_expires(osc_time_cookie());
          Cookie::newInstance()->push('oc_userId', $user_id);
          Cookie::newInstance()->push('oc_userSecret', $secret);
          Cookie::newInstance()->set();
        }

        // Flash messages
        if($code == 0) {
          osc_add_flash_error_message(__('This account does not exist.', 'google_onetap_login'));
        } else if($code == 1) {
          osc_add_flash_error_message(__('This account has not been activated.', 'google_onetap_login'));
        } else if($code == 2) {
          osc_add_flash_error_message(__('This account has been blocked.', 'google_onetap_login'));
        } else if($code == 3) {
          osc_add_flash_ok_message(__('You have been successfully logged in.', 'google_onetap_login'));
        } else if($code == 11) {
          osc_add_flash_error_message(__('Your current email is not allowed.', 'google_onetap_login'));
        } else if($code == 12) {
          osc_add_flash_error_message(__('Your current IP is not allowed.', 'google_onetap_login'));
        } else if($code == 13) {
          osc_add_flash_error_message(__('Your current email & IP is not allowed.', 'google_onetap_login'));
        }  

        $url_redirect = osc_user_dashboard_url();

        if($user_id > 0) {
          $user = User::newInstance()->findByPrimaryKey($user_id);
          osc_run_hook('after_login', $user, $url_redirect);
        }
        
        //header('Location: ' . osc_apply_filter('correct_login_url_redirect', $url_redirect));
        exit;
      } else {
        osc_add_flash_error_message(__('User not found or could not be created.', 'google_onetap_login'));
        
        //header('Location: ' . osc_base_url());
        exit;
      }
      
    } else {
      // token is not verified or expired
      osc_add_flash_error_message(__('Login has failed.', 'google_onetap_login'));
      
      //header('Location: ' . osc_base_url());
      exit;
    }
  }
}

osc_add_hook('init', 'got_login_callback', 2);


// INSTALL FUNCTION - DEFINE VARIABLES
function got_call_after_install() {
  osc_set_preference('enabled', 0, 'plugin-google_onetap_login', 'INTEGER');
  osc_set_preference('client_id', '', 'plugin-google_onetap_login', 'STRING');
  osc_set_preference('exclude_pages', 'LOGIN-RECOVER,CONTACT,PAGE,ITEM-ITEM_EDIT', 'plugin-google_onetap_login', 'STRING');

  ModelGOT::newInstance()->install();
}


function got_call_after_uninstall() {
  ModelGOT::newInstance()->uninstall();
}



// ADMIN MENU
function got_menu($title = NULL) {
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/google_onetap_login/css/admin.css?v=' . date('YmdHis') . '" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/google_onetap_login/css/bootstrap-switch.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/google_onetap_login/css/tipped.css" rel="stylesheet" type="text/css" />';
  echo '<link href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet" type="text/css" />';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/google_onetap_login/js/admin.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/google_onetap_login/js/tipped.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/google_onetap_login/js/bootstrap-switch.js"></script>';



  if($title == '') { $title = __('Configure', 'google_onetap_login'); }

  $text  = '<div class="mb-head">';
  $text .= '<div class="mb-head-left">';
  $text .= '<h1>' . $title . '</h1>';
  $text .= '<h2>Google OneTap Login Osclass Plugin</h2>';
  $text .= '</div>';
  $text .= '<div class="mb-head-right">';
  $text .= '<ul class="mb-menu">';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=google_onetap_login/admin/configure.php"><i class="fa fa-wrench"></i><span>' . __('Configure', 'google_onetap_login') . '</span></a></li>';
  $text .= '</ul>';
  $text .= '</div>';
  $text .= '</div>';

  echo $text;
}



// ADMIN FOOTER
function got_footer() {
  $pluginInfo = osc_plugin_get_info('google_onetap_login/index.php');
  $text  = '<div class="mb-footer">';
  $text .= '<a target="_blank" class="mb-developer" href="https://osclasspoint.com"><img src="https://osclasspoint.com/favicon.ico" alt="MB Themes" /> osclasspoint.com</a>';
  $text .= '<a target="_blank" href="' . $pluginInfo['support_uri'] . '"><i class="fa fa-bug"></i> ' . __('Report Bug', 'google_onetap_login') . '</a>';
  $text .= '<a target="_blank" href="https://forums.osclasspoint.com/"><i class="fa fa-comments"></i> ' . __('Support Forums', 'google_onetap_login') . '</a>';
  $text .= '<a target="_blank" class="mb-last" href="mailto:info@osclasspoint.com"><i class="fa fa-envelope"></i> ' . __('Contact Us', 'google_onetap_login') . '</a>';
  $text .= '<span class="mb-version">v' . $pluginInfo['version'] . '</span>';
  $text .= '</div>';

  return $text;
}



// ADD MENU LINK TO PLUGIN LIST
function got_admin_menu() {
echo '<h3><a href="#">Google OneTap Login Osclass Plugin</a></h3>
<ul> 
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php') . '">&raquo; ' . __('Configure', 'google_onetap_login') . '</a></li>
</ul>';
}


// ADD MENU TO PLUGINS MENU LIST
osc_add_hook('admin_menu','got_admin_menu', 1);



// DISPLAY CONFIGURE LINK IN LIST OF PLUGINS
function got_conf() {
  osc_admin_render_plugin(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php');
}

osc_add_hook(osc_plugin_path(__FILE__) . '_configure', 'got_conf');	


// CALL WHEN PLUGIN IS ACTIVATED - INSTALLED
osc_register_plugin(osc_plugin_path(__FILE__), 'got_call_after_install');

// SHOW UNINSTALL LINK
osc_add_hook(osc_plugin_path(__FILE__) . '_uninstall', 'got_call_after_uninstall');

?>