<?php


// CREATE ITEM LOG
function hsl_create_item_log($item, $action) {
  if(!isset($item['pk_i_id'])) {
    return false;
  }
  
  if(hsl_param('log_item_data') != 1) {
    return false; 
  }
  
  $data = array(
    'fk_i_item_id' => $item['pk_i_id'],
    'fk_i_user_id' => $item['fk_i_user_id'],
    's_ip' => hsl_get_ip(),
    's_contact_name' => $item['s_contact_name'],
    's_contact_email' => $item['s_contact_email'],
    's_contact_phone' => isset($item['s_contact_phone']) ? $item['s_contact_phone'] : null,
    's_contact_other' => isset($item['s_contact_other']) ? $item['s_contact_other'] : null,
    's_category' => $item['s_category'],
    's_location' => hsl_location_from_data($item),
    's_title' => $item['s_title'],
    's_description' => osc_highlight($item['s_description'], 10000),
    's_attribute' => hsl_attribute_from_data($item),
    'i_price' => $item['i_price'],
    'dt_pub_date' => $item['dt_pub_date'],
    's_status' => hsl_status_from_data($item),
    's_action' => $action,
    'dt_date' => date('Y-m-d H:i:s')
  );
  
  ModelHSL::newInstance()->insertItemLog($data);
}

osc_add_hook('posted_item', function($item) {
  hsl_create_item_log($item, 'CREATE');
});

osc_add_hook('edited_item', function($item) {
  hsl_create_item_log($item, 'EDIT');
});

osc_add_hook('activate_item', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'ACTIVATE');
});

osc_add_hook('deactivate_item', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'DEACTIVATE');
});

osc_add_hook('renew_item', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'RENEW');
});

osc_add_hook('enable_item', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'ENABLE');
});

osc_add_hook('disable_item', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'DISABLE');
});

osc_add_hook('item_spam_on', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'SPAM_ON');
});

osc_add_hook('item_spam_off', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'SPAM_OFF');
});

osc_add_hook('item_premium_on', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'PREMIUM_ON');
});

osc_add_hook('item_premium_off', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'PREMIUM_OFF');
});

osc_add_hook('delete_item', function($id) {
  $item = Item::newInstance()->findByPrimaryKey($id);
  hsl_create_item_log($item, 'DELETE');
});


// EXTRACT LOCATION DATA
function hsl_location_from_data($data) {
  $loc = array();
  $loc[] = isset($data['s_country']) ? $data['s_country'] : '';
  $loc[] = isset($data['s_region']) ? $data['s_region'] : '';
  $loc[] = isset($data['s_city']) ? $data['s_city'] : '';
  $loc[] = isset($data['s_city_area']) ? $data['s_city_area'] : '';
  $loc[] = isset($data['s_zip']) ? $data['s_zip'] : '';
  $loc[] = isset($data['s_address']) ? $data['s_address'] : '';

  $loc = array_filter($loc);
  $loc = implode(', ', $loc);
  
  return osc_highlight($loc, 300);
}


// GET CUSTOM FIELDS DATA
function hsl_attribute_from_data($data) {
  if(!isset($data['pk_i_id'])) {
    return '';
  }
  
  $meta = ModelHSL::newInstance()->getItemMeta($data['pk_i_id']);
  $atr = array();
  
  if(is_array($meta) && count($meta) > 0) {
    foreach($meta as $m) {
      $atr[] = '[' . $m['fk_i_field_id'] . '] ' . $m['s_name'] . ': ' . (trim($m['s_value']) <> '' ? trim($m['s_value']) : $m['s_multi']);
    }
  }
  
  $atr = array_filter($atr);
  $atr = implode('; ', $atr);
  
  return osc_highlight($atr, 2000);
}


// GET STATUS FROM DATA
function hsl_status_from_data($data) {
  $sts = array();
  $sts[] = isset($data['b_enabled']) ? ($data['b_enabled'] == 1 ? __('Enabled', 'history_logger') : __('Disabled', 'history_logger')) : '';
  $sts[] = isset($data['b_active']) ? ($data['b_active'] == 1 ? __('Active', 'history_logger') : __('Inactive', 'history_logger')) : '';
  $sts[] = isset($data['b_spam']) ? ($data['b_spam'] == 1 ? __('Spam', 'history_logger') : __('Non-Spam', 'history_logger')) : '';
  $sts[] = isset($data['b_premium']) ? ($data['b_premium'] == 1 ? __('Premium', 'history_logger') : __('Non-Premium', 'history_logger')) : '';
  $sts[] = isset($data['i_renewed']) ? ($data['i_renewed'] > 0 ? __('Renewed', 'history_logger') : __('Non-Renewed', 'history_logger')) : '';
  
  $sts = array_filter($sts);
  $sts = implode(', ', $sts);
  
  return osc_highlight($sts, 100);
}

// MAKE KEY NAME NICE
function hsl_key_name($key) {
  $elems = explode('_', $key);
  $text = '';
  
  if($key == 'pk_i_id') {
    $text = 'ID';
  } else if($key == 's_ip') {
    $text = 'IP';
  } else {
    foreach($elems as $e) {
      if(strlen($e) > 2) {
        $text .= ucfirst($e) . ' ';
      }
    }
  
    if(strlen(trim($text)) <= 0) {
      return $key;
    }
  }
  
  return trim($text);
}


// CREATE USER LOG
function hsl_create_user_log($user, $action) {
  if(!isset($user['pk_i_id'])) {
    return false;
  }
  
  if(hsl_param('log_user_data') != 1) {
    return false; 
  }
  
  $data = array(
    'fk_i_user_id' => $user['pk_i_id'],
    's_ip' => hsl_get_ip(),
    's_name' => $user['s_name'],
    's_email' => $user['s_email'],
    's_phone' => implode(', ', array_filter(array($user['s_phone_mobile'], $user['s_phone_land']))),
    's_location' => hsl_location_from_data($user),
    's_description' => hsl_user_info_from_data($user),
    'dt_reg_date' => $user['dt_reg_date'],
    's_status' => hsl_status_from_data($user),
    's_action' => $action,
    'dt_date' => date('Y-m-d H:i:s')
  );
  
  ModelHSL::newInstance()->insertUserLog($data);
}


osc_add_hook('user_register_completed', function($id) {
  $user = User::newInstance()->findByPrimaryKey($id);
  hsl_create_user_log($user, 'CREATE');
});

osc_add_hook('user_edit_completed', function($id) {
  $user = User::newInstance()->findByPrimaryKey($id);
  hsl_create_user_log($user, 'EDIT');
});

osc_add_hook('activate_user', function($user) {
  hsl_create_user_log($user, 'ACTIVATE');
});

osc_add_hook('dectivate_user', function($user) {
  hsl_create_user_log($user, 'DEACTIVATE');
});

osc_add_hook('enable_user', function($user) {
  hsl_create_user_log($user, 'ENABLE');
});

osc_add_hook('disable_user', function($user) {
  hsl_create_user_log($user, 'DISABLE');
});

osc_add_hook('validate_user', function($user) {
  hsl_create_user_log($user, 'VALIDATE');
});

osc_add_hook('delete_user', function($id) {
  $user = User::newInstance()->findByPrimaryKey($id);
  hsl_create_user_log($user, 'DELETE');
});




// GENERATE PAGINATION
function hsl_admin_paginate($file, $page_id, $per_page, $count_all, $class = '', $params = '') {
  $html = '';
  $page_id = (int)$page_id;
  $page_id = ($page_id <= 0 ? 1 : $page_id);
  $base_link = osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=' . $file . $params;

  if($per_page < $count_all) {
    $html .= '<div id="mb-pagination" class="' . $class . '">';
    $html .= '<div class="mb-pagination-wrap">';
    $html .= '<div>' . __('Page:', 'history_logger') . '</div>';

    $pages = ceil($count_all/$per_page); 
    $page_actual = ($page_id == '' ? 1 : $page_id);

    if($pages > 6) {

      // Too many pages to list them all
      if($page_id == 1) { 
        $ids = array(1,2,3, $pages);

      } else if ($page_id > 1 && $page_id < $pages) {
        $ids = array(1,$page_id-1, $page_id, $page_id+1, $pages);

      } else {
        $ids = array(1, $page_id-2, $page_id-1, $page_id);
      }

      $old = -1;
      $ids = array_unique(array_filter($ids));

      foreach($ids as $i) {
        $url = $base_link . '&pageId=' . $i;
        
        if($old <> -1 && $old <> $i - 1) {
          $html .= '<span>&middot;&middot;&middot;</span>';
        }

        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="mb-active"' : '') . '>' . $i . '</a>';
        $old = $i;
      }

    } else {

      // List all pages
      for ($i = 1; $i <= $pages; $i++) {
        $url = $base_link . '&pageId=' . $i;
        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="mb-active"' : '') . '>' . $i . '</a>';
      }
    }

    $html .= '</div>';
    $html .= '</div>';
  }

  return $html;
}


// GET USER INFO FROM USER OBJECT
function hsl_user_info_from_data($data) {
  if(isset($user['locale']) && is_array($user['locale']) && count($user['locale']) > 0) {
    foreach($user['locale'] as $loc) {
      if(@$loc['s_info'] != '') {
        return osc_highlight($loc['s_info'], 10000);
      }
    }
  }

  return null;
}



// COUNT ITEM LOGS
function hsl_count_item_logs() {
  return ModelHSL::newInstance()->countItemLogs(); 
}


// COUNT USER LOGS
function hsl_count_user_logs() {
  return ModelHSL::newInstance()->countUserLogs(); 
}


// GET CURRENT USER IP
function hsl_get_ip() {
  if(function_exists('osc_get_ip')) {
    return osc_get_ip();
  } else {
    if (isset($_SERVER["HTTP_CF_CONNECTING_IP"])) {
      $_SERVER['REMOTE_ADDR'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
      $_SERVER['HTTP_CLIENT_IP'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
    }
    
    $client = @$_SERVER['HTTP_CLIENT_IP'];
    $forward = @$_SERVER['HTTP_X_FORWARDED_FOR'];
    $remote = @$_SERVER['REMOTE_ADDR'];

    if(filter_var($client, FILTER_VALIDATE_IP)) {
      $ip = $client;
    } elseif(filter_var($forward, FILTER_VALIDATE_IP)) {
      $ip = $forward;
    } else {
      $ip = $remote;
    }

    return $ip;
  }
}



// CORE FUNCTIONS
function hsl_param($name) {
  return osc_get_preference($name, 'plugin-history_logger');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function hsl_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}




// CATEGORIES WORK
function hsl_cat_tree($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
      $array[$c['pk_i_id']]['sub'] = hsl_cat_sub($list, $c['pk_i_id']);
    }
  }

  return $array;
}

function hsl_cat_sub($list, $parent_id) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {  echo $c['s_name'];
        $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
        $array[$c['pk_i_id']]['sub'] = hsl_cat_sub($list, $c['pk_i_id']);
      }
    }
  }
      
  return $array;
}

function hsl_cat_list($selected = array(), $categories = '', $level = 0) {
  if($categories == '' || $level == 0) {
    $categories = hsl_cat_tree($categories);
  }


  foreach($categories as $c) {
    echo '<option value="' . $c['pk_i_id'] . '" ' . (in_array($c['pk_i_id'], $selected) ? 'selected="selected"' : '') . '>' . str_repeat('-', $level) . ($level > 0 ? ' ' : '') . $c['s_name'] . '</option>';

    if(@count($c['sub']) > 0) {
      hsl_cat_list($selected, $c['sub'], $level + 1);
    }
  }
}


function hsl_list_values_ol($values) {
 if(count($values) > 0 && is_array($values)) {
    foreach($values as $v) {
      ?>

      <li class="mb-val" id="val_<?php echo $v['pk_i_id']; ?>">
        <?php hsl_div_value($v); ?>
      
        <ol>
          <?php 
            if(isset($v['values']) && count($v['values']) > 0) { 
              hsl_list_values_ol($v['values']); 
            }
          ?>
        </ol>
      </li>
    <?php
    }
  }
}

?>