<?php
/*
  Plugin Name: History Logger Plugin
  Plugin URI: https://osclasspoint.com/osclass-plugins/protection-and-spam/history-logger-osclass-plugin-i180
  Description: Store historical data about users & listings those are available even after removal from standard tables.
  Version: 1.0.4
  Author: MB Themes
  Author URI: https://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: history_logger
  Plugin update URI: history-logger
  Support URI: https://forums.osclasspoint.com/
  Product Key: DxMMDzXC3d39tvALXnGE
*/

require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'model/ModelHSL.php';
require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'functions.php';


// REMOVE OLD ITEM DATA
function hsl_purge_item_logs($all = false) {
  $limit = (hsl_param('retention') > 0 ? hsl_param('retention') : 24);
  $limit_date = date('Y-m-d H:i:s', strtotime('-' . $limit . ' months'));

  if($all === true) {
    $limit_date = date('Y-m-d H:i:s');
  }
  
  ModelHSL::newInstance()->purgeItemLogs($limit_date);
}

osc_add_hook('cron_daily', 'hsl_purge_item_logs');


// REMOVE OLD USER DATA
function hsl_purge_user_logs($all = false) {
  $limit = (hsl_param('retention') > 0 ? hsl_param('retention') : 24);
  $limit_date = date('Y-m-d H:i:s', strtotime('-' . $limit . ' months'));

  if($all === true) {
    $limit_date = date('Y-m-d H:i:s');
  }
  
  ModelHSL::newInstance()->purgeUserLogs($limit_date);
}

osc_add_hook('cron_daily', 'hsl_purge_user_logs');


// INSTALL FUNCTION - DEFINE VARIABLES
function hsl_call_after_install() {
  osc_set_preference('log_user_data', 0, 'plugin-history_logger', 'INTEGER');
  osc_set_preference('log_item_data', 0, 'plugin-history_logger', 'INTEGER');
  osc_set_preference('retention', 24, 'plugin-history_logger', 'INTEGER');

  ModelHSL::newInstance()->install();
}


function hsl_call_after_uninstall() {
  ModelHSL::newInstance()->uninstall();
}


// ADMIN MENU
function hsl_menu($title = NULL) {
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/history_logger/css/admin.css?v=' . date('YmdHis') . '" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/history_logger/css/bootstrap-switch.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/history_logger/css/tipped.css" rel="stylesheet" type="text/css" />';
  echo '<link href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet" type="text/css" />';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/history_logger/js/admin.js?v=' . date('YmdHis') . '"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/history_logger/js/tipped.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/history_logger/js/bootstrap-switch.js"></script>';

  if( $title == '') { $title = __('Configure', 'history_logger'); }

  $text  = '<div class="mb-head">';
  $text .= '<div class="mb-head-left">';
  $text .= '<h1>' . $title . '</h1>';
  $text .= '<h2>Visitors Checking Item Counter Plugin</h2>';
  $text .= '</div>';
  $text .= '<div class="mb-head-right">';
  $text .= '<ul class="mb-menu">';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=history_logger/admin/configure.php"><i class="fa fa-wrench"></i><span>' . __('Configure', 'history_logger') . '</span></a></li>';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=history_logger/admin/logs_item.php"><i class="fa fa-list"></i><span>' . __('Item Logs', 'history_logger') . '</span></a></li>';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=history_logger/admin/logs_user.php"><i class="fa fa-user"></i><span>' . __('User Logs', 'history_logger') . '</span></a></li>';
  $text .= '</ul>';
  $text .= '</div>';
  $text .= '</div>';

  echo $text;
}



// ADMIN FOOTER
function hsl_footer() {
  $pluginInfo = osc_plugin_get_info('history_logger/index.php');
  $text  = '<div class="mb-footer">';
  $text .= '<a target="_blank" class="mb-developer" href="https://osclasspoint.com"><img src="https://osclasspoint.com/favicon.ico" alt="OsclassPoint Market" /> OsclassPoint Market</a>';
  $text .= '<a target="_blank" href="' . $pluginInfo['support_uri'] . '"><i class="fa fa-bug"></i> ' . __('Report Bug', 'history_logger') . '</a>';
  $text .= '<a target="_blank" href="https://forums.osclasspoint.com/"><i class="fa fa-handshake-o"></i> ' . __('Support Forums', 'history_logger') . '</a>';
  $text .= '<a target="_blank" class="mb-last" href="mailto:info@osclasspoint.com"><i class="fa fa-envelope"></i> ' . __('Contact Us', 'history_logger') . '</a>';
  $text .= '<span class="mb-version">v' . $pluginInfo['version'] . '</span>';
  $text .= '</div>';

  return $text;
}



// ADD MENU LINK TO PLUGIN LIST
function hsl_admin_menu() {
echo '<h3><a href="#">Visitors Checking Item Counter Plugin</a></h3>
<ul> 
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php') . '">&raquo; ' . __('Configure', 'history_logger') . '</a></li>
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/logs_item.php') . '">&raquo; ' . __('Item Logs', 'history_logger') . '</a></li>
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/logs_user.php') . '">&raquo; ' . __('User Logs', 'history_logger') . '</a></li>
</ul>';
}


// ADD MENU TO PLUGINS MENU LIST
osc_add_hook('admin_menu','hsl_admin_menu', 1);



// DISPLAY CONFIGURE LINK IN LIST OF PLUGINS
function hsl_conf() {
  osc_admin_render_plugin( osc_plugin_path( dirname(__FILE__) ) . '/admin/configure.php' );
}

osc_add_hook( osc_plugin_path( __FILE__ ) . '_configure', 'hsl_conf' );	


// CALL WHEN PLUGIN IS ACTIVATED - INSTALLED
osc_register_plugin(osc_plugin_path(__FILE__), 'hsl_call_after_install');

// SHOW UNINSTALL LINK
osc_add_hook(osc_plugin_path(__FILE__) . '_uninstall', 'hsl_call_after_uninstall');

?>