<?php
class ModelHSL extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}

public function getTable_item() {
  return DB_TABLE_PREFIX.'t_item';
}

public function getTable_item_description() {
  return DB_TABLE_PREFIX.'t_item_description';
}

public function getTable_item_location() {
  return DB_TABLE_PREFIX.'t_item_location';
}

public function getTable_item_meta() {
  return DB_TABLE_PREFIX.'t_item_meta';
}

public function getTable_meta() {
  return DB_TABLE_PREFIX.'t_meta_fields';
}

public function getTable_item_log() {
  return DB_TABLE_PREFIX.'t_hsl_item';
}

public function getTable_user() {
  return DB_TABLE_PREFIX.'t_user';
}

public function getTable_user_description() {
  return DB_TABLE_PREFIX.'t_user_description';
}

public function getTable_user_log() {
  return DB_TABLE_PREFIX.'t_hsl_user';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelHSL<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install($version = '') {
  if($version == '') {
    $this->import('history_logger/model/struct.sql');
    osc_set_preference('version', 100, 'plugin-history_logger', 'INTEGER');
  }

}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_item_log()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_user_log()));

  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-history_logger'";
  $this->dao->query($query);
}



// GET ITEM LOG
public function getItemLog($id) {
  $this->dao->select();
  $this->dao->from($this->getTable_item_log());
  $this->dao->where('pk_i_id', $id);
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }
  
  return false;
}


// GET USER LOG
public function getUserLog($id) {
  $this->dao->select();
  $this->dao->from($this->getTable_user_log());
  $this->dao->where('pk_i_id', $id);
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }
  
  return false;
}


// COUNT ALL ITEM LOGS
public function countItemLogs() {
  $this->dao->select('count(*) as i_count');
  $this->dao->from($this->getTable_item_log());
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return isset($data['i_count']) ? $data['i_count'] : 0;
  }
  
  return 0;
}


// COUNT ALL USER LOGS
public function countUserLogs() {
  $this->dao->select('count(*) as i_count');
  $this->dao->from($this->getTable_user_log());
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return isset($data['i_count']) ? $data['i_count'] : 0;
  }
  
  return 0;
}


// GET ITEM META FIELDS
public function getItemMeta($item_id) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_item_meta() . ' as i');
  $this->dao->join($this->getTable_meta() . ' as m', 'i.fk_i_field_id = m.pk_i_id', 'LEFT OUTER');

  $this->dao->where('i.fk_i_item_id', $item_id);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}



// SEARCH ITEM LOGS
public function searchItemLogs($params, $only_count = false) {
  $selector = 'DISTINCT i.*';
  
  if($only_count === true) {
    $selector = 'count(DISTINCT i.pk_i_id) as i_count';
  }
  
  $this->dao->select($selector);
  $this->dao->from($this->getTable_item_log() . ' as i');

  if(isset($params['itemId']) && $params['itemId'] !== '') {
    $this->dao->where('i.fk_i_item_id', $params['itemId']);
  }
  
  if(isset($params['logAction']) && $params['logAction'] !== '') {
    $this->dao->where('i.s_action', $params['logAction']);
  }
  
  if(isset($params['user']) && $params['user'] !== '') {
    $this->dao->like('concat(convert(coalesce(i.fk_i_user_id, 0), char), coalesce(i.s_contact_name, ""), coalesce(i.s_contact_email, ""), coalesce(i.s_contact_phone, ""), coalesce(i.s_contact_other, ""))', $params['user']);
  }
  
  if(isset($params['text']) && $params['text'] !== '') {
    $this->dao->like('concat(coalesce(i.s_title, ""), coalesce(i.s_description, ""), coalesce(i.s_attribute, ""))', $params['text']);
  }
  
  if(isset($params['location']) && $params['location'] !== '') {
    $this->dao->like('i.s_location', $params['location']);
  }
  
  if(isset($params['status']) && $params['status'] !== '') {
    $this->dao->like('i.s_status', $params['status']);
  }

  if(isset($params['date']) && $params['date'] !== '') {
    $this->dao->like('i.dt_date', $params['date']);
  }
  
  if(isset($params['pubDate']) && $params['pubDate'] !== '') {
    $this->dao->like('i.dt_pub_date', $params['pubDate']);
  }


  if($only_count !== true) {
    // $limit[0] == page; $limit[1] == limit
    $page = (isset($params['pageId']) ? $params['pageId'] : 0);
    $per_page = (isset($params['per_page']) ? $params['per_page'] : 30);
    
    if($page > 0) {
      $this->dao->limit(($page-1)*$per_page, $per_page);
    } else if($per_page > 0) {
      $this->dao->limit($per_page);
    }


    if(isset($params['sort']) && $params['sort'] !== '') {
      if($params['sort'] == 'ID_ASC') {
        $this->dao->orderby('i.pk_i_id', 'ASC');
      } else if($params['sort'] == 'ID_DESC') {
        $this->dao->orderby('i.pk_i_id', 'DESC');
      } else if($params['sort'] == 'DATE_ASC') {
        $this->dao->orderby('i.dt_date', 'ASC');
      } else if($params['sort'] == 'DATE_DESC') {
        $this->dao->orderby('i.dt_date', 'DESC');
      } else if($params['sort'] == 'PUBDATE_ASC') {
        $this->dao->orderby('i.dt_pub_date', 'ASC');
      } else if($params['sort'] == 'PUBDATE_DESC') {
        $this->dao->orderby('i.dt_pub_date', 'DESC');
      }
    } else {
      $this->dao->orderby('i.pk_i_id', 'DESC');
    }
  }

  $result = $this->dao->get();
  
  if($result) {
    if($only_count === true) {
      $data = $result->row();
      return isset($data['i_count']) ? $data['i_count'] : 0;
    } else {
      return $result->result();
    }
  }

  return ($only_count ? 0 : array());
}




// SEARCH USER LOGS
public function searchUserLogs($params, $only_count = false) {
  $selector = 'DISTINCT i.*';
  
  if($only_count === true) {
    $selector = 'count(DISTINCT i.pk_i_id) as i_count';
  }
  
  $this->dao->select($selector);
  $this->dao->from($this->getTable_user_log() . ' as i');

  if(isset($params['userId']) && $params['userId'] !== '') {
    $this->dao->where('i.fk_i_user_id', $params['userId']);
  }
  
  if(isset($params['logAction']) && $params['logAction'] !== '') {
    $this->dao->where('i.s_action', $params['logAction']);
  }
 
  if(isset($params['name']) && $params['name'] !== '') {
    $this->dao->like('i.s_name', $params['name']);
  } 
  
  if(isset($params['email']) && $params['email'] !== '') {
    $this->dao->like('i.s_email', $params['email']);
  }
  
  if(isset($params['phone']) && $params['phone'] !== '') {
    $this->dao->like('i.s_phone', $params['phone']);
  }
  
  if(isset($params['text']) && $params['text'] !== '') {
    $this->dao->like('i.s_description', $params['text']);
  }
  
  if(isset($params['location']) && $params['location'] !== '') {
    $this->dao->like('i.s_location', $params['location']);
  }
  
  if(isset($params['status']) && $params['status'] !== '') {
    $this->dao->like('i.s_status', $params['status']);
  }

  if(isset($params['date']) && $params['date'] !== '') {
    $this->dao->like('i.dt_date', $params['date']);
  }
  
  if(isset($params['regDate']) && $params['regDate'] !== '') {
    $this->dao->like('i.dt_reg_date', $params['dt_reg_date']);
  }


  if($only_count !== true) {
    // $limit[0] == page; $limit[1] == limit
    $page = (isset($params['pageId']) ? $params['pageId'] : 0);
    $per_page = (isset($params['per_page']) ? $params['per_page'] : 30);
    
    if($page > 0) {
      $this->dao->limit(($page-1)*$per_page, $per_page);
    } else if($per_page > 0) {
      $this->dao->limit($per_page);
    }


    if(isset($params['sort']) && $params['sort'] !== '') {
      if($params['sort'] == 'ID_ASC') {
        $this->dao->orderby('i.pk_i_id', 'ASC');
      } else if($params['sort'] == 'ID_DESC') {
        $this->dao->orderby('i.pk_i_id', 'DESC');
      } else if($params['sort'] == 'DATE_ASC') {
        $this->dao->orderby('i.dt_date', 'ASC');
      } else if($params['sort'] == 'DATE_DESC') {
        $this->dao->orderby('i.dt_date', 'DESC');
      } else if($params['sort'] == 'REGDATE_ASC') {
        $this->dao->orderby('i.dt_reg_date', 'ASC');
      } else if($params['sort'] == 'REGDATE_DESC') {
        $this->dao->orderby('i.dt_reg_date', 'DESC');
      }
    } else {
      $this->dao->orderby('i.pk_i_id', 'DESC');
    }
  }

  $result = $this->dao->get();
  
  if($result) {
    if($only_count === true) {
      $data = $result->row();
      return isset($data['i_count']) ? $data['i_count'] : 0;
    } else {
      return $result->result();
    }
  }

  return ($only_count ? 0 : array());
}







// INSERT ITEM LOG
public function insertItemLog($data) {
  $this->dao->insert($this->getTable_item_log(), $data);
}


// INSERT USER LOG
public function insertUserLog($data) {
  $this->dao->insert($this->getTable_user_log(), $data);
}

// REMOVE ITEM LOG
public function removeItemLog($id) {
  $this->dao->delete($this->getTable_item_log(), array('pk_i_id' => $id));
}

// REMOVE USER LOG
public function removeUserLog($id) {
  $this->dao->delete($this->getTable_user_log(), array('pk_i_id' => $id));
}


// REMOVE OLD ITEM LOGS
public function purgeItemLogs($limit_date) {
  $query = sprintf('DELETE FROM %s WHERE dt_date < "%s"', $this->getTable_item_log(), $limit_date);
  $this->dao->query($query);
}


// REMOVE OLD USER LOGS
public function purgeUserLogs($limit_date) {
  $query = sprintf('DELETE FROM %s WHERE dt_date < "%s"', $this->getTable_user_log(), $limit_date);
  $this->dao->query($query);
}

}
?>