<?php
/*
  Plugin Name: Image Required Plugin
  Plugin URI: https://osclasspoint.com/osclass-plugins/design-and-appearance/image-required-plugin-i48
  Description: Set picture upload as required element when publishing a listing.
  Version: 1.3.0
  Author: MB Themes
  Author URI: https://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: image_required
  Plugin update URI: image-required-plugin
  Support URI: https://forums.osclasspoint.com/image-required-plugin/
  Product Key: KHrh5ixSocGe8COBeMc7
*/


function ir_call_after_install() {
  osc_set_preference('image_required', 1, 'plugin-image_required', 'INTEGER');
  osc_set_preference('image_min', 1, 'plugin-image_required', 'INTEGER');
}


function ir_call_after_uninstall() {
  osc_delete_preference('image_required', 'plugin-image_required');
  osc_delete_preference('image_min', 'plugin-image_required');
}


// DEFINE CSS STYLES FOR FRONT-END
function ir_user_js() {
  if(ir_get_min_img() > 0 && ir_param('image_required') == 1 && OC_ADMIN !== true) { 
?><script type="text/javascript">
  $(document).ready(function() {
    var ir_img_count;
    setInterval(function(){ 
      if($('.qq-uploader').length || $('.qq-upload-list').length || $('.qq-upload-success').length) {
        var ir_img_count_pre = $('.qq-uploader img[alt^="qqfile_"]').length;
        var ir_img_count_mid = $('.qq-upload-list img[alt^="qqfile_"], .qq-upload-list img[src*="_uppyfile_"]').length;
        var ir_img_count_fin = $('.qq-upload-success img[src*="_thumbnail."]').length;
        var ir_img_count_fin_new = $('.qq-upload-success img[src*="_qqfile_"]').length;

        ir_img_count = ir_img_count_pre + ir_img_count_mid + ir_img_count_fin + ir_img_count_fin_new;
        
        //console.log('Images uploaded: ' + ir_img_count);
        
        if($('#ir_image_check').length) {
          $('#ir_image_check').val(ir_img_count);
        }
      }
    }, 500);

    jQuery.validator.addMethod("irMinPictures", jQuery.validator.methods.min, jQuery.validator.format("<?php echo osc_esc_html(__('Upload at least {0} picture(s)', 'image_required')); ?>"));
    jQuery.validator.addClassRules("ir_image_check", {irMinPictures: <?php echo ir_get_min_img(); ?>});
  });
</script><?php
  }
}

osc_add_hook('footer', function() {
  if(osc_is_publish_page() || osc_is_edit_page()) {
    ir_user_js();
  }
});


osc_add_hook('header', function() {
  if(osc_is_publish_page() || osc_is_edit_page()) {
    osc_enqueue_script('jquery-validate');
  }
});


// GET MINIMUM IMAGES
function ir_get_min_img() {
  $min_img = (ir_param('image_min') > 0 ? ir_param('image_min') : 0);
  return $min_img;
}


// ADD HELP INPUT TO PUBLISH PAGE
function ir_add_input() {
  if(ir_param('image_required') == 1 && OC_ADMIN !== true) {
    $min_img = ir_get_min_img();

    if($min_img > 0) {
      //echo '<input name="ir_image_check" id="ir_image_check" type="text" value="0" class="{required:true, min:' . $min_img . ', messages: {min: \'' . sprintf(__('Upload at least %d picture(s)', 'image_required'), $min_img) . '\'}}"/>';
      echo '<input name="ir_image_check" id="ir_image_check" type="number" style="width:1px;height:1px;min-width:1px;min-height:1px;margin:0;padding:0;border:none;background:transparent;opacity:0.01!important;" value="0" class="ir_image_check"/>';
    }
  }
}

osc_add_hook('item_form', 'ir_add_input', 1);
osc_add_hook('item_edit', 'ir_add_input', 1);



// CHECK IMAGE COUNT IN ADD FUNCTION
function ir_check_img_add($aItem = NULL) {
  $error = false;

  if(ir_param('image_required') == 1 && OC_ADMIN !== true) {
    $min_img = ir_get_min_img();

    if(isset($aItem['idItem']) AND $aItem['idItem'] <> 0 AND $aItem['idItem'] <> '') {
      $img_exists = ItemResource::newInstance()->countResources($aItem['idItem']);
    } else {
      $img_exists = 0;
    }

    if(array_key_exists('name', $aItem['photos'])) {
      $img_uploading = count($aItem['photos']['name']);
      $img_count = $img_exists + $img_uploading;
    } else {
      $img_count = 0;
    }

    if($img_count < $min_img) {
      $error = true;
      osc_add_flash_error_message(sprintf(__('You need to upload at least %d picture(s). You have uploaded only %d picture(s).', 'image_required'), $min_img, $img_count));
    }

    if(!$error) return true;

    header('Location: ' . osc_item_post_url());
    exit;
  }
}


// CHECK IF RUNNING ON DEMO
function ir_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}

function ir_check_img_edit( $aItem = NULL) {
  $error = false;
    
  if(ir_param('image_required') == 1 && OC_ADMIN !== true) {
    $min_img = ir_get_min_img();

    if($aItem['idItem'] <> 0 AND $aItem['idItem'] <> '') {
      $img_exists = ItemResource::newInstance()->countResources( $aItem['idItem'] );
    } else {
      $img_exists = 0;
    }

    if(is_array($aItem['photos']) && array_key_exists('name', $aItem['photos'])) {
      $img_uploading = count($aItem['photos']['name']);
      $img_count = $img_uploading;
    } else {
      $img_count = 0;
    }

    $img_count = $img_count + $img_exists;

    if( $img_count < $min_img) {
      $error = true;
      osc_add_flash_error_message(sprintf(__('You need to upload at least %d picture(s). You have uploaded only %d picture(s).', 'image_required'), $min_img, $img_count));
    }

    if(!$error) return true;

    header('Location: ' . osc_item_edit_url());
    exit;
  }
}

osc_add_hook('pre_item_add', 'ir_check_img_add');
osc_add_hook('pre_item_edit', 'ir_check_img_edit');


// CORE FUNCTIONS
function ir_param($name) {
  return osc_get_preference($name, 'plugin-image_required');
}


if(!function_exists('ir_param_update')) {
  function ir_param_update( $param_name, $update_param_name, $type = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, 'plugin-image_required' ) != '' ) ? osc_get_preference( $param_name, 'plugin-image_required' ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, 'plugin-image_required') != '' ) ? osc_get_preference( $param_name, 'plugin-image_required' ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name ) == '') {
        osc_set_preference( $param_name, $val, 'plugin-image_required', 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => "plugin-image_required", "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}

if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}

if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}

function ir_menu($title = NULL) {
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/image_required/css/admin.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/image_required/css/bootstrap-switch.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/image_required/css/tipped.css" rel="stylesheet" type="text/css" />';
  echo '<link href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet" type="text/css" />';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/image_required/js/admin.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/image_required/js/tipped.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/image_required/js/bootstrap-switch.js"></script>';

  if( $title == '') { $title = __('Configure', 'image_required'); }

  $text  = '<div class="mb-head">';
  $text .= '<div class="mb-head-left">';
  $text .= '<h1>' . $title . '</h1>';
  $text .= '<h2>Image Required Plugin</h2>';
  $text .= '</div>';
  $text .= '<div class="mb-head-right">';
  $text .= '<ul class="mb-menu">';
  $text .= '<li><a href="' . osc_base_url() . 'oc-admin/index.php?page=plugins&action=renderplugin&file=image_required/admin/configure.php"><i class="fa fa-wrench"></i><span>' . __('Configure', 'image_required') . '</span></a></li>';
  $text .= '</ul>';
  $text .= '</div>';
  $text .= '</div>';

  echo $text;
}


function ir_footer() {
  $pluginInfo = osc_plugin_get_info('image_required/index.php');
  $text  = '<div class="mb-footer">';
  $text .= '<a target="_blank" class="mb-developer" href="https://osclasspoint.com"><img src="https://osclasspoint.com/favicon.ico" alt="OsclassPoint Market" /> OsclassPoint Market</a>';
  $text .= '<a target="_blank" href="' . $pluginInfo['support_uri'] . '"><i class="fa fa-bug"></i> ' . __('Report Bug', 'image_required') . '</a>';
  $text .= '<a target="_blank" href="https://forums.osclasspoint.com/"><i class="fa fa-handshake-o"></i> ' . __('Support Forums', 'image_required') . '</a>';
  $text .= '<a target="_blank" class="mb-last" href="mailto:info@osclasspoint.com"><i class="fa fa-envelope"></i> ' . __('Contact Us', 'image_required') . '</a>';
  $text .= '<span class="mb-version">v' . $pluginInfo['version'] . '</span>';
  $text .= '</div>';

  return $text;
}


// Add menu link to Plugin list
function ir_admin_menu() {
echo '<h3><a href="#">Image Required Plugin</a></h3>
<ul> 
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php') . '">&raquo; ' . __('Configure', 'image_required') . '</a></li>
</ul>';
}

// Display configure link
function ir_conf() {
  osc_admin_render_plugin( osc_plugin_path( dirname(__FILE__) ) . '/admin/configure.php' ) ;
}

// This is a hack to show a Configure link at plugins table (you could also use some other hook to show a custom option panel)
osc_add_hook( osc_plugin_path( __FILE__ ) . '_configure', 'ir_conf' ) ;  

//Adding sub menu to plugins menu in dashboard
osc_add_hook('admin_menu','ir_admin_menu', 1);

// This is needed in order to be able to activate the plugin
osc_register_plugin(osc_plugin_path(__FILE__), 'ir_call_after_install') ;

// This is a hack to show a Uninstall link at plugins table (you could also use some other hook to show a custom option panel)
osc_add_hook(osc_plugin_path(__FILE__) . '_uninstall', 'ir_call_after_uninstall');

?>