<?php

// SINGLE VALIDATE FUNCTIONS
function inr_verify_register() { inr_verify_form('register'); }
function inr_verify_login() { inr_verify_form('login'); }
function inr_verify_admin_login() { inr_verify_form('admin_login'); }
function inr_verify_publish() { inr_verify_form('publish'); }
function inr_verify_comment() { inr_verify_form('comment'); }
function inr_verify_friend($item = '') { inr_verify_form('friend', $item); }
function inr_verify_item_contact($item = '') { 
  if(Params::getParam('page') == 'contact' && Params::getParam('action') == 'contact_post'){
    inr_verify_form('item_contact', $item); 
  }
}


// CREATE OWN FILE GET CONTENTS FOR BETTER SUPPORT
function inr_get_contents($url) {
  $ch = curl_init();
  $timeout = 5;
  curl_setopt($ch,CURLOPT_URL,$url);
  curl_setopt($ch,CURLOPT_RETURNTRANSFER,1);
  curl_setopt($ch,CURLOPT_CONNECTTIMEOUT,$timeout);
  $data = curl_exec($ch);
  curl_close($ch);
  return $data;
}


// VALIDATE FUNCTION
function inr_verify_form($hook = '', $item = '') {
  if(!inr_verify()[0]) {
    $action = $hook;

    if(isset($item['pk_i_id'])) {
      View::newInstance()->_exportVariableToView('item', $item);
    }

    osc_add_flash_error_message(sprintf(__('Error: ReCaptcha failed (%s)', 'invisible_recaptcha'), @inr_verify()[1])) ;
    osc_redirect_to(inr_get_url($action));
  }
}


// VALIDATE RECAPTCHA ON SERVER SIDE
function inr_verify() {
  $secret = inr_param('secret_key'); 
  $response = Params::getParam('g-recaptcha-response');
  $remoteip = inr_ip_address();

  $logged_hide = inr_param('logged_hide');
  
  if((osc_is_web_user_logged_in() || osc_is_admin_user_logged_in()) && $logged_hide == 1) {
    return array(true);
  }

  if($secret == '') {
    return array(true);
  }

  if(!$response) {
    return array(false, __('Unable to get recaptcha parameter, invisible recaptcha form was probably not added to this page.', 'invisible_recaptcha'));
  }

  if(!$remoteip) {
    return array(false, __('Unable to retrieve IP address, location is suspicious.', 'invisible_recaptcha'));
  }
	
  $request = inr_get_contents('https://www.google.com/recaptcha/api/siteverify?secret=' . $secret . '&response=' . $response . '&remoteip=' . $remoteip);
  $result = json_decode($request, true);

  if(true == $result['success']) {
    return array(true);
  } else {
    return array(false, __('Google replied that verification has failed. Ensure API keys are correct and that verification works on selected page.', 'invisible_recaptcha'));
  }
}


// GET RETURN LINK BASED ON HOOK
function inr_get_url($action = '') {
  if($action == '') {
    if(@$_SERVER['HTTP_REFERER'] <> '') {
      return @$_SERVER['HTTP_REFERER'];
    } else {
      return osc_base_url();
    }
  } else if($action == 'register') {
    return osc_register_account_url();
  } else if($action == 'admin_login') {
    return osc_admin_base_url(true) . '?page=login';
  } else if($action == 'login') {
    return osc_user_login_url();
  } else if($action == 'publish') {
    return osc_item_post_url();
  } else if($action == 'contact') {
    return osc_contact_url();
  } else if($action == 'item_contact') {
    return osc_item_url();
  } else if($action == 'friend') {
    return osc_item_send_friend_url();
  } else if($action == 'comment') {
    return osc_item_url();
  } }


// FUNCTION GENERATE RECAPTCHA
function inr_generate_fields() {
  if((osc_is_web_user_logged_in() || osc_is_admin_user_logged_in()) && inr_param('logged_hide') == 1) {
    return false;
  }

  $id = inr_get_id();

  echo '<div class="g-recaptcha" id="inr_id_' . $id . '" data-inr-id="' . $id . '" data-callback="onSubmit' . $id . '" data-sitekey="' . inr_param('site_key') . '" data-size="invisible" data-badge="bottomright"></div>';
}

osc_add_hook('invisible_recaptcha', 'inr_generate_fields');


// COUNT ACTIVE RECAPTCHA ON SITE
function inr_get_id() {
  ModelINR::$count++;
  $count = ModelINR::init()->count();
  return $count;
}



// CREATE SCRIPTS TO BE ADDED INTO FOOTER
function inr_footer_scripts() {
  if((osc_is_web_user_logged_in() || osc_is_admin_user_logged_in()) && inr_param('logged_hide') == 1) {
    return false;
  }

  $count = ModelINR::init()->count();

  if($count > 0) {
  ?>

    <script type="text/javascript">
      <?php for ($i = 1; $i <= $count; $i++) { ?>
        var inr_widget_<?php echo $i; ?>;
      <?php } ?>

      var InrOnLoadCallback = function() {
        <?php for ($i = 1; $i <= $count; $i++) { ?>
          inr_widget_<?php echo $i; ?> = grecaptcha.render('inr_id_<?php echo $i; ?>', {
            'sitekey' : '<?php echo inr_param('site_key'); ?>',
            'callback' : onSubmit<?php echo $i; ?>, 
            'badge': '<?php echo (inr_param('badge_position') <> '' ? inr_param('badge_position') : 'inline'); ?>'
          });
        <?php } ?>
      };

      <?php for ($i = 1; $i <= $count; $i++) { ?>
        function onSubmit<?php echo $i; ?>(token) {
          console.log('ReCaptcha OK, ID <?php echo $i; ?>');
          
          $(document).ready(function() { 
            $('#inr_id_<?php echo $i; ?>').closest('form').submit(); 
          });
        };
      <?php } ?>


      function validate(event, id) {
        event.preventDefault();

        if($.validator) {
          if($('#inr_id_' + id).closest('form').valid()) {
            grecaptcha.execute(window['inr_widget_' + id]);
          }
        } else {
          grecaptcha.execute(window['inr_widget_' + id]);
        }
      };


      $(document).ready(function() {
        $('div[id^="inr_id_"]').each(function() {
          var inr_id = $(this).attr('data-inr-id');

          $(this).closest('form').find('button[type="submit"], input[type="submit"]').click(function(e) {
            // HTML5 validation
            var osForm = $(this).closest('form')[0];

            if(!osForm.checkValidity()) {
              osForm.reportValidity();
              console.log('ReCaptcha HTML5 validation failed, ID ' + inr_id);
              return false;
            }
            
            // jQuery validation
            validate(e, inr_id);
          });
        });
      });
    </script>

    <script src="https://www.google.com/recaptcha/api.js?onload=InrOnLoadCallback&render=explicit" async defer/>
    <script>InrOnLoadCallback();</script>

  <?php
  }
  ?>


  <?php if(osc_is_admin_user_logged_in() && inr_param('report') == 1) { ?>
    <script>
      // check if recaptcha was not placed twice into same form
      $(document).ready(function() {
        $('form').each(function() {
          var cnt = $(this).find('div[id^="inr_id_"]').length;
          var msg = '<strong>Invisible Recaptcha Plugin - This message is visible to admin only</strong><br/>';
          msg = msg + 'Action: ' + $(this).find('input[name="action"]').val() + '<br/>';

          if(cnt <= 0) {
            msg = msg + 'This form does not contain recaptcha validation code - form is not protected, add validation code to this form if you want to protect it, as this form does not contain hook';
          } else if(cnt == 1) { 
            msg = msg + 'This form contains 1 recaptcha validation code - that is correct and no action is needed';
          } else if(cnt > 1) { 
            msg = msg + 'This form contains more than 1 recaptcha validation code - that is wrong, remove code you have added as this form contains hook';
          }

          $(this).prepend('<div class="inr-warn">' + msg + '</div>');
        });
      });
    </script>

    <style>.inr-warn {display:inline-block;width:100%;padding:10px;background:#333;color:#fff;font-size:14px;line-height:18px;border-radius:4px;margin:3px 0 15px 0;clear:both;}</style>
  <?php } ?>

<?php
}

osc_add_hook('footer', 'inr_footer_scripts', 9);



// CREATE CLASS TO CALCULATE ACTIVE CAPTCHA FIELDS
class ModelINR {
  public static $instance;
	
  public static $count = 0;
	
  public static function init() {
    if(!self::$instance instanceof self) {
      self::$instance = new self;
    }

    return self::$instance;
  }
		
  public static function count() {
    return self::$count;
  }
}


function inr_ip_address() {
  // check for shared internet/ISP IP
  if (!empty($_SERVER['HTTP_CLIENT_IP']) && inr_validate_ip($_SERVER['HTTP_CLIENT_IP'])) {
    return $_SERVER['HTTP_CLIENT_IP'];
  }

  // check for IPs passing through proxies
  if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
    // check if multiple ips exist in var
    if (strpos($_SERVER['HTTP_X_FORWARDED_FOR'], ',') !== false) {
      $iplist = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']);
      foreach ($iplist as $ip) {
        if (inr_validate_ip($ip))
          return $ip;
      }
    } else {
      if (inr_validate_ip($_SERVER['HTTP_X_FORWARDED_FOR']))
        return $_SERVER['HTTP_X_FORWARDED_FOR'];
    }
  }
  if (!empty($_SERVER['HTTP_X_FORWARDED']) && inr_validate_ip($_SERVER['HTTP_X_FORWARDED']))
    return $_SERVER['HTTP_X_FORWARDED'];
  if (!empty($_SERVER['HTTP_X_CLUSTER_CLIENT_IP']) && inr_validate_ip($_SERVER['HTTP_X_CLUSTER_CLIENT_IP']))
    return $_SERVER['HTTP_X_CLUSTER_CLIENT_IP'];
  if (!empty($_SERVER['HTTP_FORWARDED_FOR']) && inr_validate_ip($_SERVER['HTTP_FORWARDED_FOR']))
    return $_SERVER['HTTP_FORWARDED_FOR'];
  if (!empty($_SERVER['HTTP_FORWARDED']) && inr_validate_ip($_SERVER['HTTP_FORWARDED']))
    return $_SERVER['HTTP_FORWARDED'];

  // return unreliable ip since all else failed
  return $_SERVER['REMOTE_ADDR'];
}


function inr_validate_ip($ip) {
  if (strtolower($ip) === 'unknown')
    return false;

  // generate ipv4 network address
  $ip = ip2long($ip);

  // if the ip is set and not equivalent to 255.255.255.255
  if ($ip !== false && $ip !== -1) {
    // make sure to get unsigned long representation of ip
    // due to discrepancies between 32 and 64 bit OSes and
    // signed numbers (ints default to signed in PHP)
    $ip = sprintf('%u', $ip);
    // do private network range checking
    if ($ip >= 0 && $ip <= 50331647) return false;
    if ($ip >= 167772160 && $ip <= 184549375) return false;
    if ($ip >= 2130706432 && $ip <= 2147483647) return false;
    if ($ip >= 2851995648 && $ip <= 2852061183) return false;
    if ($ip >= 2886729728 && $ip <= 2887778303) return false;
    if ($ip >= 3221225984 && $ip <= 3221226239) return false;
    if ($ip >= 3232235520 && $ip <= 3232301055) return false;
    if ($ip >= 4294967040) return false;
  }
  return true;
}



// CORE FUNCTIONS
function inr_param($name) {
  return osc_get_preference($name, 'plugin-invisible_recaptcha');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function inr_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if( !function_exists('osc_is_contact_page') ) {
  function osc_is_contact_page() {
    $location = Rewrite::newInstance()->get_location();
    $section = Rewrite::newInstance()->get_section();
    if( $location == 'contact' ) {
      return true ;
    }

    return false ;
  }
}


// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}

?>