<?php

function itl_user_is_eligible_for_limit($user_id) {
  if($user_id > 0 && itl_param('disable_membership') == 1) {
    if(function_exists('osp_get_user_group')) {
      if(osp_get_user_group($user_id) > 0) {
        return false;
      }
    }
  }
  
  return true;
}


// CALCULATE LOGGED USER LIMITS
function itl_get_user_limit($user_id) {
  $user = User::newInstance()->findByPrimaryKey($user_id);

  $limit_personal = ((int)itl_param('limit_personal') <= 0 ? -1 : (int)itl_param('limit_personal'));
  $limit_business = ((int)itl_param('limit_business') <= 0 ? -1 : (int)itl_param('limit_business'));
  
  $limit = 0;
  
  if($user !== false && isset($user['pk_i_id']) && $user > 0) {
    $user_limit = ModelITL::newInstance()->getUserLimit($user_id);
    $count_items = ModelITL::newInstance()->countUserItems($user_id, $user['s_email']);

    if($user_limit !== false) {
      if($user_limit == -1) {
        $limit = -1;   // -1 is no limit 
      } else if ($user_limit > 0) {
        $limit = $user_limit;
      }
    } else {
      
      if(@$user['b_company'] == 1) {
        $limit = $limit_business;
      } else {
        $limit = $limit_personal;
      }
    }
  } else {
    $limit = false;
    $count_items = 0;
  }
    
  return array(
    'limit' => $limit,
    'items' => $count_items
  );
}

// USER FORM
function itl_user_limit_form() {
  $user_id = Params::getParam('id');
  $user_limit = ModelITL::newInstance()->getUserLimit($user_id);
  
  $current_url = (isset($_SERVER['HTTPS']) ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

  // for oc-admin
  if ($user_id > 0 && (strpos($current_url, osc_admin_base_url()) !== false || defined('OC_ADMIN'))) {
  ?>
  <div class="form-row">
    <div class="form-label"><?php _e('User item limit', 'item_limit'); ?></div>
    <div class="form-controls">
      <input id="itl_item_limit" type="number" name="itl_item_limit" value="<?php echo ($user_limit == 0 ? '' : $user_limit); ?>">
      <p class="help-inline">
        <?php
          echo __('Enter how many active listings can user have. Enter -1 for unlimited, enter 0 or keep blank to remove this limit.', 'item_limit');
          echo '<br/>';
          
          if($user_limit == 0 || $user_limit == '') {
            _e('No customized limit has been set for this user and is following general (personal/business) limits.', 'item_limit');
          } else if ($user_limit < 0) {
            _e('User has been set for unlimited listings.', 'item_limit');
          } else {
            echo sprintf(__('User can have maximum %s active listings.', 'item_limit'), $user_limit);
          }
        ?>
      </p>
    </div>
  </div>
  <?php
  }
}

osc_add_hook('user_form', 'itl_user_limit_form');


function itl_user_limit_update($user_id) {
  $current_url = (isset($_SERVER['HTTPS']) ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

  // for oc-admin
  if ($user_id > 0 && Params::existParam('itl_item_limit') && (strpos($current_url, osc_admin_base_url()) !== false || defined('OC_ADMIN'))) {
    $item_limit = Params::getParam('itl_item_limit');
    $item_limit = (trim($item_limit) == '' ? 0 : $item_limit);
    $item_limit = ($item_limit < 0 ? -1 : $item_limit);
    
    if($item_limit == 0) {
      ModelITL::newInstance()->removeLimit($user_id);
    } else {
      ModelITL::newInstance()->insertLimit($user_id, $item_limit);
    }
  } 
}

osc_add_hook('user_edit_completed', 'itl_user_limit_update');



// CORE FUNCTIONS
function itl_param($name) {
  return osc_get_preference($name, 'plugin-item_limit');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}

// CATEGORIES LIST
function itl_category_list() {
  $list = itl_get_categories(Category::newInstance()->toTree());
  return $list;
}


function itl_map_category_limit_options($type) {
  if($type == 'NONLOGGED') {
    return 'i_limit_nonlogged';
  } else if($type == 'PERSONAL') {
    return 'i_limit_personal';
  } else if($type == 'BUSINESS') {
    return 'i_limit_business';
  }
}


// GET MAIN CATEGORIES
function itl_get_categories($categories) {
  $list = array();

  foreach($categories as $c) {
    $level = 1;
    $list[] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name'], 'level' => $level, 'fk_i_parent_id' => $c['fk_i_parent_id'] );
   
    if(isset($c['categories']) && is_array($c['categories'])) {
      $list = array_merge($list, itl_get_subcategories($c['categories'], $level, $c['fk_i_parent_id']));
    }
  }

  return $list;
}


// GET SUBCATEGORIES
function itl_get_subcategories($categories, $level = 0, $parent = 0) {
  $level++;
  $list = array();

  foreach($categories as $c) {
    if($level == 2) {
      $parent = $c['fk_i_parent_id'];
    }

    $list[] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name'], 'level' => $level, 'fk_i_parent_id' => $parent );
        
    if(isset($c['categories']) && is_array($c['categories'])) {
      $list = array_merge($list, itl_get_subcategories($c['categories'], $level, $parent));
    }
  }

  return $list;
}


// CREATE CATEGORY TABS
function itl_category_tabs( $level ) {
  $tab = '';

  if( $level == 2) {
    $tab = '<i class="fa fa-angle-right"></i>&nbsp;';
  } else if( $level == 3) {
    $tab = '&nbsp;&nbsp;<i class="fa fa-angle-right"></i><i class="fa fa-angle-right"></i>&nbsp;';
  } else if( $level == 4) {
    $tab = '&nbsp;&nbsp;&nbsp;&nbsp;<i class="fa fa-angle-right"></i><i class="fa fa-angle-right"></i><i class="fa fa-angle-right"></i>&nbsp;';
  }

  return $tab;
}



// CHECK IF RUNNING ON DEMO
function itl_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}

?>