<?php
/*
  Plugin Name: Item Limits Plugin
  Plugin URI: https://osclasspoint.com/osclass-plugins/extra-fields-and-other/item-limits-osclass-plugin-i168
  Description: Plugin allows admin to limit how many listings can single user, personal or company publish
  Version: 1.2.0
  Author: MB Themes
  Author URI: https://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: item_limit
  Plugin update URI: item_limit
  Support URI: https://forums.osclasspoint.com/
  Product Key: oPs2Gjn9C8GS86JspY2R
*/

require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'model/ModelITL.php';
require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'functions.php';



// CREATE ITEM LIMIT
function itl_limit_items($action = 'post') {
  $action = Params::getParam('action');

  $limit_nonlogged = ((int)itl_param('limit_nonlogged') <= 0 ? -1 : (int)itl_param('limit_nonlogged'));
  $limit_personal = ((int)itl_param('limit_personal') <= 0 ? -1 : (int)itl_param('limit_personal'));
  $limit_business = ((int)itl_param('limit_business') <= 0 ? -1 : (int)itl_param('limit_business'));

  $user_id = osc_logged_user_id();
  
  if(itl_user_is_eligible_for_limit($user_id) !== true) {
    return false;
  }
  
  $user = User::newInstance()->findByPrimaryKey($user_id);

  if(Params::getParam('contactEmail') <> '') {
    $email = Params::getParam('contactEmail');
  } else {
    $email = osc_logged_user_email();
  }
  
  if($user_id > 0) {
    $user_limit = ModelITL::newInstance()->getUserLimit($user_id);
    
    if($user_limit !== false) {
      if($user_limit == -1) {
        return false;   // -1 is no limit 
      } else if ($user_limit > 0) {
        $limit = $user_limit;
      }
    } else {
      if(@$user['b_company'] == 1) {
        $limit = $limit_business;
      } else {
        $limit = $limit_personal;
      }
    }
  } else {
    $limit = $limit_nonlogged;
  }

  $item_id = NULL;
  
  if(in_array($action, array('item_edit_post', 'item_edit')) && osc_item_id() > 0) {
    $item_id = osc_item_id();
  }
  
  $count_items = ModelITL::newInstance()->countUserItems($user_id, $email, '', $item_id);
  
  if($count_items >= $limit && $limit > 0) {
    osc_add_flash_error_message(sprintf(__('You reached the maximum number of ads for your subscription. Your limit is %s active listings, but you already have %s active listings.', 'item_limit'), $limit, $count_items));

    header('Location:' . osc_base_url());
    exit;
  }
  
  
  // START OF CATEGORY LIMITS CHECKING
  $category_id = Params::getParam('catId');
  $category_limit = ModelITL::newInstance()->getCategoryLimit($category_id);

  $limit_nonlogged = -1;
  if(isset($category_limit['i_limit_nonlogged']) && $category_limit['i_limit_nonlogged'] !== null && $category_limit['i_limit_nonlogged'] !== '') {
    $limit_nonlogged = $category_limit['i_limit_nonlogged'];
  }

  $limit_personal = -1;
  if(isset($category_limit['i_limit_personal']) && $category_limit['i_limit_personal'] !== null && $category_limit['i_limit_personal'] !== '') {
    $limit_personal = $category_limit['i_limit_personal'];
  } 
  
  $limit_business = -1;
  if(isset($category_limit['i_limit_business']) && $category_limit['i_limit_business'] !== null && $category_limit['i_limit_business'] !== '') {
    $limit_business = $category_limit['i_limit_business'];
  }
  
  
  if($category_id > 0 && $category_limit !== false) {
    $limit = -1;
    
    if(isset($user['pk_i_id']) && $user['pk_i_id'] > 0) {
      if($user['b_company'] == 1) {
        $limit = $limit_business;
      } else {
        $limit = $limit_personal;
      }
    } else {
      $limit = $limit_nonlogged;
    }

    $count_items = ModelITL::newInstance()->countUserItems($user_id, $email, $category_id, $item_id);
    
    if($count_items >= $limit && $limit > 0) {
      osc_add_flash_error_message(sprintf(__('You reached the maximum number of ads in this category. Limit is %s active listings, but you already have %s active listings in this category.', 'item_limit'), $limit, $count_items));

      header('Location:' . osc_base_url());
      exit;
    }  
  }
}



// NOTIFY USER ABOUT ITEM LIMITS
function itl_user_limit_message() {
  if(osc_get_osclass_location() == 'user' && (osc_get_osclass_section() == 'dashboard' || osc_get_osclass_section() == 'items')) {
    if(itl_user_is_eligible_for_limit(osc_logged_user_id()) !== true) {
      return false;
    }

    $data = itl_get_user_limit(osc_logged_user_id());
    $limit = (int)$data['limit'];
    $items = (int)$data['items'];
    
    if($limit > 0) {
      if($limit > $items) {
        osc_add_flash_info_message(sprintf(__('Your current listings limit is %s, you have %s active listings.', 'item_limit'), $limit, $items));
      } else {
        osc_add_flash_error_message(sprintf(__('You have reached your current listings limit %s, you have %s active listings and will not be able to publish new ads.', 'item_limit'), $limit, $items));
      }
    } else {
      osc_add_flash_info_message(__('Your current listings limit is set to unlimited.', 'item_limit'));
    }
  }
}

osc_add_hook('init', 'itl_user_limit_message');


// LIMIT WHEN PUBLISHING ITEM, ONLY IN FRONT
function itl_limit_items_pre_publish() {
  if(!(Params::getParam('page') == 'items' && Params::getParam('action') == 'item_edit_post')) {
    itl_limit_items();
  }
}

osc_add_hook('pre_item_post', 'itl_limit_items_pre_publish');



// LIMIT ENTERING TO PUBLISH PAGE
function itl_limit_items_publish_page() {
  if(osc_is_publish_page()) {
    itl_limit_items();
  }
}

osc_add_hook('init', 'itl_limit_items_publish_page');



// ACTIVATE ITEM LIMIT
function itl_limit_items_activate($item_id) {
  $item = Item::newInstance()->findByPrimaryKey($item_id);

  $limit_nonlogged = ((int)itl_param('limit_nonlogged') <= 0 ? -1 : (int)itl_param('limit_nonlogged'));
  $limit_personal = ((int)itl_param('limit_personal') <= 0 ? -1 : (int)itl_param('limit_personal'));
  $limit_business = ((int)itl_param('limit_business') <= 0 ? -1 : (int)itl_param('limit_business'));
  
  $user_id = osc_logged_user_id();
  
  if(itl_user_is_eligible_for_limit($user_id) !== true) {
    return false;
  }
  
  $user = User::newInstance()->findByPrimaryKey($user_id);

  if(Params::getParam('contactEmail') <> '') {
    $email = Params::getParam('contactEmail');
  } else if(@$item['s_contact_email'] != '') {
    $email = $item['s_contact_email'];
  } else {
    $email = osc_logged_user_email();
  }

  if($user_id > 0) {
    $user_limit = ModelITL::newInstance()->getUserLimit($user_id);
    
    if($user_limit !== false) {
      if($user_limit == -1) {
        return false;   // -1 is no limit 
      } else if ($user_limit > 0) {
        $limit = $user_limit;
      }
    } else {
      if(@$user['b_company'] == 1) {
        $limit = $limit_business;
      } else {
        $limit = $limit_personal;
      }
    }
  } else {
    $limit = $limit_nonlogged;
  }

  $count_items = ModelITL::newInstance()->countUserItems($user_id, $email);
  
  if($count_items >= $limit && $limit > 0) {
    osc_add_flash_error_message(sprintf(__('You reached the maximum number of ads for your subscription. Your limit is %s active listings, but you already have %s active listings. Listing has not been activated.', 'item_limit'), $limit, $count_items));
    ModelITL::newInstance()->deactivateItem($item_id);

    header('Location:' . osc_base_url());
    exit;
  }
  
  
  // START OF CATEGORY LIMITS CHECKING
  $category_id = (isset($item['fk_i_category_id']) ? $item['fk_i_category_id'] : null);
  $category_limit = ModelITL::newInstance()->getCategoryLimit($category_id);

  $limit_nonlogged = -1;
  if(isset($category_limit['i_limit_nonlogged']) && $category_limit['i_limit_nonlogged'] !== null && $category_limit['i_limit_nonlogged'] !== '') {
    $limit_nonlogged = $category_limit['i_limit_nonlogged'];
  }

  $limit_personal = -1;
  if(isset($category_limit['i_limit_personal']) && $category_limit['i_limit_personal'] !== null && $category_limit['i_limit_personal'] !== '') {
    $limit_personal = $category_limit['i_limit_personal'];
  } 
  
  $limit_business = -1;
  if(isset($category_limit['i_limit_business']) && $category_limit['i_limit_business'] !== null && $category_limit['i_limit_business'] !== '') {
    $limit_business = $category_limit['i_limit_business'];
  }
  
  
  if($category_id > 0 && $category_limit !== false) {
    $limit = -1;
    
    if(isset($user['pk_i_id']) && $user['pk_i_id'] > 0) {
      if($user['b_company'] == 1) {
        $limit = $limit_business;
      } else {
        $limit = $limit_personal;
      }
    } else {
      $limit = $limit_nonlogged;
    }

    $count_items = ModelITL::newInstance()->countUserItems($user_id, $email, $category_id);
    
    if($count_items >= $limit && $limit > 0) {
      osc_add_flash_error_message(sprintf(__('You reached the maximum number of ads in this category. Limit is %s active listings, but you already have %s active listings in this category. Listing has not been activated.', 'item_limit'), $limit, $count_items));
      ModelITL::newInstance()->deactivateItem($item_id);

      header('Location:' . osc_base_url());
      exit;
    }  
  }
}

osc_add_hook('activate_item', 'itl_limit_items_activate');


// INSTALL FUNCTION - DEFINE VARIABLES
function itl_call_after_install() {
  osc_set_preference('limit_nonlogged', 3, 'plugin-item_limit', 'INTEGER');
  osc_set_preference('limit_personal', 10, 'plugin-item_limit', 'INTEGER');
  osc_set_preference('limit_business', 50, 'plugin-item_limit', 'INTEGER');
  osc_set_preference('disable_membership', 1, 'plugin-item_limit', 'INTEGER');
  osc_set_preference('count_inactive', 1, 'plugin-item_limit', 'INTEGER');
  osc_set_preference('count_spam', 1, 'plugin-item_limit', 'INTEGER');
  osc_set_preference('count_disabled', 1, 'plugin-item_limit', 'INTEGER');
  osc_set_preference('count_expired', 0, 'plugin-item_limit', 'INTEGER');

  ModelITL::newInstance()->install();
}


function itl_call_after_uninstall() {
  ModelITL::newInstance()->uninstall();
}



// ADMIN MENU
function itl_menu($title = NULL) {
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/item_limit/css/admin.css?v=' . date('YmdHis') . '" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/item_limit/css/bootstrap-switch.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/item_limit/css/tipped.css" rel="stylesheet" type="text/css" />';
  echo '<link href="https://fonts.googleapis.com/css?family=Open+Sans:300,600&amp;subset=latin,latin-ext" rel="stylesheet" type="text/css" />';
  echo '<link href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet" type="text/css" />';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/item_limit/js/admin.js?v=' . date('YmdHis') . '"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/item_limit/js/tipped.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/item_limit/js/bootstrap-switch.js"></script>';

  if( $title == '') { $title = __('Configure', 'item_limit'); }

  $text  = '<div class="mb-head">';
  $text .= '<div class="mb-head-left">';
  $text .= '<h1>' . $title . '</h1>';
  $text .= '<h2>Item Limits Plugin</h2>';
  $text .= '</div>';
  $text .= '<div class="mb-head-right">';
  $text .= '<ul class="mb-menu">';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=item_limit/admin/configure.php"><i class="fa fa-wrench"></i><span>' . __('Configure', 'item_limit') . '</span></a></li>';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=item_limit/admin/category.php"><i class="fa fa-gears"></i><span>' . __('Category limits', 'item_limit') . '</span></a></li>';
  $text .= '</ul>';
  $text .= '</div>';
  $text .= '</div>';

  echo $text;
}



// ADMIN FOOTER
function itl_footer() {
  $pluginInfo = osc_plugin_get_info('item_limit/index.php');
  $text  = '<div class="mb-footer">';
  $text .= '<a target="_blank" class="mb-developer" href="https://osclasspoint.com"><img src="https://osclasspoint.com/favicon.ico" alt="OsclassPoint Market" /> OsclassPoint Market</a>';
  $text .= '<a target="_blank" href="' . $pluginInfo['support_uri'] . '"><i class="fa fa-bug"></i> ' . __('Report Bug', 'item_limit') . '</a>';
  $text .= '<a target="_blank" href="https://forums.osclasspoint.com/"><i class="fa fa-handshake-o"></i> ' . __('Support Forums', 'item_limit') . '</a>';
  $text .= '<a target="_blank" class="mb-last" href="mailto:info@osclasspoint.com"><i class="fa fa-envelope"></i> ' . __('Contact Us', 'item_limit') . '</a>';
  $text .= '<span class="mb-version">v' . $pluginInfo['version'] . '</span>';
  $text .= '</div>';

  return $text;
}


// PLUGIN UPDATE
function itl_update_version() {
  ModelITL::newInstance()->versionUpdate();
}

osc_add_hook(osc_plugin_path(__FILE__) . '_enable', 'itl_update_version');


// ADD MENU LINK TO PLUGIN LIST
function itl_admin_menu() {
echo '<h3><a href="#">Item Limits Plugin</a></h3>
<ul> 
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php') . '">&raquo; ' . __('Configure', 'item_limit') . '</a></li>
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/category.php') . '">&raquo; ' . __('Category limits', 'item_limit') . '</a></li>
</ul>';
}


// ADD MENU TO PLUGINS MENU LIST
osc_add_hook('admin_menu','itl_admin_menu', 1);



// DISPLAY CONFIGURE LINK IN LIST OF PLUGINS
function itl_conf() {
  osc_admin_render_plugin( osc_plugin_path( dirname(__FILE__) ) . '/admin/configure.php' );
}

osc_add_hook( osc_plugin_path( __FILE__ ) . '_configure', 'itl_conf' );	


// CALL WHEN PLUGIN IS ACTIVATED - INSTALLED
osc_register_plugin(osc_plugin_path(__FILE__), 'itl_call_after_install');

// SHOW UNINSTALL LINK
osc_add_hook(osc_plugin_path(__FILE__) . '_uninstall', 'itl_call_after_uninstall');

?>