<?php
class ModelITL extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_item() {
  return DB_TABLE_PREFIX.'t_item';
}

public function getTable_user() {
  return DB_TABLE_PREFIX.'t_user';
}

public function getTable_user_limit() {
  return DB_TABLE_PREFIX.'t_itl_user_limit';
}

public function getTable_category_limit() {
  return DB_TABLE_PREFIX.'t_itl_category_limit';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelITL<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


// DO QUERIES ON VERSION UPDATE
public function versionUpdate() {
  $version = (itl_param('version') <> '' ? itl_param('itl_param') : 100);    // v100 is initial

  if($version == '' || $version < 101) { 
    $this->import('item_limit/model/struct_update_101.sql');
    osc_set_preference('version', 101, 'plugin-item_limit', 'INTEGER');
  }
  
  osc_reset_preferences();
}


public function install($version = '') {
  if($version == '') {
    $this->import('item_limit/model/struct.sql');
    osc_set_preference('version', 100, 'plugin-item_limit', 'INTEGER');
  }
}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_user_limit()));

  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-item_limit'";
  $this->dao->query($query);
}


// GET USER LIMIT
public function getUserLimit($user_id) {
  $this->dao->select();
  $this->dao->from($this->getTable_user_limit());
  $this->dao->where('fk_i_user_id', $user_id);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['fk_i_user_id']) && $data['fk_i_user_id'] > 0) {
      return $data['i_limit'];
    }
  }
  
  return false;
}


// GET CATEGORY LIMIT
public function getCategoryLimit($category_id) {
  if($category_id === null || $category_id <= 0) {
    return false;  
  }
  
  $this->dao->select();
  $this->dao->from($this->getTable_category_limit());
  $this->dao->where('fk_i_category_id', $category_id);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['fk_i_category_id']) && $data['fk_i_category_id'] > 0) {
      return $data;
    }
  }
  
  return false;
}


// GET CATEGORY LIMIT
public function getAllCategoryLimits() {
  $this->dao->select();
  $this->dao->from($this->getTable_category_limit());
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    $output = array();
    
    if(is_array($data) && count($data) > 0) {
      foreach($data as $d) {
        $output[$d['fk_i_category_id']] = $d;
      }
    }
    
    return $output;
  }
  
  return array();
}



// COUNT USER ITEMS
public function countUserItems($id = '', $email = '', $category_id = NULL, $exclude_item_id = NULL) {
  if((int)$id <= 0 && $email == '') {
    return 0;
  }
  
  $this->dao->select('count(*) as i_count');
  $this->dao->from($this->getTable_item());

  if($id > 0) {
    $this->dao->where('fk_i_user_id', $id);
  } else if ($email <> '') {
    $this->dao->where('s_email', $email);
  } else {
    return 0;
  }
  
  if($category_id > 0) {
    $this->dao->where('fk_i_category_id', $category_id);
  }

  if($exclude_item_id > 0) {
    $this->dao->where('pk_i_id != ' . $exclude_item_id);
  }

  if(itl_param('count_disabled') != 1) { 
    $this->dao->where('b_enabled', 1);
  }
  
  if(itl_param('count_inactive') != 1) { 
    $this->dao->where('b_active', 1);
  }
  
  if(itl_param('count_spam') != 1) { 
    $this->dao->where('b_spam', 0);
  }

  if(itl_param('count_expired') != 1) { 
    $this->dao->where('(b_premium = 1 || dt_expiration >= "' . date('Y-m-d H:i:s') .'")');
  }
  
  
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['i_count']) && (int)$data['i_count'] > 0) {
      return (int)$data['i_count'];
    }
  }

  return 0;
}



// INSERT/UPDATE LIMIT
public function updateItemNoExpiration($item_id) {
  return $this->dao->update($this->getTable_item(), array('dt_expiration' => '9999-12-31 23:59:59'), array('pk_i_id' => $item_id));
}

// INSERT/UPDATE LIMIT
public function insertLimit($user_id, $limit) {
  return $this->dao->replace($this->getTable_user_limit(), array('fk_i_user_id' => $user_id, 'i_limit' => $limit), array('fk_i_user_id' => $user_id));
}

// REMOVE LIMIT
public function removeLimit($user_id) {
  return $this->dao->delete($this->getTable_user_limit(), array('fk_i_user_id' => $user_id));
}

// DEACTIVATE ITEM
public function deactivateItem($item_id) {
  return $this->dao->update($this->getTable_item(), array('b_active' => 0), array('pk_i_id' => $item_id));
}

// INSERT/UPDATE LIMIT
public function insertCategoryLimit($category_id, $limit_nonlogged, $limit_personal, $limit_business) {
  return $this->dao->replace($this->getTable_category_limit(), array('fk_i_category_id' => $category_id, 'i_limit_nonlogged' => $limit_nonlogged, 'i_limit_personal' => $limit_personal, 'i_limit_business' => $limit_business), array('fk_i_category_id' => $category_id));
}

// DEACTIVATE ITEM
public function updateCategoryLimit($category_id, $field, $limit) {
  if($limit === '' || $limit === 0 || $limit === null) {
    $limit = null;
  }
  
  if($this->getCategoryLimit($category_id) !== false) {
    return $this->dao->update($this->getTable_category_limit(), array($field => $limit), array('fk_i_category_id' => $category_id));
  } else {
    return $this->dao->insert($this->getTable_category_limit(), array('fk_i_category_id' => $category_id, $field => $limit));
  }
}

// REMOVE LIMIT
public function removeCategoryLimit($category_id) {
  return $this->dao->delete($this->getTable_category_limit(), array('fk_i_category_id' => $category_id));
}

}
?>