<?php

// SHOW ITEM COUNTS
function iou_show_counts($item_id = NULL) {
  if($item_id === NULL) {
    $item_id = osc_item_id();
  }

  if($item_id > 0) {
    $count = iou_item_count($item_id);

    if($count !== false) {
      if(iou_param('category') != '') {
        $cats = explode(',', iou_param('category'));
        $item = Item::newInstance()->findByPrimaryKey($item_id);
        
        if(!in_array($item['fk_i_category_id'], $cats)) {
          return false;
        }
      }
      
      if($count == 1) {
        $message = __('1 person viewed this listing recently.', 'item_online_users');
      } else {
        $message = sprintf(__('%d people viewed this listing recently.', 'item_online_users'), $count);
      }
      
      $position = strtolower(iou_param('position') <> '' ? iou_param('position') : 'TOP-RIGHT');
      $type = strtolower(iou_param('type') <> '' ? iou_param('type') : 'FLOATING');
      $color = strtolower(iou_param('color') <> '' ? iou_param('color') : 'LIGHT');
      $fade = (iou_param('fade') > 0 ? iou_param('fade') : 0)*1000;
      ?>
      <div id="iou-box" class="iou-body iou-box iou-type-<?php echo $type; ?> iou-color-<?php echo $color; ?> iou-position-<?php echo $position; ?>" data-fade="<?php echo $fade; ?>">
        <span><?php echo $message; ?></span>      
        <svg class="iou-close" width="28px" height="28px" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 352 512"><path fill="currentColor" d="M242.72 256l100.07-100.07c12.28-12.28 12.28-32.19 0-44.48l-22.24-22.24c-12.28-12.28-32.19-12.28-44.48 0L176 189.28 75.93 89.21c-12.28-12.28-32.19-12.28-44.48 0L9.21 111.45c-12.28 12.28-12.28 32.19 0 44.48L109.28 256 9.21 356.07c-12.28 12.28-12.28 32.19 0 44.48l22.24 22.24c12.28 12.28 32.2 12.28 44.48 0L176 322.72l100.07 100.07c12.28 12.28 32.2 12.28 44.48 0l22.24-22.24c12.28-12.28 12.28-32.19 0-44.48L242.72 256z"></path></svg>
      </div>      
      <?php
    }
  }
}


// HOOK SHOW ITEM COUNT ON ITEM DETAIL 
function iou_show_counts_hook_detail($item_id) {
  if(iou_param('hook') == 1 && iou_param('type') == 'INLINE' && osc_is_ad_page()) {
    iou_show_counts($item_id);
  }
}

osc_add_hook('item_detail', 'iou_show_counts_hook_detail');


// HOOK SHOW ITEM COUNT ON HEADER
function iou_show_counts_hook_header() {
  if(iou_param('hook') == 1 && iou_param('type') == 'FLOATING' && osc_is_ad_page()) {
    iou_show_counts(osc_item_id());
  }
}

osc_add_hook('footer', 'iou_show_counts_hook_header');


// GET ITEM COUNTS
function iou_item_count($item_id = NULL) {
  if(iou_param('demo') == 1) {
    return rand(50, 500);
  }
  
  if($item_id === NULL) {
    $item_id = osc_item_id();
  }
  
  if($item_id > 0) {
    $ip = iou_get_ip();
    $limit = (iou_param('limit') > 0 ? iou_param('limit') : 30);

    $limit_date = date('Y-m-d H:i:s', strtotime('-' . $limit . ' minutes'));
    
    $count = ModelIOU::newInstance()->getViews($item_id, $limit_date, '');  // not filtering by IP
    
    $multiplier = (iou_param('multiplier') > 0 ? iou_param('multiplier') : 1);
    $base = (iou_param('base') > 0 ? iou_param('base') : 0);
    $min = (iou_param('minimum') > 0 ? iou_param('minimum') : 1);
    
    if($count >= $min) {
      $count_refine = ($base + $count)*$multiplier;
      return round($count_refine);
    }      
  }
  
  return false;
}


// UPDATE COUNTS
function iou_update_views($item_id = NULL) {
  if($item_id === NULL) {
    $item_id = osc_item_id();
    
    if($item_id <= 0) {
      if(Params::getParam('page') == 'item') {
        $item_id = Params::getParam('id');
      }
    }
  }

  if($item_id > 0) {
    $ip = iou_get_ip();

    if($ip != '' && (iou_param('only_logged') == 1 && osc_is_web_user_logged_in() || iou_param('only_logged') == 0)) {
      $data = array(
        'fk_i_item_id' => $item_id,
        's_ip' => $ip,
        'dt_date' => date('Y-m-d H:i:s')
      );
      
      ModelIOU::newInstance()->updateViews($data);
    }
  }
}


// HOOK UPDATE COUNTS
function iou_update_views_hook() {
  if(osc_is_ad_page()) {
    iou_update_views();
  }
}

osc_add_hook('init', 'iou_update_views_hook');


// COUNT ALL DATA
function iou_count_item_online_users() {
  return ModelIOU::newInstance()->countAll(); 
}


// GET CURRENT USER IP
function iou_get_ip() {
  if(function_exists('osc_get_ip')) {
    return osc_get_ip();
  } else {
    if (isset($_SERVER["HTTP_CF_CONNECTING_IP"])) {
      $_SERVER['REMOTE_ADDR'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
      $_SERVER['HTTP_CLIENT_IP'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
    }
    
    $client = @$_SERVER['HTTP_CLIENT_IP'];
    $forward = @$_SERVER['HTTP_X_FORWARDED_FOR'];
    $remote = @$_SERVER['REMOTE_ADDR'];

    if(filter_var($client, FILTER_VALIDATE_IP)) {
      $ip = $client;
    } elseif(filter_var($forward, FILTER_VALIDATE_IP)) {
      $ip = $forward;
    } else {
      $ip = $remote;
    }

    return $ip;
  }
}



// CORE FUNCTIONS
function iou_param($name) {
  return osc_get_preference($name, 'plugin-item_online_users');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function iou_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}




// CATEGORIES WORK
function iou_cat_tree($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
      $array[$c['pk_i_id']]['sub'] = iou_cat_sub($list, $c['pk_i_id']);
    }
  }

  return $array;
}

function iou_cat_sub($list, $parent_id) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {  echo $c['s_name'];
        $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
        $array[$c['pk_i_id']]['sub'] = iou_cat_sub($list, $c['pk_i_id']);
      }
    }
  }
      
  return $array;
}

function iou_cat_list($selected = array(), $categories = '', $level = 0) {
  if($categories == '' || $level == 0) {
    $categories = iou_cat_tree($categories);
  }


  foreach($categories as $c) {
    echo '<option value="' . $c['pk_i_id'] . '" ' . (in_array($c['pk_i_id'], $selected) ? 'selected="selected"' : '') . '>' . str_repeat('-', $level) . ($level > 0 ? ' ' : '') . $c['s_name'] . '</option>';

    if(@count($c['sub']) > 0) {
      iou_cat_list($selected, $c['sub'], $level + 1);
    }
  }
}


function iou_list_values_ol($values) {
 if(count($values) > 0 && is_array($values)) {
    foreach($values as $v) {
      ?>

      <li class="mb-val" id="val_<?php echo $v['pk_i_id']; ?>">
        <?php iou_div_value($v); ?>
      
        <ol>
          <?php 
            if(isset($v['values']) && count($v['values']) > 0) { 
              iou_list_values_ol($v['values']); 
            }
          ?>
        </ol>
      </li>
    <?php
    }
  }
}

?>