<?php
/*
  Plugin Name: Item Validation Plugin
  Plugin URI: https://osclasspoint.com/osclass-plugins/protection-and-spam/item-validation-plugin-i60
  Description: Allow admins to control listings published by users and validate it before going live
  Version: 1.4.4
  Author: MB Themes
  Author URI: http://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: item_validation
  Plugin update URI: item-validation-plugin
  Support URI: http://forums.osclasspoint.com/item-validation-plugin/
  Product Key: 0QDgBnwfOyvG8sUOCizo
*/

require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'model/ModelValidation.php';
require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'email.php';



// INSTALL FUNCTION - DEFINE VARIABLES
function iv_call_after_install() {
  ModelValidation::newInstance()->install();
  
  // General settings
  osc_set_preference('enable', 1, 'plugin-item_validation', 'INTEGER');
  osc_set_preference('validate_edit', 0, 'plugin-item_validation', 'INTEGER');
  osc_set_preference('edit_reapproval', 0, 'plugin-item_validation', 'INTEGER');
  osc_set_preference('min_auto_validated', 3, 'plugin-item_validation', 'INTEGER');
  osc_set_preference('notify_reject', 1, 'plugin-item_validation', 'INTEGER');
  osc_set_preference('notify_approve', 1, 'plugin-item_validation', 'INTEGER');


  // UPLOAD EMAIL TEMPLATES
  foreach(osc_listLocales() as $loc) {
    $email_text  = '<p>Hi {CONTACT_NAME},</p>';
    $email_text .= '<p>You have published new listing {ITEM_URL} to our site.</p>';
    $email_text .= '<p>Your listing has been <strong>APPROVED</strong> and is visible on our site.</p>';
    $email_text .= '<p>Comment: {COMMENT}</p>';
    $email_text .= '<p><br/></p>';
    $email_text .= '<p>You can modify your listing on following link:</p>';
    $email_text .= '<p>{ITEM_EDIT_URL}</p>';
    $email_text .= '<p>If you are registered user, please login into your {ACCOUNT_URL} and modify listing from here.</p>';
    $email_text .= '<p><br/></p>';
    $email_text .= '<p>Thank you, <br />{WEB_TITLE}</p>';

    $iv_notify_approved[$loc['code']]['s_title'] = '[{WEB_TITLE}] Your listing has been approved';
    $iv_notify_approved[$loc['code']]['s_text'] = $email_text;
  }


  foreach(osc_listLocales() as $loc) {
    $email_text  = '<p>Hi {CONTACT_NAME},</p>';
    $email_text .= '<p>You have published new listing {ITEM_URL} to our site.</p>';
    $email_text .= '<p>Unfortunately we could not approve this listing as it did not meet our terms and conditions of use.</p>';
    $email_text .= '<p>Your listing has been <strong>REJECTED</strong> and will not be visible to other users on our site until it is modified and validated by admin.</p>';
    $email_text .= '<p>Comment: {COMMENT}</p>';
    $email_text .= '<p><br/></p>';
    $email_text .= '<p>You can modify your listing on following link:</p>';
    $email_text .= '<p>{ITEM_EDIT_URL}</p>';
    $email_text .= '<p>If you are registered user, please login into your {ACCOUNT_URL} and modify listing from here.</p>';
    $email_text .= '<p></p>';
    $email_text .= '<p>Once you edit your listing, we will re-validate it again.</p>';
    $email_text .= '<p><br/></p>';
    $email_text .= '<p>Thank you, <br />{WEB_TITLE}</p>';

    $iv_notify_rejected[$loc['code']]['s_title'] = '[{WEB_TITLE}] Your listing has been rejected';
    $iv_notify_rejected[$loc['code']]['s_text'] = $email_text;
  }

  Page::newInstance()->insert(array('s_internal_name' => 'iv_email_notify_approved', 'b_indelible' => '1'), $iv_notify_approved);
  Page::newInstance()->insert(array('s_internal_name' => 'iv_email_notify_rejected', 'b_indelible' => '1'), $iv_notify_rejected);
}


function iv_call_after_uninstall() {
  ModelValidation::newInstance()->uninstall();
  osc_delete_preference('enable', 'plugin-item_validation');
  osc_delete_preference('validate_edit', 'plugin-item_validation');
  osc_delete_preference('edit_reapproval', 'plugin-item_validation');
  osc_delete_preference('min_auto_validated', 'plugin-item_validation');
  osc_delete_preference('notify_reject', 'plugin-item_validation');
  osc_delete_preference('notify_approve', 'plugin-item_validation');

  // get list of primary keys of static pages (emails) that should be deleted on uninstall
  $pages = ModelValidation::newInstance()->getPages();  
  foreach($pages as $page) {
    Page::newInstance()->deleteByPrimaryKey($page['pk_i_id']);
  }
}


// PLUGIN UPDATE
function iv_update_version() {
  ModelValidation::newInstance()->versionUpdate();
}

osc_add_hook(osc_plugin_path(__FILE__) . '_enable', 'iv_update_version');



// INCLUDE MAILER SCRIPT
function iv_include_mailer() {
  if(file_exists(osc_lib_path() . 'phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'phpmailer/class.phpmailer.php';
  } else if(file_exists(osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php';
  }
}


// FUNCTION CHANGE ACTIVE TO 1 WHEN SHOWING LISTING TO AVOID SHOWING WEIRD MESSAGES
function iv_fix_active($item) {
  $item['b_active'] = 1;
  return $item;
}

osc_add_filter('pre_show_item', 'iv_fix_active');


// REMOVE ACTIVATE LINKS IN BACKOFFICE
function iv_admin_js() {
  if(osc_get_preference('enable','plugin-item_validation') == 1) {
  ?>
  <script>
    $(document).ready(function() {
      $('body.page-items select[name="bulk_actions"] option[value="activate_all"]').attr('disabled', true).attr('title', '<?php echo osc_esc_js(__('Only available in Item Validation Plugin', 'item_validation')); ?>');
      $('body.page-items.actions a[href*="ACTIVE"], body.page-items .page-items a[href*="ACTIVE"]').each(function() {
        $(this).attr('href', '#').attr('disabled', true).css('opacity', '0.4').addClass('btn-disabled disabled').attr('title', '<?php echo osc_esc_js(__('Only available in Item Validation Plugin', 'item_validation')); ?>').click(function() { return false; });
      });
    });
  </script>
  <?php
  }
}

osc_add_hook('admin_header', 'iv_admin_js', 10);


// WHEN LISTING IS BROWSED, SHOW MESSAGES RELATED TO STATUS
function iv_show_item_messages($item) {
  $enabled = osc_get_preference('enable','plugin-item_validation');
  $valid = ModelValidation::newInstance()->getValidByItemId($item['pk_i_id']);
  $status = isset($valid['i_status']) ? $valid['i_status'] : 0;
  $type = '';
  $text = '';
  
  if($enabled == 1 && isset($item['b_active']) && $item['b_active'] != 1) {
    if($status == 0) {
      $type = 'info';
      $text = __('Your listing is not visible yet, it needs to be validated by admin', 'item_validation');

    } else if ($status == 9) {
      $type = 'error';
      $text = __('Your listing was rejected by admin an will not be visible to other visitors.', 'item_validation');
    }

    if($type != '' && !iv_flash_exists($text, $type)) {
      if($type == 'info') {
        osc_add_flash_info_message($text); 
      } else if ($type == 'error') {
        osc_add_flash_error_message($text); 
      }
    }
  }
}

osc_add_hook('show_item', 'iv_show_item_messages');


// CHECK IF FLASH MESSAGE ALREADY EXISTS
function iv_flash_exists($text, $type = '') {
  $messages = Session::newInstance()->_getMessage('pubMessages');

  if(is_array($messages) && count($messages) > 0) {
    foreach($messages as $m) {
      if($m['msg'] == $text) {
        if($type == '' || $type == $m['type']) {
          return true; 
        }
      }
    }
  }
  
  return false;
}


// COUNT USER VALID ITEMS
function iv_count_validated($user_id = NULL) { 
  if(isset($user_id) && $user_id <> '') {
    $user = ModelValidation::newInstance()->countValidByUserId($user_id, 1);

    if(isset($user['item_count']) && $user['item_count'] <> '') {
      $user_count = $user['item_count'];
    } else {
      $user_count = 0;
    }

  } else {
    $user_count = 0;
  }

  return $user_count;
}

// NEW LISTING HAS BEEN ADDED
function iv_add_item($item) {
  $enabled = osc_get_preference('enable','plugin-item_validation');
  $min_auto_validated = osc_get_preference('min_auto_validated','plugin-item_validation');

  $user_id = $item['fk_i_user_id'];
  $user_count = iv_count_validated($user_id);

  $count_pass = true;
  if($min_auto_validated <= 0 || $min_auto_validated == '') {
    $count_pass = false;
  } else {
    if($min_auto_validated <= $user_count) {
      $count_pass = true;
    } else {
      $count_pass = false;
    }
  }

  if(isset($item['pk_i_id']) && $item['pk_i_id'] > 0 && $enabled == 1) {
    if(!$count_pass) {
      ModelValidation::newInstance()->insertValid($item['pk_i_id'], 0);
      ModelValidation::newInstance()->deactivateItem($item['pk_i_id']);
      iv_update_stats($item, 0);  //decrease stats

      osc_add_flash_info_message(__('Your listing is not visible yet, it needs to be validated by admin', 'item_validation'));
    } else {
      ModelValidation::newInstance()->insertValid($item['pk_i_id'], 1, __('Item auto-validated (user is reliable)', 'item_validation'));
    }
  }
}

osc_add_hook('posted_item', 'iv_add_item', 8);



// LISTING HAS BEEN EDITED
function iv_edit_item($item) {
  $enabled = osc_get_preference('enable','plugin-item_validation');
  $validate_edit = osc_get_preference('validate_edit','plugin-item_validation');
  $edit_reapproval = osc_get_preference('edit_reapproval','plugin-item_validation');
  $min_auto_validated = osc_get_preference('min_auto_validated','plugin-item_validation');
  $status = ModelValidation::newInstance()->getValidByItemId($item['pk_i_id']);
  $status = isset($status['i_status']) ? $status['i_status'] : '';

  $user_id = $item['fk_i_user_id'];
  $user_count = iv_count_validated($user_id);

  $count_pass = false;
  if($min_auto_validated <= 0 || $min_auto_validated == '') {
    $count_pass = false;
  } else {
    if($min_auto_validated <= $user_count) {
      $count_pass = true;
    } else {
      $count_pass = false;
    }
  }

  if(isset($item['pk_i_id']) && $item['pk_i_id'] > 0 && $enabled == 1 && $validate_edit == 1) {
    if(!$count_pass || $status == 9) {
      if($edit_reapproval == 1 && $status == 9) {
        ModelValidation::newInstance()->updateValid($item['pk_i_id'], 0, __('Item send to re-validation (was rejected)', 'item_validation'), 1);
      } else {
        ModelValidation::newInstance()->updateValid($item['pk_i_id'], 0, NULL, 1);
      }


      // IF RE-APPROVAL IS DISABLED AND STATUS IS REJECTED, MAKE SURE LISITNG DOES NOT GO FOR APPROVAL
      if($edit_reapproval == 0 && $status == 9) {
         ModelValidation::newInstance()->updateValid($item['pk_i_id'], 9, __('Item not send to re-validation (keep rejected)', 'item_validation'), 1);
      }

      ModelValidation::newInstance()->deactivateItem($item['pk_i_id']);
      iv_update_stats($item, 0);  //decrease stats
    } else {
      ModelValidation::newInstance()->updateValid($item['pk_i_id'], 1, __('Item auto-validated (user is reliable)', 'item_validation'), 1);
    }
  }
}

osc_add_hook('edited_item', 'iv_edit_item');




// LISTING HAS BEEN ACTIVATED
function iv_activate_item($item_id) {
  $item = Item::newInstance()->findByPrimaryKey($item_id);
  $enabled = osc_get_preference('enable','plugin-item_validation');
  $min_auto_validated = osc_get_preference('min_auto_validated','plugin-item_validation');

  $user_id = $item['fk_i_user_id'];
  $user_count = iv_count_validated($user_id);

  $count_pass = true;
  if($min_auto_validated <= 0 || $min_auto_validated == '') {
    $count_pass = false;
  } else {
    if($min_auto_validated <= $user_count) {
      $count_pass = true;
    } else {
      $count_pass = false;
    }
  }

  $status = ModelValidation::newInstance()->getValidByItemId($item['pk_i_id']);
  $status = isset($status['i_status']) ? $status['i_status'] : '';


  if(isset($item['pk_i_id']) && $item['pk_i_id'] > 0 && $enabled == 1) {
    if(!$count_pass || ($status <> 1 && status <> 2)) {
      ModelValidation::newInstance()->insertValid($item['pk_i_id'], 0);
      ModelValidation::newInstance()->deactivateItem($item['pk_i_id']);
      iv_update_stats($item, 0);  //decrease stats
      
      if(defined('OC_ADMIN') && OC_ADMIN) {
        osc_get_flash_message('admin', true);  // drop messages
        $link = osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=item_validation/admin/validation.php';
        osc_add_flash_error_message(sprintf(__('This function is only available in Item Validation Plugin: %s', 'item_validation'), '<a href="' . $link . '">' . $link . '</a>'), 'admin');

        $redirect_url = osc_admin_base_url(true) . '?page=items';
        
        if(Params::getServerParam('HTTP_REFERER', false, false) != '') {
          $redirect_url = Params::getServerParam('HTTP_REFERER', false, false);
        }
        
        header('Location:' . $redirect_url);
        exit;
      } else {
        osc_add_flash_error_message(__('Your listing could not be activated. All listings are validated by Administrator.', 'item_validation'));
      }
    } else {
      ModelValidation::newInstance()->insertValid($item['pk_i_id'], 1, __('Item auto-validated (user is reliable)', 'item_validation'));
    }
  }
}

osc_add_hook('activate_item', 'iv_activate_item', 10);



// HELP FUNCTIONS
if(!function_exists('mb_param_update')) {
  function mb_param_update($param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL) {
  
    $val = '';
    if($type == 'check') {

      // Checkbox input
      if(Params::getParam($param_name) == 'on') {
        $val = 1;
      } else {
        if(Params::getParam($update_param_name) == 'done') {
          $val = 0;
        } else {
          $val = (osc_get_preference($param_name, $plugin_var_name) != '') ? osc_get_preference($param_name, $plugin_var_name) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if(Params::getParam($update_param_name) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam($param_name);
      } else {
        $val = (osc_get_preference($param_name, $plugin_var_name) != '') ? osc_get_preference($param_name, $plugin_var_name) : '';
      }
    }


    // If save button was pressed, update param
    if(Params::getParam($update_param_name) == 'done') {

      if(osc_get_preference($param_name) == '') {
        osc_set_preference($param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update(array("s_value" => $val), array("s_section" => $plugin_var_name, "s_name" => $param_name));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}



// ADD NOTIFICATION TO ADMIN TOOLBAR MENU
function iv_admin_toolbar(){
  if(!osc_is_moderator()) {
    $total = ModelValidation::newInstance()->countNotValidated();
    $total = isset($total['total_count']) ? $total['total_count'] : 0;

    if($total > 0) {
      $title = '<i class="circle circle-red">'.$total.'</i>' . ($total == 1 ? __('Item to be validated', 'item_validation') : __('Items to be validated', 'item_validation'));
      AdminToolbar::newInstance()->add_menu(
        array(
          'id' => 'item_validation',
          'title' => $title,
          'href'  => osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=item_validation/admin/validation.php',
          'meta'  => array('class' => 'action-btn action-btn-black')
       )
     );
    }
  }
}

osc_add_hook('add_admin_toolbar_menus', 'iv_admin_toolbar', 1);



// CHECK IF RUNNING ON DEMO
function iv_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


// WHEN ITEM IS ACTIVATED/DEACTIVATED, FIX STATS
function iv_update_stats($item, $type) {

  if($type == 0) {

    // decrease
    if($item['fk_i_user_id']!=null) {
      User::newInstance()->decreaseNumItems($item['fk_i_user_id']);
    }

    CategoryStats::newInstance()->decreaseNumItems($item['fk_i_category_id']);
    CountryStats::newInstance()->decreaseNumItems($item['fk_c_country_code']);
    RegionStats::newInstance()->decreaseNumItems($item['fk_i_region_id']);
    CityStats::newInstance()->decreaseNumItems($item['fk_i_city_id']);

    osc_run_hook('item_decrease_stat', $item);

  } else {

    // increase
    if($item['fk_i_user_id']!=null) {
      User::newInstance()->increaseNumItems($item['fk_i_user_id']);
    }

    CategoryStats::newInstance()->increaseNumItems($item['fk_i_category_id']);
    CountryStats::newInstance()->increaseNumItems($item['fk_c_country_code']);
    RegionStats::newInstance()->increaseNumItems($item['fk_i_region_id']);
    CityStats::newInstance()->increaseNumItems($item['fk_i_city_id']);

    osc_run_hook('item_increase_stat', $item);

  }
}


// CORE FUNCTIONS
function iv_param($name) {
  return osc_get_preference($name, 'plugin-item_validation');
}


if(!function_exists('message_ok')) {
  function message_ok($text) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}

if(!function_exists('message_error')) {
  function message_error($text) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// ADMIN MENU
function iv_menu($title = NULL) {
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/item_validation/css/admin.css?v=' . date('YmdHis') . '" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/item_validation/css/bootstrap-switch.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/item_validation/css/tipped.css" rel="stylesheet" type="text/css" />';
  echo '<link href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet" type="text/css" />';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/item_validation/js/admin.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/item_validation/js/tipped.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/item_validation/js/bootstrap-switch.js"></script>';



  if($title == '') { $title = __('Configure', 'item_validation'); }

  $text  = '<div class="mb-head">';
  $text .= '<div class="mb-head-left">';
  $text .= '<h1>' . $title . '</h1>';
  $text .= '<h2>Item Validation Plugin</h2>';
  $text .= '</div>';
  $text .= '<div class="mb-head-right">';
  $text .= '<ul class="mb-menu">';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=item_validation/admin/configure.php"><i class="fa fa-wrench"></i><span>' . __('Configure', 'item_validation') . '</span></a></li>';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=item_validation/admin/validation.php"><i class="fa fa-check"></i><span>' . __('Item Validation', 'item_validation') . '</span></a></li>';
  $text .= '</ul>';
  $text .= '</div>';
  $text .= '</div>';

  echo $text;
}



// ADMIN FOOTER
function iv_footer() {
  $pluginInfo = osc_plugin_get_info('item_validation/index.php');
  $text  = '<div class="mb-footer">';
  $text .= '<a target="_blank" class="mb-developer" href="http://osclasspoint.com"><img src="http://osclasspoint.com/favicon.ico" alt="MB Themes" /> osclasspoint.com</a>';
  $text .= '<a target="_blank" href="' . $pluginInfo['support_uri'] . '"><i class="fa fa-bug"></i> ' . __('Report Bug', 'item_validation') . '</a>';
  $text .= '<a target="_blank" href="http://forums.osclasspoint.com/"><i class="fa fa-comments"></i> ' . __('Support Forums', 'item_validation') . '</a>';
  $text .= '<a target="_blank" class="mb-last" href="mailto:info@osclasspoint.com"><i class="fa fa-envelope"></i> ' . __('Contact Us', 'item_validation') . '</a>';
  $text .= '<span class="mb-version">v' . $pluginInfo['version'] . '</span>';
  $text .= '</div>';

  return $text;
}



// ADD MENU LINK TO PLUGIN LIST
function iv_admin_menu() {
echo '<h3><a href="#">Item Validation Plugin</a></h3>
<ul> 
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php') . '">&raquo; ' . __('Configure', 'item_validation') . '</a></li>
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/validation.php') . '">&raquo; ' . __('Item Validation', 'item_validation') . '</a></li>
</ul>';
}


// ADD MENU TO PLUGINS MENU LIST
osc_add_hook('admin_menu','iv_admin_menu', 1);



// DISPLAY CONFIGURE LINK IN LIST OF PLUGINS
function iv_conf() {
  osc_admin_render_plugin(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php');
}

osc_add_hook(osc_plugin_path(__FILE__) . '_configure', 'iv_conf');	


// CALL WHEN PLUGIN IS ACTIVATED - INSTALLED
osc_register_plugin(osc_plugin_path(__FILE__), 'iv_call_after_install');

// SHOW UNINSTALL LINK
osc_add_hook(osc_plugin_path(__FILE__) . '_uninstall', 'iv_call_after_uninstall');

?>