<?php

class ModelValidation extends DAO {
private static $instance;

public static function newInstance() {
  if(!self::$instance instanceof self) {
    self::$instance = new self ;
  }
  return self::$instance ;
}

function __construct() {
  parent::__construct();
}

public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);
  if(!$this->dao->importSQL($sql)){ throw new Exception("Error importSQL::ModelValidation<br>".$file.'<br>'.$path.'<br><br>Please check your database for if there are no plugin tables. <br>If any of those tables exists in your database, drop them!');} 
}

public function install($version = '') {
  if($version == '') {
    $this->import('item_validation/model/struct.sql');
    osc_set_preference('version', 101, 'plugin-item_validation', 'INTEGER');
  }
}


public function versionUpdate() {
  $version = (iv_param('version') <> '' ? iv_param('version') : 100);    // v100 is initial

  if($version < 101) { 
    $this->dao->query(sprintf("ALTER TABLE %st_item_validation ADD COLUMN b_revalidate TINYINT(1) DEFAULT 0;", DB_TABLE_PREFIX));
    osc_set_preference('version', 101, 'plugin-item_validation', 'INTEGER');
  }

}

public function uninstall() {
  $this->dao->query('DROP TABLE '. $this->getTable_Valid());

}

public function getTable_Valid() {
  return DB_TABLE_PREFIX.'t_item_validation' ;
}

public function getTable_Item() {
  return DB_TABLE_PREFIX.'t_item' ;
}

public function getTable_Page() {
  return DB_TABLE_PREFIX.'t_pages' ;
}


public function insertValid($item_id, $status, $comment = NULL, $revalidate = 0) {
  $data = array(
    'fk_i_item_id' => $item_id,
    'i_status' => $status,
    'b_revalidate' => $revalidate,
    's_comment' => $comment
 );

  return $this->dao->insert($this->getTable_Valid(), $data);
}



public function getValidByItemId($item_id) {
  $this->dao->select();
  $this->dao->from($this->getTable_Valid());
  $this->dao->where('fk_i_item_id', $item_id);

  $result = $this->dao->get();
  if(!$result) { return array(); }
  return $result->row();
}


public function getValidByUserId($user_id, $status = NULL) {
  $this->dao->select('v.*');
  $this->dao->from($this->getTable_Valid() . ' as v, ' . $this->getTable_Item() . ' as i');
  $this->dao->where('v.fk_i_item_id = i.pk_i_id');
  $this->dao->where('i.fk_i_user_id', $user_id);
  
  if(isset($status)) {
    $this->dao->where('v.i_status', $status);
  }
  
  $this->dao->limit(100);
  $this->dao->orderby('v.fk_i_item_id DESC');

  $result = $this->dao->get();
  if(!$result) { return array(); }
  $prepare = $result->result();
  return $prepare;
}


public function countValidByUserId($user_id, $status = NULL) {
  $this->dao->select('count(v.fk_i_item_id) as item_count');
  $this->dao->from($this->getTable_Valid() . ' as v, ' . $this->getTable_Item() . ' as i');
  $this->dao->where('v.fk_i_item_id = i.pk_i_id');
  $this->dao->where('i.fk_i_user_id', $user_id);
  
  if(isset($status)) {
    $this->dao->where('v.i_status', $status);
  }

  $result = $this->dao->get();
  if(!$result) { return array(); }
  return $result->row();
}



public function countNotValidated() {
  $this->dao->select('count(*) as total_count');
  $this->dao->from($this->getTable_Valid());
  
  $this->dao->where('i_status', 0);

  $result = $this->dao->get();
  if(!$result) { return array(); }
  return $result->row();
}


public function getValidByStatus($status = NULL) {
  $this->dao->select();
  $this->dao->from($this->getTable_Valid());
  
  if(isset($status)) {
    $this->dao->where('i_status', $status);
  }

  $this->dao->limit(100);
  $this->dao->orderby('b_revalidate DESC');
  $this->dao->orderby('fk_i_item_id DESC');
  
  $result = $this->dao->get();
  if(!$result) { return array(); }
  $prepare = $result->result();
  return $prepare;
}


public function updateValid($item_id, $status, $comment = NULL, $revalidate = 1) {
  $data = array(
    'i_status' => $status,
    's_comment' => $comment,
    'b_revalidate' => $revalidate
 );

  $where = array('fk_i_item_id' => $item_id);

  return $this->_update($this->getTable_Valid(), $data, $where);
}


public function activateItem($item_id) {
  $data = array(
    'b_active' => 1
 );

  $where = array('pk_i_id' => $item_id);

  return $this->_update($this->getTable_Item(), $data, $where);
}


public function deactivateItem($item_id) {
  $data = array(
    'b_active' => 0
 );

  $where = array('pk_i_id' => $item_id);

  return $this->_update($this->getTable_Item(), $data, $where);
}



public function deleteValid($item_id) {
  return $this->dao->delete($this->getTable_Valid(), array('fk_i_item_id' => $item_id)) ;
}



public function getPages() {
  $this->dao->select('pk_i_id');
  $this->dao->from($this->getTable_Page());
  $this->dao->where('s_internal_name like "iv_%"');

  $result = $this->dao->get();

  if(!$result) { return array(); }
  $prepare = $result->result();
  return $prepare;
}

  

// update function
function _update($table, $values, $where) {
  $this->dao->from($table);
  $this->dao->set($values);
  $this->dao->where($where);
  return $this->dao->update();
}

// End of DAO Class
}
?>