<?php

// INCLUDE MAILER SCRIPT
function lnt_include_mailer() {
  if(file_exists(osc_lib_path() . 'phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'phpmailer/class.phpmailer.php';
  } else if(file_exists(osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php';
  }
}



// IDENTIFY UNUSUAL LOGIN
function lnt_monitor($user) {
  $trigger = false;
  $user_id = @$user['pk_i_id'];
  
  if(lnt_param('enable') == 1 && $user_id > 0) {
    
    // ip check
    if(lnt_param('ip_check') == 1) {
      $ip_months = (lnt_param('ip_old_months') > 0 ? lnt_param('ip_old_months') : 6);
      $ip = lnt_get_ip();
      $old_ip = $user['s_access_ip'];
      
      if(lnt_param('ip_submask') == 1) {
        $ip = lnt_ip_submask($ip);
        $old_ip = lnt_ip_submask($old_ip);
      }
        
      $log = ModelLNT::newInstance()->getByIp($user_id, $ip);
      
      if(($ip != $old_ip || $log !== false) && ($log === false || ($log !== false && strtotime($log['dt_last_date']) < strtotime('-' . $ip_months . ' months')))) {
        lnt_email_notification($user, 'IP');
        lnt_update_warnings($user['s_name'] . ' (' . $user['s_email'] . '), ' . lnt_get_ip() . ', ' . date('Y-m-d H:i:s') . ', ' . __('Reason: IP', 'login_notification'));
        $trigger = true;
      }    
    } 

    // date check
    if($trigger === false && lnt_param('date_check') == 1) {
      $months = (lnt_param('date_months') > 0 ? lnt_param('date_months') : 3);
      
      if(@$user['dt_access_date'] != '' && strtotime($user['dt_access_date']) < strtotime('-' . $months . ' months')) {
        lnt_email_notification($user, 'DATE');
        lnt_update_warnings($user['s_name'] . ' (' . $user['s_email'] . '), ' . lnt_get_ip() . ', ' . date('Y-m-d H:i:s') . ', ' . __('Reason: Date', 'login_notification'));
        $trigger = true;
      }
    }
    

    $log = ModelLNT::newInstance()->getByIp($user_id, lnt_get_ip());

    if($log !== false) {
      ModelLNT::newInstance()->updateData($log['pk_i_id'], array(
        'dt_last_date' => date('Y-m-d H:i:s'),
        'i_used' => $log['i_used'] + 1
      ));
    } else {
      ModelLNT::newInstance()->insertData(array(
        'fk_i_user_id' => $user_id,
        's_ip' => lnt_get_ip(),
        'dt_first_date' => date('Y-m-d H:i:s'),
        'dt_last_date' => date('Y-m-d H:i:s'),
        'i_used' => 1
      ));
    }
  }  
}

osc_add_hook('after_login', 'lnt_monitor', 3);



// ADD DOWNLOADS LOG SECTION TO USER PROFILE
function lnt_user_ips() {
  if(defined('OC_ADMIN') && OC_ADMIN == true && Params::getParam('page') && Params::getParam('id') > 0) {
    $logs = ModelLNT::newInstance()->getLogsByUserId(Params::getParam('id'));
    ?>
    <h3 class="render-title"><?php _e('List of IP addresses used to access account', 'login_notification'); ?></h3>
    <div class="form-row">
      <div class="form-label"><?php _e('Access logs', 'login_notification'); ?></div>
      
      <div class="form-controls" style="width:100%;">
        <textarea readonly style="width:100%;min-width:100%;max-width:100%;height:auto;min-height:180px;max-height:360px;white-space:pre;font-family:monospace;font-size:11px;line-height:12px;"><?php 
          if(count($logs) <= 0) {
            echo __('No IPs found', 'login_notification');
          } else {
            echo str_pad(__('Log ID', 'login_notification'), 15);
            echo str_pad(__('Access IP', 'login_notification'), 30);
            echo str_pad(__('First access date', 'login_notification'), 30);
            echo str_pad(__('Last access date', 'login_notification'), 30);
            echo str_pad(__('Access count', 'login_notification'), 20);
            echo PHP_EOL;
            
            echo str_repeat('-', 125);
            echo PHP_EOL;
              
            foreach($logs as $l) {
              echo str_pad($l['pk_i_id'], 15);
              echo str_pad($l['s_ip'], 30);
              echo str_pad($l['dt_first_date'], 30);
              echo str_pad($l['dt_last_date'], 30);
              echo str_pad($l['i_used'] . 'x', 20);
              echo PHP_EOL;
            }
          } 
        ?></textarea>
      </div>
    </div>
    <?php
  }
}

osc_add_hook('user_form', 'lnt_user_ips', 3);


// GET IP SUBMASK
// Enters 192.168.1.13, returns 192.168.1
// return false for input not of format ip v4
function lnt_ip_submask($ip) {
  $parts = explode('.', $ip);
  
  if(count($parts) >= 3) {
    array_pop($parts);
    return implode('.', $parts);    
  }

  return false;
}


// GET CURRENT USER IP
function lnt_get_ip() {
  if(function_exists('osc_get_ip')) {
    return osc_get_ip();
  } else {
    if (isset($_SERVER["HTTP_CF_CONNECTING_IP"])) {
      $_SERVER['REMOTE_ADDR'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
      $_SERVER['HTTP_CLIENT_IP'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
    }
    
    $client = @$_SERVER['HTTP_CLIENT_IP'];
    $forward = @$_SERVER['HTTP_X_FORWARDED_FOR'];
    $remote = @$_SERVER['REMOTE_ADDR'];

    if(filter_var($client, FILTER_VALIDATE_IP)) {
      $ip = $client;
    } elseif(filter_var($forward, FILTER_VALIDATE_IP)) {
      $ip = $forward;
    } else {
      $ip = $remote;
    }

    return $ip;
  }
}


// UPDATE WARNINGS
function lnt_update_warnings($warning) {
  $raw = trim(lnt_param('last_warnings'));
  $list = ($raw != '' ? json_decode($raw, true) : array());

  if(count($list) >= 25) {
    unset($list[0]);
  }
  
  $list[] = $warning;
  $list = array_values(array_filter($list));
  
  osc_set_preference('last_warnings', json_encode($list), 'plugin-login_notification', 'STRING');
}


// GET WARNINGS
function lnt_get_warnings() {
  $raw = trim(lnt_param('last_warnings'));
  $list = array_reverse($raw != '' ? json_decode($raw, true) : array());
  return $list;
}


// GET USER AGENT/BROWSER
function lnt_get_browser($return_array = false) {
  $u_agent = (isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '');
  $bname = 'Unknown';
  $platform = 'Unknown';
  $version= "";

  //First get the platform?
  if (preg_match('/linux/i', $u_agent)) {
    $platform = 'linux';
  }elseif (preg_match('/macintosh|mac os x/i', $u_agent)) {
    $platform = 'mac';
  }elseif (preg_match('/windows|win32/i', $u_agent)) {
    $platform = 'windows';
  }

  // Next get the name of the useragent yes seperately and for good reason
  if(preg_match('/MSIE/i',$u_agent) && !preg_match('/Opera/i',$u_agent)){
    $bname = 'Internet Explorer';
    $ub = "MSIE";
  } elseif(preg_match('/Firefox/i',$u_agent)){
    $bname = 'Mozilla Firefox';
    $ub = "Firefox";
  } elseif(preg_match('/OPR/i',$u_agent)){
    $bname = 'Opera';
    $ub = "Opera";
  } elseif(preg_match('/Chrome/i',$u_agent) && !preg_match('/Edge/i',$u_agent)){
    $bname = 'Google Chrome';
    $ub = "Chrome";
  } elseif(preg_match('/Safari/i',$u_agent) && !preg_match('/Edge/i',$u_agent)){
    $bname = 'Apple Safari';
    $ub = "Safari";
  } elseif(preg_match('/Netscape/i',$u_agent)){
    $bname = 'Netscape';
    $ub = "Netscape";
  } elseif(preg_match('/Edge/i',$u_agent)){
    $bname = 'Edge';
    $ub = "Edge";
  } elseif(preg_match('/Trident/i',$u_agent)){
    $bname = 'Internet Explorer';
    $ub = "MSIE";
  }

  // finally get the correct version number
  $known = array('Version', $ub, 'other');
  $pattern = '#(?<browser>' . join('|', $known) . ')[/ ]+(?<version>[0-9.|a-zA-Z.]*)#';
  if (!preg_match_all($pattern, $u_agent, $matches)) {
    // we have no matching number just continue
  }
  // see how many we have
  $i = count($matches['browser']);
  if ($i != 1) {
    //we will have two since we are not using 'other' argument yet
    //see if version is before or after the name
    if (strripos($u_agent,"Version") < strripos($u_agent,$ub)){
      $version= $matches['version'][0];
    } else {
      $version= $matches['version'][1];
    }
  }else {
    $version= $matches['version'][0];
  }

  // check if we have a number
  if ($version==null || $version=="") {$version="?";}

  if($return_array == true) {
    return array(
      'userAgent' => $u_agent,
      'name'      => $bname,
      'version'   => $version,
      'platform'  => $platform,
      'pattern'    => $pattern
    );
  }
  
  $return = trim($bname . ' ' . $version);
  $return = ($return <> '' ? $return : $u_agent);
  
  return ($return != '' ? $return : __('Unknown browser', 'login_notification'));
}


// CORE FUNCTIONS
function lnt_param($name) {
  return osc_get_preference($name, 'plugin-login_notification');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function lnt_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}



// CATEGORIES WORK
function lnt_cat_tree($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
      $array[$c['pk_i_id']]['sub'] = lnt_cat_sub($list, $c['pk_i_id']);
    }
  }

  return $array;
}

function lnt_cat_sub($list, $parent_id) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {  echo $c['s_name'];
        $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
        $array[$c['pk_i_id']]['sub'] = lnt_cat_sub($list, $c['pk_i_id']);
      }
    }
  }
      
  return $array;
}

function lnt_cat_list($selected = array(), $categories = '', $level = 0) {
  if($categories == '' || $level == 0) {
    $categories = lnt_cat_tree($categories);
  }


  foreach($categories as $c) {
    echo '<option value="' . $c['pk_i_id'] . '" ' . (in_array($c['pk_i_id'], $selected) ? 'selected="selected"' : '') . '>' . str_repeat('-', $level) . ($level > 0 ? ' ' : '') . $c['s_name'] . '</option>';

    if(@count($c['sub']) > 0) {
      lnt_cat_list($selected, $c['sub'], $level + 1);
    }
  }
}


function lnt_list_values_ol($values) {
 if(count($values) > 0 && is_array($values)) {
    foreach($values as $v) {
      ?>

      <li class="mb-val" id="val_<?php echo $v['pk_i_id']; ?>">
        <?php lnt_div_value($v); ?>
      
        <ol>
          <?php 
            if(isset($v['values']) && count($v['values']) > 0) { 
              lnt_list_values_ol($v['values']); 
            }
          ?>
        </ol>
      </li>
    <?php
    }
  }
}

?>