<?php
class ModelLNT extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}

public function getTable_log() {
  return DB_TABLE_PREFIX.'t_lnt_user_ip_log';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelLNT<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install($version = '') {
  if($version == '') {
    $this->import('login_notification/model/struct.sql');
    osc_set_preference('version', 100, 'plugin-login_notification', 'INTEGER');
    
    
    $locales = OSCLocale::newInstance()->listAllEnabled();

    // NOTIFICATION MAIL
    $lnt_mail_notification = array();
    foreach($locales as $l) {
      $email_text  = '<p>Hi {USER_NAME}</p>';
      $email_text .= '<p>We\'ve detected a new login to your Account at <b>{WEB_TITLE}</b> ({WEB_URL}).<br/>If it was you, you don\'t have to take any action. If not, login to <a href="{ACCOUNT_URL}">your account</a> and change your password immediately.</p>';

      $email_text .= '<p>';
      $email_text .= '<b>Account:</b> {USER_NAME} ({USER_EMAIL})<br/>';
      $email_text .= '<b>Access IP:</b> {USER_IP}<br/>';
      $email_text .= '<b>Browser:</b> {USER_AGENT}';
      $email_text .= '</p>';

      $email_text .= '<p><br/></p>';
      $email_text .= '<p>Thank you, <br />{WEB_TITLE}</p>';

      $lnt_mail_notification[$l['pk_c_code']] = array();
      $lnt_mail_notification[$l['pk_c_code']]['s_title'] = '{WEB_TITLE} - Security warning';
      $lnt_mail_notification[$l['pk_c_code']]['s_text'] = $email_text;
    }
    
    Page::newInstance()->insert(array('s_internal_name' => 'lnt_mail_notification', 'b_indelible' => '1'), $lnt_mail_notification);
  }
}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_log()));


  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-login_notification'";
  $this->dao->query($query);
  
  
  // DELETE ALL EMAIL TEMPLATES
  $mail_notification = Page::newInstance()->findByInternalName('lnt_mail_notification');
  Page::newInstance()->deleteByPrimaryKey($mail_notification['pk_i_id']);
}


// GET LOG BY ID
public function getById($id) {
  $this->dao->select();
  $this->dao->from($this->getTable_log());
  $this->dao->where('pk_i_id', $id);
  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['pk_i_id']) && $data['pk_i_id'] > 0) {
      return $data;
    }
  }
  
  return false;
}


// GET LOG BY ID
public function getByIp($user_id, $ip) {
  $this->dao->select();
  $this->dao->from($this->getTable_log());
  $this->dao->where('fk_i_user_id', $user_id);
  //$this->dao->where('s_ip', $ip);
  $this->dao->like('s_ip', $ip, 'after');

  $this->dao->orderby('dt_last_date', 'DESC');
  $this->dao->limit(1);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['pk_i_id']) && $data['pk_i_id'] > 0) {
      return $data;
    }
  }
  
  return false;
}


// GET LOGS BY USER ID
public function getLogsByUserId($user_id) {
  $this->dao->select();
  $this->dao->from($this->getTable_log());
  $this->dao->where('fk_i_user_id', $user_id);
  $this->dao->orderby('dt_last_date', 'DESC');

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// GET LOGS BY IP
public function getLogsByIp($ip) {
  $this->dao->select();
  $this->dao->from($this->getTable_log());
  $this->dao->where('s_ip', $ip);
  $this->dao->orderby('dt_last_date', 'DESC');

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }
  
  return array();
}


// INSERT LOG
public function insertData($data) {
  $this->dao->insert($this->getTable_log(), $data);
  return $this->dao->insertedId();
}

// UPDATE LOG
public function updateData($id, $data) {
  $this->dao->update($this->getTable_log(), $data, array('pk_i_id' => $id));
}

// REMOVE OLD LOGS
public function purgeLogs($limit_date) {
  $query = sprintf('DELETE FROM %s WHERE dt_date < "%s"', $this->getTable_log(), $limit_date);
  $this->dao->query($query);
}


}
?>