<?php

// MANAGE ITEM POST
function mdr_manage_item_post_edit() {
  $item_id = Params::getParam('mdrItemId');
  $item = Item::newInstance()->findByPrimaryKey($item_id);
  $msection = (Params::getParam('mdrPluginSubaction') == 'FRONT' ? 'pubMessages' : 'admin');

  if(Params::getParam('mdrPluginAction') == 'moderate' && Params::getParam('mdrEnable') == 1 && Params::getParam('mdrIsReversal') != 1) {
    $item_ids = array();
    
    if(Params::getParam('mdrStatusApplyAll') == 1) { 
      osc_add_flash_ok_message(__('Listing status change has been applied to all user items.'), $msection);
      $item_ids = ModelMDR::newInstance()->getItemIdsByEmail(Params::getParam('contactEmail'));
    }
    
    $item_ids[] = $item_id;
    $item_ids = array_filter(array_unique($item_ids));
    
    // Revert item to previous version
    if(Params::getParam('mdrRevertToVersionId') > 0) {
      mdr_revert_item($item_id, Params::getParam('mdrRevertToVersionId'));
    }
    
    // Change item status
    if(Params::getParam('mdrChangeStatus') <> 'NONE' && Params::getParam('mdrChangeStatus') <> '') {
      osc_add_flash_ok_message(__('Listing status change has been successfully completed.'), $msection);
      mdr_change_item_status($item_ids, Params::getParam('mdrChangeStatus'));
    }
    
    // Notify user about moderation
    if(Params::getParam('mdrNotify') == 1) {
      osc_add_flash_ok_message(__('User has been notified about moderation via email.'), $msection);
      mdr_email_user($item_id, Params::getParamsAsArray());
    }

    // Ban user IP
    if(Params::getParam('mdrBan') == 1) {
      osc_add_flash_ok_message(__('User email and IP address has been banned.'), $msection);
      mdr_ban_user(Params::getParam('contactEmail'), Params::getParam('ipAddress'));
    }

    // Block user account
    if(Params::getParam('mdrUserBlock') == 1) {
      osc_add_flash_ok_message(__('User account has been blocked.'), $msection);
      mdr_block_user(isset($item['fk_i_user_id']) ? $item['fk_i_user_id'] : null);
    }

    // Remove item after moderation
    if(Params::getParam('mdrRemove') == 1) {
      osc_add_flash_ok_message(__('Listing has been removed.'), $msection);
      mdr_remove_item($item_id);
    }
    
    $status = null;
    if(Params::getParam('mdrChangeStatus') <> '' && Params::getParam('mdrChangeStatus') <> 'NONE') {
      $status = Params::getParam('mdrChangeStatus');
    }
    
    $data = array(
      'fk_i_item_id' => $item_id,
      'fk_i_archive_id' => (Params::getParam('mdrRevertToVersionId') > 0 ? Params::getParam('mdrRevertToVersionId') : null),
      'fk_i_admin_id' => osc_logged_admin_id(),
      's_admin' => osc_esc_html(osc_logged_admin_name()),
      's_status' => osc_esc_html($status),
      'b_notify' => (Params::getParam('mdrNotify') == 1 ? 1 : 0),
      's_reason' => osc_esc_html(Params::getParam('mdrReason')),
      's_comment' => osc_esc_html(Params::getParam('mdrComment')),
      's_comment_internal' => osc_esc_html(Params::getParam('mdrCommentInternal')),
      'dt_date' => date('Y-m-d H:i:s')  
    );
    
    if(Params::getParam('mdrRemove') != 1) {
      osc_add_flash_ok_message(__('Listing has been successfully moderated.'), $msection);
      $action_id = ModelMDR::newInstance()->insertAction($data);
    }
    
    if(Params::getParam('mdrPluginSubaction') == 'FRONT' && Params::getParam('mdrFormGenerated') == 1) {
      header('Location: ' . osc_item_url_from_item($item));
      exit;    
    }
    
    if(Params::getParam('mdrPluginSubaction') == 'BACK') {
      header('Location: ' . osc_admin_base_url(true) . '?page=items&action=item_edit&id=' . $item_id);
      exit;    
    }
  }  
}

osc_add_hook('posted_item', 'mdr_manage_item_post_edit', 5);
osc_add_hook('edited_item', 'mdr_manage_item_post_edit', 5);

osc_add_hook('init', function() {
  if(osc_is_admin_user_logged_in() && Params::getParam('mdrPluginSubaction') == 'FRONT' && Params::getParam('mdrFormGenerated') == 1) {
    mdr_manage_item_post_edit();
  }  
}, 5);


// REVERT ITEM TO PREVIOUS VERSION
function mdr_revert_item($item_id, $archive_id) {
  $msection = (Params::getParam('mdrPluginSubaction') == 'FRONT' ? 'pubMessages' : 'admin');

  if($item_id > 0 && $archive_id > 0) {
    $item = Item::newInstance()->findByPrimaryKey($item_id);
    $archive = ModelMDR::newInstance()->getArchiveById($archive_id);
    
    if($item !== false && isset($item['pk_i_id'])) {
      if($archive !== false && isset($archive['pk_i_id']) && $archive['fk_i_item_id'] == $item_id) {
        $data = @json_decode($archive['s_data'], true);
        $data['octoken'] = Params::getParam('octoken');
        $data['mdrIsReversal'] = 1;

        if(count($data) > 0) {
          foreach($data as $key => $val) {
            Params::setParam($key, $val);
          }
        }
        
        $mItems = new ItemActions(false);
        $mItems->prepareData(false);

        if ($item !== false && isset($item['pk_i_id'])) {
          View::newInstance()->_exportVariableToView('item', $item);
          $success = $mItems->edit();
          Session::newInstance()->_clearVariables();

          if($success == 1) {
            osc_add_flash_ok_message(__('Listing has been reversed to selected version.'), $msection);
          } else {
            osc_add_flash_error_message(__('Listing reversal has failed.'), $msection);
          }
        }
      }
    }
  }
}


// BLOCK USER ACCOUNT
function mdr_block_user($user_id) {
  if($user_id > 0) {
    require_once LIB_PATH . 'osclass/UserActions.php';

    $userAction = new UserActions(true);
    $userAction->disable($user_id);
  }
}


// CREATE NEW BAN
function mdr_ban_user($email, $ip) {
  BanRule::newInstance()->insert(array(
    's_name' => __('Moderator > Banned by admin', 'moderator'), 
    's_ip' => $ip, 
    's_email' => strtolower($email), 
    'dt_date' => date('Y-m-d H:i:s')
  ));
}


// CHANGE ITEM(S) STATUS
function mdr_change_item_status($item_ids, $status) {
  $itemAction = new ItemActions(true);
  
  if(!is_array($item_ids) && $item_ids > 0) {
    $item_ids = array($item_ids);
  }

  $item_ids = array_filter(array_unique($item_ids));
  
  if(is_array($item_ids) && count($item_ids) > 0) {
    foreach($item_ids as $id) {
      switch($status):
        case 'ACTIVATE':
          $itemAction->activate($id);
          break;
          
        case 'DEACTIVATE':
          $itemAction->deactivate($id);
          break;
          
        case 'BLOCK':
          $itemAction->disable($id);
          break;
          
        case 'UNBLOCK':
          $itemAction->enable($id);
          break;
          
        case 'SPAM':
          $itemAction->spam($id, true);
          break;
          
        case 'UNSPAM':
          $itemAction->spam($id, false);
          break;

        case 'RENEW':
          $itemAction->renew($id);
          break;
          
      endswitch;
    }
  }      
}

// SEND ITEM TO REMOVAL QUEUE
function mdr_remove_item($id) {
  $itemAction = new ItemActions(true);

  if($id > 0) {
    $item = Item::newInstance()->findByPrimaryKey($id);
    
    if($item !== false && isset($item['pk_i_id'])) {
      $itemAction->delete($item['s_secret'], $item['pk_i_id']);
    }
  }
}


// SHOW MODERATE BOX IN FRONT
function mdr_moderate_box_front($generate_form = true) {
?>
  <div id="mdr-frontbox">
    <?php if($generate_form) { ?>
      <form action="<?php echo osc_base_url(); ?>" method="POST" enctype="multipart/form-data" name="moderate">
        <input type="hidden" name="mdrFormGenerated" value="1"/>
    <?php } ?>
    
      <?php mdr_moderate_box('FRONT'); ?>
      
    <?php if($generate_form) { ?>
      </form>
    <?php } ?>
  </div>
<?php
}

osc_add_hook('footer', function() {
  if(osc_is_ad_page() && osc_is_admin_user_logged_in() && mdr_param('show_in_front') == 1) {
    mdr_moderate_box_front(true);
  }
});


// MODERATE BOX WITH ALL DETAILS
function mdr_moderate_box($subaction = '') { 
  $item_id = (osc_item_id() > 0 ? osc_item_id() : (Params::getParam('itemId') > 0 ? Params::getParam('itemId') : Params::getParam('id')));
  $item = Item::newInstance()->findByPrimaryKey($item_id);
  $open = (mdr_param('show_opened') == 1 ? true : false);
  $open = (in_array($subaction, array('FRONT', 'BACK')) ? false : $open);
  $versions = ModelMDR::newInstance()->getArchivesByItemId($item_id, array('ADD','EDIT','ADMIN_ADD','ADMIN_EDIT'));
  $versions_all = ModelMDR::newInstance()->getArchivesByItemId($item_id);
  $actions = ModelMDR::newInstance()->getActionsByItemId($item_id);
?>
  <link href="<?php echo osc_base_url(); ?>oc-content/plugins/moderator/css/user.css?v=<?php echo date('YmdHis'); ?>" rel="stylesheet" type="text/css" />
  <script type="text/javascript" src="<?php echo osc_base_url(); ?>oc-content/plugins/moderator/js/user.js?v=<?php echo date('YmdHis'); ?>"></script>

  <div id="mdr-box">
    <input type="hidden" name="mdrPluginAction" value="moderate"/>
    <input type="hidden" name="mdrPluginSubaction" value="<?php echo $subaction; ?>"/>
    <input type="hidden" name="mdrArchiveUniqueHash" value="<?php echo mb_generate_rand_string(8); ?>"/>
    <input type="hidden" name="mdrItemId" value="<?php echo $item_id; ?>"/>
    
    <div class="mdr-head">
      <?php 
        if(in_array($subaction, array('FRONT', 'BACK'))) { 
          _e('Moderate this item', 'moderator');
        } else {
          _e('Moderate options', 'moderator');
        }
      ?>
      <i class="mdr-arrow fa <?php echo ($open ? 'fa-caret-up' : 'fa-caret-down'); ?>"></i>
    </div>
    
    <div class="mdr-wrap" <?php if(!$open) { ?>style="display:none;"<?php } ?>>
      <div class="mdr-row mdr-row-checkbox mdr-row-first">
        <strong class="mdr-about-head"><?php _e('Enable moderation options', 'moderator'); ?></strong>
        <div class="mdr-about-row"><?php _e('In order to moderate this listing, "Moderate listing" option must be selected, otherwise this section will be ignored.', 'moderator'); ?></div>

        <?php if(!in_array($subaction, array('FRONT', 'BACK'))) { ?>
          <div class="mdr-about-row"><?php _e('By hitting "Submit" button, it will also update listing data, so in case you\'ve modified something, changes will be stored.', 'moderator'); ?></div>
        <?php } ?>
        
        <label>
          <input type="checkbox" id="mdrEnable" name="mdrEnable" <?php if($open) { ?>checked<?php } ?> value="1">
          <span><?php _e('Moderate listing', 'moderator'); ?></span>
        </label>
      </div>
      
      <div class="mdr-row">
        <label for="mdrComment"><?php _e('Comment/feedback for user', 'moderator'); ?></label>
        <textarea id="mdrComment" name="mdrComment"></textarea>
      </div>

      <div class="mdr-row">
        <label for="mdrCommentInternal"><?php _e('Internal comment (not visible to user)', 'moderator'); ?></label>
        <textarea id="mdrCommentInternal" name="mdrCommentInternal"></textarea>
      </div>
      
      <div class="mdr-row mdr-reasons-row">
        <label for="mdrReason"><?php _e('Reason for moderation', 'moderator'); ?></label>
        
        <?php 
          $i = 1; 
          $limit = (mdr_param('shown_reasons') > 0 ? mdr_param('shown_reasons') : 5); 
        ?>
        <?php foreach(mdr_reasons() as $key => $name) { ?>
          <span <?php if($i > $limit) { ?>class="mdr-hide"<?php } ?>><input type="radio" id="<?php echo osc_esc_html($key); ?>" name="mdrReason" value="<?php echo $key; ?>" <?php if($i == 1) { ?>checked<?php } ?>> <label for="<?php echo osc_esc_html($key); ?>"><?php echo $name; ?></label></span>
          <?php $i++; ?>
        <?php } ?>
        
        <?php if(count(mdr_reasons()) > $limit) { ?>
          <a href="#" class="mdr-more-reasons"><?php _e('Show more', 'moderator'); ?> <i class="fa fa-angle-down"></i></a>
        <?php } ?>
      </div>

      <div class="mdr-row">
        <label for="mdrChangeStatus"><?php _e('Change item status', 'moderator'); ?></label>
        <select name="mdrChangeStatus" id="mdrChangeStatus">
          <option value="" selected="selected"><?php _e('Select a status...', 'moderator'); ?></option>

          <?php foreach(mdr_statues() as $key => $name) { ?>
            <option value="<?php echo $key; ?>"><?php echo $name; ?></option>
          <?php } ?>
        </select>
        
        <div class="mdr-help"><?php _e('If you select status change, other updates made in item form will be discarded.', 'moderator'); ?></div>
      </div>


      <div class="mdr-row">
        <label for="mdrRevertToVersionId"><?php _e('Revert item data to previous version', 'moderator'); ?></label>
        <select name="mdrRevertToVersionId" id="mdrRevertToVersionId">
          <?php if(count($versions) <= 0) { ?>
            <option value="" selected="selected"><?php _e('No older versions found', 'moderator'); ?></option>
          <?php } else { ?>
            <option value="" selected="selected"><?php _e('Select an archived version...', 'moderator'); ?></option>
            
            <?php foreach($versions as $v) { ?>
               <?php $mods = @json_decode($v['s_data_changed'], true); ?>
              <option value="<?php echo $v['pk_i_id']; ?>"><?php echo __('Version', 'moderator') . ' #' . $v['pk_i_id'] . ', ' . sprintf(__('%d mod(s)', 'moderator'), count($mods)) . ', ' . mdr_action_name($v['s_action']) . ', ' . date('j. M Y H:i:s', strtotime($v['dt_date'])); ?></option>
            <?php } ?>
          <?php } ?>
        </select>
        
        <div class="mdr-help">
          <?php _e('You can only revert to archive created by add or edit action.', 'moderator'); ?><br/>
          <?php _e('Item pictures will not be reverted!', 'moderator'); ?>
        </div>
      </div>

      <div class="mdr-row mdr-row-label">
        <label><?php _e('Additional actions', 'moderator'); ?></label>
      </div>
        
      <div class="mdr-row mdr-row-checkbox">
        <label>
          <input type="checkbox" id="mdrNotify" name="mdrNotify" value="1" <?php if(mdr_param('default_notify') == 1) { ?>checked<?php } ?>>
          <span><?php _e('Notify user about changes', 'moderator'); ?></span>
        </label>
      </div>
      
      <div class="mdr-row mdr-row-checkbox">
        <label>
          <input type="checkbox" id="mdrRemove" name="mdrRemove" value="1">
          <span><?php _e('Remove item after moderation', 'moderator'); ?></span>
        </label>
      </div>
      
      <div class="mdr-row mdr-row-checkbox">
        <label>
          <input type="checkbox" id="mdrBan" name="mdrBan" value="1">
          <span><?php _e('Ban user IP', 'moderator'); ?></span>
        </label>
      </div>

      <div class="mdr-row mdr-row-checkbox">
        <label>
          <input type="checkbox" id="mdrUserBlock" name="mdrUserBlock" value="1">
          <span><?php _e('Block user account', 'moderator'); ?></span>
        </label>
      </div>
      
      <div class="mdr-row mdr-row-checkbox">
        <label>
          <input type="checkbox" id="mdrStatusApplyAll" name="mdrStatusApplyAll" value="1">
          <span><?php _e('Apply item actions (change status/removal) to all user items', 'moderator'); ?> (<?php echo ModelMDR::newInstance()->countItemIdsByEmail(@$item['s_contact_email']); ?>)</span>
        </label>
      </div>
      
      <div class="mdr-button">
        <button type="submit"><?php _e('Submit', 'moderator'); ?></button>
      </div>
      
      <div class="mdr-versions">
        <div class="mdr-subhead"><?php _e('Previous item versions', 'moderator'); ?></div>
        
        <div class="mdr-subwrap">
          <?php if(count($versions_all) > 0) { ?>
            <?php foreach($versions_all as $v) { ?>
              <?php $mods = @json_decode($v['s_data_changed'], true); ?>
              <div class="mdr-version-line">
                <div class="mdr-icon mdr-has-tooltip" title="<?php _e('Archive ID', 'moderator'); ?> <?php echo $v['pk_i_id']; ?>"><svg width="10" height="10" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path d="M160 80C160 112.8 140.3 140.1 112 153.3V241.1C130.8 230.2 152.7 224 176 224H272C307.3 224 336 195.3 336 160V153.3C307.7 140.1 288 112.8 288 80C288 35.82 323.8 0 368 0C412.2 0 448 35.82 448 80C448 112.8 428.3 140.1 400 153.3V160C400 230.7 342.7 288 272 288H176C140.7 288 112 316.7 112 352V358.7C140.3 371 160 399.2 160 432C160 476.2 124.2 512 80 512C35.82 512 0 476.2 0 432C0 399.2 19.75 371 48 358.7V153.3C19.75 140.1 0 112.8 0 80C0 35.82 35.82 0 80 0C124.2 0 160 35.82 160 80V80zM80 104C93.25 104 104 93.25 104 80C104 66.75 93.25 56 80 56C66.75 56 56 66.75 56 80C56 93.25 66.75 104 80 104zM368 56C354.7 56 344 66.75 344 80C344 93.25 354.7 104 368 104C381.3 104 392 93.25 392 80C392 66.75 381.3 56 368 56zM80 456C93.25 456 104 445.3 104 432C104 418.7 93.25 408 80 408C66.75 408 56 418.7 56 432C56 445.3 66.75 456 80 456z"/></svg></div>
                <div class="mdr-id">#<?php echo $v['i_version']; ?></div>
                <div class="mdr-updates"><?php echo sprintf(__('%d mod(s)', 'moderator'), count($mods)); ?></div>
                <div class="mdr-action"><?php echo mdr_action_name($v['s_action']); ?></div>
                <div class="mdr-date mdr-has-tooltip" title="<?php echo osc_esc_html(date('j. M Y H:i:s', strtotime($v['dt_date']))); ?>"><?php echo date('j. M Y', strtotime($v['dt_date'])); ?></div>
                <div class="mdr-subarrow"><i class="fa fa-caret-down"></i></div>
                
                <div class="mdr-change-list">
                  <?php if(!is_array($mods) || count($mods) <= 0) { ?>
                    <div class="mdr-empty"><?php _e('No captured changes has been found.', 'moderator'); ?></div>
                  <?php } else { ?>
                    <div class="mdr-top"><?php _e('List of identified updates comparing to previous version', 'moderator'); ?></div>
                  
                    <?php foreach($mods as $m) { ?>
                      <div class="mdr-row">
                        <strong class="mdr-col"><?php echo $m['name']; ?><?php echo (@$m['full_key'] <> $m['name'] ? ' (' . $m['full_key'] . ')' : ''); ?> <em><?php echo mdr_edit_type_name($m['type']); ?></em>:</strong>

                        <div class="mdr-vals">
                          <?php if($m['type'] == 'NEW' || $m['type'] == 'MODIFIED') { ?>
                            <div class="mdr-val mdr-green">+ <?php echo $m['new']; ?></div>
                          <?php } ?>
                          
                          <?php if($m['type'] == 'REMOVED' || $m['type'] == 'MODIFIED') { ?>
                            <div class="mdr-val mdr-red">- <?php echo $m['old']; ?></div>
                          <?php } ?>
                        </div>
                      </div>
                    <?php } ?>
                  <?php } ?>

                  <?php if($subaction != 'FRONT') { ?>
                    <div class="mdr-remove-line"><a href="<?php echo osc_admin_base_url(true); ?>?mdrPluginAction=removeVersion&mdrVersionId=<?php echo $v['pk_i_id']; ?>" onclick="return confirm('<?php echo osc_esc_js(__('Are you sure you want to remove this entry? Action cannot be undone', 'moderator')); ?>');"><?php _e('Remove this version', 'moderator'); ?></a></div>
                  <?php } ?>
                </div>
              </div>
            <?php } ?>
          <?php } else { ?>
            <div class="mdr-empty"><?php _e('No previous versions found', 'moderator'); ?></div>
          <?php } ?>
        </div>
      </div>


      <div class="mdr-moderations">
        <div class="mdr-subhead"><?php _e('Previous item moderations', 'moderator'); ?></div>

        <div class="mdr-subwrap">
          <?php if(count($actions) > 0) { ?>
            <?php foreach($actions as $a) { ?>
              <div class="mdr-mod-line">
                <div class="mdr-icon"><svg width="10" height="10" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path d="M490.3 40.4C512.2 62.27 512.2 97.73 490.3 119.6L460.3 149.7L362.3 51.72L392.4 21.66C414.3-.2135 449.7-.2135 471.6 21.66L490.3 40.4zM172.4 241.7L339.7 74.34L437.7 172.3L270.3 339.6C264.2 345.8 256.7 350.4 248.4 353.2L159.6 382.8C150.1 385.6 141.5 383.4 135 376.1C128.6 370.5 126.4 361 129.2 352.4L158.8 263.6C161.6 255.3 166.2 247.8 172.4 241.7V241.7zM192 63.1C209.7 63.1 224 78.33 224 95.1C224 113.7 209.7 127.1 192 127.1H96C78.33 127.1 64 142.3 64 159.1V416C64 433.7 78.33 448 96 448H352C369.7 448 384 433.7 384 416V319.1C384 302.3 398.3 287.1 416 287.1C433.7 287.1 448 302.3 448 319.1V416C448 469 405 512 352 512H96C42.98 512 0 469 0 416V159.1C0 106.1 42.98 63.1 96 63.1H192z"/></svg></div>
                <div class="mdr-id">#<?php echo $a['pk_i_id']; ?></div>
                <div class="mdr-admin"><?php echo $a['s_admin']; ?></div>
                <div class="mdr-reason"><?php echo mdr_reason_name($a['s_reason']); ?></div>
                <div class="mdr-subarrow"><i class="fa fa-caret-down"></i></div>
                
                <div class="mdr-mod-list">
                  <div class="mdr-row">
                    <strong class="mdr-col"><?php _e('Comment/feedback for user', 'moderator'); ?>:</strong>
                    <div class="mdr-val"><?php echo nl2br(trim($a['s_comment']) <> '' ? $a['s_comment'] : '-'); ?></div>
                  </div>
                  
                  <div class="mdr-row">
                    <strong class="mdr-col"><?php _e('Internal comment (not visible to user)', 'moderator'); ?>:</strong>
                    <div class="mdr-val"><?php echo nl2br(trim($a['s_comment_internal']) <> '' ? $a['s_comment_internal'] : '-'); ?></div>
                  </div>
                  
                  <div class="mdr-row">
                    <strong class="mdr-col"><?php _e('Change item status', 'moderator'); ?>:</strong>
                    <div class="mdr-val"><?php echo mdr_status_name($a['s_status']); ?></div>
                  </div>
                  
                  <div class="mdr-row">
                    <strong class="mdr-col"><?php _e('Notify user about changes', 'moderator'); ?>:</strong>
                    <div class="mdr-val"><?php echo ($a['b_notify'] == 1 ? __('Yes', 'moderator') : __('No', 'moderator')); ?></div>
                  </div>

                  <div class="mdr-row">
                    <strong class="mdr-col"><?php _e('Revert item data to previous version', 'moderator'); ?>:</strong>
                    <div class="mdr-val"><?php echo ($a['fk_i_archive_id'] > 0 ? __('Archive record', 'moderator') . ' #' . $a['fk_i_archive_id'] : __('No', 'moderator')); ?></div>
                  </div>
                  
                  <div class="mdr-row">
                    <strong class="mdr-col"><?php _e('Moderation date', 'moderator'); ?>:</strong>
                    <div class="mdr-val"><?php echo date('j. M Y H:i:s'); ?></div>
                  </div>
                  
                  <?php if($subaction != 'FRONT') { ?>
                    <div class="mdr-remove-line"><a href="<?php echo osc_admin_base_url(true); ?>?mdrPluginAction=removeAction&mdrActionId=<?php echo $a['pk_i_id']; ?>" onclick="return confirm('<?php echo osc_esc_js(__('Are you sure you want to remove this entry? Action cannot be undone', 'moderator')); ?>');"><i class="fa fa-trash"></i> <?php _e('Remove this moderation', 'moderator'); ?></a></div>
                  <?php } ?>
               </div>
              </div>
            <?php } ?>
          <?php } else { ?>
            <div class="mdr-empty"><?php _e('No previous moderations found', 'moderator'); ?></div>
          <?php } ?>
        </div>
      </div>
    </div>
  </div>
<?php
}

osc_add_hook('admin_items_form_right_top', 'mdr_moderate_box');


// SHOW MODERATE BOX IN BACK
function mdr_moderate_box_back() {
  ?>
    <div id="mdr-backbox">
      <?php mdr_moderate_box('BACK'); ?>
    </div>
  <?php
}

osc_add_hook('item_edit', function() {
  if(defined('OC_ADMIN') && OC_ADMIN === true) {
    if(osc_version() <= 801) {
      mdr_moderate_box_back();
    }
  } else if(osc_is_admin_user_logged_in() && mdr_param('show_in_front') == 1) {
    mdr_moderate_box_front(false);
  }
});


// MANAGE REMOVAL OF VERSION
function mdr_manage_version_remove() {
  $msection = (Params::getParam('mdrPluginSubaction') == 'FRONT' ? 'pubMessages' : 'admin');
  
  if(Params::getParam('mdrVersionId') > 0 && Params::getParam('mdrPluginAction') == 'removeVersion') {
    ModelMDR::newInstance()->removeArchiveById(Params::getParam('mdrVersionId'));
    osc_add_flash_ok_message(__('Item version has been successfully removed', 'moderator'), $msection);
    
    if(@$_SERVER['HTTP_REFERER'] != '') {
      $redirect = $_SERVER['HTTP_REFERER'];
    } else {
      $redirect = osc_admin_base_url();
    }
    
    header('Location: ' . $redirect);
    exit;
  }
}

osc_add_hook('init_admin', 'mdr_manage_version_remove');


// MANAGE REMOVAL OF ACTIVITY
function mdr_manage_activity_remove() {
  $msection = (Params::getParam('mdrPluginSubaction') == 'FRONT' ? 'pubMessages' : 'admin');
  
  if(Params::getParam('mdrActionId') > 0 && Params::getParam('mdrPluginAction') == 'removeAction') {
    ModelMDR::newInstance()->removeActionById(Params::getParam('mdrActionId'));
    osc_add_flash_ok_message(__('Moderate record has been successfully removed', 'moderator'), $msection);
    
    if(@$_SERVER['HTTP_REFERER'] != '') {
      $redirect = $_SERVER['HTTP_REFERER'];
    } else {
      $redirect = osc_admin_base_url();
    }
    
    header('Location: ' . $redirect);
    exit;
  }
}

osc_add_hook('init_admin', 'mdr_manage_activity_remove');


// PREPARE LIST OF UPDATES
function mdr_prepare_archive_data($item_id, $action) {
  $resources = ItemResource::newInstance()->getAllResourcesFromItem($item_id);
  $last_row = ModelMDR::newInstance()->getLastestItemArchive($item_id, '', array('ADD','EDIT','ADMIN_ADD','ADMIN_EDIT'));

  if(in_array($action, array('ADD', 'EDIT')) && Params::getParam('mdrPluginSubaction') != 'FRONT') {
    $data = Params::getParamsAsArray();
    
    if(!empty($resources) && count($resources) > 0) {
      $j = 1;
      foreach($resources as $r) {
        $data['resource_' . $j] = $r['s_path'] . $r['s_name'] . '.' . $r['s_extension'];
        $j++;
      }
    }
    
    unset($data['octoken']);
    
    if(count($data) > 0) {
      foreach($data as $key => $row) {
        if(strpos($key, 'mdr') === 0) {
          unset($data[$key]);
        }
      }
    }
    
  } else {
    if(isset($last_row['s_data'])) {
      $data = @json_decode($last_row['s_data'], true);
    } else {
      $data = array(); 
    }
  }
  
  if($action == 'ACTIVATE') {
    $data['b_status_active'] = 1;
  } else if($action == 'DEACTIVATE') {
    $data['b_status_active'] = 0;
  } else if($action == 'RENEW') {
    $data['b_status_renew'] = 1;
  } else if($action == 'BLOCK') {
    $data['b_status_block'] = 1;
  } else if($action == 'UNBLOCK') {
    $data['b_status_block'] = 0;
  } else if($action == 'MARK_SPAM') {
    $data['b_status_spam'] = 1;
  } else if($action == 'UNMARK_SPAM') {
    $data['b_status_spam'] = 0;
  } else if($action == 'MARK_PREMIUM') {
    $data['b_status_premium'] = 1;
  } else if($action == 'UNMARK_PREMIUM') {
    $data['b_status_premium'] = 0;
  } 
  
  if(is_array($data)) {
    $data = json_encode($data, JSON_NUMERIC_CHECK|JSON_UNESCAPED_SLASHES);
  }
  
  return $data;
}


// CREATE ITEM ARCHIVE
function mdr_create_archive($item, $action = '') {
  if(mdr_param('enable_archive') == 1) {
    if(is_array($item) && !empty($item) && isset($item['pk_i_id']) && $item['pk_i_id'] > 0) {
      $item_id = $item['pk_i_id'];
    } else {
      return false;
    }
    
    $resources = ItemResource::newInstance()->getAllResourcesFromItem($item_id);
    $item['resource'] = $resources;
    $item['meta'] = Field::newInstance()->findByCategoryItem($item_id, $item['fk_i_category_id']);
    
    $item_data = json_encode($item, JSON_NUMERIC_CHECK|JSON_UNESCAPED_SLASHES);
    $data = mdr_prepare_archive_data($item_id, $action);

    $version = 1;
    $old_data = '';

    $last_row = ModelMDR::newInstance()->getLastestItemArchive($item_id, '', array('ADD','EDIT','ADMIN_ADD','ADMIN_EDIT'));

    if($last_row !== false && isset($last_row['i_version'])) {
      $version = $last_row['i_version'] + 1;
      $old_data = $last_row['s_data'];
    }
    
    if(defined('OC_ADMIN') && OC_ADMIN === true) { 
      //if(in_array($action, array('ADD','EDIT'))) {
        $action = 'ADMIN_' . $action;
      //}
    }
   
    ModelMDR::newInstance()->insertArchive(array(
      'fk_i_item_id' => $item_id,
      'i_version' => $version,
      's_action' => $action,
      's_data' => $data,
      's_data_changed' => mdr_get_diff($old_data, $data, $action),
      's_item_data' => $item_data,
      's_hash' => Params::getParam('mdrArchiveUniqueHash'),
      'dt_date' => date('Y-m-d H:i:s')
    ));
  }
}

osc_add_hook('edited_item', function($item) { mdr_create_archive($item, 'EDIT'); }, 1);
osc_add_hook('posted_item', function($item) { mdr_create_archive($item, 'ADD'); }, 1);
osc_add_hook('activate_item', function($item_id) { mdr_create_archive(Item::newInstance()->findByPrimaryKey($item_id), 'ACTIVATE'); });
osc_add_hook('renew_item', function($item_id) { mdr_create_archive(Item::newInstance()->findByPrimaryKey($item_id), 'RENEW'); });
osc_add_hook('deactivate_item', function($item_id) { mdr_create_archive(Item::newInstance()->findByPrimaryKey($item_id), 'DEACTIVATE'); });
osc_add_hook('enable_item', function($item_id) { mdr_create_archive(Item::newInstance()->findByPrimaryKey($item_id), 'UNBLOCK'); });
osc_add_hook('disable_item', function($item_id) { mdr_create_archive(Item::newInstance()->findByPrimaryKey($item_id), 'BLOCK'); });
osc_add_hook('item_premium_on', function($item_id) { mdr_create_archive(Item::newInstance()->findByPrimaryKey($item_id), 'MARK_PREMIUM'); });
osc_add_hook('item_premium_off', function($item_id) { mdr_create_archive(Item::newInstance()->findByPrimaryKey($item_id), 'UNMARK_PREMIUM'); });
osc_add_hook('item_spam_on', function($item_id) { mdr_create_archive(Item::newInstance()->findByPrimaryKey($item_id), 'UNMARK_SPAM'); });
osc_add_hook('item_spam_off', function($item_id) { mdr_create_archive(Item::newInstance()->findByPrimaryKey($item_id), 'MARK_SPAM'); });



// CHECK DIFFERENCES IN 2 ARRAYS (JSON ENCODED)
function mdr_get_diff($old, $new, $action = '') {
  if(!is_string($old) || !is_string($new) || $old == '' || $new == '') {
    return json_encode(array());
  }
  
  $old = @json_decode($old, true); 
  $new = @json_decode($new, true);
  
  if(Params::getParam('mdrChangeStatus') <> 'NONE' && Params::getParam('mdrChangeStatus') <> '') {
    $status = Params::getParam('mdrChangeStatus');
    
    if($status == 'ACTIVATE') {
      $new['b_status_active'] = 1;
    } else if($status == 'DEACTIVATE') {
      $new['b_status_active'] = 0;
    } else if($status == 'RENEW') {
      $new['b_status_renew'] = 1;
    } else if($status == 'BLOCK') {
      $new['b_status_block'] = 1;
    } else if($status == 'UNBLOCK') {
      $new['b_status_block'] = 0;
    } else if($status == 'SPAM') {
      $new['b_status_spam'] = 1;
    } else if($status == 'UNSPAM') {
      $new['b_status_spam'] = 0;
    } 
  }

  $diff = mdr_compare_arrays($old, $new);
  
  // Exclude "remove" diffs for non-edit actions
  if(count($diff) > 0 && $action != '' && !in_array($action, array('ADD','EDIT','ADMIN_ADD','ADMIN_EDIT'))) {
    $diff_without_remove = array();
    
    foreach($diff as $d) {
      if($d['type'] != 'REMOVED') {
        $diff_without_remove[] = $d;
      }
    }
    
    $diff = $diff_without_remove;
  }

  return json_encode($diff, JSON_NUMERIC_CHECK|JSON_UNESCAPED_SLASHES);
}


// RECURSIVELY COMPARE ARRAYS
function mdr_compare_arrays($old, $new, $parent_key = '') {
  $old = !is_array($old) ? array() : $old;
  $new = !is_array($new) ? array() : $new;
  
  $diff = array();
  
  // Do not run diff check if old or new data are empty - can be mistake
  if(empty($new) || empty($old)) {
    return $diff;
  }

  if(count($new) > 0) {
    foreach($new as $key => $val) {
      $full_key = ($parent_key <> '' ? $parent_key . '/' : '') . $key;
      
      if(!in_array($full_key, mdr_ignored_fields()) && strpos($key, 'mdr') !== 0) {
        if(!is_array($val)) {
          if(trim($val != '' ? $val : '') != trim(isset($old[$key]) ? $old[$key] : '')) {
            if(in_array($key, array('catId', 'select_1', 'select_2', 'select_3', 'select_4'))) {
              if($val > 0) {
                $cat = Category::newInstance()->findByPrimaryKey($val);
                
                if(isset($cat['s_name'])) {
                  $val = $cat['s_name'] . ' (#' . $val . ')'; 
                }
              }

              if(isset($old[$key]) && $old[$key] > 0) {
                $cat_o = Category::newInstance()->findByPrimaryKey($old[$key]);
                
                if(isset($cat_o['s_name'])) {
                  $old[$key] = $cat_o['s_name'] . ' (#' . $old[$key] . ')'; 
                }
              }
            }

            if(strpos($key, 'b_status_') === 0) {
              $val = ($val == 1 ? __('Yes', 'moderator') : __('No', 'moderator'));
              
              if(isset($old[$key])) {
                $old[$key] = ($old[$key] == 1 ? __('Yes', 'moderator') : __('No', 'moderator'));
              }
            }
  
            $diff[] = array(
              'key' => $key,
              'parent_key' => $parent_key,
              'full_key' => $full_key,
              'name' => mdr_field_name($full_key, $key),
              'type' => !isset($old[$key]) ? 'NEW' : 'MODIFIED',
              'old' => !isset($old[$key]) ? '' : $old[$key],
              'new' => $val          
            );
          }
        } else {
          $diff_nest = mdr_compare_arrays(isset($old[$key]) ? $old[$key] : array(), $val, $full_key);
          
          if(!empty($diff_nest)) {
            $diff = array_merge($diff, $diff_nest);
          }
        }
      }
    }
  }
  
  if(count($old) > 0) {
    foreach($old as $key => $val) {
      $full_key = ($parent_key <> '' ? $parent_key . '/' : '') . $key;

      if(!in_array($full_key, mdr_ignored_fields()) && strpos($key, 'mdr') !== 0 && strpos($key, 'b_status_') !== 0) {
        if(!is_array($val)) {
          if(!isset($new[$key]) && trim($val != '' ? $val : '') != '') {
            $diff[] = array(
              'key' => $key,
              'parent_key' => $parent_key,
              'full_key' => $full_key,
              'name' => mdr_field_name($full_key, $key),
              'type' => 'REMOVED',
              'old' => $val,
              'new' => ''          
            );
          }
        } else {
          $diff_nest = mdr_compare_arrays(isset($old[$key]) ? $old[$key] : array(), $val, $full_key);
          
          if(!empty($diff_nest)) {
            $diff = array_merge($diff, $diff_nest);
          }
        }
      }
    }
  }
  
  return $diff;
}


// MODERATE ACTION NAME
function mdr_action_name($action) {
  $list = mdr_actions();
  
  $admin_string = '';
  if(strpos($action, 'ADMIN_') === 0) {
    $admin_string = ' (' . __('by admin', 'moderator') . ')';
    $action = str_replace('ADMIN_', '', $action);
  }
  
  if(isset($list[$action]) && $list[$action] != '') {
    return $list[$action] . $admin_string;
  }
  
  return ucwords(str_replace('_', ' ', $action)) . $admin_string;  
}


// ACTIONS LIST
function mdr_actions() {
  $actions = array(
    'MODERATE' => __('Moderate', 'moderator'),
    'ADD' => __('Add', 'moderator'),
    'EDIT' => __('Edit', 'moderator'),
    'ACTIVATE' => __('Activate', 'moderator'),
    'DEACTIVATE' => __('Deactivate', 'moderator'),
    'RENEW' => __('Renew', 'moderator'),
    'BLOCK' => __('Block', 'moderator'),
    'UNBLOCK' => __('Unblock', 'moderator'),
    'MARK_PREMIUM' => __('Mark as Premium', 'moderator'),
    'UNMARK_PREMIUM' => __('Unmark as Premium', 'moderator'),
    'MARK_SPAM' => __('Mark as Spam', 'moderator'),
    'UNMARK_SPAM' => __('Unmark as Spam', 'moderator')
  );
  
  return osc_apply_filter('mdr_actions', $actions);
}
  

// MODERATE REASON NAME
function mdr_reason_name($reason) {
  $list = mdr_reasons();
  
  if(isset($list[$reason]) && $list[$reason] != '') {
    return $list[$reason];
  }
  
  return ucwords(str_replace('_', ' ', $reason));  
}

// LIST OF AVAILABLE REASONS
function mdr_reasons() {
  $reasons = array(
    'NONE' => __('Standard update', 'moderator'),
    'DUPLICATE' => __('Duplicated item', 'moderator'),
    'ILLEGAL' => __('Illegal item', 'moderator'),
    'SPAM' => __('Spam item', 'moderator'),
    'OFFENSIVE' => __('Offensive content', 'moderator'),
    'INAPPROPRIATE' => __('Inappropriate item', 'moderator'),
    'BAD_CATEGORY' => __('Wrong category', 'moderator'),
    'MISSING_IMAGE' => __('Item must contain images', 'moderator'),
    'MISSING_DATA' => __('Missing data', 'moderator'),
    'MISSING_PRICE' => __('Item must have price defined', 'moderator'),
    'MISSING_LOCATION' => __('Item must have location defined', 'moderator'),
    'INCOMPLETE' => __('Item details/attributes are not complete', 'moderator'),
    'COMMERCIAL' => __('Commercial listing', 'moderator'),
    'MISLEADING' => __('Content is misleading', 'moderator'),
    'SHORT' => __('Title or description too short', 'moderator'),
    'DATA_ISSUE' => __('Data issue', 'moderator'),
    'TERMS' => __('Does not meet Terms & Conditions', 'moderator')  
  );
  
  $custom = array_filter(array_unique(array_map('trim', explode(',', mdr_param('custom_reasons')))));
  
  if(count($custom) > 0) {
    foreach($custom as $c) {
      if(!in_array($c, $reasons) && $c != '') {
        $reasons[$c] = $c;
      }
    }
  }
  
  return osc_apply_filter('mdr_reasons', $reasons);
}


// MODERATE STATUS NAME
function mdr_status_name($status) {
  $list = mdr_statues();
  
  if($status == '') {
    return '-';
  }
  
  if(isset($list[$status]) && $list[$status] != '') {
    return $list[$status];
  }
  
  return ucwords(str_replace('_', ' ', $status));  
}


// LIST OF AVAILABLE STATUS
function mdr_statues() {
  $statues = array(
    //'NONE' => __('No status (change)', 'moderator'),
    'ACTIVATE' => __('Activate', 'moderator'),
    'DEACTIVATE' => __('Deactivate', 'moderator'),
    'RENEW' => __('Renew', 'moderator'),
    'BLOCK' => __('Block', 'moderator'),
    'UNBLOCK' => __('Unblock', 'moderator'),
    'SPAM' => __('Mark as Spam', 'moderator'),
    'UNSPAM' => __('Unmark as Spam', 'moderator')
  );
  
  return osc_apply_filter('mdr_statues', $statues);
}


// EDIT TYPE NAME
function mdr_edit_type_name($type) {
  if($type == 'NEW') {
    return __('Added', 'moderator');
  } else if($type == 'MODIFIED') {
    return __('Modified', 'moderator');
  } else if($type == 'REMOVED') {
    return __('Removed', 'moderator');
  }
 
  return $type;
}


// GET PROPER FIELD NAME
function mdr_field_name($full_key, $key = '') {
  $map = mdr_fields_map();
  $locales = osc_get_locales();
  $keys = array_filter(explode('/', $full_key));
  $top_key = $keys[0];
  $text = array();
  
  if(count($keys) > 0) {
    $j = 0;
    foreach($keys as $k) {
      $k_ = explode('_', $k);
      $match_code = array_search($k, array_column($locales, 'pk_c_code'));

      if($match_code !== false) {
        $text[] = $locales[$match_code]['s_name'];
      } else if($k_[0] == 'resource') {
        $text[] = $map['resource'] . (isset($k_[1]) ? ' #' . $k_[1] : '');
      } else if($top_key == 'meta' && $j == 1) {
        $field = Field::newInstance()->findByPrimaryKey($k);
        
        if(isset($field['s_name']) && $field['s_name'] != '') {
          $text[] = $field['s_name'];
        } else {
          $text[] = 'ID ' . $k;
        }
        
      } else {
        $text[] = isset($map[$k]) ? $map[$k] : ucwords(str_replace(array('_', '-'), ' ', $key));
      }
      
      $j++;
    }
  }
  
  $text = implode(' / ', $text);
  
  if($text == '' && isset($map[$key])) {
    $text = $map[$key];
  }
  
  if($text == '') {
    $text = ucwords(str_replace(array('_', '-'), ' ', $key));  
  }

  return $text;  
}


// FIELDS/KEYS MAPPING
function mdr_fields_map() {
  $map = array(
    'id' => __('ID', 'moderator'),
    'title' => __('Title', 'moderator'),
    'description' => __('Description', 'moderator'),
    'price' => __('Price', 'moderator'),
    'currency' => __('Currency', 'moderator'),
    'sTransaction' => __('Transaction', 'moderator'),
    'sCondition' => __('Condition', 'moderator'),
    'ipAddress' => __('IP Address', 'moderator'),
    'catId' => __('Category', 'moderator'),
    'select_1' => __('Category (level 1)', 'moderator'),
    'select_2' => __('Category (level 2)', 'moderator'),
    'select_3' => __('Category (level 3)', 'moderator'),
    'select_4' => __('Category (level 4)', 'moderator'),
    'resource' => __('Resource', 'moderator'),
    'country' => __('Country', 'moderator'),
    'countryId' => __('Country', 'moderator'),
    'region' => __('Region', 'moderator'),
    'regionId' => __('Region', 'moderator'),
    'city' => __('City', 'moderator'),
    'cityId' => __('City', 'moderator'),
    'cityArea' => __('City area', 'moderator'),
    'cityAreaId' => __('City area', 'moderator'),
    'zip' => __('ZIP', 'moderator'),
    'address' => __('Address', 'moderator'),
    'contactName' => __('Contact name', 'moderator'),
    'contactEmail' => __('Contact email', 'moderator'),
    'contactPhone' => __('Contact phone', 'moderator'),
    'contactOther' => __('Other contact', 'moderator'),
    'tc' => __('GDPR (Terms & Conditions)', 'moderator'),
    'pp' => __('GDPR (Privacy policy)', 'moderator'),
    'nw' => __('GDPR (Newsletter)', 'moderator'),
    'dt_expiration' => __('Expiration date', 'moderator'),
    'showPhone' => __('Show phone', 'moderator'),
    'showEmail' => __('Show email', 'moderator'),
    'meta' => __('Custom field', 'moderator'),
    'b_status_active' => __('Item is active', 'moderator'),
    'b_status_renew' => __('Item was renewed', 'moderator'),
    'b_status_block' => __('Item is blocked', 'moderator'),
    'b_status_spam' => __('Item is marked as spam', 'moderator'),
    'b_status_premium' => __('Item is marked as premium', 'moderator')
  );
  
  return osc_apply_filter('mdr_fields_map', $map);
}


// FIELDS/KEYS MAPPING
function mdr_email_whitelisted_keys() {
  $map = array(
    'title',
    'description',
    'price',
    'currency',
    'sTransaction',
    'sCondition',
    //'catId',
    'select_1',
    'select_2',
    'select_3',
    'select_4',
    'resource',
    'country',
    'countryId',
    'region',
    'regionId',
    'city',
    'cityId',
    'cityArea',
    'cityAreaId',
    'zip',
    'address',
    'contactName',
    'contactEmail',
    'contactPhone',
    'contactOther',
    'dt_expiration',
    'showPhone',
    'showEmail',
    'meta',
    'b_status_active',
    'b_status_renew',
    'b_status_block',
    'b_status_spam',
    'b_status_premium'
  );
  
  return osc_apply_filter('mdr_email_whitelisted_keys', $map);
}


// LIST OF IGNORED FIELDS IN COMPARE
function mdr_ignored_fields() {
  return array(
    'secret',
    's_secret',
    'password',
    's_password',
    'octoken',
    'page',
    'action',
    'itemId',
    'fk_i_item_id',
    'id'
  );
}



// CORE FUNCTIONS
function mdr_param($name) {
  return osc_get_preference($name, 'plugin-moderator');
}


if(!function_exists('mdr_param_update')) {
  function mdr_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? ($type == 'array' ? json_decode(osc_get_preference($param_name, $plugin_var_name), true) : osc_get_preference($param_name, $plugin_var_name)) : ($type == 'array' ? array() : '');
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, ($type == 'array' ? json_encode($val, JSON_UNESCAPED_SLASHES) : $val), $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => ($type == 'array' ? json_encode($val, JSON_UNESCAPED_SLASHES) : $val)), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}



// CHECK IF RUNNING ON DEMO
function mdr_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}



// CATEGORIES WORK
function mdr_cat_tree($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
      $array[$c['pk_i_id']]['sub'] = mdr_cat_sub($list, $c['pk_i_id']);
    }
  }

  return $array;
}

function mdr_cat_sub($list, $parent_id) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {  echo $c['s_name'];
        $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
        $array[$c['pk_i_id']]['sub'] = mdr_cat_sub($list, $c['pk_i_id']);
      }
    }
  }
      
  return $array;
}

function mdr_cat_list($selected = array(), $categories = '', $level = 0) {
  if($categories == '' || $level == 0) {
    $categories = mdr_cat_tree($categories);
  }


  foreach($categories as $c) {
    echo '<option value="' . $c['pk_i_id'] . '" ' . (in_array($c['pk_i_id'], $selected) ? 'selected="selected"' : '') . '>' . str_repeat('-', $level) . ($level > 0 ? ' ' : '') . $c['s_name'] . '</option>';

    if(@count($c['sub']) > 0) {
      mdr_cat_list($selected, $c['sub'], $level + 1);
    }
  }
}


function mdr_list_values_ol($values) {
 if(count($values) > 0 && is_array($values)) {
    foreach($values as $v) {
      ?>

      <li class="mb-val" id="val_<?php echo $v['pk_i_id']; ?>">
        <?php mdr_div_value($v); ?>
      
        <ol>
          <?php 
            if(isset($v['values']) && count($v['values']) > 0) { 
              mdr_list_values_ol($v['values']); 
            }
          ?>
        </ol>
      </li>
    <?php
    }
  }
}

?>