<?php
class ModelMDR extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_action() {
  return DB_TABLE_PREFIX.'t_mdr_action';
}

public function getTable_archive() {
  return DB_TABLE_PREFIX.'t_mdr_item_archive';
}

public function getTable_user() {
  return DB_TABLE_PREFIX.'t_user';
}

public function getTable_item() {
  return DB_TABLE_PREFIX.'t_item';
}

public function getTable_item_location() {
  return DB_TABLE_PREFIX.'t_item_location';
}

public function getTable_item_description() {
  return DB_TABLE_PREFIX.'t_item_description';
}

public function getTable_category() {
  return DB_TABLE_PREFIX.'t_category';
}

public function getTable_admin() {
  return DB_TABLE_PREFIX.'t_admin';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelMDR<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install($version = '') {
  if($version == '') {
    $this->import('moderator/model/struct.sql');
    osc_set_preference('version', 100, 'plugin-moderator', 'INTEGER');
    
    $locales = OSCLocale::newInstance()->listAllEnabled();

    $email_data = array();
    foreach($locales as $l) {
      $email_text  = '<p>Hi {CONTACT_NAME}</p>';
      $email_text .= '<p>We would like to notify you, that your listing has been moderated by administrator.</p>';

      $email_text .= '<p>';
      $email_text .= '<b>What has been changed:</b><br/>';
      $email_text .= '{CHANGES}';
      $email_text .= '</p>';
      
      $email_text .= '<p>';
      $email_text .= '<b>Reason for moderation:</b><br/>';
      $email_text .= '{REASON}';
      $email_text .= '</p>';
      
      $email_text .= '<p>';
      $email_text .= '<b>Additional comment from moderator:</b><br/>';
      $email_text .= '{COMMENT}';
      $email_text .= '</p>';
      
      $email_text .= '<p><br/></p>';
      $email_text .= '<p>Thank you, <br />{WEB_TITLE}</p>';

      $email_data[$l['pk_c_code']] = array();
      $email_data[$l['pk_c_code']]['s_title'] = '{WEB_TITLE} - Your listing has been moderated';
      $email_data[$l['pk_c_code']]['s_text'] = $email_text;
    }
    
    Page::newInstance()->insert(array('s_internal_name' => 'mdr_email_user_notify', 'b_indelible' => '1'), $email_data);
  }

}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_action()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_archive()));


  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-moderator'";
  $this->dao->query($query);
}


public function getArchiveById($id) {
  if($id <= 0) { return false; }
  
  $this->dao->select();
  $this->dao->from($this->getTable_archive());
  $this->dao->where('pk_i_id', $id);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }

  return false;
}


public function getArchivesByItemId($item_id, $actions = array()) {
  if($item_id <= 0) { return false; }

  $this->dao->select();
  $this->dao->from($this->getTable_archive());
  $this->dao->where('fk_i_item_id', $item_id);
  
  if(is_array($actions) && !empty($actions)) {
    $this->dao->where('s_action in ("' . implode('","', $actions) . '")');
  }
  
  $this->dao->orderby('dt_date', 'DESC');

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }

  return array();
}


public function getItemArchiveLastVersion($item_id) {
  if($item_id <= 0) { return false; }

  $this->dao->select('max(i_version) as i_version');
  $this->dao->from($this->getTable_archive());
  $this->dao->where('fk_i_item_id', $item_id);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    
    if(isset($data['i_version']) && $data['i_version'] > 0) {
      return $data['i_version'];
    }
  }

  return 0;
}


public function getLastestItemArchive($item_id, $not_hash = '', $actions = array()) {
  if($item_id <= 0) { return false; }

  $this->dao->select();
  $this->dao->from($this->getTable_archive());
  $this->dao->where('fk_i_item_id', $item_id);
  
  if($not_hash != '') {
    $this->dao->where('s_hash != "' . $not_hash . '"');
  }
  
  if(is_array($actions) && !empty($actions)) {
    $this->dao->where('s_action in ("' . implode('","', $actions) . '")');
  }
  
  $this->dao->orderby('dt_date', 'DESC');
  $this->dao->limit(1);

  $result = $this->dao->get();

  if($result) {
    $data = $result->row();
    return $data;
  }

  return false;
}


public function getActionById($id) {
  if($id <= 0) { return false; }
  
  $this->dao->select();
  $this->dao->from($this->getTable_action());
  $this->dao->where('pk_i_id', $id);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return $data;
  }

  return false;
}


public function getActionsByItemId($item_id) {
  if($item_id <= 0) { return false; }

  $this->dao->select();
  $this->dao->from($this->getTable_action());
  $this->dao->where('fk_i_item_id', $item_id);
  $this->dao->orderby('dt_date', 'DESC');

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }

  return array();
}


public function getActionsByAdminId($admin_id, $limit = 100) {
  if($item_id <= 0) { return false; }

  $this->dao->select();
  $this->dao->from($this->getTable_action());
  $this->dao->where('fk_i_admin_id', $admin_id);
  $this->dao->orderby('dt_date', 'DESC');
  $this->dao->limit($limit);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return $data;
  }

  return array();
}


public function getItemIdsByEmail($email) {
  if($email == '') { return false; }

  $this->dao->select('pk_i_id');
  $this->dao->from($this->getTable_item());
  $this->dao->where('s_contact_email', $email);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->result();
    return array_column($data, 'pk_i_id');
  }

  return array();
}


public function countItemIdsByEmail($email) {
  if($email == '') { return false; }

  $this->dao->select('count(pk_i_id) as i_count');
  $this->dao->from($this->getTable_item());
  $this->dao->where('s_contact_email', $email);

  $result = $this->dao->get();
  
  if($result) {
    $data = $result->row();
    return isset($data['i_count']) ? $data['i_count'] : 0;
  }

  return 0;
}





public function insertAction($data) {
  $this->dao->insert($this->getTable_action(), $data);
  return $this->dao->insertedId();
}

public function insertArchive($data) {
  $this->dao->insert($this->getTable_archive(), $data);
  return $this->dao->insertedId();
}



public function updateArchive($id, $data) {
  $this->dao->update($this->getTable_archive(), $data, array('pk_i_id' => $id));
}

public function updateAction($id, $data) {
  $this->dao->update($this->getTable_action(), $data, array('pk_i_id' => $id));
}



public function removeActionById($id) {
  return $this->dao->delete($this->getTable_action(), array('pk_i_id' => $id));
}

public function removeActionByItemId($item_id) {
  return $this->dao->delete($this->getTable_action(), array('fk_i_item_id' => $item_id));
}

public function removeArchiveById($id) {
  return $this->dao->delete($this->getTable_archive(), array('pk_i_id' => $id));
}

public function removeArchiveByItemId($item_id) {
  return $this->dao->delete($this->getTable_archive(), array('fk_i_item_id' => $item_id));
}

}
?>