<?php

// INCLUDE MAILER SCRIPT
function nsl_include_mailer() {
  if(file_exists(osc_lib_path() . 'phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'phpmailer/class.phpmailer.php';
  } else if(file_exists(osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php';
  }
}


// GET TESTING MAILS LIST
function nsl_test_emails($as_string = false) {
  $mails = array_filter(array_unique(array_map('trim', explode(',', nsl_param('test_email')))));
  
  if(is_array($mails) && count($mails) > 0) {
    if($as_string === true) {
      return implode(',', $mails);
    }
    
    return $mails;
  }
  
  return false;
}
  

// MASK EMAIL ON DEMO
function nsl_mask_email($email) {
  if(!nsl_is_demo()) {
    return $email;
  } else {
    return preg_replace('/[\w]/', '*', $email);
  }
}


// GET USER FROM MAIL LIST STATUS
function nsl_mail_status($id, $error) {
  $html = '<span class="st' . $id . ' er' . ($error > 0 ? $error : 0) . '">';

  if($error > 0) {
    $html .= '<i class="fa fa-times"></i> ' . __('Error', 'newsletter');
  } else if($id == 0) {
    $html .= '<i class="fa fa-hourglass-o"></i> ' . __('Pending to be sent', 'newsletter');
  } else if($id == 1) {
    $html .= '<i class="fa fa-check"></i> ' . __('Sent', 'newsletter');
  } else if($id == 2) {
    $html .= '<i class="fa fa-sign-out"></i> ' . __('Unsubscribed', 'newsletter');
  }
 
  $html .= '</span>';
  return $html;
}



// GET CAMPAIGN STATUS
function nsl_status($id) {
  $html = '<span class="st' . $id . '">';

  if($id == 0) {
    $html .= '<i class="fa fa-hourglass-o"></i> ' . __('Not started', 'newsletter');
  } else if($id == 1) {
    $html .= '<i class="fa fa-clock-o"></i> ' . __('In progress', 'newsletter');
  } else if($id == 2) {
    $html .= '<i class="fa fa-check"></i> ' . __('Completed', 'newsletter');
  }
 
  $html .= '</span>';
  return $html;
}

// GET PLUGIN PARAMETER
function nsl_param($name) {
  return osc_get_preference($name, 'plugin-newsletter');
}


// CHECK IF RUNNING ON DEMO
function nsl_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


// GET CURRENT OR DEFAULT ADMIN LOCALE
function nsl_get_locale() {
  $locales = OSCLocale::newInstance()->listAllEnabled();

  if(Params::getParam('nslLocale') <> '') {
    $current = Params::getParam('nslLocale');
  } else {
    $current = (osc_current_user_locale() <> '' ? osc_current_user_locale() : osc_current_admin_locale());
    $current_exists = false;

    // check if current locale exist in front-office
    foreach( $locales as $l ) {
      if($current == $l['pk_c_code']) {
        $current_exists = true;
      }
    }

    if( !$current_exists ) {
      $i = 0;
      foreach( $locales as $l ) {
        if( $i==0 ) {
          $current = $l['pk_c_code'];
        }

        $i++;
      }
    }
  }

  return $current;
}


// CREATE LOCALE SELECT BOX
function nsl_locale_box( $file ) {
  $html = '';
  $locales = OSCLocale::newInstance()->listAllEnabled();
  $current = nsl_get_locale();

  $html .= '<select rel="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=newsletter/admin/' . $file . '" class="mb-select mb-select-locale" id="nslLocale" name="nslLocale">';

  foreach( $locales as $l ) {
    $html .= '<option value="' . $l['pk_c_code'] . '" ' . ($current == $l['pk_c_code'] ? 'selected="selected"' : '') . '>' . $l['s_name'] . '</option>';
  }
 
  $html .= '</select>';
  return $html;
}



// CORE FUNCTIONS
if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}



if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if( !function_exists('osc_is_contact_page') ) {
  function osc_is_contact_page() {
    $location = Rewrite::newInstance()->get_location();
    $section = Rewrite::newInstance()->get_section();
    if( $location == 'contact' ) {
      return true ;
    }

    return false ;
  }
}


// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}

?>