<?php
class ModelNSL extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}


public function getTable_campaign() {
  return DB_TABLE_PREFIX.'t_nsl_campaign';
}

public function getTable_list() {
  return DB_TABLE_PREFIX.'t_nsl_list';
}

public function getTable_unsubscribed() {
  return DB_TABLE_PREFIX.'t_nsl_unsubscribed';
}

public function getTable_user() {
  return DB_TABLE_PREFIX.'t_user';
}


public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelNSL<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install($version = '') {
  if($version == '') {
    $this->import('newsletter/model/struct.sql');

    osc_set_preference('version', 100, 'plugin-newsletter', 'INTEGER');
  }
}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_campaign()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_list()));


  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-newsletter'";
  $this->dao->query($query);
}



// GET CAMPAIGNS
public function getCampaigns($status = -1) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_campaign());

  if($status >= 0) {
    $this->dao->where('i_status', $status);
  }


  $result = $this->dao->get();
  
  if($result) {
    $campaigns = $result->result();
    return $campaigns;
  }

  return array();
}



// GET CAMPAIGN
public function getCampaign($id) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_campaign());
  $this->dao->where('pk_i_id', $id);

  $result = $this->dao->get();
  
  if($result) {
    $campaign = $result->row();
    return $campaign;
  }

  return false;
}


// GET UNSUBSCRIBED
public function getUnsubscribed($user_id) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_unsubscribed());
  $this->dao->where('fk_i_user_id', $user_id);

  $result = $this->dao->get();
  
  if($result) {
    $user = $result->row();
    return $user;
  }

  return false;
}


// GET MAIL LIST
public function getList($campaign_id, $status = -1, $error = -1, $limit = 500, $offset = 0) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_list());
  $this->dao->where('fk_i_campaign_id', $campaign_id);

  if($status >= 0) {
    $this->dao->where('i_status', $status);
  }

  if($error >= 0) {
    $this->dao->where('i_error', $error);
  }

  if(isset($limit) && $limit <> '' && $limit > 0 ) {
    if(isset($offset) && $offset <> '' && $offset > 0) {
      $this->dao->limit( $offset, $limit );
    } else {
      $this->dao->limit( $limit );
    }
  }

  $this->dao->orderby('pk_i_id ASC');

  $result = $this->dao->get();
  
  if($result) {
    $campaigns = $result->result();
    return $campaigns;
  }

  return array();
}


// GET MAIL LIST
public function getListForSend($campaign_id) {
  $this->dao->select('*');
  $this->dao->from($this->getTable_list());
  $this->dao->where('fk_i_campaign_id', $campaign_id);
  $this->dao->where('i_status', 0);

  $this->dao->orderby('pk_i_id ASC');

  $result = $this->dao->get();
  
  if($result) {
    $campaigns = $result->result();
    return $campaigns;
  }

  return array();
}


// COUNT MAIL LIST
public function countList($campaign_id, $status = -1, $error = -1) {
  $this->dao->select('count(*) as i_count');
  $this->dao->from($this->getTable_list());
  $this->dao->where('fk_i_campaign_id', $campaign_id);

  if($status >= 0) {
    if($status == 1) {
      $this->dao->where('i_status in (1,2)');
    } else {
      $this->dao->where('i_status', $status);
    }
  }

  if($error >= 0) {
    $this->dao->where('i_error', $error);
  }


  $result = $this->dao->get();
  
  if($result) {
    $campaigns = $result->row()['i_count'];
    return $campaigns;
  }

  return 0;
}



// REMOVE CAMPAIGN
public function removeCampaign($id) {
  $this->dao->delete($this->getTable_campaign(), array('pk_i_id' => $id));
  $this->dao->delete($this->getTable_list(), array('fk_i_campaign_id' => $id));
}


// REMOVE LIST
public function removeList($id) {
  return $this->dao->delete($this->getTable_list(), array('pk_i_id' => $id));
}

// REMOVE LIST BY CAMPAIGN ID
public function removeListByCampaignId($id) {
  return $this->dao->delete($this->getTable_list(), array('fk_i_campaign_id' => $id));
}

// REMOVE USER FROM LIST
public function removeUser($campaign_id, $id) {
  return $this->dao->delete($this->getTable_list(), array('fk_i_campaign_id' => $campaign_id, 'pk_i_id' => $id));
}



// INSERT NEW CAMPAIGN
public function insertCampaign($values) {
  $this->dao->insert($this->getTable_campaign(), $values);
  $id = $this->dao->insertedId();

  $this->insertList($id);

  return $id;
}


// UNSUBSCRIBE USER
public function insertUnsubscribe($user_id, $email) {
  if(!$this->getUnsubscribed($user_id)) {
    $this->dao->insert($this->getTable_unsubscribed(), array('fk_i_user_id' => $user_id, 'dt_date' => date("Y-m-d h:i:s")));
  }

  $this->dao->update($this->getTable_list(), array('i_status' => 2, 's_comment' => sprintf(__('Unsubscribed on %s', 'newsletter'), date("Y-m-d h:i:s"))), array('s_email' => $email));
  $this->dao->update($this->getTable_list(), array('i_status' => 2, 's_comment' => sprintf(__('Unsubscribed on %s', 'newsletter'), date("Y-m-d h:i:s"))), array('fk_i_user_id' => $user_id));
}


// INSERT NEW LIST
public function insertList($campaign_id) {
  $clause = '';
  if(function_exists('gdpr_param') && nsl_param('ignore_gdpr') != 1) {
    $clause = ' AND i_gdpr_tc = 1 AND i_gdpr_pp = 1 AND i_gdpr_nw = 1';
  }

  $this->dao->query('INSERT INTO ' . $this->getTable_list() . ' (fk_i_campaign_id, fk_i_user_id, s_name, s_email, s_secret) SELECT ' . $campaign_id . ' as camp_id, pk_i_id, s_name, s_email, s_secret FROM ' . $this->getTable_user() . ' WHERE b_enabled = 1 AND b_active = 1' . $clause);

  $list_id = $this->dao->insertedId();
  $this->dao->query('DELETE FROM ' . $this->getTable_list() . ' WHERE fk_i_campaign_id = ' . $campaign_id . ' AND fk_i_user_id in (SELECT fk_i_user_id FROM ' . $this->getTable_unsubscribed() . ')');
}


// ADD USERS TO LIST - SPECIAL
public function addUsersToListSpecial($campaign_id, $company, $active, $has_phone, $has_location, $has_items) {
  $conds = array();

  if($company <> '') {
    $conds[] = 'b_company = ' . $company;
  }

  if($active <> '') {
    $conds[] = 'b_active = ' . $active;
  }

  if($has_phone <> '') {
    $conds[] = '(coalesce(s_phone_land, "") <> "" or coalesce(s_phone_mobile, "") <> "")';
  }

  if($has_location <> '') {
    $conds[] = '(coalesce(s_region, "") <> "" or coalesce(s_city, "") <> "" or coalesce(fk_i_region_id, 0) > 0 or coalesce(fk_i_city_id, 0) > 0)';
  }

  if($has_items <> '') {
    if($has_items == 0) {
      $conds[] = 'i_items = 0';
    } else if($has_items > 0) {
      $conds[] = 'i_items > 0';
    }
  }

  if(function_exists('gdpr_param') && nsl_param('ignore_gdpr') != 1) {
    $conds[] = '(i_gdpr_tc = 1 AND i_gdpr_pp = 1 AND i_gdpr_nw = 1)';
  }


  $clause = '';

  if(!empty($conds)) {
    $clause = implode(' AND ', $conds);
  }

  if($clause <> '') {
    $clause = ' AND ' . $clause;
  }

  $this->dao->query('INSERT INTO ' . $this->getTable_list() . ' (fk_i_campaign_id, fk_i_user_id, s_name, s_email, s_secret) SELECT ' . $campaign_id . ' as camp_id, pk_i_id, s_name, s_email, s_secret FROM ' . $this->getTable_user() . ' as u WHERE NOT EXISTS (SELECT 1 FROM ' . $this->getTable_list() . ' as e WHERE u.pk_i_id = e.fk_i_user_id AND e.fk_i_campaign_id = ' . $campaign_id . ') AND b_enabled = 1' . $clause);

  $list_id = $this->dao->insertedId();
  $this->dao->query('DELETE FROM ' . $this->getTable_list() . ' WHERE fk_i_campaign_id = ' . $campaign_id . ' AND fk_i_user_id in (SELECT fk_i_user_id FROM ' . $this->getTable_unsubscribed() . ')');
}




// UPDATE CAMPAIGN
public function updateCampaign($data) {
  if(!isset($data['pk_i_id']) || $data['pk_i_id'] <= 0) {
    if(isset($data['pk_i_id'])) {
      unset($data['pk_i_id']);
    }

    //$this->dao->insert($this->getTable_campaign(), $data);
    $id = $this->insertCampaign($data);

    Params::setParam('campaignId', $id);
  } else {
    Params::setParam('campaignId', $data['pk_i_id']);
    $where = array('pk_i_id' => $data['pk_i_id']);

    $this->dao->update($this->getTable_campaign(), $data, $where);
  }
}


// RESET MAILING LIST
public function resetList($campaign_id) {
  $this->dao->update($this->getTable_list(), array('i_status' => 0, 'i_error' => 0, 's_comment' => null, 'dt_send_date' => NULL), array('fk_i_campaign_id' => $campaign_id));
}


// UPDATE SENT EMAIL
public function updateListById($data) {
  $this->dao->update($this->getTable_list(), $data, array('pk_i_id' => $data['pk_i_id']));
}

// UPDATE CAMPAIGN STATUS
public function updateCampaignStatus($id, $status) {
  $this->dao->update($this->getTable_campaign(), array('i_status' => $status), array('pk_i_id' => $id));
}



}
?>