<?php

// GENERATE SERVICE WORKERS
function pwa_generate_service_worker() {
  $content = file_get_contents(osc_base_path() . 'oc-content/plugins/pwa/js/pwa-service-worker.js');

  $version = (pwa_param('sw_version') > 0 ? pwa_param('sw_version') : 1);
  $assets = array();
  $assets_pre = (pwa_param('assets') <> '' ? array_filter(json_decode(pwa_param('assets'), true)) : array());
  
  if(is_array($assets_pre) && !empty($assets_pre) && count($assets_pre) > 0) {
    foreach($assets_pre as $a) {
      $asset_line = array_filter(explode(',', $a));
      
      foreach($asset_line as $els) {
        $assets[] = trim($els);
      }
    }
    
    $assets = json_encode(array_unique(array_map('trim', $assets)), JSON_UNESCAPED_SLASHES);
  } else {
    $assets = '[]';
  }
  
  $assets = ($assets == '' ? '[]' : $assets);

  $cache_name = PWA_CACHE_NAME;
  $offline_image_url = pwa_offline_image_url();
  $debug = (pwa_param('debug') == 1 ? 1 : 0);
  $periodic_sync = (pwa_param('periodic_sync') == 1 ? 1 : 0);
  $background_sync = (pwa_param('background_sync') == 1 ? 1 : 0);
  $strategy = (pwa_param('strategy') == '' ? 'NOCACHE' : pwa_param('strategy'));
  $oc_admin_dir = 'oc-admin';
  
  if(defined('OC_ADMIN_FOLDER') && OC_ADMIN_FOLDER != '') {
    $oc_admin_dir = OC_ADMIN_FOLDER;
  }
  
  $words = array();
  $words[] = array('{VERSION}', '{CACHE_ASSETS}', '{BASE_URL}', '{BASE_PATH}', '{OFFLINE_IMAGE_URL}', '{PERIODIC_SYNC}', '{STRATEGY}', '{BACKGROUND_SYNC}', '{OC_ADMIN_DIR}', '{CACHE_NAME}', '{DEBUG}');
  $words[] = array($version, $assets, osc_base_url(), osc_base_path(), $offline_image_url, $periodic_sync, $strategy, $background_sync, $oc_admin_dir, $cache_name, $debug);

  $content = osc_mailBeauty($content, $words);

  $path = osc_base_path() . 'pwa-service-worker.js';
  $ulr = osc_base_url() . 'pwa-service-worker.js';

  if(!file_put_contents($path, $content)){
    return false;
  }
  
  return true;  
}


// OFFLINE PAGE
function pwa_generate_offline_page() {
  $content = file_get_contents(osc_base_path() . 'oc-content/plugins/pwa/form/offline.html');

  $title = __('You are offline', 'pwa');
  $desc = __('Looks like you\'re not connected to a network. Check your settings and try again.', 'pwa');
  $list_name = __('Try', 'pwa');
  $list_elem1 = __('Turning off <b>Airplane mode</b>', 'pwa');
  $list_elem2 = __('Turning on <b>Mobile data</b> or <b>Wi-Fi</b>', 'pwa');
  $list_elem3 = __('Checking the signal in your area', 'pwa');
  $reload = __('Reload', 'pwa');
  
  $image_url = pwa_offline_image_url();
  
  $words = array();
  $words[] = array('{TITLE}', '{DESCRIPTION}', '{LIST_NAME}', '{LIST_ELEM1}', '{LIST_ELEM2}', '{LIST_ELEM3}', '{BASE_URL}', '{IMAGE_URL}', '{RELOAD}');
  $words[] = array($title, $desc, $list_name, $list_elem1, $list_elem2, $list_elem3, osc_base_url(), $image_url, $reload);

  $content = osc_mailBeauty($content, $words);

  $path = osc_base_path() . 'offline.html';
  $ulr = osc_base_url() . 'offline.html';

  if(!file_put_contents($path, $content)){
    return false;
  }
  
  return true;  
}


// HOOK MANIFEST TO HEAD
function pwa_hook_head() {
  if(pwa_param('meta_tags') == 1) {
    echo '<meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=5.0">' . PHP_EOL;
    
    if(pwa_param('theme_color') != '') {
      echo '<meta name="theme-color" content="' . osc_esc_html(pwa_param('theme_color')) . '">' . PHP_EOL;
      echo '<meta name="msapplication-TileColor" content="' . osc_esc_html(pwa_param('theme_color')) . '">' . PHP_EOL;
    }
    
    if(pwa_param('favicon_ico') != '') {
      echo '<link rel="shortcut icon" type="image/x-icon" href="' . osc_esc_html(pwa_param('favicon_ico')) . '">' . PHP_EOL;
    }

    if(pwa_param('favicon_16') != '') {
      echo '<link rel="icon" type="image/png" sizes="16x16" href="' . osc_esc_html(pwa_param('favicon_16')) . '">' . PHP_EOL;
    }
    
    if(pwa_param('favicon_32') != '') {
      echo '<link rel="icon" type="image/png" sizes="32x32" href="' . osc_esc_html(pwa_param('favicon_32')) . '">' . PHP_EOL;
    }
    
    if(pwa_param('mask_icon') != '') {
      echo '<link rel="mask-icon" href="' . osc_esc_html(pwa_param('mask_icon')) . '" color="' . osc_esc_html(pwa_param('theme_color') != '' ? pwa_param('theme_color') : '#5bbad5') . '">' . PHP_EOL;
    }
    
    if(pwa_param('apple_icon') != '') {
      echo '<link rel="apple-touch-icon" sizes="180x180" href="' . osc_esc_html(pwa_param('apple_icon')) . '">' . PHP_EOL;
    }

    if(pwa_param('name') != '') {
      echo '<meta name="apple-mobile-web-app-title" content="' . osc_esc_html(pwa_param('name')) . '">' . PHP_EOL;
      echo '<meta name="application-name" content="' . osc_esc_html(pwa_param('name')) . '">' . PHP_EOL;
    }
  }

  if(pwa_param('hook_manifest') == 1 && file_exists(osc_base_path() . 'manifest.webmanifest')) {
    echo '<link rel="manifest" href="' . osc_base_url() . 'manifest.webmanifest">' . PHP_EOL;
  }
  
  $strategy = (pwa_param('strategy') == '' ? 'NOCACHE' : pwa_param('strategy'));
  $periodic_sync = (pwa_param('periodic_sync') == 1 ? 1 : 0);
  $background_sync = (pwa_param('background_sync') == 1 ? 1 : 0);

  $oc_admin_dir = 'oc-admin';
  
  if(defined('OC_ADMIN_FOLDER') && OC_ADMIN_FOLDER != '') {
    $oc_admin_dir = OC_ADMIN_FOLDER;
  }
  
  if(pwa_param('hook_worker') == 1 && file_exists(osc_base_path() . 'pwa-service-worker.js') && !(defined('OC_ADMIN') && OC_ADMIN === true)) {
  ?>
  
<script>
  <?php if($background_sync == 1 && $strategy != 'NOCACHE') { ?>
  async function registerBackgroundSync() {
    const registration = await navigator.serviceWorker.ready;
    
    // Check if periodicSync is supported
    if ('sync' in registration) {
      // Request permission
      const status = await navigator.permissions.query({
        name: 'background-sync',
      });

      if (status.state === 'granted') {
        try {
          // Register new sync
          await registration.sync.register('update-cached-content-background');
          pwaConsoleLog2('[PWA] Background sync registered.');
        } catch(e) {
          pwaConsoleLog2('[PWA] Background sync failed.', e);
        }
      } else {
        pwaConsoleLog2('[PWA] Background sync is not granted.');
      }
    } else {
      pwaConsoleLog2('[PWA] Background sync is not supported.');
    }
  }
  <?php } ?>
    
  <?php if($periodic_sync == 1 && $strategy != 'NOCACHE') { ?>
  async function registerPeriodicSync() {
    const registration = await navigator.serviceWorker.ready;
    // Check if periodicSync is supported
    if ('periodicSync' in registration) {
      // Request permission
      const status = await navigator.permissions.query({
        name: 'periodic-background-sync',
      });

      if (status.state === 'granted') {
        try {
          // Register new sync every 24 hours
          await registration.periodicSync.register('update-cached-content-periodic', {
            minInterval: 24 * 60 * 60 * 1000, // 1 day
          });
          pwaConsoleLog2('[PWA] Periodic background sync registered.');
        } catch(e) {
          pwaConsoleLog2('[PWA] Periodic background sync failed.', e);
        }
      } else {
        pwaConsoleLog2('[PWA] Periodic background sync is not granted.');
      }
    } else {
      pwaConsoleLog2('[PWA] Periodic background sync is not supported.');
    }
  }
  <?php } ?>

  window.addEventListener("load", () => {
    if("serviceWorker" in navigator) {
      navigator.serviceWorker.register("<?php echo osc_base_url(); ?>pwa-service-worker.js");
      
      pwaConsoleLog2('[PWA] Configuration - Cache name: <?php echo PWA_CACHE_NAME; ?>, Version: <?php echo pwa_param('sw_version'); ?>, Strategy: <?php echo $strategy; ?>, Background sync: <?php echo $background_sync; ?>, Periodic sync: <?php echo $periodic_sync; ?>');

      <?php if($strategy != 'NOCACHE') { ?>
      // Submit assets to cache
      window.setTimeout(pullCacheLinks, 1000);
      <?php } ?>
      
      <?php if($background_sync == 1 && $strategy != 'NOCACHE') { ?>
      // Register the background sync
      registerBackgroundSync();
      <?php } ?>
      
      <?php if($periodic_sync == 1 && $strategy != 'NOCACHE') { ?>
      // Register the periodic background sync
      registerPeriodicSync();
      <?php } ?>
    } else {
      pwaConsoleLog2('[PWA] Service workers not supported by browser.');
    }
  });
  
  <?php if($strategy != 'NOCACHE') { ?>
  // Get all stylesheets and scripts links and add them into cache
  // in case link contains ?v=, it will be ignored/removed from link
  function pullCacheLinks() {
    let linksFound = [];
    
    // Get stylesheets
    document.querySelectorAll('link').forEach(function(elem) {
      if(elem.rel == 'stylesheet' && elem.type.includes('css') && elem.href && elem.href !== '') {
        if (!elem.href.includes('google') && !elem.href.includes('adservice') && !elem.href.includes('browser-sync') && !elem.href.includes('<?php echo $oc_admin_dir; ?>')) {
          linksFound.push(elem.href.split('?v=')[0]);
        }
      }
    });

    // Get scripts
    document.querySelectorAll('script').forEach(function(elem) {
      if(elem.type.includes('javascript') && elem.src && elem.src !== '') {
        if (!elem.src.includes('google') && !elem.src.includes('adservice') && !elem.src.includes('browser-sync') && !elem.src.includes('<?php echo $oc_admin_dir; ?>')) {
          linksFound.push(elem.src.split('?v=')[0]);
        }
      }
    });
    
    caches.open("<?php echo PWA_CACHE_NAME; ?>").then(function(cache) {
      cache.addAll(linksFound);
    });
  }
  <?php } ?>
  
  // Log to console based on preference
  async function pwaConsoleLog2(text) {
    if("<?php echo pwa_param('debug'); ?>" == "1") {
      console.log(text);
    }
  }
</script>

<?php
  }
}

osc_add_hook('header', 'pwa_hook_head');


// UNREGISTER WORKER
function pwa_unregister_worker() {
?>
  
<script>
  navigator.serviceWorker.getRegistrations().then(function(registrations) {
    if(registrations.length > 0) {
      pwaConsoleLog3('[PWA] Service worker unregistering in backoffice...');

      for(let registration of registrations) {
        pwaConsoleLog3(registration);
        registration.unregister();
      }

      pwaConsoleLog3('[PWA] Service worker unregistered in backoffice!');
    }
  });
  
  // Log to console based on preference
  async function pwaConsoleLog3(text) {
    if("<?php echo pwa_param('debug'); ?>" == "1") {
      console.log(text);
    }
  }
</script>

<?php
}

osc_add_hook('admin_header', 'pwa_unregister_worker', 1);



// GENERATE MANIFEST BASED ON STORED PARAMETERS
function pwa_generate_manifest() {
  $output = array();
  
  if(pwa_param('add_schema') == 1) {
    $output['$schema'] = 'https://json.schemastore.org/web-manifest-combined.json';
  }

  $list = array('name', 'short_name', 'description', 'display', 'display_override', 'orientation', 'scope', 'start_url', 'dir', 'lang', 'background_color', 'theme_color', 'prefer_related_applications', 'categories', 'iarc_rating_id');

  foreach($list as $l) {
    $val = trim(pwa_param($l));
    
    if($val != '') {
      if($l == 'prefer_related_applications') {
        $val = ($val == 1 ? true : false);
      } else if ($l == 'categories') {
        if(substr($val, 0, 1) == '[') {
          $val = str_replace('"', '', str_replace("'", "", substr($val, 1, strlen($val) - 2)));
          $val = array_filter(array_map('trim', explode(',', $val)));
        } else {
          $val = str_replace('"', '', str_replace("'", "", $val));
        }
      }
      
      $output[$l] = $val;
    }
  }
  
  
  // Icons
  $data = json_decode(pwa_param('icons'), true);
  
  if(is_array($data) && count($data) > 0) {
    $output['icons'] = array();
    
    foreach($data as $d) {
      $row = array();
      
      if(@$d['src'] != '' && @$d['sizes'] != '') {
        $row['src'] = $d['src'];
        $row['sizes'] = $d['sizes'];
        
        if(@$d['type'] != '') {
          $row['type'] = $d['type'];
        }
        
        if(@$d['purpose'] != '') {
          $row['purpose'] = $d['purpose'];
        }
      }
      
      if(!empty($row)) {
        $output['icons'][] = $row;
      }
    }
    
    if(empty($output['icons'])) {
      unset($output['icons']);
    }
  }
  
  
  // Related apps
  $data = json_decode(pwa_param('apps'), true);
  
  if(is_array($data) && count($data) > 0) {
    $output['related_applications'] = array();
    
    foreach($data as $d) {
      $row = array();
      
      if(@$d['platform'] != '' && @$d['id'] != '' && @$d['url'] != '') {
        $row['platform'] = $d['platform'];
        $row['id'] = $d['id'];
        $row['url'] = $d['url'];
      }
      
      if(!empty($row)) {
        $output['related_applications'][] = $row;
      }
    }
    
    if(empty($output['related_applications'])) {
      unset($output['related_applications']);
    }
  }

 
  // Shortcut
  $data = json_decode(pwa_param('shortcuts'), true);
  
  if(is_array($data) && count($data) > 0) {
    $output['shortcuts'] = array();
    
    foreach($data as $d) {
      $row = array();
      
      if(@$d['name'] != '' && @$d['url'] != '') {
        $row['name'] = $d['name'];
        $row['url'] = $d['url'];
        
        if(@$d['short_name'] != '') {
          $row['short_name'] = $d['short_name'];
        }
        
        $icon = (isset($d['icon']) ? $d['icon'] : array());
        $subrow = array();
        
        if(@$icon['src'] != '' && @$icon['sizes'] != '') {
          $subrow['src'] = $icon['src'];
          $subrow['sizes'] = $icon['sizes'];
          
          if(@$icon['type'] != '') {
            $subrow['type'] = $icon['type'];
          }
          
          if(@$icon['purpose'] != '') {
            $subrow['purpose'] = $icon['purpose'];
          }          
        }
        
        if(!empty($subrow)) {
          $row['icons'] = array();
          $row['icons'][] = $subrow;
        }
      }
      
      if(!empty($row)) {
        $output['shortcuts'][] = $row;
      }
    }
    
    if(empty($output['shortcuts'])) {
      unset($output['shortcuts']);
    }
  }


  // Screenshots
  $data = json_decode(pwa_param('screenshots'), true);
  
  if(is_array($data) && count($data) > 0) {
    $output['screenshots'] = array();
    
    foreach($data as $d) {
      $row = array();
      
      if(@$d['src'] != '' && @$d['sizes'] != '') {
        $row['src'] = $d['src'];
        $row['sizes'] = $d['sizes'];
        
        if(@$d['type'] != '') {
          $row['type'] = $d['type'];
        }
        
        if(@$d['platform'] != '') {
          $row['platform'] = $d['platform'];
        }
      }
      
      if(!empty($row)) {
        $output['screenshots'][] = $row;
      }
    }
    
    if(empty($output['screenshots'])) {
      unset($output['screenshots']);
    }
  }
  
  
  $json = json_encode($output, JSON_PRETTY_PRINT|JSON_UNESCAPED_SLASHES);
  
  $path = osc_base_path() . 'manifest.webmanifest';
  $ulr = osc_base_url() . 'manifest.webmanifest';

  if(!file_put_contents($path, $json)){
    return false;
  }
  
  return true;  
}


// LOAD FILES
function pwa_get_files($type = 'icons') {
  $path = osc_uploads_path() . 'pwa/' . $type;
  $url = osc_base_url() . 'oc-content/uploads/pwa/' . $type;
  $output = array();
  
  if(file_exists($path)) {
    $files = array_diff(scandir($path), array('.', '..'));
    
    if(is_array($files) && count($files) > 0) {
      foreach($files as $f) {
        $ext = pathinfo($f, PATHINFO_EXTENSION);

        if(!in_array($ext, array('php','pdf','html','txt','','.','.htaccess'))) {
          $output[] = array(
            'name' => $f,
            'ext' => $ext,
            'path' => $path . '/' . $f,
            'url' => $url . '/' . $f
          );
        }
      }
    }
  }
  
  return $output;
}

// GET OFFLINE PAGE IMAGE URL
function pwa_offline_image_url() {
  if(pwa_param('offline_image') != '') {
    $image_url = pwa_param('offline_image');
  } else {
    $image_url = osc_base_url() . 'oc-content/plugins/pwa/img/offline.png';
  }
  
  return $image_url;
}
  

// LIST OF UPLOADED ICONS
function pwa_select_icons() {
  $list = pwa_get_files('icons')
  ?>
  <select id="mb-sel-stick" class="mb-sel-stick">
    <option value=""><?php _e('Choose', 'pwa'); ?></option>
    
    <?php foreach($list as $l) { ?>
      <option value="<?php echo $l['url']; ?>"><?php echo $l['name']; ?></option>
    <?php } ?>
  </select>
  <?php  
}

// LIST OF UPLOADED ICONS
function pwa_select_screenshots() {
  $list = pwa_get_files('screenshots')
  ?>
  <select id="mb-sel-stick" class="mb-sel-stick">
    <option value=""><?php _e('Choose', 'pwa'); ?></option>
    
    <?php foreach($list as $l) { ?>
      <option value="<?php echo $l['url']; ?>"><?php echo $l['name']; ?></option>
    <?php } ?>
  </select>
  <?php  
}


// LIST OF SUPPORTED ICON SIZES
function pwa_select_sizes() {
  $list = array('16x16','32x32','64x64','96x96','144x144','180x180','192x192', '512x512');
  ?>
  <select id="mb-sel-stick" class="mb-sel-stick">
    <option value=""><?php _e('Choose', 'pwa'); ?></option>
    
    <?php foreach($list as $l) { ?>
      <option value="<?php echo $l; ?>"><?php echo $l; ?></option>
    <?php } ?>
  </select>
  <?php  
}


// LIST OF SUPPORTED SCREENSHOT SIZES
function pwa_select_sizes2() {
  $list = array('800x600', '1024x768','1280x720','1366×768','1920×1080','3840x2160');
  ?>
  <select id="mb-sel-stick" class="mb-sel-stick">
    <option value=""><?php _e('Choose', 'pwa'); ?></option>
    
    <?php foreach($list as $l) { ?>
      <option value="<?php echo $l; ?>"><?php echo $l; ?></option>
    <?php } ?>
  </select>
  <?php  
}


// LIST OF SUPPORTED TYPES
function pwa_select_types() {
  $list = array('image/png','image/svg+xml','image/webp','image/gif','image/jpg','image/jpeg');
  ?>
  <select id="mb-sel-stick" class="mb-sel-stick">
    <option value=""><?php _e('Choose', 'pwa'); ?></option>
    
    <?php foreach($list as $l) { ?>
      <option value="<?php echo $l; ?>"><?php echo $l; ?></option>
    <?php } ?>
  </select>
  <?php  
}


// LIST OF SUPPORTED PURPOSES
function pwa_select_purposes() {
  $list = array('any','maskable','monochrome');
  ?>
  <select id="mb-sel-stick" class="mb-sel-stick">
    <option value=""><?php _e('Choose', 'pwa'); ?></option>
    
    <?php foreach($list as $l) { ?>
      <option value="<?php echo $l; ?>"><?php echo $l; ?></option>
    <?php } ?>
  </select>
  <?php  
}

// LIST OF SUPPORTED PLATFORMS
function pwa_select_platforms() {
  $list = array('chrome_web_store','play','itunes','webapp','windows','f-droid','amazon');
  ?>
  <select id="mb-sel-stick" class="mb-sel-stick">
    <option value=""><?php _e('Choose', 'pwa'); ?></option>
    
    <?php foreach($list as $l) { ?>
      <option value="<?php echo $l; ?>"><?php echo $l; ?></option>
    <?php } ?>
  </select>
  <?php  
}

// LIST OF SUPPORTED PLATFORMS2
function pwa_select_platforms2() {
  $list = array('wide','narrow','android','chromeos','ios','kaios','macos','windows','xbox','chrome_web_store','play','itunes','microsoft-inbox','microsoft-store');
  ?>
  <select id="mb-sel-stick" class="mb-sel-stick">
    <option value=""><?php _e('Choose', 'pwa'); ?></option>
    
    <?php foreach($list as $l) { ?>
      <option value="<?php echo $l; ?>"><?php echo $l; ?></option>
    <?php } ?>
  </select>
  <?php  
}


// CORE FUNCTIONS
function pwa_param($name) {
  return osc_get_preference($name, 'plugin-pwa');
}


if(!function_exists('pwa_param_update')) {
  function pwa_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? ($type == 'array' ? json_decode(osc_get_preference($param_name, $plugin_var_name), true) : osc_get_preference($param_name, $plugin_var_name)) : ($type == 'array' ? array() : '');
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, ($type == 'array' ? json_encode($val, JSON_UNESCAPED_SLASHES) : $val), $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => ($type == 'array' ? json_encode($val, JSON_UNESCAPED_SLASHES) : $val)), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function pwa_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}



// CATEGORIES WORK
function pwa_cat_tree($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
      $array[$c['pk_i_id']]['sub'] = pwa_cat_sub($list, $c['pk_i_id']);
    }
  }

  return $array;
}

function pwa_cat_sub($list, $parent_id) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {  echo $c['s_name'];
        $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
        $array[$c['pk_i_id']]['sub'] = pwa_cat_sub($list, $c['pk_i_id']);
      }
    }
  }
      
  return $array;
}

function pwa_cat_list($selected = array(), $categories = '', $level = 0) {
  if($categories == '' || $level == 0) {
    $categories = pwa_cat_tree($categories);
  }


  foreach($categories as $c) {
    echo '<option value="' . $c['pk_i_id'] . '" ' . (in_array($c['pk_i_id'], $selected) ? 'selected="selected"' : '') . '>' . str_repeat('-', $level) . ($level > 0 ? ' ' : '') . $c['s_name'] . '</option>';

    if(@count($c['sub']) > 0) {
      pwa_cat_list($selected, $c['sub'], $level + 1);
    }
  }
}


function pwa_list_values_ol($values) {
 if(count($values) > 0 && is_array($values)) {
    foreach($values as $v) {
      ?>

      <li class="mb-val" id="val_<?php echo $v['pk_i_id']; ?>">
        <?php pwa_div_value($v); ?>
      
        <ol>
          <?php 
            if(isset($v['values']) && count($v['values']) > 0) { 
              pwa_list_values_ol($v['values']); 
            }
          ?>
        </ol>
      </li>
    <?php
    }
  }
}

?>