<?php
/*
  Plugin Name: Progressive Web App (PWA) Plugin
  Plugin URI: https://osclasspoint.com/osclass-plugins/social-and-authentication/progressive-web-app-pwa-plugin-i192
  Description: Setup manifest and service workers and other PWA essential parameters enhance your website with PWA features.
  Version: 1.0.1
  Author: MB Themes
  Author URI: https://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: pwa
  Plugin update URI: pwa
  Support URI: https://forums.osclasspoint.com/pwa-profressive-web-app-osclass-plugin/
  Product Key: CUZuRRxIiooPd46TbQFo
*/

define('PWA_CACHE_NAME', 'pwa-osclass');

require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'model/ModelPWA.php';
require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'functions.php';


// INSTALL FUNCTION - DEFINE VARIABLES
function pwa_call_after_install() {
  osc_set_preference('mode', 'ADVANCED', 'plugin-pwa', 'STRING');
  osc_set_preference('debug', 1, 'plugin-pwa', 'INTEGER');
  osc_set_preference('meta_tags', 1, 'plugin-pwa', 'INTEGER');
  osc_set_preference('hook_manifest', 1, 'plugin-pwa', 'INTEGER');
  osc_set_preference('hook_worker', 1, 'plugin-pwa', 'INTEGER');
  osc_set_preference('add_schema', 1, 'plugin-pwa', 'INTEGER');
  osc_set_preference('sw_version', 1, 'plugin-pwa', 'INTEGER');
  osc_set_preference('background_sync', 1, 'plugin-pwa', 'INTEGER');
  osc_set_preference('periodic_sync', 1, 'plugin-pwa', 'INTEGER');
  osc_set_preference('strategy', 'NOCACHE', 'plugin-pwa', 'STRING');
  osc_set_preference('offline_image', '', 'plugin-pwa', 'STRING');
  osc_set_preference('favicon_ico', '', 'plugin-pwa', 'STRING');
  osc_set_preference('favicon_16', '', 'plugin-pwa', 'STRING');
  osc_set_preference('favicon_32', '', 'plugin-pwa', 'STRING');
  osc_set_preference('mask_icon', '', 'plugin-pwa', 'STRING');
  osc_set_preference('apple_icon', '', 'plugin-pwa', 'STRING');

  ModelPWA::newInstance()->install();
}


function pwa_call_after_uninstall() {
  ModelPWA::newInstance()->uninstall();
  pwa_unregister_worker();
  @unlink(osc_base_path() . 'manifest.webmanifest');
  @unlink(osc_base_path() . 'pwa-service-worker.js');
  @unlink(osc_base_path() . 'offline.html');
}


// ADMIN MENU
function pwa_menu($title = NULL) {
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/pwa/css/admin.css?v=' . date('YmdHis') . '" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/pwa/css/bootstrap-switch.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/pwa/css/tipped.css" rel="stylesheet" type="text/css" />';
  echo '<link href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet" type="text/css" />';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/pwa/js/admin.js?v=' . date('YmdHis') . '"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/pwa/js/tipped.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/pwa/js/bootstrap-switch.js"></script>';

  if( $title == '') { $title = __('Configure', 'pwa'); }

  $text  = '<div class="mb-head">';
  $text .= '<div class="mb-head-left">';
  $text .= '<h1>' . $title . '</h1>';
  $text .= '<h2>Progressive Web App (PWA) Plugin</h2>';
  $text .= '</div>';
  $text .= '<div class="mb-head-right">';
  $text .= '<ul class="mb-menu">';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=pwa/admin/configure.php"><i class="fa fa-wrench"></i><span>' . __('Configure', 'pwa') . '</span></a></li>';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=pwa/admin/manifest.php"><i class="fa fa-file-text-o"></i><span>' . __('Manifest', 'pwa') . '</span></a></li>';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=pwa/admin/worker.php"><i class="fa fa-gears"></i><span>' . __('Service Workers', 'pwa') . '</span></a></li>';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=pwa/admin/upload.php"><i class="fa fa-upload"></i><span>' . __('Upload Icons', 'pwa') . '</span></a></li>';
  $text .= '</ul>';
  $text .= '</div>';
  $text .= '</div>';

  echo $text;
}



// ADMIN FOOTER
function pwa_footer() {
  $pluginInfo = osc_plugin_get_info('pwa/index.php');
  $text  = '<div class="mb-footer">';
  $text .= '<a target="_blank" class="mb-developer" href="https://osclasspoint.com"><img src="https://osclasspoint.com/favicon.ico" alt="OsclassPoint Market" /> OsclassPoint Market</a>';
  $text .= '<a target="_blank" href="' . $pluginInfo['support_uri'] . '"><i class="fa fa-bug"></i> ' . __('Report Bug', 'pwa') . '</a>';
  $text .= '<a target="_blank" href="https://forums.osclasspoint.com/"><i class="fa fa-handshake-o"></i> ' . __('Support Forums', 'pwa') . '</a>';
  $text .= '<a target="_blank" class="mb-last" href="mailto:info@osclasspoint.com"><i class="fa fa-envelope"></i> ' . __('Contact Us', 'pwa') . '</a>';
  $text .= '<span class="mb-version">v' . $pluginInfo['version'] . '</span>';
  $text .= '</div>';

  return $text;
}



// ADD MENU LINK TO PLUGIN LIST
function pwa_admin_menu() {
echo '<h3><a href="#">Progressive Web App (PWA) Plugin</a></h3>
<ul> 
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php') . '">&raquo; ' . __('Configure', 'pwa') . '</a></li>
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/manifest.php') . '">&raquo; ' . __('Manifest', 'pwa') . '</a></li>
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/worker.php') . '">&raquo; ' . __('Service Workers', 'pwa') . '</a></li>
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/upload.php') . '">&raquo; ' . __('Upload Icons', 'pwa') . '</a></li>
</ul>';
}


// ADD MENU TO PLUGINS MENU LIST
osc_add_hook('admin_menu','pwa_admin_menu', 1);



// DISPLAY CONFIGURE LINK IN LIST OF PLUGINS
function pwa_conf() {
  osc_admin_render_plugin( osc_plugin_path( dirname(__FILE__) ) . '/admin/configure.php' );
}

osc_add_hook( osc_plugin_path( __FILE__ ) . '_configure', 'pwa_conf' );	


// CALL WHEN PLUGIN IS ACTIVATED - INSTALLED
osc_register_plugin(osc_plugin_path(__FILE__), 'pwa_call_after_install');

// SHOW UNINSTALL LINK
osc_add_hook(osc_plugin_path(__FILE__) . '_uninstall', 'pwa_call_after_uninstall');

?>