<?php
/*
  Plugin Name: QR Code Plugin
  Plugin URI: https://osclasspoint.com/
  Description: Add a qr code to your ad page, print it and share it offline
  Version: 1.3.1
  Author: MB Themes
  Author URI: https://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: qrcode
  Plugin update URI: qrcode
  Support URI: https://forums.osclasspoint.com/free-plugins/
  Product Key: RvR9IqCQA0IVv57n7PFx
*/

if(!defined('QRCODE_MARGIN')) {
  define('QRCODE_MARGIN', 0);
}

function qrcode_uploads_path() {
  return osc_content_path().'uploads/qrcode/';
}

function qrcode_uploads_url() {
  return osc_base_url().'oc-content/uploads/qrcode/';
}

function qrcode_install() {
  @mkdir(osc_content_path().'uploads/qrcode/');
  $conn= getConnection();
  osc_set_preference('code_size', '3', 'qrcode', 'INTEGER');
  $conn->commit();
}

function qrcode_uninstall() {
  $conn= getConnection();
  osc_delete_preference('code_size', 'qrcode');
  
  $conn->commit();
  $files = glob(qrcode_uploads_path() . "*.png");
  foreach($files as $f) {
    @unlink($f);
  }
  @rmdir(qrcode_uploads_path());
}


function qrcode_admin_menu() {
  if(osc_version()<320) {
    echo '<h3><a href="#">QR Code</a></h3>
    <ul>
      <li><a href="' . osc_admin_render_plugin_url(osc_plugin_folder(__FILE__) . 'conf.php') . '">&raquo; ' . __('Settings', 'qrcode') . '</a></li>
      <li><a href="' . osc_admin_render_plugin_url(osc_plugin_folder(__FILE__) . 'help.php') . '">&raquo; ' . __('Help', 'qrcode') . '</a></li>
    </ul>';
  } else {
    osc_add_admin_submenu_divider('plugins', 'QR Codes', 'qrcode_divider', 'administrator');
    osc_add_admin_submenu_page('plugins', __('QR Settings', 'qrcode'), osc_admin_render_plugin_url(osc_plugin_folder(__FILE__) . 'conf.php'), 'qrcode_settings', 'administrator');
    osc_add_admin_submenu_page('plugins', __('QR Help', 'qrcode'), osc_admin_render_plugin_url(osc_plugin_folder(__FILE__) . 'help.php'), 'qrcode_help', 'administrator');
  }
}

function qrcode_delete_item($itemId) {
  $files = glob(qrcode_uploads_path() .$itemId."_*");
  foreach($files as $f) {
    @unlink($f);
  }
}

function qrcode_size() {
  $size = (osc_get_preference("code_size", "qrcode") > 0 ? osc_get_preference("code_size", "qrcode") : 2);
  $size = ($size > 10 ? 10 : $size);

  return $size;  
}

function qrcode_generateqr($data, $id = '') {
  include "lib/qrlib.php";
  
  $size = qrcode_size();
  
  if($id!='') {
    $filename = $id."_".md5($data)."_".$size.".png";
  } else {
    $filename = md5($data)."_".$size.".png";
  }
  $filename = qrcode_uploads_path().$filename;
  QRcode::png($data, $filename, 'M', $size, QRCODE_MARGIN);
}

function show_qrcode() {
  $filename = osc_item_id()."_".md5(osc_item_url())."_".qrcode_size().".png";
  if(!file_exists(qrcode_uploads_path().$filename)) {
    qrcode_generateqr(osc_item_url(), osc_item_id());
  }
  
  echo '<img src="'.qrcode_uploads_url().$filename.'" alt="QR CODE" id="qrcode_'.osc_item_id().'" class="qrcode" />';
}


/**
 * ADD HOOKS
 */
osc_register_plugin(osc_plugin_path(__FILE__), 'qrcode_install');
osc_add_hook(osc_plugin_path(__FILE__)."_uninstall", 'qrcode_uninstall');

// DELETE ITEM
osc_add_hook('delete_item', 'qrcode_delete_item');

// FANCY MENU
if(osc_version()<320) {
  osc_add_hook('admin_menu', 'qrcode_admin_menu');
} else {
  osc_add_hook('admin_menu_init', 'qrcode_admin_menu');
}



// DISPLAY CONFIGURE LINK IN LIST OF PLUGINS
function qrcode_conf() {
  osc_admin_render_plugin( osc_plugin_path( dirname(__FILE__) ) . '/conf.php' );
}

osc_add_hook( osc_plugin_path( __FILE__ ) . '_configure', 'qrcode_conf' );	


?>