<?php
/*
  Plugin Name: SMS Payments
  Plugin URI: https://osclasspoint.com/osclass-plugins/messaging-and-communication/sms-notification-and-verification-plugin_i104
  Description: Plugin allows users to Promote and Move to Top their listings via SMS
  Version: 1.1.1
  Author: MB Themes
  Author URI: https://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: sms_payments
  Plugin update URI: sms-payment-plugin
  Support URI: https://forums.osclasspoint.com/sms-payments-plugin/
  Product Key: 2mdX3OOMOsFdKnYHx4Cp
*/

require_once 'model/ModelSP.php';
require_once 'email.php';


function sp_call_after_install() {
  ModelSP::newInstance()->import('sms_payments/model/struct.sql');
  osc_set_preference('promote_service_id', '', 'plugin-sms_payments', 'STRING');
  osc_set_preference('promote_secret', '', 'plugin-sms_payments', 'STRING');
  osc_set_preference('promote_expiration_days', 30, 'plugin-sms_payments', 'INTEGER');
  osc_set_preference('promote_test_mode', 1, 'plugin-sms_payments', 'INTEGER');
  osc_set_preference('promote_button', 1, 'plugin-sms_payments', 'INTEGER');

  osc_set_preference('movetop_service_id', '', 'plugin-sms_payments', 'STRING');
  osc_set_preference('movetop_secret', '', 'plugin-sms_payments', 'STRING');
  osc_set_preference('movetop_test_mode', 1, 'plugin-sms_payments', 'INTEGER');
  osc_set_preference('movetop_button', 1, 'plugin-sms_payments', 'INTEGER');

  osc_set_preference('log_history_days', 180, 'plugin-sms_payments', 'INTEGER');


  
  //upload email templates
  foreach(osc_listLocales() as $loc) {
    // email template - promotion has ended
    $des_promotion[$loc['code']]['s_title'] = '{WEB_TITLE} - Promotion of listing #{ITEM_ID} has ended';
    $des_promotion[$loc['code']]['s_text'] = '<p>Hi {CONTACT_NAME}!</p> <p>Let us inform you, that promotion of your listing: <strong>{ITEM_TITLE} (#{ITEM_ID})</strong> has ended today.</p> <p>If you want to promote this listing again, please visit our site on following link:</p> <p><strong><span style="color: #1400ff;">{ITEM_URL}</span></strong></p> <p>Regards, <br />{WEB_TITLE}</p>';
  }
  
  Page::newInstance()->insert( array('s_internal_name' => 'sp_promotion_ended', 'b_indelible' => '1'), $des_promotion );
}


function sp_call_after_uninstall() {
  ModelSP::newInstance()->uninstall();
  osc_delete_preference('promote_service_id', 'plugin-sms_payments');
  osc_delete_preference('promote_secret', 'plugin-sms_payments');
  osc_delete_preference('promote_expiration_days', 'plugin-sms_payments');
  osc_delete_preference('promote_test_mode', 'plugin-sms_payments');
  osc_delete_preference('promote_button', 'plugin-sms_payments');
  osc_delete_preference('movetop_service_id', 'plugin-sms_payments');
  osc_delete_preference('movetop_secret', 'plugin-sms_payments');
  osc_delete_preference('movetop_test_mode', 'plugin-sms_payments');
  osc_delete_preference('movetop_button', 'plugin-sms_payments');
  osc_delete_preference('log_history_days', 'plugin-sms_payments');
}


// DEFINE CSS STYLES FOR FRONT-END
function sp_user_styles() {
  osc_enqueue_style('sp-style', osc_base_url() . 'oc-content/plugins/sms_payments/css/front.css');
  osc_enqueue_style('open-sans', '//fonts.googleapis.com/css?family=Open+Sans:300,600&amp;subset=latin,latin-ext');
  osc_enqueue_style('font-awesome47', '//maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css');
}


// DEFINE JAVASCRIPT FOR FRONT-END
function sp_user_scripts() {
  if(osc_is_ad_page()) {
    echo '<script src="https://assets.fortumo.com/fmp/fortumopay.js" type="text/javascript" data-cfasync="false"></script>';
  }
}


// INCLUDE MAILER SCRIPT
function sp_include_mailer() {
  if(file_exists(osc_lib_path() . 'phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'phpmailer/class.phpmailer.php';
  } else if(file_exists(osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php')) {
    require_once osc_lib_path() . 'vendor/phpmailer/phpmailer/class.phpmailer.php';
  }
}


// CHECK IF RUNNING ON DEMO
function sp_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}

  


// PAYMENT MANAGER FOR PROMOTIONS
function sp_manage_payment_promotion() {   
  $secret = osc_get_preference('promote_secret', 'plugin-sms_payments');         // Secret key from application
  $days = osc_get_preference('promote_expiration_days', 'plugin-sms_payments');  // Secret key from application
  $sender = Params::getParam('sender');                                          // Payment sender
  $amount = Params::getParam('amount');                                          // Amount of payment
  $price = Params::getParam('price');                                            // Price for payment
  $revenue = Params::getParam('revenue');                                        // Revenue for transaction
  $currency = Params::getParam('currency');                                      
  $country = Params::getParam('country');                                        
  $item_id = Params::getParam('cuid');                                           // Item ID
  $payment_id = Params::getParam('payment_id');                                  // ID of payment

  
  if(!is_numeric($amount)) {
    $amount = 1;
  }

  $days = $amount * $days;

  if( $sender <> '' AND $item_id > 0 AND payment_id <> '' AND !empty($secret) AND sp_signature_verification(Params::getParamsAsArray(), $secret)) {
    
    $check = ModelSP::newInstance()->getExpire( $item_id );
    $today = date('Y-m-d');


    if(date('Y-m-d', strtotime($check['expire_date'])) > $today) {
      $start_date = $check['expire_date'];
    } else {
      $start_date = $today;
    }

    $expire = date('Y-m-d', strtotime(" + " . $days . " day", strtotime($start_date)));



    // Make sure request comming from fortumo servers
    // It may be required to update IP list after some time
    if(!in_array($_SERVER['REMOTE_ADDR'], array('54.72.6.126','54.72.6.27','54.72.6.17','54.72.6.23','79.125.125.1','79.125.5.95','79.125.5.205'))) {
      sp_log_payment_short( $item_id, 0, 'promote', 'Accessing script from unauthorized IP on ' . date('Y-m-d H:i:s') );
      header("HTTP/1.0 403 Forbidden");
      die("ERROR: Invalid request origin (IP)");
    }


    // Check status of payment
    if(preg_match("/failed/i", Params::getParam('status'))) {
      // Payment has failed	
      sp_log_payment_short( $item_id, 0, 'promote', 'Unsuccessful payment for item #' . $item_id . ' on ' . date('Y-m-d H:i:s') );

    } else {
      // Payment successful
      sp_log_payment( $item_id, 1, 'promote', $payment_id, $sender, $country, $amount, $price, $revenue, $currency, $expire );

      sp_change_premium( $item_id, 1, $expire );       
    }
  }
}



// PAYMENT MANAGER FOR MOVE TO TOP
function sp_manage_payment_move_top() {   
  $secret = osc_get_preference('movetop_secret', 'plugin-sms_payments');         // Secret key from application
  $sender = Params::getParam('sender');                                          // Payment sender
  $amount = Params::getParam('amount');                                          // Amount of payment
  $price = Params::getParam('price');                                            // Price for payment
  $revenue = Params::getParam('revenue');                                        // Revenue for transaction
  $currency = Params::getParam('currency');                                      
  $country = Params::getParam('country');                                        
  $item_id = Params::getParam('cuid');                                           // Item ID
  $payment_id = Params::getParam('payment_id');                                  // ID of payment


  if(!is_numeric($amount)) {
    $amount = 1;
  }

  if( $sender <> '' AND $item_id > 0 AND payment_id <> '' AND !empty($secret) AND sp_signature_verification(Params::getParamsAsArray(), $secret)) {

    // Make sure request comming from fortumo servers
    // It may be required to update IP list after some time
    if(!in_array($_SERVER['REMOTE_ADDR'], array('54.72.6.126','54.72.6.27','54.72.6.17','54.72.6.23','79.125.125.1','79.125.5.95','79.125.5.205'))) {
      sp_log_payment_short( $item_id, 0, 'promote', 'Accessing script from unauthorized IP on ' . date('Y-m-d H:i:s') );
      header("HTTP/1.0 403 Forbidden");
      die("ERROR: Invalid request origin (IP)");
    }


    // Check status of payment
    if(preg_match("/failed/i", Params::getParam('status'))) {
      // Payment has failed	
      sp_log_payment_short( $item_id, 0, 'movetop', 'Unsuccessful payment for item #' . $item_id . ' on ' . date('Y-m-d H:i:s') );

    } else {
      // Payment successful
      sp_log_payment( $item_id, 1, 'movetop', $payment_id, $sender, $country, $amount, $price, $revenue, $currency );

      sp_move_top( $item_id );       
    }
  }
}


// MANAGE BOTH PAYMENTS
function sp_manage_payments() {
  $secret_promotion = osc_get_preference('promote_secret', 'plugin-sms_payments');   // Secret key from application (promotion)
  $secret_move_top = osc_get_preference('movetop_secret', 'plugin-sms_payments');    // Secret key from application (move to top)
  $sender = Params::getParam('sender');                                              // Payment sender
  $item_id = Params::getParam('cuid');                                               // Item ID
  $payment_id = Params::getParam('payment_id');                                      // ID of payment

  sp_manage_payment_move_top();
  sp_manage_payment_promotion();


  // CHECK IF SIGNATURE VALIDATION DID NOT FAILED
  if( $sender <> '' AND $item_id > 0 AND payment_id <> '' ) {

    if(($secret_promotion <> '' AND !sp_signature_verification(Params::getParamsAsArray(), $secret_promotion)) AND ($secret_move_top <> '' AND !sp_signature_verification(Params::getParamsAsArray(), $secret_move_top))) {
      $message = __('Secret key for Promotion or Move-to-Top service is incorrect', 'sms_payments');
    }


    if($message <> '') {
      osc_add_flash_error_message( __('Signature validation has failed. Secret keys in your API settings does not match to one provided by Fortumo!') . ' ' . $message);
      sp_log_payment_short( $item_id, 0, 'both', 'Signature validation has failed. Secret keys in your API settings does not match to one provided by Fortumo!' . ' ' . $message );
    }
  }
}

osc_add_hook('before_html', 'sp_manage_payments');



// FUNCTION TO HANDLE VISITOR SENT BACK AFTER PAYMENT
function sp_redirect_payment()  {
  $secret_promotion = osc_get_preference('promote_secret', 'plugin-sms_payments');   // Secret key from application (promotion)
  $secret_move_top = osc_get_preference('movetop_secret', 'plugin-sms_payments');    // Secret key from application (move to top)


  // PROMOTION
  if( Params::getParam('page') == 'sms-payment' && Params::getParam('action') == 'promote' && Params::getParam('item_id') <> '' )  {
    osc_add_flash_ok_message( __('Listing has been promoted', 'sms_payments'));
      sp_log_payment_short( $item_id, 0, 'both', 'Signature validation has failed. Secret keys in your API settings does not match to one provided by Fortumo!' . ' ' . $message );

    $item_id = Params::getParam('item_id');
    BaseModel::redirectTo( osc_item_url_ns($item_id) );
  }


  // MOVE TO TOP
  if( Params::getParam('page') == 'sms-payment' && Params::getParam('action') == 'movetop' && Params::getParam('item_id') <> '' )  {
    osc_add_flash_ok_message( __('Listing has been moved to top', 'sms_payments'));

    $item_id = Params::getParam('item_id');
    BaseModel::redirectTo( osc_item_url_ns($item_id) );
  }
}



// REMOVE PREMIUMS WHEN IT EXPIRE
function sp_clear_premium() {
  $today = date('Y-m-d');

  $items = ModelSP::newInstance()->getExpiredItems($today);

  foreach($items as $i) {
    sp_change_premium($i['item_id'], 0);
    email_promotion_ended($i['item_id']);
  }
}



// REMOVE OLD PAYMENT LOGS
function sp_clear_logs() {
  $log_days_remove = osc_get_preference('log_days_remove', 'plugin-sms_payments');                 // After how many days should be logs removed
  $log_days_remove = ($log_days_remove > 0 ? $log_days_remove : 180);

  $today = date('Y-m-d');
  $max_date = date('Y-m-d', strtotime(" - " . $log_days_remove . " day", strtotime($today)));

  ModelSP::newInstance()->removeOldLogs($max_date);
}

function sp_log_payment( $item_id, $status, $type, $payment_id, $customer_id, $country, $amount, $price, $revenue, $currency, $expire = NULL ) {
  ModelSP::newInstance()->logPayment( $item_id, $status, $type, $payment_id, $customer_id, $country, $amount, $price, $revenue, $currency, $expire );
}


function sp_log_payment_short( $item_id, $status, $type, $value = NULL) {
  ModelSP::newInstance()->logPaymentShort( $item_id, $status, $type, $value );
}


function sp_change_premium( $item_id, $premium, $date = NULL) {
  ModelSP::newInstance()->updatePremiumByItemId( $item_id, $premium );                    // change b_premium in t_item table

  // CHANGE EXPIRATION
  $check = ModelSP::newInstance()->getExpire( $item_id );

  if( isset($check['expire_date']) && $check['expire_date'] <> '' ) {
    ModelSP::newInstance()->updateExpireByItemId( $item_id, $date );
  } else {
    ModelSP::newInstance()->insertExpireByItemId( $item_id, $date );
  }
}



function sp_move_top( $item_id ) {
  ModelSP::newInstance()->moveTopByItemId( $item_id );                                   // reset listing publish date to current date
}



function sp_button_promote( $item_id )  {
  $item = Item::newInstance()->findByPrimaryKey( $item_id );
  $check = ModelSP::newInstance()->getExpire( $item_id );

  $service_id = osc_get_preference('promote_service_id', 'plugin-sms_payments');
  $test_mode = osc_get_preference('promote_test_mode', 'plugin-sms_payments');
  $days = osc_get_preference('promote_expiration_days', 'plugin-sms_payments');
  $days = ($days > 0 ? $days : 30);

  if(isset($check['expire_date']) && $check['expire_date'] <> '') {
    $start = $check['expire_date'];
  } else {
    $start = date('Y-m-d');
    $check['expire_date'] = '';
  }

  $will_end = date('d. M Y', strtotime(" + " . $days . " day", strtotime($start)));


  $link = $service_id . '/' . $item_id . ($test_mode == 1 ? '?test=ok' : '');

  // Check if item is already premium
  if( $item['b_premium'] == 0 ) {
    $text = __('Promote listing up to', 'sms_payments') . ' ' . $will_end;
  } else {
    $text = __('Extend promotion up to', 'sms_payments') . ' ' . $will_end;
  }

  $result  = '<a id="fmp-button" href="#" class="sp-promote-button" rel="' . $link . '" title="' . $text . '">';
  $result .= '<span class="sp-top">' . __('Promote listing', 'sms_payments') . '</span>';
  $result .= '<span class="sp-bot">' . __('for', 'sms_payments') . ' ' . ($check['expire_date'] <> '' ? __('next', 'sms_payments') . ' ' : '' ) . $days . ' ' . ($days == 1 ? __('day', 'sms_payments') : __('days', 'sms_payments')) . '</span>';
  $result .= '<i class="fa fa-mobile"></i>';
  $result .= '</a>';

  return $result;
}
  

function sp_button_move_top( $item_id )  {
  $service_id = osc_get_preference('movetop_service_id', 'plugin-sms_payments');
  $test_mode = osc_get_preference('movetop_test_mode', 'plugin-sms_payments');

  $link = $service_id . '/' . $item_id . ($test_mode == 1 ? '?test=ok' : '');

  $result  = '<a id="fmp-button" href="#" class="sp-movetop-button" rel="' . $link . '" title="' . __('Move listing to top of list and get more views', 'sms_payments') . '">';
  $result .= '<span class="sp-top">' . __('Move to Top', 'sms_payments') . '</span>';
  $result .= '<span class="sp-bot">' . __('and get more views', 'sms_payments') . '</span>';
  $result .= '<i class="fa fa-arrow-up"></i>';
  $result .= '</a>';

  return $result;
}


function sp_buttons( $item_id ) {
  $result = '';

  if( osc_get_preference('promote_button', 'plugin-sms_payments') == 1 ) {
    $result .= sp_button_promote( $item_id );
  }

  if( osc_get_preference('movetop_button', 'plugin-sms_payments') == 1 ) {
    $result .= sp_button_move_top( $item_id );
  }

  return $result;
}


function sp_signature_verification($params_array, $secret) {
  //ksort($params_array);

  $str = '';
  foreach ($params_array as $k=>$v) {
    if($k != 'sig') {
      $str .= "$k=$v";
    }
  }
  
  $str .= $secret;
  $signature = md5($str);

  return ($params_array['sig'] == $signature);
}


if(!function_exists('sp_param_update')) {
  function sp_param_update( $param_name, $update_param_name, $type = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, 'plugin-sms_payments' ) != '' ) ? osc_get_preference( $param_name, 'plugin-sms_payments' ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, 'plugin-sms_payments') != '' ) ? osc_get_preference( $param_name, 'plugin-sms_payments' ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name ) == '') {
        osc_set_preference( $param_name, $val, 'plugin-sms_payments', 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => "plugin-sms_payments", "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}

if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}

function sp_menu($title = NULL) {
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/sms_payments/css/admin.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/sms_payments/css/bootstrap-switch.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/sms_payments/css/tipped.css" rel="stylesheet" type="text/css" />';
  echo '<link href="//fonts.googleapis.com/css?family=Open+Sans:300,600&amp;subset=latin,latin-ext" rel="stylesheet" type="text/css" />';
  echo '<link href="//maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet" type="text/css" />';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/sms_payments/js/admin.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/sms_payments/js/tipped.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/sms_payments/js/bootstrap-switch.js"></script>';

  if( $title == '') { $title = __('Configure', 'sms_payments'); }

  $text  = '<div class="mb-head">';
  $text .= '<div class="mb-head-left">';
  $text .= '<h1>' . $title . '</h1>';
  $text .= '<h2>SMS Payments Plugin</h2>';
  $text .= '</div>';
  $text .= '<div class="mb-head-right">';
  $text .= '<ul class="mb-menu">';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=sms_payments/admin/configure.php"><i class="fa fa-wrench"></i><span>' . __('Configure', 'sms_payments') . '</span></a></li>';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=sms_payments/admin/logs.php"><i class="fa fa-money"></i><span>' . __('Payment Logs', 'sms_payments') . '</span></a></li>';
  $text .= '</ul>';
  $text .= '</div>';
  $text .= '</div>';

  echo $text;
}


function sp_footer() {
  $pluginInfo = osc_plugin_get_info('sms_payments/index.php');
  $text  = '<div class="mb-footer">';
  $text .= '<a target="_blank" class="mb-developer" href="http://osclasspoint.com"><img src="http://osclasspoint.com/favicon.ico" alt="MB Themes" /> osclasspoint.com</a>';
  $text .= '<a target="_blank" href="' . $pluginInfo['support_uri'] . '"><i class="fa fa-bug"></i> ' . __('Report Bug', 'sms_payments') . '</a>';
  $text .= '<a target="_blank" href="http://forums.osclasspoint.com/"><i class="fa fa-comments"></i> ' . __('Support Forums', 'sms_payments') . '</a>';
  $text .= '<a target="_blank" class="mb-last" href="mailto:info@osclasspoint.com"><i class="fa fa-envelope"></i> ' . __('Contact Us', 'sms_payments') . '</a>';
  $text .= '<span class="mb-version">v' . $pluginInfo['version'] . '</span>';
  $text .= '</div>';

  return $text;
}


// Add menu link to Plugin list
function sp_admin_menu() {
echo '<h3><a href="#">SMS Payments</a></h3>
<ul> 
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php') . '">&raquo; ' . __('Configure', 'sms_payments') . '</a></li>
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/logs.php') . '">&raquo; ' . __('Payment Logs', 'sms_payments') . '</a></li>
</ul>';
}

// Display configure link
function sp_conf() {
  osc_admin_render_plugin( osc_plugin_path( dirname(__FILE__) ) . '/admin/configure.php' ) ;
}

// This is a hack to show a Configure link at plugins table (you could also use some other hook to show a custom option panel)
osc_add_hook( osc_plugin_path( __FILE__ ) . '_configure', 'sp_conf' ) ;  

//Adding sub menu to plugins menu in dashboard
osc_add_hook('admin_menu','sp_admin_menu', 1);

// This is needed in order to be able to activate the plugin
osc_register_plugin(osc_plugin_path(__FILE__), 'sp_call_after_install') ;

// This is a hack to show a Uninstall link at plugins table (you could also use some other hook to show a custom option panel)
osc_add_hook(osc_plugin_path(__FILE__) . '_uninstall', 'sp_call_after_uninstall');


// Redirect successful payment to listing page
osc_add_hook('before_html', 'sp_redirect_payment');

// Add styles to front-end
osc_add_hook('header', 'sp_user_styles');

// Add scripts to front-end
osc_add_hook('footer', 'sp_user_scripts');

// Remove old logs every day
osc_add_hook('cron_daily', 'sp_clear_logs');

// Remove premium from expired listings
osc_add_hook('cron_daily', 'sp_clear_premium');
?>