<?php
class ModelSP extends DAO {
  private static $instance;

  public static function newInstance() {
    if( !self::$instance instanceof self ) {
      self::$instance = new self;
    }
    return self::$instance;
  }

  function __construct() { 
    parent::__construct(); 
  }

  public function getTable_Item() {
    return DB_TABLE_PREFIX.'t_item';
  }

  public function getTable_PremiumExpire() {
    return DB_TABLE_PREFIX.'t_item_sms_premium_expire';
  }

  public function getTable_Logs() {
    return DB_TABLE_PREFIX.'t_log_sms_payment';
  }

  public function getTable_User() {
    return DB_TABLE_PREFIX.'t_user';
  }


  public function import($file) {
    $path = osc_plugin_resource($file) ;
    $sql = file_get_contents($path);

    if(!$this->dao->importSQL($sql)){ throw new Exception("Error importSQL::ModelSP<br>".$file.'<br>'.$path.'<br><br>Please check your database for tables t_item_sms_premium_expire, t_log_sms_payment.<br>If any of those tables exist in your database, delete them!');}
  }

  public function uninstall() {
    $this->dao->query('DROP TABLE '.$this->getTable_PremiumExpire());
    $this->dao->query('DROP TABLE '.$this->getTable_Logs());
  }
  
  
  // LOGS
  public function logPayment( $item_id, $successful, $type, $payment_id, $customer_id, $country, $amount, $price, $revenue, $currency, $expire ) {
    $aSet = array(
      'item_id' => $item_id,
      'successful' => $successful,
      'payment_type' => $type,
      'payment_id' => $payment_id,
      'customer_id' => $customer_id,
      'country' => $country,
      'amount' => $amount,
      'price' => $price,
      'revenue' => $revenue,
      'currency' => $currency,
      'expire_date' => $expire,
      'log_date' => date('Y-m-d H:i:s')
    );

    $this->dao->insert( $this->getTable_Logs(), $aSet);
    $id = $this->dao->insertedId();

    osc_run_hook('sms_log_saved', $id);
  
    return $id;
  }  


  public function logPaymentShort( $item_id, $successful, $type, $value ) {
    $aSet = array(
      'item_id' => $item_id,
      'successful' => $successful,
      'payment_type' => $type,
      'message' => $value,
      'log_date' => date('Y-m-d H:i:s')
    );

    $this->dao->insert( $this->getTable_Logs(), $aSet);
    return $this->dao->insertedId();
  }  

  
  public function getLogs( $type = NULL, $successful = NULL ) {
    $this->dao->select();
    $this->dao->from( $this->getTable_Logs() );

    if($type <> '') {
      $this->dao->where( 'payment_type', $type );
    }

    if($successful <> '') {
      $this->dao->where( 'successful', $successful );
    }

    $this->dao->orderBy( 'record_id DESC' );


    $result = $this->dao->get();

    if( !$result ) { return array(); }
    $prepare = $result->result();
    return $prepare;
  }


  public function getLogsStats() {
    $this->dao->select('extract(year from log_date) as log_year, extract(month from log_date) as log_month, currency, sum(amount) as amount, sum(price) as price, sum(revenue) as revenue');
    $this->dao->from( $this->getTable_Logs() );
    $this->dao->where( 'successful = 1' );
    $this->dao->groupBy( 'extract(year from log_date), extract(month from log_date), currency' );
    $this->dao->orderBy( 'log_date DESC' );

    $result = $this->dao->get();

    if( !$result ) { return array(); }
    $prepare = $result->result();
    return $prepare;
  }

  public function getLogsSummary() {
    $this->dao->select('currency, sum(amount) as amount, sum(price) as price, sum(revenue) as revenue');
    $this->dao->from( $this->getTable_Logs() );
    $this->dao->where( 'successful = 1' );
    $this->dao->groupBy( 'currency' );

    $result = $this->dao->get();

    if( !$result ) { return array(); }
    $prepare = $result->result();
    return $prepare;
  }
  
  
  public function removeOldLogs( $max_date ) {
    return $this->dao->query('DELETE FROM '.$this->getTable_Logs().' WHERE log_date <= "' . $max_date . '"');
  }
  

  public function updatePremiumByItemId( $item_id, $premium ) {
    $aSet = array('b_premium' => $premium);
    $aWhere = array('pk_i_id' => $item_id);

    return $this->_update($this->getTable_Item(), $aSet, $aWhere);
  }  
 

  public function updateExpireByItemId( $item_id, $date = NULL ) {
    $aSet = array('expire_date' => $date);
    $aWhere = array('item_id' => $item_id);

    return $this->_update($this->getTable_PremiumExpire(), $aSet, $aWhere);
  } 


  public function insertExpireByItemId( $item_id, $date ) {
    $aSet = array(
      'item_id' => $item_id,
      'expire_date' => $date
    );

    return $this->dao->insert( $this->getTable_PremiumExpire(), $aSet);
  }  


  public function moveTopByItemId( $item_id ) {
    $aSet = array('dt_pub_date' => date('Y-m-d H:i:s'));
    $aWhere = array('pk_i_id' => $item_id);

    return $this->_update($this->getTable_Item(), $aSet, $aWhere);
  } 
  

  public function getExpire( $item_id ) {
    $this->dao->select();
    $this->dao->from( $this->getTable_PremiumExpire() );
    $this->dao->where('item_id', $item_id );

    $result = $this->dao->get();
    if( !$result ) { return array(); }
    return $result->row();
  }


  public function getExpiredItems( $today ) {
    $this->dao->select();
    $this->dao->from( $this->getTable_PremiumExpire() );
    $this->dao->where('date(expire_date) = "' . $today . '"');

    $result = $this->dao->get();

    if( !$result ) { return array(); }
    $prepare = $result->result();
    return $prepare;
  }
  

  // update
  function _update($table, $values, $where) {
    $this->dao->from($table);
    $this->dao->set($values);
    $this->dao->where($where);
    return $this->dao->update();
  }
}
?>