<?php
  // Create menu
  $title = __('Configure', 'two_factor_auth');
  tfa_menu($title);
 

  // GET & UPDATE PARAMETERS
  // $variable = mb_param_update( 'param_name', 'form_name', 'input_type', 'plugin_var_name' );
  // input_type: check or value
  $required = mb_param_update('required', 'plugin_action', 'check', 'plugin-two_factor_auth');
  $discrepancy = mb_param_update('discrepancy', 'plugin_action', 'value', 'plugin-two_factor_auth');
  $app_name = mb_param_update('app_name', 'plugin_action', 'value', 'plugin-two_factor_auth');
  $success_message = mb_param_update('success_message', 'plugin_action', 'check', 'plugin-two_factor_auth');
  $hook_link = mb_param_update('hook_link', 'plugin_action', 'check', 'plugin-two_factor_auth');
  $threshold_days = mb_param_update('threshold_days', 'plugin_action', 'value', 'plugin-two_factor_auth');


  if(Params::getParam('plugin_action') == 'done') {
    message_ok( __('Settings were successfully saved', 'two_factor_auth') );
  }
  
  tfa_load_libs();
  $tfa = new RobThree\Auth\TwoFactorAuth(tfa_app_name());
?>

<div class="mb-body">
  <!-- CONFIGURE SECTION -->
  
  <div class="mb-notes">
    <div class="mb-line"><?php echo sprintf(__('Make sure your server-time is %s! Depending on the discrepancy allowed your time cannot drift too much from the users\' time!', 'two_factor_auth'), '<a href="http://en.wikipedia.org/wiki/Network_Time_Protocol">' . __('NTP-synced', 'two_factor_auth') . '</a>'); ?></div>

    <?php
      try {
        @$tfa->ensureCorrectTime();
        echo '<div class="mb-line">' . __('Your hosts time seems to be correct / within margin.', 'two_factor_auth') . '</div>';
      } catch (RobThree\Auth\TwoFactorAuthException $e) {
        $code = $e->getCode();
        $err = $e->getMessage();

        if($code === 0) {
          echo '<div class="mb-line">' . __('Unable to verify if time on your hosting is correct, socket functions not available.', 'two_factor_auth') . '</div>';
        } else {
          echo '<div class="mb-line">' . sprintf(__('Warning: Your hosts time seems to be off: %s.', 'two_factor_auth'), $err) . '</div>';
        }
      }
    ?>
    
    <div class="mb-line"><?php echo sprintf(__('Your current PHP timestamp is %s (%s), your unix timestamp is %s (%s).', 'two_factor_auth'), @$_SERVER['REQUEST_TIME'], date('Y-m-d H:i:s', @$_SERVER['REQUEST_TIME']), time(), date('Y-m-d H:i:s', time())); ?></div>
  </div>
  
  <div class="mb-box">
    <div class="mb-head">
      <i class="fa fa-wrench"></i> <?php _e('Configure', 'two_factor_auth'); ?>
    </div>

    <div class="mb-inside">
      <form name="promo_form" action="<?php echo osc_admin_base_url(true); ?>" method="POST" enctype="multipart/form-data" >
        <input type="hidden" name="page" value="plugins" />
        <input type="hidden" name="action" value="renderplugin" />
        <input type="hidden" name="file" value="<?php echo osc_plugin_folder(__FILE__); ?>configure.php" />
        <input type="hidden" name="plugin_action" value="done" />


        <div class="mb-row">
          <label for="required"><span><?php _e('Required', 'two_factor_auth'); ?></span></label> 
          <input name="required" id="required" type="checkbox" class="element-slide" <?php echo ($required == 1 ? 'checked' : ''); ?>/>
          
          <div class="mb-explain">
            <div class="mb-line"><?php _e('When enabled, users will be forced to setup Two-factor authentication if they did not do that yet.', 'two_factor_auth'); ?></div>
          </div>
        </div>
        
        <div class="mb-row">
          <label for="discrepancy"><span><?php _e('Tolerance of Expired Codes', 'two_factor_auth'); ?></span></label> 
          <input name="discrepancy" id="discrepancy" type="number" value="<?php echo osc_esc_html($discrepancy); ?>" min="1" max="10"/>
          <div class="mb-input-desc"><?php _e('codes', 'two_factor_auth'); ?></div>

          <div class="mb-explain">
            <div class="mb-line"><?php _e('The value of discrepancy is the number of timeslices checked in both directions of the current one, so in case time on your server is little bit shifted comparing to real time, increasing this parameter may help.', 'two_factor_auth'); ?></div>
          </div>
        </div>

        <div class="mb-row">
          <label for="threshold_days"><span><?php _e('Threshold Days', 'two_factor_auth'); ?></span></label> 
          <input name="threshold_days" id="threshold_days" type="number" value="<?php echo osc_esc_html($threshold_days); ?>" min="0"/>
          <div class="mb-input-desc"><?php _e('days', 'two_factor_auth'); ?></div>
          
          <div class="mb-explain">
            <div class="mb-line"><?php _e('If set to 0, two-factor authentication is initiated after each login. If set to value greater than 0, then it means user must use two-factor authentication after entered number of days.', 'two_factor_auth'); ?></div>
            <div class="mb-line"><?php _e('Recommended value is between 7 and 30 days, so authentication is not annoying.', 'two_factor_auth'); ?></div>
          </div>
        </div>
        
        <div class="mb-row">
          <label for="app_name"><span><?php _e('Application Name', 'two_factor_auth'); ?></span></label> 
          <input name="app_name" id="app_name" type="text" value="<?php echo osc_esc_html($app_name); ?>" size="60"/>
          
          <div class="mb-explain">
            <div class="mb-line"><?php _e('Will be displayed in the users app as the default issuer name when using QR code to import the secret.', 'two_factor_auth'); ?></div>
          </div>
        </div>
        
        <div class="mb-row">
          <label for="success_message"><span><?php _e('Success Message after Login', 'two_factor_auth'); ?></span></label> 
          <input name="success_message" id="success_message" type="checkbox" class="element-slide" <?php echo ($success_message == 1 ? 'checked' : ''); ?>/>
          
          <div class="mb-explain">
            <div class="mb-line"><?php _e('When enabled, successful message is shown after login. By default, no message is shown once user successfully login.', 'two_factor_auth'); ?></div>
          </div>
        </div>
        
        <div class="mb-row">
          <label for="hook_link"><span><?php _e('Hook Link to User Menu', 'two_factor_auth'); ?></span></label> 
          <input name="hook_link" id="hook_link" type="checkbox" class="element-slide" <?php echo ($hook_link == 1 ? 'checked' : ''); ?>/>
          
          <div class="mb-explain">
            <div class="mb-line"><?php _e('When enabled, link to configure two-factor authentication will be added to user menu.', 'two_factor_auth'); ?></div>
          </div>
        </div>
        


        <div class="mb-row">&nbsp;</div>

        <div class="mb-foot">
          <?php if(tfa_is_demo()) { ?>
            <a class="mb-button mb-has-tooltip disabled" onclick="return false;" style="cursor:not-allowed;opacity:0.5;" title="<?php echo osc_esc_html(__('This is demo site', 'two_factor_auth')); ?>"><?php _e('Save', 'two_factor_auth');?></a>
          <?php } else { ?>
            <button type="submit" class="mb-button"><?php _e('Save', 'two_factor_auth');?></button>
          <?php } ?>
        </div>
      </form>
    </div>
  </div>



  <!-- PLUGIN INTEGRATION -->
  <div class="mb-box">
    <div class="mb-head"><i class="fa fa-wrench"></i> <?php _e('Plugin Setup', 'two_factor_auth'); ?></div>

    <div class="mb-inside">
      <div class="mb-row"><?php _e('No theme modification are required to use all functions of plugin, but in order to customize position of two-factor authentication link, you may use following functions.', 'two_factor_auth'); ?></div>
      <div class="mb-row">
        <strong><?php _e('Configure link code:', 'two_factor_auth'); ?></strong>
        <span class="mb-code">&lt;?php echo tfa_configure_link($raw = false, $text = ''); ?&gt;</span>
      </div>
    </div>
  </div>
</div>


<?php echo tfa_footer(); ?>