<?php


// 2FA APP NAME
function tfa_app_name() {
  if(trim(tfa_param('app_name') != '')) {
    return trim(tfa_param('app_name'));
  } 
  
  return osc_get_domain();
}

// LOAD LIBRARIES
function tfa_load_libs() {
  spl_autoload_register(function($className) {
    include_once str_replace(array('RobThree\\Auth', '\\'), array(osc_plugins_path() . osc_plugin_folder(__FILE__) . '/src/TwoFactorAuth/lib', '/'), $className) . '.php';
  });
}


// CATCH 2FA CODE SUBMISSION
function tfa_validate_data() {
  if(Params::getParam('tfa') == 'AUTH') {
    $user_id = Params::getParam('userId');
    $secret = Params::getParam('secret');
    $code = Params::getParam('code1') . Params::getParam('code2') . Params::getParam('code3') . Params::getParam('code4') . Params::getParam('code5') . Params::getParam('code6');
  
    $user = User::newInstance()->findByPrimaryKey($user_id);
    
    if(!isset($user['pk_i_id']) || @$user['s_secret'] != Params::getParam('secret')) {
      osc_add_flash_error_message(__('Authentication data mismatch.', 'two_factor_auth'));
      header('Location:' . osc_base_url());
      exit;
    }
    
    $data = ModelTFA::newInstance()->getUserData($user['pk_i_id']);
    
    tfa_load_libs();

    // __construct($issuer = null, $digits = 6, $period = 30, $algorithm = 'sha1', IQRCodeProvider $qrcodeprovider = null, IRNGProvider $rngprovider = null, ITimeProvider $timeprovider = null)
    $tfa = new RobThree\Auth\TwoFactorAuth(tfa_app_name());

    $result = false;
    $message = '';
    
    try {
      // verifyCode($secret, $code, $discrepancy = 1, $time = null, &$timeslice = 0)
      $result = $tfa->verifyCode($data['s_secret'], $code, tfa_param('discrepancy') > 1 ? tfa_param('discrepancy') : 1);

      if($result == true) {
        ModelTFA::newInstance()->updateData($user_id, array(
          's_code' => osc_esc_html($code), 
          'dt_last_auth_date' => date('Y-m-d H:i:s'))
        );
        
        if(tfa_param('success_message') == 1) {
          osc_add_flash_ok_message(sprintf(__('Hello %s! You have successfully logged in to your account.', 'two_factor_auth'), $user['s_name']));
        }
        
        require_once LIB_PATH . 'osclass/UserActions.php';
        $uActions = new UserActions(false);
        $logged = $uActions->bootstrap_login($user_id);
      }
    } catch (RobThree\Auth\TwoFactorAuthException $e) {
      $message = $e->getMessage();
    }


    if($result != true) {
      osc_add_flash_error_message(sprintf(__('Two-factor authentication has failed, incorrect or invalid code provided. %s', 'two_factor_auth'), trim($message) <> '' ? '(' . trim($message) . ')' : ''));
      header('Location:' . osc_base_url());
      exit;
    }
  }
}

osc_add_hook('init', 'tfa_validate_data');


// CREATE LINK/BUTTON TO CONFIGURE TFA
function tfa_configure_link($raw = false, $text = '') {
  $url = osc_route_url('tfa-overview');
  
  if(!$raw) {
    return '<a href="' . $url . '" class="tfa-conf-link">' . ($text != '' ? $text : __('Two-factor authentication', 'two_factor_auth')) . '</a>';
  }
  
  return $url;
}


// CHECK IF 2FA AUTH IS REQUIRED
function tfa_auth_required($user) {
  if(!isset($user['pk_i_id'])) {
    return false;
  }
 
  $data = ModelTFA::newInstance()->getUserData($user['pk_i_id']);
  $threshold_days = tfa_param('threshold_days');
  
  if(@$data['b_enabled'] == 0) {
    return false;
  }  
  
  if($threshold_days <= 0) {
    return true;
  } else {
    $datediff = time() - strtotime($data['dt_last_auth_date']);
    $days_diff = round($datediff / (60 * 60 * 24));
    
    if($days_diff > $threshold_days) {
      return true; 
    }
  }
 
  return false;
}


// CATCH PRE-LOGIN AND DO REDIRECT FOR 2FA AUTH
function tfa_before_login() {
  $email = trim(Params::getParam('email'));

  if(osc_validate_email($email)) {
    $user = User::newInstance()->findByEmail($email);
  }
  
  if(!isset($user['pk_i_id'])) {
    $user = User::newInstance()->findByUsername($email);
  }
  
  // user not found, this is not expected!
  if(!isset($user['pk_i_id'])) {
    return false;
  }
  
  if(tfa_auth_required($user)) {
    header('Location:' . osc_route_url('tfa-auth', array('userId' => $user['pk_i_id'], 'secret' => $user['s_secret'])));
    exit;
  }
}

osc_add_hook('before_login', 'tfa_before_login');


// FORCE USER TO SETUP 2FA
function tfa_force_setup() {
  if(tfa_param('required') == 1) {
    $user_id = osc_logged_user_id();
    $data = ModelTFA::newInstance()->getUserData($user_id);

    if(@$data['b_enabled'] != 1) {
      osc_add_flash_info_message(__('In order to make your account and data more secure, please configure two-factor authentication on your account.', 'two_factor_auth'));
      header('Location:' . osc_route_url('tfa-overview'));
      exit;
    }
  }
}

osc_add_hook('after_login', 'tfa_force_setup');


// CORE FUNCTIONS
function tfa_param($name) {
  return osc_get_preference($name, 'plugin-two_factor_auth');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function tfa_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}

?>