<?php
/*
  Plugin Name: Two-Factor Authentication Plugin
  Plugin URI: https://osclasspoint.com/
  Description: Enhance security of your classifieds and enable two-factor authentication to your users.
  Version: 1.0.1
  Author: MB Themes
  Author URI: https://osclasspoint.com
  Author Email: info@osclasspoint.com
  Short Name: two_factor_auth
  Plugin update URI: two-factor-auth
  Support URI: https://forums.osclasspoint.com/
  Product Key: viI5pwIYUEk2IpfpZc40
*/

// https://github.com/RobThree/TwoFactorAuth

define('TFA_DEMO', true);

require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'model/ModelTFA.php';
require_once osc_plugins_path() . osc_plugin_folder(__FILE__) . 'functions.php';

osc_enqueue_style('tfa-user-style', osc_base_url() . 'oc-content/plugins/two_factor_auth/css/user.css?v=' . date('YmdHis'));
osc_register_script('tfa-user', osc_base_url() . 'oc-content/plugins/two_factor_auth/js/user.js?v=' . date('YmdHis'), array('jquery'));
osc_enqueue_script('tfa-user');


osc_add_route('tfa-auth', 'tfa/authenticate/([0-9]+)/(.+)', 'tfa/authenticate/{userId}/{secret}', osc_plugin_folder(__FILE__).'form/auth.php', false, 'tfa', 'authenticate');
osc_add_route('tfa-overview', 'tfa/overview', 'tfa/overview', osc_plugin_folder(__FILE__).'user/user.php', true, 'tfa', 'overview');
osc_add_route('tfa-setup', 'tfa/setup', 'tfa/setup', osc_plugin_folder(__FILE__).'user/setup.php', true, 'tfa', 'setup');
osc_add_route('tfa-reset', 'tfa/reset', 'tfa/reset', osc_plugin_folder(__FILE__).'user/reset.php', true, 'tfa', 'reset');
osc_add_route('tfa-disable', 'tfa/disable', 'tfa/disable', osc_plugin_folder(__FILE__).'user/disable.php', true, 'tfa', 'disable');


// INSTALL FUNCTION - DEFINE VARIABLES
function tfa_call_after_install() {
  osc_set_preference('required', 0, 'plugin-two_factor_auth', 'INTEGER');
  osc_set_preference('discrepancy', 1, 'plugin-two_factor_auth', 'INTEGER');
  osc_set_preference('app_name', '', 'plugin-two_factor_auth', 'STRING');
  osc_set_preference('success_message', 1, 'plugin-two_factor_auth', 'INTEGER');
  osc_set_preference('threshold_days', 7, 'plugin-two_factor_auth', 'INTEGER');
  osc_set_preference('hook_link', 1, 'plugin-two_factor_auth', 'INTEGER');

  ModelTFA::newInstance()->install();
}


function tfa_call_after_uninstall() {
  ModelTFA::newInstance()->uninstall();
}


// ADD TFA CONFIGURE LINK TO USER MENU
function tfa_user_menu_link(){
  if(tfa_param('hook_link') == 1) {
    if(osc_current_web_theme() == 'veronika' || osc_current_web_theme() == 'stela' || (defined('USER_MENU_ICONS') && USER_MENU_ICONS == 1)) {
      echo '<li class="opt_tfa"><a href="' . osc_route_url('tfa-overview') .'" ><i class="fa fa-key"></i> '.__('Two-factor authentication', 'two_factor_auth') . '</a></li>';
    } else {
      echo '<li class="opt_tfa"><a href="' . osc_route_url('tfa-overview') .'" >'.__('Two-factor authentication', 'two_factor_auth') . '</a></li>';
    }
  }
}

osc_add_hook('user_menu', 'tfa_user_menu_link');


// ADMIN MENU
function tfa_menu($title = NULL) {
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/two_factor_auth/css/admin.css?v=' . date('YmdHis') . '" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/two_factor_auth/css/bootstrap-switch.css" rel="stylesheet" type="text/css" />';
  echo '<link href="' . osc_base_url() . 'oc-content/plugins/two_factor_auth/css/tipped.css" rel="stylesheet" type="text/css" />';
  echo '<link href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet" type="text/css" />';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/two_factor_auth/js/admin.js?v=' . date('YmdHis') . '"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/two_factor_auth/js/tipped.js"></script>';
  echo '<script src="' . osc_base_url() . 'oc-content/plugins/two_factor_auth/js/bootstrap-switch.js"></script>';

  if( $title == '') { $title = __('Configure', 'two_factor_auth'); }

  $text  = '<div class="mb-head">';
  $text .= '<div class="mb-head-left">';
  $text .= '<h1>' . $title . '</h1>';
  $text .= '<h2>Two-Factor Authentication Plugin</h2>';
  $text .= '</div>';
  $text .= '<div class="mb-head-right">';
  $text .= '<ul class="mb-menu">';
  $text .= '<li><a href="' . osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=two_factor_auth/admin/configure.php"><i class="fa fa-wrench"></i><span>' . __('Configure', 'two_factor_auth') . '</span></a></li>';
  $text .= '</ul>';
  $text .= '</div>';
  $text .= '</div>';

  echo $text;
}



// ADMIN FOOTER
function tfa_footer() {
  $pluginInfo = osc_plugin_get_info('two_factor_auth/index.php');
  $text  = '<div class="mb-footer">';
  $text .= '<a target="_blank" class="mb-developer" href="https://osclasspoint.com"><img src="https://osclasspoint.com/favicon.ico" alt="OsclassPoint Market" /> OsclassPoint Market</a>';
  $text .= '<a target="_blank" href="' . $pluginInfo['support_uri'] . '"><i class="fa fa-bug"></i> ' . __('Report Bug', 'two_factor_auth') . '</a>';
  $text .= '<a target="_blank" href="https://forums.osclasspoint.com/"><i class="fa fa-handshake-o"></i> ' . __('Support Forums', 'two_factor_auth') . '</a>';
  $text .= '<a target="_blank" class="mb-last" href="mailto:info@osclasspoint.com"><i class="fa fa-envelope"></i> ' . __('Contact Us', 'two_factor_auth') . '</a>';
  $text .= '<span class="mb-version">v' . $pluginInfo['version'] . '</span>';
  $text .= '</div>';

  return $text;
}



// ADD MENU LINK TO PLUGIN LIST
function tfa_admin_menu() {
echo '<h3><a href="#">Two-Factor Authentication Plugin</a></h3>
<ul> 
  <li><a style="color:#2eacce;" href="' . osc_admin_render_plugin_url(osc_plugin_path(dirname(__FILE__)) . '/admin/configure.php') . '">&raquo; ' . __('Configure', 'two_factor_auth') . '</a></li>
</ul>';
}


// ADD MENU TO PLUGINS MENU LIST
osc_add_hook('admin_menu','tfa_admin_menu', 1);



// DISPLAY CONFIGURE LINK IN LIST OF PLUGINS
function tfa_conf() {
  osc_admin_render_plugin( osc_plugin_path( dirname(__FILE__) ) . '/admin/configure.php' );
}

osc_add_hook( osc_plugin_path( __FILE__ ) . '_configure', 'tfa_conf' );	


// CALL WHEN PLUGIN IS ACTIVATED - INSTALLED
osc_register_plugin(osc_plugin_path(__FILE__), 'tfa_call_after_install');

// SHOW UNINSTALL LINK
osc_add_hook(osc_plugin_path(__FILE__) . '_uninstall', 'tfa_call_after_uninstall');

?>