<?php
$user_id = osc_logged_user_id();
$user = User::newInstance()->findByPrimaryKey($user_id);

tfa_load_libs();

// construct($issuer = null, $digits = 6, $period = 30, $algorithm = 'sha1', IQRCodeProvider $qrcodeprovider = null, IRNGProvider $rngprovider = null, ITimeProvider $timeprovider = null)
$tfa = new RobThree\Auth\TwoFactorAuth(tfa_app_name());
$data = ModelTFA::newInstance()->getUserData($user['pk_i_id']);

$secret = @$data['s_secret'];
$message = '';

// Check if secret is already defined
if($secret == '') {
  try {
    $secret = $tfa->createSecret();
    
    if($secret != '') {
      ModelTFA::newInstance()->replaceData(array(
        'fk_i_user_id' => $user_id,
        'b_enabled' => 0,
        's_secret' => $secret,
        's_code' => null,
        'dt_last_auth_date' => null
      ));
    }
  } catch (RobThree\Auth\TwoFactorAuthException $e) {
    $message = $e->getMessage();
  }
  
  if($secret == '') {
    osc_add_flash_error_message(sprintf(__('There was problem generating secret code for your profile. %s', 'two_factor_auth'), trim($message) <> '' ? '(' . trim($message) . ')' : ''));
    header('Location:' . osc_route_url('tfa-overview'));
    exit;
  }
}

if(Params::getParam('tfa') == 'SETUP') {
  $code = Params::getParam('code1') . Params::getParam('code2') . Params::getParam('code3') . Params::getParam('code4') . Params::getParam('code5') . Params::getParam('code6');

  $result = false;
  $message = '';
    
  try {
    // verifyCode($secret, $code, $discrepancy = 1, $time = null, &$timeslice = 0)
    $result = $tfa->verifyCode($secret, $code, tfa_param('discrepancy') > 1 ? tfa_param('discrepancy') : 1);

    if($result == true) {
      ModelTFA::newInstance()->updateData($user_id, array(
        'b_enabled' => 1,
        's_code' => osc_esc_html($code),
        'dt_last_auth_date' => date('Y-m-d H:i:s'))
      );
      
      osc_add_flash_ok_message(__('Two-factor authentication has been successfully setup.', 'two_factor_auth'));
      header('Location:' . osc_route_url('tfa-overview'));
      exit;
    }
  } catch (RobThree\Auth\TwoFactorAuthException $e) {
    $message = $e->getMessage();
  }

  if($result != true) {
    osc_add_flash_error_message(sprintf(__('Two-factor authentication has failed, incorrect or invalid code provided. %s', 'two_factor_auth'), trim($message) <> '' ? '(' . trim($message) . ')' : ''));
    header('Location:' . osc_route_url('tfa-setup'));
    exit;
  }
}
?>

<div class="tfa-account tfa-body">
  <form class="tfa-form2 nocsrf" action="<?php echo osc_route_url('tfa-setup'); ?>" method="POST">
    <input type="hidden" name="tfa" value="SETUP"/>
    <input type="hidden" name="userId" value="<?php echo $user_id; ?>"/>
    <input type="hidden" name="secret" value="<?php echo $secret; ?>"/>

    <h2><?php _e('Two-factor Authentication Setup', 'two_factor_auth'); ?></h2>
    <div class="tfa-row"><?php echo sprintf(__('Scan QR code into your authentication application or enter code manually: %s', 'two_factor_auth'), '<br/><strong>' . chunk_split($secret, 4, ' ') . '</strong>'); ?></div>
    <div class="tfa-row tfa-qr">
      <img src="<?php echo $tfa->getQRCodeImageAsDataUri(osc_logged_user_email(), $secret); ?>">
    </div>
    
    <div class="tfa-row">
      <?php _e('Please enter 6-digits code from your authentication application.', 'two_factor_auth'); ?>
      
      <?php if(defined('TFA_DEMO') && TFA_DEMO == true) { echo ' (' . $tfa->getCode($secret) . ')'; } ?>
    </div>

    <div class="tfa-row tfa-row-code">
      <div class="tfa-left">
        <input name="code1" type="text" maxlength="1" required autocomplete="off" data-id="1"/>
        <input name="code2" type="text" maxlength="1" required autocomplete="off" data-id="2"/>
        <input name="code3" type="text" maxlength="1" required autocomplete="off" data-id="3"/>
      </div>
      
      <div class="tfa-center">-</div>
      
      <div class="tfa-right">
        <input name="code4" type="text" maxlength="1" required autocomplete="off" data-id="4"/>
        <input name="code5" type="text" maxlength="1" required autocomplete="off" data-id="5"/>
        <input name="code6" type="text" maxlength="1" required autocomplete="off" data-id="6"/>
      </div>
    </div>
    
    <div class="tfa-row tfa-row-buttons">
      <button type="submit" class="tfa-btn"><?php _e('Submit', 'two_factor_auth'); ?></button>
    </div>
  </form>
</div>