<?php
class ModelVCR extends DAO {
private static $instance;

public static function newInstance() {
  if( !self::$instance instanceof self ) {
    self::$instance = new self;
  }
  return self::$instance;
}

function __construct() {
  parent::__construct();
}

public function getTable_voucher() {
  return DB_TABLE_PREFIX.'t_vcr_voucher';
}

public function getTable_voucher_stats() {
  return DB_TABLE_PREFIX.'t_vcr_voucher_stats';
}

public function getTable_user() {
  return DB_TABLE_PREFIX.'t_user';
}




public function import($file) {
  $path = osc_plugin_resource($file);
  $sql = file_get_contents($path);

  if(!$this->dao->importSQL($sql) ){
    throw new Exception("Error importSQL::ModelVCR<br>" . $file . "<br>" . $this->dao->getErrorLevel() . " - " . $this->dao->getErrorDesc() );
  }
}


public function install($version = '') {
  if($version == '') {
    $this->import('voucher/model/struct.sql');

    osc_set_preference('version', 100, 'plugin-voucher', 'INTEGER');
  }
}


public function uninstall() {
  // DELETE ALL TABLES
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_voucher()));
  $this->dao->query(sprintf('DROP TABLE %s', $this->getTable_voucher_stats()));



  // DELETE ALL PREFERENCES
  $db_prefix = DB_TABLE_PREFIX;
  $query = "DELETE FROM {$db_prefix}t_preference WHERE s_section = 'plugin-voucher'";
  $this->dao->query($query);
}



// GET VOUCHERS
public function getVouchers() {
  $this->dao->select();
  $this->dao->from($this->getTable_voucher());
  $this->dao->orderby('pk_i_id DESC');

  $result = $this->dao->get();
  
  if($result) {
    return $result->result();
  }

  return array();
}


// GET VOUCHER BY ID
public function getVoucher($id) {
  $this->dao->select();
  $this->dao->from($this->getTable_voucher());
  $this->dao->where('pk_i_id', $id);

  $result = $this->dao->get();
  
  if($result) {
    return $result->row();
  }

  return array();
}



// GET VOUCHER STATS
public function getVoucherStats($voucher_id, $user_id) {
  $this->dao->select('count(*) as i_count');
  $this->dao->from($this->getTable_voucher_stats());

  $this->dao->where('fk_i_voucher_id', $voucher_id);
  $this->dao->where('fk_i_user_id', $user_id);
  
  $result = $this->dao->get();
  
  if($result) {
    return @$result->row()['i_count'];
  }
  
  return 0;
}


// REMOVE VOUCHER
public function removeVoucher($id) {
  $this->dao->delete($this->getTable_voucher(), array('pk_i_id' => $id));
  $this->dao->delete($this->getTable_voucher_stats(), array('fk_i_voucher_id' => $id));
}


// INSERT VOUCHER
public function insertVoucher() {
  $data = array(
    's_description' => __('New voucher!', 'voucher'),
    's_type' => 'PERCENT',
    's_code' => strtoupper(mb_generate_rand_string(6)),
    'i_quantity' => 100,
    'i_quantity_per_user' => 1,
    'i_quantity_used' => 0,
    'd_amount' => 25,
    'd_min_amount' => 0,
    'dt_date_from' => date('Y-m-d'),
    'dt_date_to' => date('Y-m-d', strtotime('+ 1 years')),
    's_user_ids' => '',
    's_group_ids' => '',
    's_restricted_products' => '',
    'i_active' => 0,
    'dt_date_created' => date('Y-m-d')
  );

  $this->dao->insert($this->getTable_voucher(), $data);
  return $this->dao->insertedId();
}


// UPDATE VOUCHER
public function updateVoucher($data) {
  $this->dao->update($this->getTable_voucher(), $data, array('pk_i_id' => $data['pk_i_id']));
}


}
?>