<?php


// REMOVE ALL WEBP IMAGES
function wbp_remove_webp_images() {
  $path = osc_content_path() . 'uploads/';
  $dirs = array_filter(glob($path . '*'), 'is_dir');

  if(is_array($dirs) && count($dirs) > 0) {
    foreach($dirs as $dir) {
      $dir_name = basename($dir);

      if(is_numeric($dir_name)) {
        //$files = glob($dir . '/*.webp');
        array_map('unlink', glob($dir . '/*.webp'));
      }
    }
  }  
}


// DISCOVER ALL WEBP IMAGES
function wbp_count_webp_images() {
  $path = osc_content_path() . 'uploads/';
  $dirs = array_filter(glob($path . '*'), 'is_dir');
  $count = 0;
  
  if(is_array($dirs) && count($dirs) > 0) {
    foreach($dirs as $dir) {
      $dir_name = basename($dir);

      if(is_numeric($dir_name)) {
        $files = glob($dir . '/*.webp');
        $count += count($files);
      }
    }
  }  
  
  return $count;
}


// GET IMAGE TYPES
function wbp_image_sizes() {
  $types = trim(wbp_param('types'));
  $types = ($types == '' ? 'thumbnail,preview,normal' : $types);
  $types = array_filter(array_map('trim', explode(',', $types)));
  
  return $types;  
}


// CHECK IF GD LIBRARY CAN BE USED
function wbp_can_use_gd() {
  if(function_exists('imagewebp')) {
    return true;
  }
  
  return false;
}


// CHECK IF IMAGEMAGICK CAN BE USED
function wbp_can_use_imagemagick() {
  if(extension_loaded('imagick')) {
    if(class_exists('Imagick')) {
      $formats = array_map('strtolower', Imagick::queryFormats());
      
      if(in_array('webp', $formats)) {
        return true;
      }
    }
  }
  
  return false;
}


// CHECK IF IMAGEMAGICK SHOULD BE USED
function wbp_use_imagemagick() {
  if(wbp_can_use_imagemagick()) {
    if(wbp_param('use_imagemagick') == 1) {
      return true;
    }
  }
  
  return false;  
}


// CONVERT JPG/JPEG/PNG TO WEBP
// Parameters are path to images/files
// Keep target empty to generate it from source
function wbp_image_to_webp($source, $target = '') {
  $quality = 70;  // (0 - 100)

  if(trim($source) == '' || !file_exists($source)) {
    return false;
  }
  
  $ext = strtolower((string)pathinfo($source, PATHINFO_EXTENSION));

  if($ext == '' || !in_array($ext, array('jpg','jpeg','png','gif'))) {
    return false;
  }
  
  if($target == '') {
    $target = preg_replace('"\.(jpg|jpeg|png|gif)$"', '.webp', $source);
  }

  if(wbp_can_use_gd() === true && wbp_use_imagemagick() === false) {
    if($ext == 'jpg' || $ext == 'jpeg') {
      $im = imagecreatefromjpeg($source);

    } else if($ext == 'png') {
      $im = imagecreatefrompng($source);

    } else if($ext == 'gif') {
      $im = imagecreatefromgif($source);
    }

    imagewebp($im, $target, $quality);
    imagedestroy($im);
    
  } else if(wbp_use_imagemagick() === true) {
    $im_quality = strval(round($quality/100*6));
    
    $im = new Imagick();
    $im->pingImage($source);
    $im->readImage($source);
    $im->setImageFormat('webp');
    $im->setOption('webp:method', $im_quality); 
    $im->writeImage($target);           
  }
  
  return $target;
}


// GENERATE WEBP IMAGES ONCE RESOURCE IS CREATED OR LOADED
function wbp_generate_item_resource_webp($resource) {
  // Link: osc_base_url() . $resource['s_path'] . $resource['pk_i_id'] . '.' . $resource['s_extension'];
  $quality = 70;  // (0 - 100)
  $sizes = wbp_image_sizes();
  
  if(count($sizes) > 0 && isset($resource['pk_i_id']) && $resource['pk_i_id'] > 0) {
    foreach($sizes as $size) {
      if($size == 'normal') {
        $slug = '';
      } else {
        $slug = '_' . $size;
      }
      
      $source = osc_base_path() . $resource['s_path'] . $resource['pk_i_id'] . $slug . '.' . $resource['s_extension'];

      $res = wbp_image_to_webp($source);   // return false if failed or there was problem
    }
  }
}

osc_add_hook('uploaded_file', 'wbp_generate_item_resource_webp');
osc_add_hook('regenerated_image', 'wbp_generate_item_resource_webp');
//osc_run_hook('uploaded_file', ItemResource::newInstance()->findByPrimaryKey(80));


// REMOVE WEBP IMAGES ONCE RESOURCE IS REMOVED
function wbp_remove_item_resource_webp($resource) {
  $sizes = wbp_image_sizes();
  
  if(count($sizes) > 0 && isset($resource['pk_i_id']) && $resource['pk_i_id'] > 0) {
    foreach($sizes as $size) {
      if($size == 'normal') {
        $slug = '';
      } else {
        $slug = '_' . $size;
      }
      
      $source = osc_base_path() . $resource['s_path'] . $resource['pk_i_id'] . $slug . '.webp';
      
      if(file_exists($source)) {
        @unlink($source); 
      }
    }
  }
}

osc_add_hook('delete_resource', 'wbp_remove_item_resource_webp');


// CONVERT URL TO PATH
function wbp_url_to_path($url) {
  if($url == '') {
    return false;
  }

  if(strpos($url, osc_base_url()) !== false) {    // file can be on external storage
    $path = preg_replace('/^' . preg_quote(osc_base_url(), '/') . '/', '', $url);
    return $path;
  } 
  
  return false;
}


// CONVERT PATH TO URL
function wbp_path_to_url($path) {
  if($path == '') {
    return false;
  }

  if(strpos($path, osc_base_path()) !== false) {    // file can be on external storage
    $url = preg_replace('/^' . preg_quote(osc_base_path(), '/') . '/', '', $path);
    return $url;
  }
  
  return false;
}


// GET WEBP VARIANT OF IMAGE, GENERATE IF DOES NOT EXISTS
// input param is URL of image
// return $source_url if there are problems to get target
function wbp_get_webp_img($source_url, $generate_if_missing = true, $generate_force = false) {
  if(trim((string)$source_url) == '') {
    return $source_url;
  }
  
  $source_path = wbp_url_to_path($source_url);
  
  $ext = strtolower((string)pathinfo($source_url, PATHINFO_EXTENSION));     // $source_path may be false

  if($ext == '' || !in_array($ext, array('jpg','jpeg','png','gif'))) {
    return $source_url;
  }
  
  $target_url = preg_replace('"\.(jpg|jpeg|png|gif)$"', '.webp', $source_url);
  $target_path = false;
  
  if($source_path !== false) {
    $target_path = preg_replace('"\.(jpg|jpeg|png|gif)$"', '.webp', $source_path);
  }

  if($source_path === false || (file_exists($target_path) && !$generate_force)) {
    return $target_url;
  } else if($source_path !== false && $target_path !== false && ($generate_if_missing || $generate_force)) {
    $res = wbp_image_to_webp($source_path, $target_path);
    
    if($res !== false) {
      return $target_url;
    }
  }
  
  return $source_url;
}


// CREATE HTML TAG WITH PROPER SUPPORT
function wbp_resource_tag($resource, $type = 'thumbnail', $alt = NULL, $class = '') {
  $ext = strtolower(@$resource['s_extension'] <> '' ? $resource['s_extension'] : 'jpeg');
  $alt = ($alt <> '' ? $alt : sprintf(__('Image #%s', 'image_generator'), isset($resource['pk_i_id']) ? $resource['pk_i_id'] : 0));
  $class = (trim($class) <> '' ? ' ' . trim($class) : '');
  ?>
  <picture class="igt-image<?php echo $class; ?>" data-id="<?php echo (isset($resource['pk_i_id']) ? $resource['pk_i_id'] : ''); ?>">
    <source srcset="<?php echo wbp_resource_url($resource, $type, 'webp'); ?>" type="image/webp">
    <source srcset="<?php echo wbp_resource_url($resource, $type); ?>" type="image/<?php echo $ext; ?>"> 
    <img src="<?php echo wbp_resource_url($resource, $type); ?>" alt="<?php echo osc_esc_html($alt); ?>">
  </picture>
  <?php  
}


// CREATE THUMBNAIL HTML TAG
function wbp_thumbnail_resource_tag($resource = NULL, $alt = '', $class = '') {
  $resource = ($resource == NULL ? osc_resource() : $resource);
  $alt = ($alt == '' ? osc_item_title() : $alt);
  
  return wbp_resource_tag($resource, 'thumbnail', $alt, $class);
}

// CREATE PREVIEW HTML TAG
function wbp_preview_resource_tag($resource = NULL, $alt = '', $class = '') {
  $resource = ($resource == NULL ? osc_resource() : $resource);
  $alt = ($alt == '' ? osc_item_title() : $alt);
  
  return wbp_resource_tag($resource, 'preview', $alt, $class);
}

// CREATE THUMBNAIL HTML TAG
function wbp_normal_resource_tag($resource = NULL, $alt = '', $class = '') {
  $resource = ($resource == NULL ? osc_resource() : $resource);
  $alt = ($alt == '' ? osc_item_title() : $alt);
  
  return wbp_resource_tag($resource, 'normal', $alt, $class);
}


// CREATE URL TO RESOURCE
function wbp_resource_url($resource, $type = 'thumbnail', $ext = NULL) {
  $generate_if_missing = (wbp_param('generate_if_missing') == 1 ? true : false);

  if(isset($resource['pk_i_id']) && $resource['pk_i_id'] > 0) {
    $ext = ($ext == NULL ? $resource['s_extension'] : $ext);
    $slug = ($type == 'normal' ? '' : '_' . $type);
    $source_url = osc_base_url() . $resource['s_path'] . $resource['pk_i_id'] . $slug . '.' . $resource['s_extension'];
    $target_url = osc_base_url() . $resource['s_path'] . $resource['pk_i_id'] . $slug . '.' . $ext;
    
    if($ext == 'webp') {
      return wbp_get_webp_img($source_url, $generate_if_missing);
    } else {
      return $target_url;
    }
  }
  
  return false;  
}


// REPLACE FILE EXTENSION
function wbp_replace_extension($source_url, $ext = 'webp') {
  $generate_if_missing = (wbp_param('generate_if_missing') == 1 ? true : false);
  return wbp_get_webp_img($source_url, $generate_if_missing);
}


// APPLY FILTERS
function wbp_filters() {
  if(wbp_param('replace_resource_thumbnail') == 1) {
    osc_add_filter('resource_thumbnail_url', 'wbp_replace_extension', 8);
  }
  
  if(wbp_param('replace_resource_preview') == 1) {
    osc_add_filter('resource_preview_url', 'wbp_replace_extension', 8);
  }
  
  if(wbp_param('replace_resource_normal') == 1) {
    osc_add_filter('resource_url', 'wbp_replace_extension', 8);
  }
}

osc_add_hook('init', 'wbp_filters');




// GENERATE ADMIN PAGINATION
function wbp_admin_paginate($file, $page_id, $per_page, $count_all, $class = '', $params = '') {
  $html = '';
  $page_id = (int)$page_id;
  $page_id = ($page_id <= 0 ? 1 : $page_id);
  $base_link = osc_admin_base_url(true) . '?page=plugins&action=renderplugin&file=' . $file . $params;

  if($per_page < $count_all) {
    $html .= '<div id="mb-pagination" class="' . $class . '">';
    $html .= '<div class="mb-pagination-wrap">';
    $html .= '<div>' . __('Page:', 'webp_images') . '</div>';

    $pages = ceil($count_all/$per_page); 
    $page_actual = ($page_id == '' ? 1 : $page_id);

    if($pages > 6) {

      // Too many pages to list them all
      if($page_id == 1) { 
        $ids = array(1,2,3, $pages);

      } else if ($page_id > 1 && $page_id < $pages) {
        $ids = array(1,$page_id-1, $page_id, $page_id+1, $pages);

      } else {
        $ids = array(1, $page_id-2, $page_id-1, $page_id);
      }

      $old = -1;
      $ids = array_unique(array_filter($ids));

      foreach($ids as $i) {
        $url = $base_link . '&pageId=' . $i;
        
        if($old <> -1 && $old <> $i - 1) {
          $html .= '<span>&middot;&middot;&middot;</span>';
        }

        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="mb-active"' : '') . '>' . $i . '</a>';
        $old = $i;
      }

    } else {

      // List all pages
      for ($i = 1; $i <= $pages; $i++) {
        $url = $base_link . '&pageId=' . $i;
        $html .= '<a href="' . $url . '" ' . ($page_actual == $i ? 'class="mb-active"' : '') . '>' . $i . '</a>';
      }
    }

    $html .= '</div>';
    $html .= '</div>';
  }

  return $html;
}


// CORE FUNCTIONS
function wbp_param($name) {
  return osc_get_preference($name, 'plugin-webp_images');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function wbp_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}




// CATEGORIES WORK
function wbp_cat_tree($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
      $array[$c['pk_i_id']]['sub'] = wbp_cat_sub($list, $c['pk_i_id']);
    }
  }

  return $array;
}

function wbp_cat_sub($list, $parent_id) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {  echo $c['s_name'];
        $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
        $array[$c['pk_i_id']]['sub'] = wbp_cat_sub($list, $c['pk_i_id']);
      }
    }
  }
      
  return $array;
}

function wbp_cat_list($selected = array(), $categories = '', $level = 0) {
  if($categories == '' || $level == 0) {
    $categories = wbp_cat_tree($categories);
  }


  foreach($categories as $c) {
    echo '<option value="' . $c['pk_i_id'] . '" ' . (in_array($c['pk_i_id'], $selected) ? 'selected="selected"' : '') . '>' . str_repeat('-', $level) . ($level > 0 ? ' ' : '') . $c['s_name'] . '</option>';

    if(@count($c['sub']) > 0) {
      wbp_cat_list($selected, $c['sub'], $level + 1);
    }
  }
}


function wbp_list_values_ol($values) {
 if(count($values) > 0 && is_array($values)) {
    foreach($values as $v) {
      ?>

      <li class="mb-val" id="val_<?php echo $v['pk_i_id']; ?>">
        <?php wbp_div_value($v); ?>
      
        <ol>
          <?php 
            if(isset($v['values']) && count($v['values']) > 0) { 
              wbp_list_values_ol($v['values']); 
            }
          ?>
        </ol>
      </li>
    <?php
    }
  }
}

?>