<?php

// CREATE VIDEO FORM
function ytb_video_form($cat_id = NULL, $item_id = NULL) {
  if($cat_id <= 0) {
    $cat_id = -1;
  }
  
  if($item_id <= 0) {
    $item_id = Params::getParam('itemId');
  }
  
  $category_ids = array_filter(explode(',', ytb_param('category')));
  
  if(ytb_param('category') == '' || in_array($cat_id, $category_ids)) {
    $url = ytb_video_url($item_id);

    $html  = '<div class="row ytb-row">';
    $html .= '<label for="ytb-video-url">' . __('Youtube video', 'youtube') . '</label>';
    $html .= '<input type="text" class="ytb-video-url" id="ytb-video-url" name="s_video_url" value="' . $url . '" placeholder="' . osc_esc_js(__('Enter youtube video url, i.e. https://www.youtube.com/watch?v=s8FKf9AK1Pc', 'youtube')) . '" />';
    $html .= '</div>';

    return $html;
  }
}

// PREPARE VIDEO FORM HOOK
function ytb_video_form_hook($cat_id = NULL, $item_id = NULL) {
  if(ytb_param('hook_post') == 1) {
    echo ytb_video_form($cat_id, $item_id);
  }
}

osc_add_hook('item_form', 'ytb_video_form_hook');
osc_add_hook('item_edit', 'ytb_video_form_hook');



// GET ITEM VIDEO LINK
function ytb_video_url($item_id) {
  if($item_id > 0) {
    $row = ModelYTB::newInstance()->getVideoByItemId($item_id); 
    return ytb_create_url(@$row['s_code']);
  }

  return '';
}

// GET VIDEO BOX
function ytb_video_box($item_id, $category_id = NULL) {
  $html = '';
  
  if($item_id <= 0) {
    return false;
  }
  
  if($category_id <= 0) {
    $item = Item::newInstance()->findByPrimaryKey($item_id);
    $category_id = $item['fk_i_category_id'];
  }
  
  $category_ids = array_filter(explode(',', ytb_param('category')));
  
  if(ytb_param('category') == '' || in_array($category_id, $category_ids)) {
    $url = ytb_video_url($item_id);
    
    $width = (ytb_param('width') > 0 ? ytb_param('width') : 560);
    $height = (ytb_param('height') > 0 ? ytb_param('height') : 315);
    
    if($url != '') {
      $html .= '<style>';
      $html .= '.ytb-video-box, #plugin-details > div.ytb-video-box {float:left;clear:both;width:' . $width . 'px;margin:10px 0 30px 0;background:#f0f0f0;position:relative;border-radius:3px;overflow:hidden;}';
      $html .= '.ytb-video-box:after, #plugin-details > div.ytb-video-box:after {z-index:2;content:"' . __('Loading video ...', 'youtube') . '";color:rgba(0,0,0,0.3);width:100%;text-align:center;line-height:30px;margin-top:-15px;position:absolute;top:50%;left:0;font-size:16px;font-weight:500;}';
      $html .= '.ytb-video-box iframe {position:relative;z-index:3;}';
      $html .= '@media screen and (max-width: 1200px) {';
      $html .= '.ytb-video-box, #plugin-details > div.ytb-video-box { position: relative; padding-bottom: 56.25%;padding-top: 25px; height: 0;float:left;clear:both;width:100%;max-width:560px;}';
      $html .= '.ytb-video-box iframe { position: absolute; top: 0; left: 0; width: 100%; height: 100%;}';
      $html .= '}';
      $html .= '</style>';

      $html .= '<div class="ytb-video-box">';
      $html .= '<iframe loading="lazy" width="' . $width . '" height="' . $height . '" src="' . $url . '" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>';
      $html .= '</div>';    
    }

    return $html;
  }
}


// PREPARE VIDEO BOX FOR HOOK
function ytb_video_box_hook($item) {
  if(ytb_param('hook_item') == 1 && isset($item['pk_i_id'])) {
    echo ytb_video_box($item['pk_i_id'], $item['fk_i_category_id']);
  }
}

osc_add_hook('item_detail', 'ytb_video_box_hook');



// SAVE YOUTUBE VIDEO WHEN LISTING IS POSTED OR EDITED
function ytb_save_video($item) {
  $url = Params::getParam('s_video_url');
  
  if($url <> '' || @$item['pk_i_id'] > 0) {
    $url = ytb_embedded_url($url);
    $code = ytb_get_video_code($url);
    
    if($code <> '') {
      ModelYTB::newInstance()->updateVideoByItemId($item['pk_i_id'], $code, $url);
    } else if (@$item['pk_i_id'] > 0) {
      ModelYTB::newInstance()->removeVideoByItemId($item['pk_i_id']);
    } else {
      osc_add_flash_error_message(__('There was problem saving your video, please edit your listing and add video URL again', 'youtube'));
    }
  }
}


osc_add_hook('posted_item', 'ytb_save_video');
osc_add_hook('edited_item', 'ytb_save_video');


// FIX URL TO STANDARD EMBEDDED FORMAT
function ytb_embedded_url($url) {
  if(preg_match('|.*?youtube.*?v[\?/=](.{11})|', $url)) {
    return preg_replace('|.*?youtube.*?v[\?/=](.{11}).*|', 'https://www.youtube.com/v/$01?fs=1', $url);
    
  } else if(preg_match('|.*?youtu\.be\/(.{11})|', $url)) {
    return preg_replace('|.*?youtu\.be\/(.{11}).*|', 'https://www.youtube.com/v/$01?fs=1', $url);
  }

  return $url;
}


// CREATE EMBEDDED URL FROM CODE
function ytb_create_url($code) {
  if($code <> '') {
    return 'https://www.youtube.com/embed/' . $code;
  }
}


// GET CODE FROM STANDARD YOUTUBE URL
function ytb_get_video_code($url) {
  $id = null;
  
  if (preg_match('/youtube\.com\/watch\?v=([^\&\?\/]+)/', $url, $id)) {
    return $id[1];
  } else if (preg_match('/youtube\.com\/embed\/([^\&\?\/]+)/', $url, $id)) {
    return $id[1];
  } else if (preg_match('/youtube\.com\/v\/([^\&\?\/]+)/', $url, $id)) {
    return $id[1];
  } else if (preg_match('/youtu\.be\/([^\&\?\/]+)/', $url, $id)) {
    return $id[1];
  } else if (preg_match('/youtube\.com\/verify_age\?next_url=\/watch%3Fv%3D([^\&\?\/]+)/', $url, $id)) {
    return $id[1];
  }
}



// CORE FUNCTIONS
function ytb_param($name) {
  return osc_get_preference($name, 'plugin-ytb');
}


if(!function_exists('mb_param_update')) {
  function mb_param_update( $param_name, $update_param_name, $type = NULL, $plugin_var_name = NULL ) {
  
    $val = '';
    if( $type == 'check') {

      // Checkbox input
      if( Params::getParam( $param_name ) == 'on' ) {
        $val = 1;
      } else {
        if( Params::getParam( $update_param_name ) == 'done' ) {
          $val = 0;
        } else {
          $val = ( osc_get_preference( $param_name, $plugin_var_name ) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
        }
      }
    } else {

      // Other inputs (text, password, ...)
      if( Params::getParam( $update_param_name ) == 'done' && Params::existParam($param_name)) {
        $val = Params::getParam( $param_name );
      } else {
        $val = ( osc_get_preference( $param_name, $plugin_var_name) != '' ) ? osc_get_preference( $param_name, $plugin_var_name ) : '';
      }
    }


    // If save button was pressed, update param
    if( Params::getParam( $update_param_name ) == 'done' ) {

      if(osc_get_preference( $param_name, $plugin_var_name ) == '') {
        osc_set_preference( $param_name, $val, $plugin_var_name, 'STRING');  
      } else {
        $dao_preference = new Preference();
        $dao_preference->update( array( "s_value" => $val ), array( "s_section" => $plugin_var_name, "s_name" => $param_name ));
        osc_reset_preferences();
        unset($dao_preference);
      }
    }

    return $val;
  }
}


// CHECK IF RUNNING ON DEMO
function ytb_is_demo() {
  if(osc_logged_admin_username() == 'admin') {
    return false;
  } else if(isset($_SERVER['HTTP_HOST']) && (strpos($_SERVER['HTTP_HOST'],'mb-themes') !== false || strpos($_SERVER['HTTP_HOST'],'abprofitrade') !== false)) {
    return true;
  } else {
    return false;
  }
}


if(!function_exists('message_ok')) {
  function message_ok( $text ) {
    $final  = '<div class="flashmessage flashmessage-ok flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}


if(!function_exists('message_error')) {
  function message_error( $text ) {
    $final  = '<div class="flashmessage flashmessage-error flashmessage-inline">';
    $final .= $text;
    $final .= '</div>';
    echo $final;
  }
}



// COOKIES WORK
if(!function_exists('mb_set_cookie')) {
  function mb_set_cookie($name, $val) {
    Cookie::newInstance()->set_expires( 86400 * 30 );
    Cookie::newInstance()->push($name, $val);
    Cookie::newInstance()->set();
  }
}


if(!function_exists('mb_get_cookie')) {
  function mb_get_cookie($name) {
    return Cookie::newInstance()->get_value($name);
  }
}

if(!function_exists('mb_drop_cookie')) {
  function mb_drop_cookie($name) {
    Cookie::newInstance()->pop($name);
  }
}


if(!function_exists('mb_generate_rand_string')) {
  function mb_generate_rand_string($length = 10) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = strlen($characters);
    $randomString = '';

    for ($i = 0; $i < $length; $i++) {
      $randomString .= $characters[rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }
}



// CATEGORIES WORK
function ytb_cat_tree($list = array()) {
  if(!is_array($list) || empty($list)) {
    $list = Category::newInstance()->listAll();
  }

  $array = array();
  //$root = Category::newInstance()->findRootCategoriesEnabled();

  foreach($list as $c) {
    if($c['fk_i_parent_id'] <= 0) {
      $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
      $array[$c['pk_i_id']]['sub'] = ytb_cat_sub($list, $c['pk_i_id']);
    }
  }

  return $array;
}

function ytb_cat_sub($list, $parent_id) {
  $array = array();
  //$cats = Category::newInstance()->findSubcategories($id);

  if(count($list) > 0) {
    foreach($list as $c) {
      if($c['fk_i_parent_id'] == $parent_id) {  echo $c['s_name'];
        $array[$c['pk_i_id']] = array('pk_i_id' => $c['pk_i_id'], 's_name' => $c['s_name']);
        $array[$c['pk_i_id']]['sub'] = ytb_cat_sub($list, $c['pk_i_id']);
      }
    }
  }
      
  return $array;
}

function ytb_cat_list($selected = array(), $categories = '', $level = 0) {
  if($categories == '' || $level == 0) {
    $categories = ytb_cat_tree($categories);
  }


  foreach($categories as $c) {
    echo '<option value="' . $c['pk_i_id'] . '" ' . (in_array($c['pk_i_id'], $selected) ? 'selected="selected"' : '') . '>' . str_repeat('-', $level) . ($level > 0 ? ' ' : '') . $c['s_name'] . '</option>';

    if(@count($c['sub']) > 0) {
      ytb_cat_list($selected, $c['sub'], $level + 1);
    }
  }
}


function ytb_list_values_ol($values) {
 if(count($values) > 0 && is_array($values)) {
    foreach($values as $v) {
      ?>

      <li class="mb-val" id="val_<?php echo $v['pk_i_id']; ?>">
        <?php ytb_div_value($v); ?>
      
        <ol>
          <?php 
            if(isset($v['values']) && count($v['values']) > 0) { 
              ytb_list_values_ol($v['values']); 
            }
          ?>
        </ol>
      </li>
    <?php
    }
  }
}


?>